/**
 * @file mainwindow.h
 * @brief client
 * @author Bagrova Polina
 */
#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QButtonGroup>
#include <QActionGroup>
#include <QTimer>
#include <QLabel>
#include "common.h"
#include <QPushButton>
#include <QKeyEvent>
#include<QMouseEvent>

QT_BEGIN_NAMESPACE
namespace Ui
{
    class MainWindow;
}
QT_END_NAMESPACE

class CCameraManager;
class CDeviceManager;
class CWebSocketClient;
class CGPSManager;
class CMcAdptManager;
class CdmmManager;
class ITcpModule;
//这个类主要是为了屏蔽点击,防止状态按钮被误点击导致指示错误
class CIconButton : public QPushButton
{
    Q_OBJECT
public:
    CIconButton(QWidget *parent = nullptr)
        : QPushButton(parent)
    {
    }

    ~CIconButton() {}

public slots:
    void mousePressEvent(QMouseEvent *)
    {
    }
};

class MainWindow : public QMainWindow
{
    Q_OBJECT

private:
    Ui::MainWindow *ui;

public:
    /**
     * @brief Client constructor
     * @param parent
     */
    MainWindow(QWidget *parent = nullptr);
    /**
     * @brief Client destructor
     */
    ~MainWindow();

    int Initialize();

signals:

public slots:
    // 点击设置按钮
    void on_actSetting_triggered();

    // 点击显示单一画面
    void on_actionactShowOneCam_triggered();

    // 显示登陆界面
    void onShowLogin();

    // 心跳命令
    bool ping();

    // 发送状态信息
    bool sendCommonStatus();

    // 上报GPS 速度信息
    bool sendGpsSpeed();

    // 关闭程序
    void closeApp();

    // gps遮挡消息处理
    void onGpsMasked(bool bMasked);

    // 车速异常消息处理
    void onSpeedAlarm(bool bAlarmd);

    // 网络异常处理
    void onNetworkAlarm(bool bAlarmd);

    // 手咪异常消息处理
    void onAudioStatus(int status);
	//检查模块消息处理
	void on4GStatus(int status);
	void oncardStatus(int status);
	void onhardStatus(int status);
	void onsoftStatus(int status);
	void ondisk(int status);

	void onaiStatus(int status);
	void onScreenshop(int bscreen);
    // 配置变更
    void onConfigChanged();

protected slots:
    void keyPressEvent(QKeyEvent *event);
	virtual void mousePressEvent(QMouseEvent *ev);
    void onSettingClicked();

private slots:
    void onUpdateImage(int idx, QPixmap &pixmap);

protected:
    void setCamViewStyle(QWidget *pWidget);

    // 更新状态
    void updateRuningStatus();
	//更新检查状态
	void updateCheckStatus();
	void updatespeedStatus();

protected:
	ITcpModule *m_pTcpModule;
    //视频管理模块
    CCameraManager *m_pVideoMng;
    // 设备管理模块
    CDeviceManager *m_pDevMng;
	CdmmManager *m_pCheckMng;
    CGPSManager *m_pGpsMng;
    CMcAdptManager *m_pAudioMng;

    QButtonGroup m_btnCamGroup;

    QActionGroup m_actCamGroup;
    CWebSocketClient *m_pWebSock;
	qint64 m_frameCon;

    QLabel *m_pWinShow[MAX_CAMERA_COUNT];
};
#endif // MAINWINDOW_H
