#include "CCameraDevice.h"
#include "assert.h"
#include <QDateTime>
#include "camera/CCameraManager.h"
#include <gst/video/video.h>
#include "base_module.h"
#include <QDebug>
#include "socket/TcpServer.h"
/*
CCamConfig::CCamConfig()
    : m_camType(CAMERA_TYPE_RTSP),
      m_PrefIdx(0),
      m_strUsername("admin"),
      m_strPassword("network123")
{
}


CCamConfig::CCamConfig(QString &strDev)
    : m_PrefIdx(0),
      m_strUsername("admin"),
      m_strPassword("network123")
{
    m_strCamDev = strDev;
    // v4l2
    if (strDev.left(4) == "/dev/video")
    {
        m_camType = CAMERA_TYPE_V4L2;
    }
    // file
    else if (strDev.left(1) == "/")
    {
        m_camType = CAMERA_TYPE_FILE;
    }
    // web
    else
    {
        m_camType = CAMERA_TYPE_RTSP;
    }
}
*/

CCamConfig::CCamConfig()
    : m_camType(CAMERA_TYPE_RTSP),
      m_PrefIdx(0),
      m_strUsername("admin"),
      m_strPassword("network123")
{
}

CCamConfig::CCamConfig(QString &strDev)
    : m_PrefIdx(0),
      m_strUsername("admin"),
      m_strPassword("network123")
{
    m_strCamDev = strDev;
    // v4l2
    if (strDev.left(4) == "/dev/video")
    {
        m_camType = CAMERA_TYPE_V4L2;
    }
    // file
    else if (strDev.left(1) == "/")
    {
        m_camType = CAMERA_TYPE_FILE;
    }
    // web
    else
    {
        m_camType = CAMERA_TYPE_RTSP;
    }
}


CCamConfig::CCamConfig(CCamConfig &cam_cfg)
{
    // 设备地址
    m_strCamDev = cam_cfg.m_strCamDev;

    // 偏好位置
    m_PrefIdx = cam_cfg.m_PrefIdx;

    // 相机登陆用户名
    m_strUsername = cam_cfg.m_strUsername;
    // 相机登陆密码
    m_strPassword = cam_cfg.m_strPassword;

    // v4l2
    if (m_strCamDev.left(10) == "/dev/video")
    {
        m_camType = CAMERA_TYPE_V4L2;
    }
    // file
    else if (m_strCamDev.left(1) == "/")
    {
        m_camType = CAMERA_TYPE_FILE;
    }
    // web
    else
    {
        m_camType = CAMERA_TYPE_RTSP;
    }
}

// 获取网络摄像头设备主地址
QString CCamConfig::getWebCamVideoMain()
{
    QString strSrc = "rtsp://" + m_strUsername + ":" + m_strPassword + "@" + m_strCamDev + ":554/h264/ch1/main/av_stream";
	//QString strSrc = "rtsp://" + m_strUsername + ":" + m_strPassword + "@" + m_strCamDev + ":554/cam/realmonitor?channel=1&subtype=0";user admin passwd abcd1234 
	printf("getWebCamVideoMain %s/n",strSrc);
    return strSrc;
}

// 获取默认主码流地址
QString CCamConfig::getVideoMain()
{
    if (m_strCamDev.isEmpty())
    {
        return "";
    }

    switch (m_camType)
    {
    case CAMERA_TYPE_V4L2:
        return m_strCamDev;
        break;
    case CAMERA_TYPE_FILE:
        return m_strCamDev;
        break;
    case CAMERA_TYPE_RTSP:
        return getWebCamVideoMain();
        break;
    default:
        break;
    }

    return "";
}

// 获取默认副码流地址
QString CCamConfig::getVideoSub()
{
    if (m_strCamDev.isEmpty())
    {
        return "";
    }

    switch (m_camType)
    {
    case CAMERA_TYPE_V4L2:
        return m_strCamDev;
        break;
    case CAMERA_TYPE_FILE:
        return m_strCamDev;
        break;
    case CAMERA_TYPE_RTSP:
        return "rtsp://" + m_strUsername + ":" + m_strPassword + "@" + m_strCamDev + ":554/h264/ch1/sub/av_stream";
        break;
    default:
        break;
    }

    return "";
}

// 获取默认gst source device
QString CCamConfig::getSourceDev()
{
    if (m_strCamDev.isEmpty())
    {
        return "";
    }

    QString strGstSrc;
    QString strDec;
    switch (m_camType)
    {
    case CAMERA_TYPE_V4L2:
        strGstSrc = QString("v4l2src device=%1 ! image/jpeg, width=(int)1920, height=(int)1080, framerate=(fraction)30/1").arg(m_strCamDev);
        break;
    case CAMERA_TYPE_FILE:
        strGstSrc = QString("filesrc location=%1").arg(m_strCamDev);
        break;
    case CAMERA_TYPE_RTSP:
        strGstSrc = QString("rtspsrc location=%1  latency=0 ! rtph264depay").arg(getVideoMain());
        break;
    default:
        break;
    }

    return strGstSrc + strDec;
}

// 获取设备地址
QString CCamConfig::getCamDev()
{
    return m_strCamDev;
}

// 设置设备地址 /dev/video[x]  /nvme0/xxxx rtsp://
void CCamConfig::setCamDev(QString cam_dev)
{
    m_strCamDev = cam_dev;
    // v4l2
    if (m_strCamDev.left(10) == "/dev/video")
    {
        m_camType = CAMERA_TYPE_V4L2;
    }
    // file
    else if (m_strCamDev.left(1) == "/")
    {
        m_camType = CAMERA_TYPE_FILE;
    }
    // rtsp
    else if (m_strCamDev.left(7) == "rtsp://")
    {
        m_camType = CAMERA_TYPE_RTSP;
    }
}

//
CCamConfig &CCamConfig::operator=(CCamConfig &cam_cfg)
{
    if (this == &cam_cfg)
    {
        return *this;
    }
    // 设备地址
    m_strCamDev = cam_cfg.m_strCamDev;
    // 主码流
    m_strVideoMain = cam_cfg.m_strVideoMain;
    // 副码流
    m_strVideoSub = cam_cfg.m_strVideoSub;

    // v4l2
    if (m_strCamDev.left(10) == "/dev/video")
    {
        m_camType = CAMERA_TYPE_V4L2;
    }
    // file
    else if (m_strCamDev.left(1) == "/")
    {
        m_camType = CAMERA_TYPE_FILE;
    }
    // web
    else
    {
        m_camType = CAMERA_TYPE_RTSP;
    }
    return cam_cfg;
}

//
bool CCamConfig::operator==(const CCamConfig &cam_cfg)
{
    if (m_strCamDev == cam_cfg.m_strCamDev &&
        m_strVideoMain == cam_cfg.m_strVideoMain &&
        m_strVideoSub == cam_cfg.m_strVideoSub)
    {
        return true;
    }

    return false;
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// //////////////////////////////////////////              CCameraDevice               /////////////////////////////////////////////
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CCameraDevice::CCameraDevice(QObject *parent)
    : QObject(parent),
      m_winIdx(0),
      m_iStatus(CAMERA_STATUS_INIT),
      m_pCamMng(nullptr),
      m_pTcpModule(nullptr),
      m_iSnapshotInterval(1000),
      m_iSnapshotLeft(0),
      m_pVideoPipe(nullptr),
      m_pGb28181Pipe(nullptr),
      m_pJt808Pipe(nullptr),
      m_pRtmpPipe(nullptr),
      m_pJt808HisPipe(nullptr)
{
    m_pVideoPipe = new CRtsp2RecordPipe();
    m_pGb28181Pipe = new CRtsp2GB28181Pipe();
    m_pJt808Pipe = new CRtsp2Jt808Pipe();
    m_pRtmpPipe = new CRtsp2RtmpPipe();
    m_pJt808HisPipe = new CFile2Jt808Pipe();
}

CCameraDevice::~CCameraDevice()
{
    stopRun();
    SAFE_DELETE(m_pVideoPipe);
    SAFE_DELETE(m_pGb28181Pipe);
    SAFE_DELETE(m_pJt808Pipe);
    SAFE_DELETE(m_pRtmpPipe);
    SAFE_DELETE(m_pJt808HisPipe);
}

pthread_t CCameraDevice::startupTask(PTHREAD_FUNC task, void *pThis)
{
    pthread_t thread_id;
#ifdef WIN32
    //起侦听线程
    _beginthreadex(nullptr, 0, (_beginthreadex_proc_type)task, pThis, 0, &thread_id);
#else
    //起侦听线程
    pthread_create(&thread_id, nullptr, task, pThis);
#endif
    return thread_id;
}

// 设置相机配置信息
bool CCameraDevice::setCamConfig(CCamConfig &cam_cfg)
{
    if (*this == cam_cfg)
    {
        return true;
    }

    (CCamConfig &)(*this) = cam_cfg;

    return true;
}

//设置输入源信息
void CCameraDevice::setSource(QString strSrcPath)
{
    assert(m_pVideoPipe != nullptr);
    m_pVideoPipe->setSource(strSrcPath);
}

// 是否关闭
bool CCameraDevice::isCameraIdel()
{
    return (m_iStatus == CAMERA_STATUS_IDEL || m_iStatus == CAMERA_STATUS_INIT) ? true : false;
}

bool CCameraDevice::isLaunchable()
{
    // 如果处于非中断态，则退出
    if (false == isCameraIdel())
    {
        return false;
    }

    return true;
}

//设置显示窗口ID
void CCameraDevice::setWindowID(WId CamWID)
{
    assert(m_pVideoPipe);
    m_pVideoPipe->setWID(CamWID);
}

QString CCameraDevice::getSavePath(QDateTime &tm, QString &file_name)
{
    QString video_path = m_pCamMng->getVideoPath();
    file_name = QString("%1_Cam%2.mp4").arg(tm.toString("yyyy-MM-dd_hh-mm")).arg(m_winIdx + 1);
    return video_path + file_name;
}

bool CCameraDevice::startRun()
{
    assert(m_pCamMng);
    QDateTime tmBegin = QDateTime::currentDateTime();
    QString file_name;
    QString file_path = getSavePath(tmBegin, file_name);
    QString strGstCmd;
    // 检查是否满足启动条件
    if (false == isLaunchable())
    {
        return false;
    }

    // 将状态设置为启动中
    m_iStatus = CAMERA_STATUS_LAUNCHING;
    m_pVideoPipe->setFileSavePath(file_path);

    if (m_pVideoPipe->startPipe())
    {
        if (!file_name.isEmpty())
        {
            int interval = m_pCamMng->getVideoInterval();
            QDateTime tmEnd = tmBegin.addSecs(interval);
            addVideo2List(file_path, tmBegin, tmEnd, m_winIdx + 1);
        }
    }
    else
    {
        m_iStatus = CAMERA_STATUS_IDEL;
    }

    m_iStatus = CAMERA_STATUS_LAUNCHED;
    return true;
}

bool CCameraDevice::save2NewFile(QDateTime &tmBegin, QDateTime &tmEnd)
{
    QString file_name;
    QString path_name = getSavePath(tmBegin, file_name);

    assert(m_pVideoPipe);
    if (false == m_pVideoPipe->save2NewFile(path_name))
    {
        m_iStatus = CAMERA_STATUS_IDEL;
        startRun();
    }
    else
    {
        addVideo2List(path_name, tmBegin, tmEnd, m_winIdx + 1);
    }

    return true;
}

void CCameraDevice::addVideo2List(QString path_name, QDateTime &tmBegin, QDateTime &tmEnd,
                                  BYTE channel, BYTE media_type, BYTE stream_code, BYTE storage_type)
{
    //给应用层发送消息,同步传感器信息
    MODULE_MESSAGE msg;
    msg.dst_id = MODULE_FILE_MNG;
    msg.dst_type = APP_TYPE_MAIN;
    msg.iCmd = CMD_ADD_FILE_TO_LIST;

    int idx = 0;
    // 通道号 (索引+1)
    msg.szBuff[idx++] = channel;
    // 资源类型
    msg.szBuff[idx++] = media_type;
    // 码流类型
    msg.szBuff[idx++] = stream_code;
    // 存储器类型
    msg.szBuff[idx++] = storage_type;

    uint64_t tm = tmBegin.toTime_t();
    memcpy(msg.szBuff + idx, &tm, sizeof(tm));
    idx += sizeof(tm);

    tm = tmEnd.toTime_t();
    memcpy(msg.szBuff + idx, &tm, sizeof(tm));
    idx += sizeof(tm);

    memcpy(msg.szBuff + idx, path_name.toStdString().c_str(), path_name.length());
    idx += path_name.length();

    if (m_pTcpModule == nullptr)
    {
        m_pTcpModule = dynamic_cast<ITcpModule *>(QueryPlugin("TcpModule"));
    }
    //模块间发送消息
    m_pTcpModule->SendMsg(msg, sizeof(msg));
}

bool CCameraDevice::stopRun()
{
    m_iStatus = CAMERA_STATUS_IDEL;
    if (m_pVideoPipe && m_pVideoPipe->isStarted())
    {
        m_pVideoPipe->stopPipe();
    }

    if (m_pGb28181Pipe && m_pGb28181Pipe->isStarted())
    {
        m_pGb28181Pipe->stopPipe();
    }

    if (m_pJt808Pipe && m_pJt808Pipe->isStarted())
    {
        m_pJt808Pipe->stopPipe();
    }

    if (m_pRtmpPipe && m_pRtmpPipe->isStarted())
    {
        m_pRtmpPipe->stopPipe();
    }

    return true;
}

void CCameraDevice::startSnapshotTask()
{
    STARTUP_TASK(TaskSnapshot);
}

//建立gstream管道
void CCameraDevice::TaskSnapshot(void *pBuff)
{
    while (!m_pCamMng->isShutdown())
    {
        if (m_iSnapshotLeft > 0)
        {
            capSnapshot();
            m_iSnapshotLeft--;
            mSleep(m_iSnapshotInterval * 1000);
            if (m_iSnapshotLeft == 0)
            {
                if (m_pCamMng)
                {
                    m_pCamMng->snapShotFinilished(m_winIdx, m_strSnapFile);
                }
            }
        }
        else
        {
            // 检查状态
            if (m_pVideoPipe)
            {
                m_pVideoPipe->checkPipeState();
            }

            mSleep(1000);
        }
    }
}

QString CCameraDevice::genSnapshotPathName()
{
    QString root_path = m_pCamMng->getSnapshotPath();
	QString devide_id = m_pCamMng->getDevideId();
    QDateTime tm = QDateTime::currentDateTime();
    //QString str_name = QString("IMG_%1_Cam%2.jpg").arg(tm.toString("yyyy-MM-dd hh:mm:ss")).arg(m_winIdx);
	QString str_name = QString("%1_%2_capture_%3Cam%4.jpg").arg(tm.toString("yyyy_M_d")).arg(devide_id).arg(QString::number(tm.toTime_t())).arg(m_winIdx);
    return root_path + str_name;
}

void CCameraDevice::snapshot(int interval, int snap_count)
{
    m_iSnapshotLeft = snap_count;
    m_iSnapshotInterval = interval;
    m_strSnapFile.clear();
}

// 截图操作函数
bool CCameraDevice::capSnapshot()
{
    assert(m_pVideoPipe);
    QString path_name = genSnapshotPathName();
	//printf("path name is ");
	qDebug() << path_name;
	bool ret = false;
	ret = m_pVideoPipe->snapshot(path_name);
    if (ret)
    {
        m_strSnapFile += path_name + ",";
    }
	return ret;
}

jpgBuf *CCameraDevice::capSnapshotString()
{
	assert(m_pVideoPipe);
	jpgBuf *ret = m_pVideoPipe->snapshotString();
	return ret;
}


void CCameraDevice::startGB28181Video(string ip, int port, string ssrc, int type)
{
    QString strSrc;
    if (type == 0)
    {
        strSrc = getVideoMain();
    }
    else
    {
        strSrc = getVideoSub();
    }

    // 码流不一致则关闭当前视频
    m_pGb28181Pipe->stopPipe();
    m_pGb28181Pipe->setServerInfo(ip, port, ssrc, type);
    m_pGb28181Pipe->setSource(strSrc);
    // 开启新码流的视频推送
    m_pGb28181Pipe->startPipe();
}

void CCameraDevice::stopGB28181Video()
{
    m_pGb28181Pipe->stopPipe();
}

// jt808 视频操作
bool CCameraDevice::startJt808RTVideo(string phone_num, string ip, int tcp_port, int udp_port, int stream_code)
{
    QString strSrc;
    if (m_pJt808Pipe->isStarted())
    {
        m_pJt808Pipe->stopPipe();
    }

    if (0 == stream_code)
    {
        strSrc = this->getVideoMain();
    }
    else
    {
        strSrc = this->getVideoSub();
    }

    ((CJt808Video *)m_pJt808Pipe)->connect2Server(ip, tcp_port, udp_port);
    m_pJt808Pipe->setSource(strSrc);
    m_pJt808Pipe->setPhoneNum(phone_num, m_winIdx + 1);
    m_pJt808Pipe->startPipe();
}

bool CCameraDevice::stopJt808RTVideo()
{
    m_pJt808Pipe->stopPipe();
}

bool CCameraDevice::pauseJt808RTVideo()
{
    m_pJt808Pipe->pausePipe();
}

bool CCameraDevice::continueJt808RTVideo()
{
    m_pJt808Pipe->playPipe();
}

// 变更码流
void CCameraDevice::changeJt808RTStreamCode(int stream_code)
{
    QString strSrc;
    if (m_pJt808Pipe->isStarted())
    {
        m_pJt808Pipe->stopPipe();
    }

    if (0 == stream_code)
    {
        strSrc = this->getVideoMain();
    }
    else
    {
        strSrc = this->getVideoSub();
    }

    m_pJt808Pipe->setSource(strSrc);
    m_pJt808Pipe->startPipe();
}

// jt808 历史视频播放
bool CCameraDevice::startJt808FileVideo(string phone_num, string ip, int tcp_port, int udp_port, int channel, string file_path)
{
    if (m_pJt808HisPipe->isStarted())
    {
        m_pJt808HisPipe->stopPipe();
    }

    ((CJt808Video *)m_pJt808HisPipe)->connect2Server(ip, tcp_port, udp_port);
    m_pJt808HisPipe->setSource(file_path.c_str());
    m_pJt808HisPipe->setPhoneNum(phone_num, channel);
    m_pJt808HisPipe->startPipe();
}
