#ifndef CCAMERADEVICE_H
#define CCAMERADEVICE_H
#include <QWidget>
#include <glib.h>
#include <QObject>
#include <string>
#include "base_module.h"
#include "CVideoPipe.h"
#include "CRtsp2GB28181Pipe.h"
#include "CRtsp2Jt808Pipe.h"
#include "CFile2Jt808Pipe.h"

using namespace std;
class CCameraManager;
class ITcpModule;

enum CAMERA_TYPE
{
    //USB 等系列v4l2驱动摄像头
    CAMERA_TYPE_V4L2 = 0,
    //网络摄像头
    CAMERA_TYPE_RTSP,
    //文件视频
    CAMERA_TYPE_FILE,
    //摄像头最大种类
    CAMERA_TYPE_MAX_COUNT,
};

enum CAMERA_STATUS
{
    // 初始状态
    CAMERA_STATUS_INIT = 0,
    // 空闲态
    CAMERA_STATUS_IDEL,
    // 启动中
    CAMERA_STATUS_LAUNCHING,
    // 启动完成
    CAMERA_STATUS_LAUNCHED,
};

class CCamConfig
{
    friend class CCameraManager;

public:
    CCamConfig();
    CCamConfig(QString &strCamDev);
    CCamConfig(CCamConfig &cam_cfg);
    CCamConfig &operator=(CCamConfig &cam_cfg);
    bool operator==(const CCamConfig &cam_cfg);

    // 设置设备地址 /dev/video[x]  /nvme0/xxxx rtsp://
    void setCamDev(QString cam_dev);
    // 获取视频源信息
    QString getSourceDev();
    // 获取设备地址
    QString getCamDev();
    // 获取主码流地址
    QString getVideoMain();
    // 获取主码流地址
    QString getVideoSub();
    // 获取网络摄像头设备主地址
    QString getWebCamVideoMain();

protected:
    // 偏好位置
    int m_PrefIdx;
    // 设备地址
    QString m_strCamDev;
    // 主码流
    QString m_strVideoMain;
    // 副码流
    QString m_strVideoSub;
    // 相机登陆用户名
    QString m_strUsername;
    // 相机登陆密码
    QString m_strPassword;
    // 摄像头类型
    CAMERA_TYPE m_camType;
};

//摄像头
class CCameraDevice : public QObject, public CCamConfig
{
    friend class CCameraManager;
    Q_OBJECT
public:
    CCameraDevice(QObject *parent = nullptr);
    ~CCameraDevice();

    virtual pthread_t startupTask(PTHREAD_FUNC task, void *pThis);

    string name();
    // 开始运行
    bool startRun();

    // 停止运行
    bool stopRun();

    // 是否空闲态
    bool isCameraIdel();

    //设置输入源信息
    void setSource(QString strSrcPath);

    //设置显示窗口ID
    void setWindowID(WId CamWID);

    //设置窗口index
    void setWinIndex(int winIdx) { m_winIdx = winIdx; }

    // 开始采集任务
    void startSnapshotTask();

    // 设置截图属性，以便截图任务开时作业
    void snapshot(int interval, int snap_count);

    // 设置相机配置信息
    bool setCamConfig(CCamConfig &cam_cfg);
	// 截屏操作
	bool capSnapshot();
	
	
	jpgBuf *capSnapshotString();
protected:
    // 获取文件保存路径
    QString getSavePath(QDateTime &tm, QString &file_name);

    // 是否满足启动条件
    bool isLaunchable();

    // 生成截图路径名称
    QString genSnapshotPathName();

    // 添加视频到列表文件
    void addVideo2List(QString file_name, QDateTime &tm, QDateTime &tmEnd,
                       BYTE channel, BYTE media_type = 2, BYTE stream_code = 1, BYTE storage_type = 1);

    // 保存到新文件
    bool save2NewFile(QDateTime &date_time, QDateTime &tmEnd);

    
    

    THREAD_DEFINITION(CCameraDevice, TaskSnapshot)

    //主消息任务，主要用于和其他模块之间的消息通讯
    virtual void TaskSnapshot(void *pBuff);

    // gb28181 视频操作
    void startGB28181Video(string ip, int port, string ssrc, int type);
    void stopGB28181Video();

    // jt808 视频操作
    bool startJt808RTVideo(string phone_num, string ip, int tcp_port, int udp_port, int stream_code);
    bool stopJt808RTVideo();
    bool pauseJt808RTVideo();
    bool continueJt808RTVideo();

    // jt808 历史视频播放
    bool startJt808FileVideo(string phone_num, string ip, int tcp_port, int udp_port, int channel, string file_path);

    // 变更码流
    void changeJt808RTStreamCode(int stream_code);
Q_SIGNALS:
    // 更新图片
    void
    notifyUpdateImage(int, QPixmap);

protected:
    // 接口类
    CCameraManager *m_pCamMng;
    ITcpModule *m_pTcpModule;

    // 窗口索引
    int m_winIdx;

    // 相机状态：
    int m_iStatus;

private:
    // 截图参数
    int m_iSnapshotInterval;
    int m_iSnapshotLeft;
    QString m_strSnapFile;

    //    CGB28181Video m_GB28181Video;

    CRtsp2RecordPipe *m_pVideoPipe;
    CRtsp2GB28181Pipe *m_pGb28181Pipe;
    CRtsp2Jt808Pipe *m_pJt808Pipe;
    CRtsp2RtmpPipe *m_pRtmpPipe;
    CFile2Jt808Pipe *m_pJt808HisPipe;
};

#endif // CCAMERADEVICE_H
