#ifndef CVIDEOMANAGER_H
#define CVIDEOMANAGER_H
#include "base_module.h"
#include <QObject>
#include <map>
#include <string>
#include <vector>
#include <QWidget>
#include "CCameraDevice.h"
#include <QtXml>
#include "camera/WebCamFinder.h"
#include "alarm/CAlarmManager.h"
#include "camera/CRtmpResponser.h"
#include <QDateTime>
#include <QVector>
#include <QList>
#include "log/CLogManager.h"
#include "Ckafkaproducer.h"
#include "cJSON.h"

class ITcpModule;
class CTimerManager;
class CDeviceManager;
class CWebSocketClient;
class CFileManager;
class CDatebase;
class CConfigManager;
class CGPSManager;
using namespace std;

typedef QMap<QString, CCamConfig *> CAM_CONFIG_MAP;

// 视频管理模块，这个模块主要负责摄像头查找，调度，视频采集，显示推送；
// 以及本地视频浏览、远程点播等视频相关的场景
class CCameraManager : public QObject, public IModule
{
    Q_OBJECT
    const QString CAMERA_DB_NAME = "camera";
    Q_PROPERTY(int video_interval READ getVideoInterval WRITE setVideoInterval)

public:
    MODULE_NAME(CCameraManager)
    LOG_DECLARE()
    CCameraManager();
    ~CCameraManager();

    // 初始化操作
    virtual bool Initialize();

    // 创建数据库
    bool InitializeDB();

    // 关闭操作，这里因为要操作视频相关的设备，需要重载一下.
    virtual void shutdown();

    // 注册消息
    bool RegistMessage();

    //添加摄像头显示WID
    void setVideoWId(int index, WId win_id);

    // 信号槽简易信号处理
    void connect2Slots();

    // 获取截图路径
    QString getSnapshotPath();
	// 获取设备id
	QString getDevideId();
    // 获取视频路径
    QString getVideoPath();

    // 服务器要求获取录像文件的文件列表
    bool videoRecordList(QVariantMap &prop_arr);
	cJSON* json_proc(jpgBuf *jb,int idx);
    // 实时视频rtmp推送
    bool realtimeVideo(QVariantMap &prop_arr);

    // 停止实时视频推流
    bool stopRealtimeVideo(QVariantMap &prop_arr);

    // 抓拍
    bool snapShot(QVariantMap &prop_arr);
    void procSnapTest(MODULE_MESSAGE *pMsg);
	QString genSnapshotPathName(int ind);
	int Send_obs(jpgBuf *jb,QString pa);
	cJSON* proc_Obs(jpgBuf *jb,int idx);
	void procSnapUpload(MODULE_MESSAGE *pMsg);
	void procDistense(MODULE_MESSAGE *pMsg);
    // 单摄像头抓拍结束
    bool snapShotFinilished(int cam_idx, QString strFileList);

    // 恢复出厂配置回调函数
    static bool restoreFactoryCallback(void *This, int type);

    // 恢复出厂值
    bool restoreFactory(int type);

    // 获取摄像头
    CCameraDevice *getCameraByIndex(int index);

    // 视频保存间隔
    int getVideoInterval();
    // 设置视频保存间隔
    bool setVideoInterval(int interval);

    // 保存数据库
    bool saveConfig();
	
	
	
Q_SIGNALS:
    //
    void notifyScreenshotTip(int);
protected slots:
    // 抓拍的结果反馈
    bool onSnapShot();
	//void keyPressEvent(QKeyEvent *event); 
public slots:
    // 数据位置
    void onDataPathChanged(QString path);

    // 命名规则
    void onNameRuleChanged(QString rule);

    // 视频保存模式：按时间长度保存（1）; 按文件大小保存(2)
    void onSaveModeChanged(int mode);

    // 按时间长度保存的时间间隔(分钟)
    void onSaveInternalChanged(int val);

    // 按文件大小保存的文件大小(MB)
    void onSaveSizeChanged(int val);

    // 获取指定索引的设备地址
    bool onCamCfgChanged(int idx, CCamConfig &cam_cfg);

    // 拆帧间隔
    void onFrameSplittingIntervalChanged(int frameInverval);

protected:
    //主消息任务，主要用于和其他模块之间的消息通讯
    THREAD_DEFINITION(CCameraManager, TaskMain)

    //实时摄像头任务，主要处理视频保存和界面推送
    THREAD_DEFINITION(CCameraManager, CameraTask)

    // 定时查找网络设备
    THREAD_DEFINITION(CCameraManager, tskCameraFind)

    //主消息任务，主要用于和其他模块之间的消息通讯
    virtual void TaskMain(void *pBuff);

    //实时摄像头任务，主要处理视频保存和界面推送
    virtual void CameraTask(void *pBuff);

    //摄像头查找任务
    virtual void tskCameraFind(void *pBuff);

    // 处理摄像头配置变更
    void procSetCamConfig(MODULE_MESSAGE *pMsg);

    // 摄像头定时更换文件
    void procSave2NewFile(MODULE_MESSAGE *pMsg);

    // 开始gb28181推流
    void procStartGB28181Video(MODULE_MESSAGE *pMsg);

    // 停止gb28181推流
    void procStopGB28181Video(MODULE_MESSAGE *pMsg);

    // 开始JT808实时推流
    void procStartJt808RTVideo(MODULE_MESSAGE *pMsg);

    // jt808实时推流控制
    void procJt808RTVideoControl(MODULE_MESSAGE *pMsg);

    // 开始JT808历史推流
    void procStartJt808HisVideo(MODULE_MESSAGE *pMsg);

    // JT808历史推流控制
    void procJt808HisVideoControl(MODULE_MESSAGE *pMsg);

    // 刷新设备列表
    void updateCamerapDevice(QString strDev);

    // 刷新相机设备回调函数
    static void updateCameraCallback(DEVICE_IPC_INFO *pInfo, void *lpContext);

    // 获取空闲摄像头窗口, 没有空闲则返回空，注意判空
    CCameraDevice *getIdelCamera(int &index);

    // 获取指定源的摄像头窗口
    CCameraDevice *getCameraBySrc(QString strSrc);

    // 设置相机配置
    void setCamConfig(CCamConfig &cam_cfg);

    // 根据IP地址获取推荐的显示位
    int getPrefIdxByIP(QString strDev);

    // 实时视频推流的结果反馈
    bool onRealtimeVideo(QString pushUrl, bool bSuccess);
	void procDeviceId(MODULE_MESSAGE *pMsg);
	
protected:
    // 依赖模块项
    ITcpModule *m_pTcpModule;
    CTimerManager *m_pTimerMng;
    CConfigManager *m_pConfigMng;
    CDatebase *m_pCamDb;
    CDeviceManager *m_pDevMng;
    CWebSocketClient *m_pWebSock;
    CFileManager *m_pFileMng;
	CGPSManager *m_pGps;
    //这里为所有配置的摄像头信息
    CCameraDevice m_vctCam[MAX_CAMERA_COUNT];

    //摄像头窗口ID数组
    WId m_vctCamWID[MAX_CAMERA_COUNT];

    // 相机配置表
    CAM_CONFIG_MAP m_mapCamConfig;
    QMutex m_mtxCamCfg;
    QMutex m_mtxCamera;
    QMutex m_mtxCamFind;

    // 摄像头查找类
    CWebCamFinder m_webCamFinder;
    // 实时播放类
    CRtmpResponser m_rtmpResponser;

    // 截图状态参数
    bool m_bSnapFinished[MAX_CAMERA_COUNT];
    int m_iSnapInterval;
    int m_iSnapCount;
    QString m_strSnapList;
	QString m_devideId;
	CKafkaProducer *kp = NULL;
	float m_dis;
};

#endif // CVIDEOMANAGER_H
