#include "CFile2Jt808Pipe.h"
#include <QDebug>

CFile2Jt808Pipe::CFile2Jt808Pipe()
    : m_jt808Sink(nullptr)
{
}

CFile2Jt808Pipe::~CFile2Jt808Pipe()
{
}

bool CFile2Jt808Pipe::setSource(QString strSrc)
{
    if (m_bStarted == true)
    {
        return false;
    }

    // gst 源
    QString strGstSource = QString("filesrc location=%1  ! h264parse ! video/x-h264, stream-format=byte-stream").arg(strSrc);

    //filesrc location=/hncy_data/video/2021-09-15_14-39_Cam4.mp4  ! h264parse ! omxh264dec ! nvvidconv ! omxh264enc ! video/x-h264, stream-format=(string)byte-stream

    // jt808
    QString strJt808Sink = " ! appsink name=jt808_sink";

    m_strLaunch = strGstSource + strJt808Sink;

    CVideoPipe::setSource(strSrc);
    return true;
}

// 开始流媒体管道
bool CFile2Jt808Pipe::startPipe()
{
    qDebug() << "gst_parse_launch:" << m_strLaunch;

    // 启动参数为空直接返回失败
    if (m_strLaunch.isEmpty() || m_strLaunch.contains("%1"))
    {
        return false;
    }

    m_gstPipe = gst_parse_launch(m_strLaunch.toStdString().c_str(), NULL);
    if (!m_gstPipe)
    {
        return false;
    }

    GstElement *jt808_sink = gst_bin_get_by_name(GST_BIN(m_gstPipe), "jt808_sink");
    m_jt808Sink = (GstAppSink *)(jt808_sink);
    if (m_jt808Sink)
    {
        //g_signal_connect(m_gstPipe, "deep-notify", G_CALLBACK(gst_object_default_deep_notify ), NULL);
        gst_app_sink_set_emit_signals(m_jt808Sink, true);
        gst_app_sink_set_drop(m_jt808Sink, true);
        gst_app_sink_set_max_buffers(m_jt808Sink, 1);
        g_signal_connect(m_jt808Sink, "new-sample", G_CALLBACK(Jt808Transform), this);
    }
    else
    {
        gst_element_set_state(m_gstPipe, GST_STATE_NULL);
        gst_object_unref(m_gstPipe);
        gst_object_unref(m_jt808Sink);
        return false;
    }

    // connect to interesting Q_SIGNALSd
    GstBus *bus = gst_element_get_bus(m_gstPipe);
    gst_bus_add_watch(bus, (GstBusFunc)CVideoPipe::postGstMessage, this);
    gst_object_unref(bus);

    // run the m_gstPipe
    GstStateChangeReturn sret = gst_element_set_state(m_gstPipe, GST_STATE_PLAYING);

    if (sret == GST_STATE_CHANGE_FAILURE)
    {
        qDebug() << "==========================================================+++++==================";
        qDebug() << "=========================GstStateChangeReturn ERROR==============================";
        qDebug() << "==========================================================+++++==================";
        gst_element_set_state(m_gstPipe, GST_STATE_NULL);
        gst_object_unref(m_gstPipe);
        gst_object_unref(m_jt808Sink);
        return false;
    }

    m_bStarted = true;

    return true;
}
// 结束流媒体管道
bool CFile2Jt808Pipe::stopPipe()
{
    if (m_bStarted)
    {
        gst_element_set_state(m_gstPipe, GST_STATE_NULL);
        gst_object_unref(m_gstPipe);
        gst_object_unref(m_jt808Sink);

        m_gstPipe = nullptr;
        m_jt808Sink = nullptr;
        m_bStarted = false;
    }
    return true;
}

/* The appsink has received a buffer */
GstFlowReturn CFile2Jt808Pipe::Jt808Transform(GstElement *sink, CFile2Jt808Pipe *pThis)
{
    GstSample *sample = nullptr;
    GstBuffer *sampleBuffer = nullptr;
    GstMapInfo mapInfo;
    /* Retrieve the buffer */
    g_signal_emit_by_name(sink, "pull-sample", &sample);
    if (sample)
    {
        // 获取buff
        sampleBuffer = gst_sample_get_buffer(sample);
        if (!sampleBuffer)
        {
            gst_sample_unref(sample);
            return GST_FLOW_ERROR;
        }

        // buff_map
        gst_buffer_map(sampleBuffer, &mapInfo, GST_MAP_READ);
        /* The only thing we do in this example is print a * to indicate a received buffer */
        //g_print("*");

        pThis->sendFrameData((char *)mapInfo.data, mapInfo.size);

        gst_sample_unref(sample);
        return GST_FLOW_OK;
    }

    return GST_FLOW_ERROR;
}
