#ifndef CGB28181_VIDEO_H
#define CGB28181_VIDEO_H
#include <stdint.h>
#include <string>
#include <QMutex>
#include "CVideoPipe.h"
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/tcp.h>

#define PS_HDR_LEN 14
#define SYS_HDR_LEN 18
#define PSM_HDR_LEN 24
#define PES_HDR_LEN 19
#define RTP_HDR_LEN 12
#define RTP_VERSION 2
#define RTP_MAX_PACKET_BUFF 1400
#define PS_PES_PAYLOAD_SIZE 64000

union LESize
{
    unsigned short int length;
    unsigned char byte[2];
};

struct bits_buffer_s
{
    unsigned char *p_data;
    unsigned char i_mask;
    int i_size;
    int i_data;
};

struct Data_Info_s
{
    int buff_offset = 0;
    uint64_t s64CurPts;
    int IFrame;
    uint16_t u16CSeq;
    uint32_t u32Ssrc;
    char szBuff[RTP_MAX_PACKET_BUFF];
};

class CGB28181Video
{
public:
    CGB28181Video();
    ~CGB28181Video();

    bool Initialize();

    void setServerInfo(std::string ip, int port, std::string ssrc, int type);
    bool setParameter(int32_t ssrc);
    int sendFrameData(char *buff, int size);
    int gb28181_streampackageForH264(char *pData, int nFrameLen, Data_Info_s *pPacker, int stream_type);
    int gb28181_make_rtp_header(char *pData, int marker_flag, unsigned short cseq, long long curpts, unsigned int ssrc);
    int gb28181_send_rtp_pack(char *databuff, int nDataLen, int mark_flag, Data_Info_s *pPacker);
    int gb28181_make_pes_header(char *pData, int stream_id, int payload_len, unsigned long long pts, unsigned long long dts);
    int gb28181_make_psm_header(char *pData);
    int gb28181_make_sys_header(char *pData);
    int gb28181_make_ps_header(char *pData, unsigned long long s64Scr);
    int SendDataBuff(char *buff, int size);
    // SPS
    int pushSPS(char *buff, int size, Data_Info_s &DataInfo);
    // PPS
    int pushPPS(char *buff, int size, Data_Info_s &DataInfo);
    // IDR
    int pushIDR(char *buff, int size, Data_Info_s &DataInfo);
    // P/B
    int pushPB(char *buff, int size, Data_Info_s &DataInfo);

    // 对于一包不能装下的,分包发送
    int sendFrameSection(char *buff, int nSize, bool bLastPkg, Data_Info_s &DataInfo);

protected:
    int _socketFd;
    int m_tcpSock;
    std::string m_strSSRC;
    Data_Info_s m_DataInfo;
    QMutex m_mtxSend;
    char *m_pSendBuff;

    std::string m_strRtpIP;
    int m_rtpPort;

    struct sockaddr_in m_servAddr;
    bool m_bSPSfilled;
    bool m_bPPSfilled;
    bool m_bIDRfilled;
    const int stream_type = 0; // 这里固定为视频
};

// 文件到GB28181管道
class CFile2GB28181Pipe : public CVideoPipe
{
    Q_OBJECT
public:
    CFile2GB28181Pipe();
    ~CFile2GB28181Pipe();

    // 开始流媒体管道
    virtual bool startPipe();
    // 结束流媒体管道
    virtual bool stopPipe();

protected:
};

#endif //CGB28181_VIDEO_H
