#include "CH264Parse.h"
#include <memory>
#include <QDebug>
#include "common.h"

#define MAX_VIDEO_TAG_BUF_SIZE 1024 * 1024

NALU_t *AllocNALU(int buffersize)
{
    NALU_t *n;

    if ((n = (NALU_t *)calloc(1, sizeof(NALU_t))) == NULL)
    {
        qDebug("AllocNALU Error: Allocate Meory To NALU_t error ");
        getchar();
    }

    n->max_size = buffersize; //Assign buffer size

    if ((n->buf = (unsigned char *)calloc(buffersize, sizeof(char))) == NULL)
    {
        free(n);
        qDebug("AllocNALU Error: Allocate Meory To NALU_t Buffer error ");
        getchar();
    }
    return n;
}

void FreeNALU(NALU_t *n)
{
    if (n)
    {
        if (n->buf)
        {
            free(n->buf);
            n->buf = NULL;
        }
        free(n);
    }
}

bool FindStartCode2(char *Buf)
{
    if (Buf[0] != 0 || Buf[1] != 0 || Buf[2] != 1) //Check whether buf is 0x000001
    {
        return false;
    }
    else
    {
        return true;
    }
}

bool FindStartCode3(char *Buf)
{
    if (Buf[0] != 0 || Buf[1] != 0 || Buf[2] != 0 || Buf[3] != 1) //Check whether buf is 0x00000001
    {
        return false;
    }
    else
    {
        return true;
    }
}

int ReadChar(unsigned char *to, unsigned char *from, int sum)
{
    int i;
    for (i = 0; i < sum; i++)
    {
        to[i] = from[i];
    }
    return sum;
}

int GetAnnexbNALU(NALU_t *nalu, char *data, int size, int *Buf_index)
{
    if (*Buf_index >= size)
    {
        qDebug("GetAnnexbNALU Buf_index error!\n");
        return 0;
    }
    if (FindStartCode2(data + *Buf_index))
    {
        nalu->startcodeprefix_len = 3; //初始化前缀位三个字节
    }
    else if (FindStartCode3(data + *Buf_index))
    {
        nalu->startcodeprefix_len = 4; //初始化前缀位三个字节
    }
    else
    {
        qDebug("GetAnnexbNALU data error!\n");
        return 0;
    }

    int i;

    int End_index;
    for (i = *Buf_index + nalu->startcodeprefix_len; i < size - 4; i++)
    {
        if (FindStartCode2(data + i) || FindStartCode3(data + i))
        {
            break;
        }
    }
    End_index = i;
    if (End_index >= size - 4)
        End_index = size;
    else
    {
        qDebug("Found SPS PPS \n");
    }

#ifdef _DEBUG_
    char *str = malloc(size * 2 + 1);
    for (i = 0; i < End_index - *Buf_index; i++)
    {
        str[i * 2] = data[i + *Buf_index] + '0';
        str[i * 2 + 1] = ',';
    }
    str[i * 2] = '\0';
    LOGI("size:%d \n", size);
    LOGI("data:%s \n", str);
    free(str);

    printf("nalu->startcodeprefix_len = %d\n", nalu->startcodeprefix_len);
    printf("data = %d,%d,%d,%d,%d\n", data[*Buf_index], data[*Buf_index + 1],
           data[*Buf_index + 2], data[*Buf_index + 3], data[*Buf_index + 4]);
#endif

    nalu->len = End_index - *Buf_index - nalu->startcodeprefix_len;              //设置包含nal 头的数据长度
    memcpy(nalu->buf, &data[*Buf_index + nalu->startcodeprefix_len], nalu->len); //拷贝一个nal 数据到数组中
    nalu->forbidden_bit = nalu->buf[0] & 0x80;                                   //1 bit  设置nal 头
    nalu->nal_reference_idc = nalu->buf[0] & 0x60;                               // 2 bit
    nalu->nal_unit_type = (nalu->buf[0]) & 0x1f;                                 // 5 bit
    printf("nal_unit_type = %d\n", nalu->nal_unit_type);
    if (End_index >= size - 4)
    {
        *Buf_index = size;
    }
    else
        *Buf_index = End_index;
    return nalu->startcodeprefix_len; //((info3 == 1)? 4 : 3);
}

int GetFrameType(NALU_t *nal)
{
    bs_t s;
    int frame_type = 0;
    unsigned char *OneFrameBuf_H264 = NULL;
    if ((OneFrameBuf_H264 = (unsigned char *)calloc(nal->len + 4,
                                                    sizeof(unsigned char))) == NULL)
    {
        qDebug("Error malloc OneFrameBuf_H264\n");
        return getchar();
    }
    if (nal->startcodeprefix_len == 3)
    {
        OneFrameBuf_H264[0] = 0x00;
        OneFrameBuf_H264[1] = 0x00;
        OneFrameBuf_H264[2] = 0x01;
        memcpy(OneFrameBuf_H264 + 3, nal->buf, nal->len);
    }
    else if (nal->startcodeprefix_len == 4)
    {
        OneFrameBuf_H264[0] = 0x00;
        OneFrameBuf_H264[1] = 0x00;
        OneFrameBuf_H264[2] = 0x00;
        OneFrameBuf_H264[3] = 0x01;
        memcpy(OneFrameBuf_H264 + 4, nal->buf, nal->len);
    }
    else
    {
        qDebug("H264 error！\n");
    }
    bs_init(&s, OneFrameBuf_H264 + nal->startcodeprefix_len + 1, nal->len - 1);

    if (nal->nal_unit_type == NAL_SLICE || nal->nal_unit_type == NAL_SLICE_IDR)
    {
        /* i_first_mb */
        bs_read_ue(&s);
        /* picture type */
        frame_type = bs_read_ue(&s);
        switch (frame_type)
        {
        case 0:
        case 5: /* P */
            nal->Frametype = FRAME_P;
            break;
        case 1:
        case 6: /* B */
            nal->Frametype = FRAME_B;
            break;
        case 3:
        case 8: /* SP */
            nal->Frametype = FRAME_P;
            break;
        case 2:
        case 7: /* I */
            nal->Frametype = FRAME_I;
            break;
        case 4:
        case 9: /* SI */
            nal->Frametype = FRAME_I;
            break;
        }
    }
    else if (nal->nal_unit_type == NAL_SEI)
    {
        nal->Frametype = NAL_SEI;
    }
    else if (nal->nal_unit_type == NAL_SPS)
    {
        nal->Frametype = NAL_SPS;
    }
    else if (nal->nal_unit_type == NAL_PPS)
    {
        nal->Frametype = NAL_PPS;
    }
    if (OneFrameBuf_H264)
    {
        free(OneFrameBuf_H264);
        OneFrameBuf_H264 = NULL;
    }
    return 1;
}

bool GainSpsPps(unsigned char *spsbuf, unsigned int *spslength,
                unsigned char *ppsbuf, unsigned int *ppslength, char *data, int size)
{
    //读取二帧数据
    NALU_t *n_1 = NULL;
    NALU_t *n_2 = NULL;
    unsigned int avcc_pos;
    n_1 = AllocNALU(MAX_VIDEO_TAG_BUF_SIZE);
    n_2 = AllocNALU(MAX_VIDEO_TAG_BUF_SIZE);
    int Buf_index = 0;
loop_1_1:
    printf("loop_1_1\n");
    if (Buf_index >= size)
    {
        qDebug("loop_1_1 error\n");
        return false;
    }
    if (GetAnnexbNALU(n_1, data, size, &Buf_index) == 0)
        return false;
    //判断帧类型
    GetFrameType(n_1);
    if (n_1->nal_unit_type == NAL_SPS)
    {
    loop_1_2:
        printf("loop_1_2\n");
        if (Buf_index >= size)
        {
            qDebug("loop_1_2 error\n");
            return false;
        }
        if (GetAnnexbNALU(n_2, data, size, &Buf_index) == 0)
            return false;
        //判断帧类型
        GetFrameType(n_2);
        if (n_2->nal_unit_type == NAL_PPS)
        {
            memcpy(spsbuf, n_1->buf, n_1->len);
            *spslength = n_1->len;
            memcpy(ppsbuf, n_2->buf, n_2->len);
            *ppslength = n_2->len;
        }
        else
        {
            goto loop_1_2;
        }
    }
    else
    {
        goto loop_1_1;
    }
    FreeNALU(n_1);
    FreeNALU(n_2);
    return true;
}

int Read_One_H264_Frame(unsigned char **buf, char *data, int size,
                        int *Is_KyeFrame)
{
    NALU_t *n = NULL;
    int startcodeprefix_size;
    unsigned int video_buf_size = 0;
    *Is_KyeFrame = false;
    //分配nal 资源
    n = AllocNALU(MAX_VIDEO_TAG_BUF_SIZE);
    int Buf_index = 0;

loop_2:
    //读取一帧数据
    if (Buf_index >= size)
    {
        qDebug("loop_2 error\n");
        return 0;
    }
    startcodeprefix_size = GetAnnexbNALU(n, data, size, &Buf_index);
    if (startcodeprefix_size == 0)
    {
        printf("loop_1_2 error \n");
        return 0;
    }
    //判断帧类型
    GetFrameType(n);

    if (n->Frametype == FRAME_I)
    {
        //将data填入bufz中
#if 0
        buf[0] = n->len >> 24;
        buf[1] = (n->len >> 16) & 0xFF;
        buf[2] = (n->len >> 8) & 0xFF;
        buf[3] = n->len & 0xFF;
        memcpy(buf + 4 ,n->buf,n->len);
        video_buf_size = n->len + 4;
#endif
        *Is_KyeFrame = true;
        *buf = (unsigned char *)calloc(n->len, sizeof(char));
        memcpy(*buf, n->buf, n->len);
        video_buf_size = n->len;
        qDebug("Is_KyeFrame\n");
    }
    else if (n->Frametype == FRAME_B || n->Frametype == FRAME_P)
    {
#if 0
        buf[0] = n->len >> 24;
        buf[1] = (n->len >> 16) & 0xFF;
        buf[2] = (n->len >> 8) & 0xFF;
        buf[3] = n->len & 0xFF;
        memcpy(buf + 4 ,n->buf,n->len);
        video_buf_size = n->len + 4;
#endif
        *buf = (unsigned char *)calloc(n->len, sizeof(char));
        memcpy(*buf, n->buf, n->len);
        video_buf_size = n->len;
        qDebug("!Is_KyeFrame, %d\n", n->Frametype);
    }
    //    else if (n->Frametype == NAL_SPS || n->Frametype == NAL_PPS)
    //    {
    //        return -1;
    //    }
    else //其它帧直接去掉
    {
        goto loop_2;
    }

    FreeNALU(n); //释放nal 资源
    return video_buf_size;
}

void bs_init(bs_t *s, void *p_data, int i_data)
{
    s->p_start = (unsigned char *)p_data; //用传入的p_data首地址初始化p_start，只记下有效数据的首地址
    s->p = (unsigned char *)p_data;       //字节首地址，一开始用p_data初始化，每读完一个整字节，就移动到下一字节首地址
    s->p_end = s->p + i_data;             //尾地址，最后一个字节的首地址?
    s->i_left = 8;                        //还没有开始读写，当前字节剩余未读取的位是8
}

int bs_read(bs_t *s, int i_count)
{
    static int i_mask[33] = {0x00,
                             0x01, 0x03, 0x07, 0x0f,
                             0x1f, 0x3f, 0x7f, 0xff,
                             0x1ff, 0x3ff, 0x7ff, 0xfff,
                             0x1fff, 0x3fff, 0x7fff, 0xffff,
                             0x1ffff, 0x3ffff, 0x7ffff, 0xfffff,
                             0x1fffff, 0x3fffff, 0x7fffff, 0xffffff,
                             0x1ffffff, 0x3ffffff, 0x7ffffff, 0xfffffff,
                             0x1fffffff, 0x3fffffff, 0x7fffffff, (int)0xffffffff};
    /* 
              数组中的元素用二进制表示如下： 
 
              假设：初始为0，已写入为+，已读取为- 
               
              字节:       1       2       3       4 
                   00000000 00000000 00000000 00000000      下标 
 
              0x00:                           00000000      x[0] 
 
              0x01:                           00000001      x[1] 
              0x03:                           00000011      x[2] 
              0x07:                           00000111      x[3] 
              0x0f:                           00001111      x[4] 
 
              0x1f:                           00011111      x[5] 
              0x3f:                           00111111      x[6] 
              0x7f:                           01111111      x[7] 
              0xff:                           11111111      x[8]    1字节 
 
             0x1ff:                      0001 11111111      x[9] 
             0x3ff:                      0011 11111111      x[10]   i_mask[s->i_left] 
             0x7ff:                      0111 11111111      x[11] 
             0xfff:                      1111 11111111      x[12]   1.5字节 
 
            0x1fff:                  00011111 11111111      x[13] 
            0x3fff:                  00111111 11111111      x[14] 
            0x7fff:                  01111111 11111111      x[15] 
            0xffff:                  11111111 11111111      x[16]   2字节 
 
           0x1ffff:             0001 11111111 11111111      x[17] 
           0x3ffff:             0011 11111111 11111111      x[18] 
           0x7ffff:             0111 11111111 11111111      x[19] 
           0xfffff:             1111 11111111 11111111      x[20]   2.5字节 
 
          0x1fffff:         00011111 11111111 11111111      x[21] 
          0x3fffff:         00111111 11111111 11111111      x[22] 
          0x7fffff:         01111111 11111111 11111111      x[23] 
          0xffffff:         11111111 11111111 11111111      x[24]   3字节 
 
         0x1ffffff:    0001 11111111 11111111 11111111      x[25] 
         0x3ffffff:    0011 11111111 11111111 11111111      x[26] 
         0x7ffffff:    0111 11111111 11111111 11111111      x[27] 
         0xfffffff:    1111 11111111 11111111 11111111      x[28]   3.5字节 
 
        0x1fffffff:00011111 11111111 11111111 11111111      x[29] 
        0x3fffffff:00111111 11111111 11111111 11111111      x[30] 
        0x7fffffff:01111111 11111111 11111111 11111111      x[31] 
        0xffffffff:11111111 11111111 11111111 11111111      x[32]   4字节 
 
     */
    int i_shr;        //
    int i_result = 0; //用来存放读取到的的结果 typedef unsigned   uint32_t;

    while (i_count > 0) //要读取的比特数
    {
        if (s->p >= s->p_end) //字节流的当前位置>=流结尾，即代表此比特流s已经读完了。
        {                     //
            break;
        }

        if ((i_shr = s->i_left - i_count) >= 0) //当前字节剩余的未读位数，比要读取的位数多，或者相等
        {                                       //i_left当前字节剩余的未读位数，本次要读i_count比特，i_shr=i_left-i_count的结果如果>=0，说明要读取的都在当前字节内
                                                //i_shr>=0，说明要读取的比特都处于当前字节内
            //这个阶段，一次性就读完了，然后返回i_result(退出了函数)
            /* more in the buffer than requested */
            i_result |= (*s->p >> i_shr) & i_mask[i_count]; //“|=”:按位或赋值，A |= B 即 A = A|B
                                                            //|=应该在最后执行，把结果放在i_result(按位与优先级高于复合操作符|=)
                                                            //i_mask[i_count]最右侧各位都是1,与括号中的按位与，可以把括号中的结果复制过来
                                                            //!=,左边的i_result在这儿全是0，右侧与它按位或，还是复制结果过来了，好象好几步都多余
            /*读取后，更新结构体里的字段值*/
            s->i_left -= i_count; //即i_left = i_left - i_count，当前字节剩余的未读位数，原来的减去这次读取的
            if (s->i_left == 0)   //如果当前字节剩余的未读位数正好是0，说明当前字节读完了，就要开始下一个字节
            {
                s->p++;        //移动指针，所以p好象是以字节为步长移动指针的
                s->i_left = 8; //新开始的这个字节来说，当前字节剩余的未读位数，就是8比特了
            }
            return (i_result); //可能的返回值之一为：00000000 00000000 00000000 00000001 (4字节长)
        }
        else /* i_shr < 0 ,跨字节的情况*/
        {
            //这个阶段，是while的一次循环，可能还会进入下一次循环，第一次和最后一次都可能读取的非整字节，比如第一次读了3比特，中间读取了2字节(即2x8比特)，最后一次读取了1比特，然后退出while循环
            //当前字节剩余的未读位数，比要读取的位数少，比如当前字节有3位未读过，而本次要读7位
            //???对当前字节来说，要读的比特，都在最右边，所以不再移位了(移位的目的是把要读的比特放在当前字节最右)
            /* less(较少的) in the buffer than requested */
            i_result |= (*s->p & i_mask[s->i_left]) << -i_shr; //"-i_shr"相当于取了绝对值
                                                               //|= 和 << 都是位操作符，优先级相同，所以从左往右顺序执行
                                                               //举例:int|char ，其中int是4字节，char是1字节，sizeof(int|char)是4字节
                                                               //i_left最大是8，最小是0，取值范围是[0,8]
            i_count -= s->i_left;                              //待读取的比特数，等于原i_count减去i_left，i_left是当前字节未读过的比特数，而此else阶段，i_left代表的当前字节未读的比特全被读过了，所以减它
            s->p++;                                            //定位到下一个新的字节
            s->i_left = 8;                                     //对一个新字节来说，未读过的位数当然是8，即本字节所有位都没读取过
        }
    }

    return (i_result); //可能的返回值之一为：00000000 00000000 00000000 00000001 (4字节长)
}

int bs_read1(bs_t *s)
{

    if (s->p < s->p_end)
    {
        unsigned int i_result;

        s->i_left--;                            //当前字节未读取的位数少了1位
        i_result = (*s->p >> s->i_left) & 0x01; //把要读的比特移到当前字节最右，然后与0x01:00000001进行逻辑与操作，因为要读的只是一个比特，这个比特不是0就是1，与0000 0001按位与就可以得知此情况
        if (s->i_left == 0)                     //如果当前字节剩余未读位数是0，即是说当前字节全读过了
        {
            s->p++;        //指针s->p 移到下一字节
            s->i_left = 8; //新字节中，未读位数当然是8位
        }
        return i_result; //unsigned int
    }

    return 0; //返回0应该是没有读到东西
}

int bs_read_ue(bs_t *s)
{
    int i = 0;

    while (bs_read1(s) == 0 && s->p < s->p_end && i < 32) //条件为：读到的当前比特=0，指针未越界，最多只能读32比特
    {
        i++;
    }
    return ((1 << i) - 1 + bs_read(s, i));
}

/*主要功能为得到一个完整的NALU并保存在NALU_t的buf中，获取他的长度，填充F,IDC,TYPE位。
* 并判断这个buf中是否还存在其他的NALU单元。
* parms:
* buf:为一帧原始码流
* len:原始码流长度，
* retBuf： 如果有多个NALU，则将已经获取的第一个NALU去掉，其余部分存储在retBuf中
* nalu： 获取的一个NALU
* 返回值：-1没有获取的NALU，0获取到NALU并且只有一个，大于0说明不止一个NALU，并且值代表被取出的NALU的长度包含前缀
*/

//int GetAnNALU(unsigned char *buf, int len, unsigned char *retBuf, NALU_t *nalu)
//{
//    int pos = 0;
//    int StartCodeFound;

//    if (len < 4)
//    {
//        return -1;
//    }

//    nalu->startcodeprefix_len = 3; //初始化码流序列的开始字符为3个字节

//    info2 = FindStartCode2(buf); //判断是否为0x000001
//    if (info2 != 1)
//    {
//        info3 = FindStartCode3(buf); //判断是否为0x00000001
//        if (info3 != 1)              //如果不是，返回-1
//        {
//            return -1;
//        }
//        else
//        {
//            //如果是0x00000001,得到开始前缀为4个字节
//            pos = 4;
//            nalu->startcodeprefix_len = 4;
//        }
//    }
//    else
//    {
//        //如果是0x000001,得到开始前缀为3个字节
//        pos = 3;
//        nalu->startcodeprefix_len = 3;
//    }
//    // 确认特征码是00 00 01还是00 00 00 01

//    //查找下一个开始字符的标志位
//    StartCodeFound = 0;
//    info2 = 0;
//    info3 = 0;

//    while (!StartCodeFound)
//    {
//        if (pos >= len - 1)
//        {
//            break;
//        }
//        pos++;
//        // 找到下一个特征码
//        info3 = FindStartCode3(&buf[pos - 4]); //判断是否为0x00000001
//        if (info3 != 1)
//        {
//            info2 = FindStartCode2(&buf[pos - 3]); //判断是否为0x000001
//        }
//        StartCodeFound = (info2 == 1 || info3 == 1);
//        // 是否找到,直到或结束
//    }

//    if (!StartCodeFound)
//    {
//        // 没找到,长度就是总长-特帧码长度
//        nalu->len = (len)-nalu->startcodeprefix_len;
//    }
//    else
//    {
//        if (info2 == 1)
//        {
//            pos = pos - 3;
//        }
//        else
//        {
//            pos = pos - 4;
//        }
//        // 找到就是下一个特征码起始位置-第一个特征码结束位置
//        nalu->len = pos - nalu->startcodeprefix_len;
//    }

//    // 拷贝净荷长度到nalu的buff
//    memcpy(nalu->buf, &buf[nalu->startcodeprefix_len], nalu->len); //拷贝一个完整NALU，不拷贝起始前缀0x000001或0x00000001
//    nalu->forbidden_bit = nalu->buf[0] & 0x80;                     //1 bit
//    nalu->nal_reference_idc = nalu->buf[0] & 0x60;                 // 2 bit
//    nalu->nal_unit_type = (nalu->buf[0]) & 0x1f;                   // 5 bit

//    if (!StartCodeFound)
//    {
//        return 0;
//    }
//    else
//    {
//        memcpy(retBuf, buf + pos, len - pos);
//        return pos;
//    }
//}

//void imageTransmitter ::sendFrame(unsigned char *buf, int len)
//{
//    NALU_HEADER *nalu_hdr;
//    FU_INDICATOR *fu_ind;
//    FU_HEADER *fu_hdr;
//    char *nalu_payload;
//    unsigned char rltBuf[800000];
//    char sendbuf[1500];

//    int rlt = GetAnNALU(buf, len, rltBuf, n);
//    if (rlt < 0)
//    {
//        printf("get null error!\n");
//        return;
//    }

//    /* 输出NALU长度和TYPE */
//    dump(n);

//    /*当一个NALU小于MAX_RTP_PKT_LENGTH字节的时候，采用一个单RTP包发送  */
//    if (n->len <= MAX_RTP_PKT_LENGTH)
//    {
//        nalu_hdr = (NALU_HEADER *)&sendbuf[0];
//        nalu_hdr->F = n->forbidden_bit;
//        nalu_hdr->NRI = n->nal_reference_idc >> 5;
//        nalu_hdr->TYPE = n->nal_unit_type;
//        nalu_payload = &sendbuf[1];
//        memcpy(nalu_payload, n->buf + 1, n->len - 1); //去掉nalu头的nalu剩余内容写入sendbuf[13]开始的字符串。

//        this->sendPacket(sendbuf, n->len);
//    }
//    else if (n->len > MAX_RTP_PKT_LENGTH)
//    {
//        int k = 0, l = 0;
//        k = n->len / MAX_RTP_PKT_LENGTH; //需要k个MAX_RTP_PKT_LENGTH字节的RTP包
//        l = n->len % MAX_RTP_PKT_LENGTH; //最后一个RTP包的需要装载的字节数
//        int t = 0;                       //用于指示当前发送的是第几个分片RTP包
//        while (t <= k)
//        {
//            if (!t) //发送一个需要分片的NALU的第一个分片，置FU HEADER的S位
//            {
//                memset(sendbuf, 0, 1500);
//                fu_ind = (FU_INDICATOR *)&sendbuf[0];
//                fu_ind->F = n->forbidden_bit;
//                fu_ind->NRI = n->nal_reference_idc >> 5;
//                fu_ind->TYPE = 28;

//                fu_hdr = (FU_HEADER *)&sendbuf[1];
//                fu_hdr->E = 0;
//                fu_hdr->R = 0;
//                fu_hdr->S = 1;
//                fu_hdr->TYPE = n->nal_unit_type;

//                nalu_payload = &sendbuf[2];
//                memcpy(nalu_payload, n->buf + 1, MAX_RTP_PKT_LENGTH); //去掉NALU头
//                this->sendPacket(sendbuf, MAX_RTP_PKT_LENGTH + 2);
//                t++;
//            }
//            else if (k == t) //发送的是最后一个分片，注意最后一个分片的长度可能超过
//            {
//                memset(sendbuf, 0, 1500);
//                fu_ind = (FU_INDICATOR *)&sendbuf[0];
//                fu_ind->F = n->forbidden_bit;
//                fu_ind->NRI = n->nal_reference_idc >> 5;
//                fu_ind->TYPE = 28;

//                fu_hdr = (FU_HEADER *)&sendbuf[1];
//                fu_hdr->R = 0;
//                fu_hdr->S = 0;
//                fu_hdr->E = 1;
//                fu_hdr->TYPE = n->nal_unit_type;

//                nalu_payload = &sendbuf[2];
//                memcpy(nalu_payload, n->buf + t * MAX_RTP_PKT_LENGTH + 1, l - 1); //将nalu最后剩余的
//                this->sendPacket(sendbuf, l + 1);
//                t++;
//            }
//            else if (t < k && 0 != t)
//            {
//                memset(sendbuf, 0, 1500);
//                fu_ind = (FU_INDICATOR *)&sendbuf[0];
//                fu_ind->F = n->forbidden_bit;
//                fu_ind->NRI = n->nal_reference_idc >> 5;
//                fu_ind->TYPE = 28;

//                fu_hdr = (FU_HEADER *)&sendbuf[1];
//                fu_hdr->R = 0;
//                fu_hdr->S = 0;
//                fu_hdr->E = 0;
//                fu_hdr->TYPE = n->nal_unit_type;

//                nalu_payload = &sendbuf[2];
//                memcpy(nalu_payload, n->buf + t * MAX_RTP_PKT_LENGTH + 1, MAX_RTP_PKT_LENGTH);
//                this->sendPacket(sendbuf, MAX_RTP_PKT_LENGTH + 2);
//                t++;
//            }
//        }
//    }

//    if (rlt > 0)
//    {
//        sendFrame(rltBuf, len - rlt);
//    }
//}
