#include "CJt808Video.h"
#include "common.h"
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include "CH264Parse.h"
#include <arpa/inet.h>
#include <QString>

int CJt808Video::rtp_inc_sn = 0;

CJt808Video::CJt808Video()
    : m_connectMode(SEND_TYPE_NONE)
{
    m_pSendBuff = new BYTE[2000];
}

CJt808Video::~CJt808Video()
{
    SAFE_DELETE_ARRAY(m_pSendBuff)
}
void CJt808Video::setPhoneNum(std::string phone, int channel)
{
    m_strPhoneNum = phone;
    m_head.channel = channel;

    /* 手机号 BCD[6] 不足位的情况下前补0*/
    int phone_len = std::min(12, (int)phone.length());
    char *phone_num = (char *)phone.c_str();

    for (int i = 0; i < phone_len; i += 2)
    {
        if (phone_len - i > 2)
        {
            m_head.sim_bcd[5 - i / 2] = ((phone_num[phone_len - i - 2] - '0') << 4) + (phone_num[phone_len - i - 1] - '0');
        }
        else
        {
            m_head.sim_bcd[5 - i / 2] = (phone_num[phone_len - i - 1] - '0');
        }
    }
}

bool CJt808Video::connect2Server(std::string ip, int tcp_port, int udp_port)
{
    // TODO: 在此添加控件通知处理程序代码
    m_jt808ServIp = ip;
    m_strJt808TcpPort = tcp_port;
    m_strJt808UdpPort = udp_port;
    return connect2Serv();
}

bool CJt808Video::connect2Serv()
{
    bool bConnected = false;
    if (m_strJt808TcpPort != 0)
    {
        m_Jt808VideoSocket = socket(AF_INET, SOCK_STREAM, 0); // 指定为tcp方式

        memset(&m_servAddr, 0, sizeof(m_servAddr));
        m_servAddr.sin_family = AF_INET;
        m_servAddr.sin_port = htons(m_strJt808TcpPort);                // htons()作用是将端口号由主机字节序转换为网络字节序的整数值。
        m_servAddr.sin_addr.s_addr = inet_addr(m_jt808ServIp.c_str()); //inet_addr()作用是将一个IP字符串转化为一个网络字节序的整数值，

        if (SOCKET_ERROR == ::connect(m_Jt808VideoSocket, (sockaddr *)&m_servAddr, sizeof(m_servAddr)))
        {
            bConnected = false;
        }
        else
        {
            m_connectMode = SEND_BY_TCP;
            bConnected = true;
        }
    }

    if (bConnected == false)
    {
        if ((m_Jt808VideoSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1)
        {
            printf("创建套接字失败:");
            return false;
        }

        memset(&m_servAddr, 0, sizeof(m_servAddr));
        m_servAddr.sin_family = AF_INET;
        m_servAddr.sin_port = htons(m_strJt808UdpPort);                // htons()作用是将端口号由主机字节序转换为网络字节序的整数值。
        m_servAddr.sin_addr.s_addr = inet_addr(m_jt808ServIp.c_str()); //inet_addr()作用是将一个IP字符串转化为一个网络字节序的整数值，

        int ul = 0;
        int ret = ioctl(m_Jt808VideoSocket, FIONBIO, &ul); //设置为非阻塞模式
        if (ret == -1)
        {
            printf("设置非阻塞失败!");
        }

        m_connectMode = SEND_BY_UDP;
    }

    return true;
}

int CJt808Video::sendFrameData(char *buff, int size)
{
    static uint64_t ts = 36;
    // 扫盲
    // h264常见的帧头数据为：
    // 00 00 00 01 67 (SPS)
    // 00 00 00 01 68 (PPS)
    // 00 00 00 01 65 ( IDR 帧)
    // 00 00 00 01 61 (P帧)

    // NALU Header常见的取值：0x67 0x68 0x65 0x61，0x47 0x48 0x45 0x41，0x27 0x28 0x25 0x21

    // NALU header 描述
    // 0x67，0x47，0x27 SPS， 序列参数集，重要级别分别为11、10、01
    // 0x68，0x48，0x28 PPS， 图像参数集，重要级别分别为11、10、01
    // 0x65，0x45，0x25 IDR帧，重要级别分别为 11、10、01
    // 0x61，0x41，0x21 非IDR帧，重要级别分别为 11、10、01

    // 组包参考https://blog.csdn.net/ichenwin/article/details/100086930;

    int flag = four_char_to_long((BYTE *)buff);

    //    if (flag == 0x00000001)
    //    {
    //        BYTE type = buff[4] & 0x0F;
    //        switch (type)
    //        {
    //        case 7: // SPS
    //        case 8: // PPS
    //            m_head.date_type = F_I;
    //            ts += 3600;
    //            break;
    //        case 5: // IDR
    //            ts += 3600;
    //            m_head.date_type = F_I;
    //        case 1: // P/B
    //            ts += 3600;
    //            m_head.date_type = F_I;
    //            break;
    //        default:
    //            m_head.date_type = F_I;
    //            ts += 3600;
    //            break;
    //        }
    //    }
    ts += 36;
    sendFrameData(buff, size, ts);

    //gb28181_streampackageForH264(buff, size, &m_DataInfo, 0); //0 表示传递的是视频数据
    //m_DataInfo.s64CurPts += 3600;
    return 0;
}

int CJt808Video::sendFrameData(char *buff, int size, uint64_t &ts)
{
    if (size > 950)
    {
        int i = 0;
    }
    // 先获取帧格式类型
    m_head.sub_pkg_mark = SPF_FIRST;
    m_head.date_type = F_I;
    m_head.ts = ts;
    // 遍历发送
    int buff_idx = 0;
    int send_len = 0;

    int i = 0;

    while (buff_idx < size)
    {
        // 根据发包计算子包类型和拷贝数量
        if (size < JT1078_MAX_LENGTH)
        {
            m_head.payload_len = size;
            m_head.sub_pkg_mark = SPF_ATOM;
            m_head.M1 = 1;
        }
        else if (size - buff_idx <= JT1078_MAX_LENGTH)
        {
            m_head.payload_len = size - buff_idx;
            m_head.sub_pkg_mark = SPF_LAST;
            m_head.M1 = 1;
        }
        else
        {
            m_head.payload_len = JT1078_MAX_LENGTH;
            if (m_head.sub_pkg_mark == SPF_FIRST)
            {
                m_head.M1 = 1;
            }
            else
            {
                m_head.M1 = 0;
            }
        }

        // 生成RTP头
        // memcpy(m_pSendBuff, &m_head, sizeof(m_head));
        m_head.SN = ++rtp_inc_sn;
        int idx = makeRtpHead(m_pSendBuff, m_head);
        // 拷贝净荷
        memcpy(m_pSendBuff + idx, buff + buff_idx, m_head.payload_len);
        send_len = m_head.payload_len + idx;

        // 码流调试代码
        // qDebug("buff长度:%d,发送长度:%d, 分包序号:%d, 子包类型:%d, M1类型:%d,净荷长度:%d",
        //        size, send_len, i++, m_head.sub_pkg_mark, m_head.M1, m_head.payload_len);

        // QString strOut = "发送1078流：";
        // for (int i = 0; i < 36; i++)
        // {
        //     QString str;
        //     strOut += str.sprintf("%02x ", m_pSendBuff[i]);
        // }
        // qDebug("%s", strOut.toStdString().c_str());

        // 发送
        if (send_len != sendData((char *)m_pSendBuff, send_len))
        {
            // 丢包或链接丢失,直接退出发送
            break;
        }

        buff_idx += m_head.payload_len;
        m_head.sub_pkg_mark = SPF_MID;
    }
}

int CJt808Video::makeRtpHead(BYTE *send_data, JT_1078_VIDEO_HEADER &head)
{

    uint8_t vpxcc = (head.V2 << 6) + (head.P1 << 5) + (head.X1 << 4) + head.CC4;

    uint8_t pt = (head.M1 << 7) + head.PT7;

    uint8_t datatype_subpackageflag = (head.date_type << 4) + head.sub_pkg_mark;

    size_t head_size = 30;
    if (head.date_type == A_F)
    {
        head_size -= 4;
    }
    else if (head.date_type == T_T)
    {
        head_size -= 12;
    }

    size_t offset = 0;
    memcpy(send_data, head.FramHead, 4);
    offset += 4;
    send_data[offset] = vpxcc;
    ++offset;
    send_data[offset] = pt;
    ++offset;

    uint16_t sn = htons(head.SN);
    // memcpy(send_data + offset, &sn, 2);
    short_to_two_char(head.SN, send_data + offset);
    offset += 2;

    memcpy(send_data + offset, head.sim_bcd, 6);
    offset += 6;

    send_data[offset] = head.channel;
    ++offset;

    send_data[offset] = datatype_subpackageflag;
    ++offset;

    if (head.date_type == T_T)
    {
    }
    else if (head.date_type == A_F)
    {
        int64_to_eight_char(head.ts, send_data + offset);
        offset += 8;
    }
    else
    {
        int64_to_eight_char(head.ts, send_data + offset);
        offset += 8;

        uint16_t tmp = htons(3600);
        // memcpy(send_data + offset, &tmp, 2);
        short_to_two_char(3600, send_data + offset);
        offset += 2;

        tmp = htons(3600);
        // memcpy(send_data + offset, &tmp, 2);
        short_to_two_char(3600, send_data + offset);
        offset += 2;
    }

    // uint16_t len = htons(head.payload_len);
    // memcpy(send_data + offset, &len, 2);
    short_to_two_char(head.payload_len, send_data + offset);
    offset += 2;

    return offset;
}

int CJt808Video::sendData(char *pData, int len)
{
#ifdef WIN32
    int con_para = 0;
#else
    int con_para = MSG_NOSIGNAL;
#endif
    int send_cnt = -1;

    switch (m_connectMode)
    {
    case SEND_BY_TCP:
        send_cnt = send(m_Jt808VideoSocket, pData, len, con_para);
        // TCP 要重连
        if (send_cnt == -1)
        {
            connect2Serv();
        }
        break;
    case SEND_BY_UDP:
        send_cnt = sendto(m_Jt808VideoSocket, pData, len, 0, (struct sockaddr *)&m_servAddr, sizeof(m_servAddr));
        break;
    default:
        break;
    }

    return send_cnt;
}
