#ifndef CJT808VIDEO_H
#define CJT808VIDEO_H
#include <QString>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/tcp.h>
#include "common.h"
#include <string>

//分包处理标记
#define RCV_ATOMIC_PACKAGE 0x00
#define RCV_FIRST_PACKAGE 0x01
#define RCV_LAST_PACKAGE 0x02
#define RCV_MIDDLE_PACKAGE 0x03
#define FIRST_RECEIVE_BYTES 16

#define PKG_HD_INIT 0               //初始状态，准备接收数据包头前15字节
#define PKG_HD_RECVING 1            //接收包头中，包头不完整，继续接收中
#define PKG_HD_REMAINING_INIT 2     //后续包头字节
#define PKG_HD__REMAINING_RECVING 3 //
#define PKG_BD_INIT 4               //包头刚好收完，准备接收包体
#define PKG_BD_RECVING 5            //接收包体中，包体不完整，继续接收中，处理后直接回到_PKG_HD_INIT状态
#define PKG_RV_FINISHED 6           //完整包收完

enum PTType
{
    PT_AAC = 19,
    PT_MP3 = 25,
    PT_H264 = 98,
    PT_H265 = 99,
};

enum PackageDataType
{
    F_I = 0x00,
    F_P = 0x01,
    F_B = 0x02,
    A_F = 0x03,
    T_T = 0x04
};

enum SubPackageFlag
{
    SPF_ATOM = 0x00,
    SPF_FIRST = 0x01,
    SPF_LAST = 0x02,
    SPF_MID = 0x03
};

//一些和网络通讯相关的结构
//包头结构
#pragma pack(1)

typedef struct JT_1078_VIDEO_HEADER
{
    uint8_t FramHead[4]{0x30, 0x31, 0x63, 0x64}; //帧头标识
    uint8_t V2 : 2;                              //固定为2
    uint8_t P1 : 1;                              //固定为0
    uint8_t X1 : 1;                              //RTP头是否需要扩展位，固定为0
    uint8_t CC4 : 4;                             //固定为1
    uint8_t M1 : 1;                              //标志位，确定是否是完整数据帧的边界
    uint8_t PT7 : 7;                             //负载类型
    uint16_t SN{0};                              //RTP数据包序号每发送一个RTP数据包序列号加
    uint8_t sim_bcd[6]{0, 0, 0, 0, 0, 0};        //SIM卡号
    uint8_t channel{1};                          //逻辑通道号
    uint8_t date_type : 4;                       //数据类型
    uint8_t sub_pkg_mark : 4;                    //分包处理标记
    uint64_t ts;                                 //时间戳
    uint16_t iframe_interval;                    //与上一帧的时间间隔
    uint16_t last_frame_interval;                //与上一帧的时间间隔
    uint16_t payload_len;                        //数据体长度

    JT_1078_VIDEO_HEADER()
        : V2(2),
          P1(0),
          X1(0),
          CC4(1),
          M1(0),
          PT7(98),
          date_type(F_I),
          sub_pkg_mark(SPF_ATOM),
          payload_len(0),
          channel(1),
          iframe_interval(0),
          last_frame_interval(0)
    {
    }
} JT_1078_VIDEO_HEADER;

#define JT_1078_VIDEO_HEADER_LEN 30
#pragma pack()

#define JT1078_MAX_LENGTH 950               //每个包的最大长度包头+包体不超过这个数字,为了留出一些空间,实际上编码包头+包体长度必须不大于这个值(_PKG_MAX_LENGTH-1000)
#define HEAD_BUFSIZE sizeof(JT_1078_HEADER) //包头大小

class CJt808Video
{
    enum VIDEO_SEND_MODE
    {
        SEND_TYPE_NONE = 0,
        SEND_BY_TCP = 1,
        SEND_BY_UDP = 2,
    };

public:
    CJt808Video();
    virtual ~CJt808Video();
    // 开始JT808实时视频传输
    // 结束jt808实时视频传输
    // 开始历史视频传输
    // 结束历史视频传输

    // 连接视频服务器
    virtual bool connect2Server(std::string ip, int tcp_port, int udp_port);

    // 发送数据
    int sendFrameData(char *buff, int size);

    // 设置管道电话号码
    void setPhoneNum(std::string phone, int channel);

protected:
    // 发送数据
    int sendData(char *pData, int len);

    // 无参数链接或重连
    bool connect2Serv();

    int sendFrameData(char *buff, int size, uint64_t &ts);
    int makeRtpHead(BYTE *buff, JT_1078_VIDEO_HEADER &head);

protected:
    int m_Jt808VideoSocket;
    std::string m_jt808ServIp;
    int m_strJt808TcpPort;
    int m_strJt808UdpPort;
    int m_connectMode;
    int m_channel;

    JT_1078_VIDEO_HEADER m_head;

    struct sockaddr_in m_servAddr;
    BYTE *m_pSendBuff;
    static int rtp_inc_sn;
    std::string m_strPhoneNum;
};

#endif // CJT808VIDEO_H
