#include "CVideoPipe.h"
#include <QDebug>
#include <string.h>
#include "CCameraManager.h"

using namespace std;

CVideoPipe::CVideoPipe()
    : m_bStarted(false),
      m_gstPipe(nullptr)
{
}

CVideoPipe::~CVideoPipe()
{
}

// 设置管道源信息
bool CVideoPipe::setSource(QString strSrc)
{
    m_strSource = strSrc;
}

// 消息回调
gboolean CVideoPipe::postGstMessage(GstBus *bus, GstMessage *message, gpointer user_data)
{
    (void)bus;
    (void)user_data;
    switch (GST_MESSAGE_TYPE(message))
    {
    case GST_MESSAGE_STATE_CHANGED:
    {
        break;
    }

    case GST_MESSAGE_TAG:
    {
        break;
    }
    case GST_MESSAGE_EOS:
    {
        break;
    }
    default:
        break;
    }
    return TRUE;
}

// 结束流媒体管道
bool CVideoPipe::pausePipe()
{
    if (m_gstPipe)
    {
        gst_element_set_state(m_gstPipe, GST_STATE_PAUSED);
    }
}

// 结束流媒体管道
bool CVideoPipe::playPipe()
{
    if (m_gstPipe)
    {
        gst_element_set_state(m_gstPipe, GST_STATE_PLAYING);
    }
}

// rtsp主源 显示 记录管道
CRtsp2RecordPipe::CRtsp2RecordPipe()
    : m_CamWID(0),
      m_vSink(nullptr),
      m_fileSink(nullptr),
      m_snapshotSink(nullptr),
      m_pauseCnt(0)
{
}

CRtsp2RecordPipe::~CRtsp2RecordPipe()
{
    stopPipe();
}

bool CRtsp2RecordPipe::setSource(QString strSrc)
{
    if (m_bStarted == true)
    {
        return false;
    }

    // gst 源
    QString strGstSource = QString("rtspsrc location=%1  latency=0 ! rtph264depay ! video/x-h264, stream-format=byte-stream ! identity sync=true ! tee name=t").arg(strSrc);
    // gst文件sink
    QString strGstFileSink = " t. ! queue ! filesink location=%1 name=filesink";
    // gst输出显示   omxh264dec
    QString strGstVideoSink = " t. ! queue ! h264parse ! nvv4l2decoder !tee name=t1 t1.! nvvidconv !  queue !  nvjpegenc quality=70 ! appsink name=snap_sink drop=true max-buffers=1 sync=false  t1.! queue ! nvvidconv ! xvimagesink sync=false name=video-sink";

    m_strLaunch = strGstSource + strGstFileSink + strGstVideoSink;

    return true;
}

// 设置文件保存路径
bool CRtsp2RecordPipe::setFileSavePath(QString path_name)
{
    if (m_bStarted == true)
    {
        return false;
    }

    // 启动参数为空直接返回失败
    if (m_strLaunch.isEmpty() || !m_strLaunch.contains("%1"))
    {
        return false;
    }
    //
    m_curFileName = path_name;
    m_strCurLaunch = m_strLaunch.arg(path_name);
    return true;
}

// 检查并恢复pipe状态
void CRtsp2RecordPipe::checkPipeState()
{
    if (m_gstPipe == nullptr)
    {
        return;
    }

    // 连续10秒pipeline状态还是paused则重启管道

    GstStateChangeReturn sret;
    GstState state, state_pause;
    sret = gst_element_get_state(m_gstPipe, &state, &state_pause, 5000);

    if (sret == GST_STATE_CHANGE_FAILURE || state == GST_STATE_PAUSED)
    {
        m_pauseCnt++;
        if (m_pauseCnt > 5)
        {
            stopPipe();
            setFileSavePath(m_curFileName);
            startPipe();
            m_pauseCnt = 0;
        }
    }
    else
    {
        m_pauseCnt = 0;
    }
}

// 保存到新文件
bool CRtsp2RecordPipe::save2NewFile(QString path_name)
{
    m_curFileName = path_name;
    QMutexLocker lock(&m_mtxPipe);
    GstStateChangeReturn sret;
    sret = gst_element_set_state(m_fileSink, GST_STATE_NULL);
	
    if (sret == GST_STATE_CHANGE_FAILURE)
    {
        gst_element_set_state(m_gstPipe, GST_STATE_NULL);
        gst_object_unref(m_gstPipe);
        gst_object_unref(m_vSink);
        gst_object_unref(m_fileSink);
        gst_object_unref(m_snapshotSink);

        m_gstPipe = nullptr;
        m_vSink = nullptr;
        m_fileSink = nullptr;
        m_snapshotSink = nullptr;
        m_bStarted = false;
        LOG_ERROR(CCameraManager, "gst_element_set_state state error!");
        return false;
    }

    string str_location = path_name.toStdString();
    qDebug() << path_name << str_location.c_str();
    g_object_set(m_fileSink, "location", str_location.c_str(), NULL);

    // run the pipeline
    sret = gst_element_set_state(m_fileSink, GST_STATE_PLAYING);
    sret = gst_element_set_state(m_gstPipe, GST_STATE_PLAYING);

    return true;
}

// 设置视频现实窗口的wid
bool CRtsp2RecordPipe::setWID(DWORD WID)
{
    m_CamWID = WID;
    return true;
}

// 消息回调
gboolean CRtsp2RecordPipe::postGstMessage(GstBus *bus, GstMessage *message, gpointer user_data)
{
    (void)bus;
    (void)user_data;
    CRtsp2RecordPipe *pipe = (CRtsp2RecordPipe *)(user_data);
    switch (GST_MESSAGE_TYPE(message))
    {
    case GST_MESSAGE_STATE_CHANGED:
    {
        if (GST_MESSAGE_SRC(message) == GST_OBJECT(pipe->m_gstPipe))
        {
            GstState old_state, new_state, pending_state;
            gst_message_parse_state_changed(message, &old_state, &new_state, &pending_state);
            g_print("Pipeline state changed from %s to %s:\n",
                    gst_element_state_get_name(old_state), gst_element_state_get_name(new_state));
        }
        break;
    }

    case GST_MESSAGE_TAG:
    {
        break;
    }
    case GST_MESSAGE_EOS:
    {
        break;
    }
    default:
        break;
    }
    return TRUE;
}
// 开始流媒体管道
bool CRtsp2RecordPipe::startPipe()
{
    QMutexLocker lock(&m_mtxPipe);
    qDebug() << "gst_parse_launch:" << m_strCurLaunch;

    // 启动参数为空直接返回失败
    if (m_strCurLaunch.isEmpty() || m_strCurLaunch.contains("%1"))
    {
        return false;
    }

    m_gstPipe = gst_parse_launch(m_strCurLaunch.toStdString().c_str(), NULL);

    if (!m_gstPipe)
    {
        return false;
    }

    // seg window id to gstreamer
    m_vSink = gst_bin_get_by_name(GST_BIN(m_gstPipe), "video-sink");
    GstElement *snap_sink = gst_bin_get_by_name(GST_BIN(m_gstPipe), "snap_sink");
    m_fileSink = gst_bin_get_by_name(GST_BIN(m_gstPipe), "filesink");
    if (m_vSink)
    {
        gst_video_overlay_set_window_handle(GST_VIDEO_OVERLAY(m_vSink), m_CamWID);
    }

    m_snapshotSink = (GstAppSink *)(snap_sink);
	
    if (m_snapshotSink)
    {
        // g_signal_connect(m_gstPipe, "deep-notify", G_CALLBACK(gst_object_default_deep_notify ), NULL);
        gst_app_sink_set_emit_signals(m_snapshotSink, true);
        gst_app_sink_set_drop(m_snapshotSink, true);
        gst_app_sink_set_max_buffers(m_snapshotSink, 1);
		
		
    }

    // connect to interesting Q_SIGNALSd
    GstBus *bus = gst_element_get_bus(m_gstPipe);
    gst_bus_add_watch(bus, (GstBusFunc)postGstMessage, this);
    gst_object_unref(bus);

    // run the m_gstPipe
    GstStateChangeReturn sret = gst_element_set_state(m_gstPipe, GST_STATE_PLAYING);

    if (sret == GST_STATE_CHANGE_FAILURE)
    {
        qDebug() << "=================================================================================";
        qDebug() << "=========================GstStateChangeReturn ERROR==============================";
        qDebug() << "=================================================================================";
        gst_element_set_state(m_gstPipe, GST_STATE_NULL);
        gst_object_unref(m_gstPipe);
        return false;
    }

    m_bStarted = true;

    return true;
}

// 结束流媒体管道
bool CRtsp2RecordPipe::stopPipe()
{
    QMutexLocker lock(&m_mtxPipe);
    if (m_bStarted)
    {
        gst_element_set_state(m_gstPipe, GST_STATE_NULL);
        gst_object_unref(m_gstPipe);
        gst_object_unref(m_vSink);
        gst_object_unref(m_fileSink);
        gst_object_unref(m_snapshotSink);

        m_gstPipe = nullptr;
        m_vSink = nullptr;
        m_fileSink = nullptr;
        m_snapshotSink = nullptr;
        m_bStarted = false;
    }
    return true;
}

bool CRtsp2RecordPipe::snapshot(QString path_name)
{
    // 如果没有appsink或pipeline 直接退出
    if (!m_gstPipe || !m_snapshotSink)
    {

        return false;
    }

    GstSample *sample = nullptr;
    GstBuffer *sampleBuffer = nullptr;
    GstMapInfo mapInfo;
    GstCaps *caps = nullptr;
    GstStructure *capsStruct = nullptr;
    bool ret;
    int arrivedWidth = 0;
    int arrivedHeight = 0;
    QImage::Format externalFormat = QImage::Format_RGB888;

    // 从appsink获取sample
    g_object_get(m_snapshotSink, "last-sample", &sample, nullptr);
    if (sample == nullptr)
    {
        return false;
    }

    // 获取caps
    caps = gst_sample_get_caps(sample);
    capsStruct = gst_caps_get_structure(caps, 0 /*index*/);
    if (capsStruct == nullptr)
    {
        qWarning("Failed getting structure from caps.");
        return false;
    }

    // 获取宽高属性
    ret = gst_structure_get_int(capsStruct, "width", &arrivedWidth);
    if (ret == false)
    {
        qWarning("Failed getting width from structure.");
        return false;
    }

    ret = gst_structure_get_int(capsStruct, "height", &arrivedHeight);
    if (ret == false)
    {
        qWarning("Failed getting height from structure.");
        return false;
    }

    // 获取buff
    sampleBuffer = gst_sample_get_buffer(sample);
    if (sampleBuffer != nullptr)
    {
        // buff_map
        gst_buffer_map(sampleBuffer, &mapInfo, GST_MAP_READ);
		
        FILE *of = fopen(path_name.toStdString().c_str(), "wb");
        if (of == NULL)
        {
            //            fprintf(stderr, "File %s could not be created\n", outfile);
        }
        else
        {
            if (fwrite(mapInfo.data, 1, mapInfo.size, of) != mapInfo.size)
            {
                //                fprintf(stderr, "Could not write to file %s\n", outfile);
            }
            fclose(of);
        }
		
        gst_buffer_unmap(sampleBuffer, &mapInfo);
    }
    gst_sample_unref(sample);
}

jpgBuf *CRtsp2RecordPipe::snapshotString()
{
	 // 如果没有appsink或pipeline 直接退出
    if (!m_gstPipe || !m_snapshotSink)
    {

        return NULL;
    }

    GstSample *sample = nullptr;
    GstBuffer *sampleBuffer = nullptr;
    GstMapInfo mapInfo;
    GstCaps *caps = nullptr;
    GstStructure *capsStruct = nullptr;
    bool ret;
    int arrivedWidth = 0;
    int arrivedHeight = 0;
    QImage::Format externalFormat = QImage::Format_RGB888;
	
	
    // 从appsink获取sample
    g_object_get(m_snapshotSink, "last-sample", &sample, nullptr);
    if (sample == nullptr)
    {
        return NULL;
    }

    // 获取caps
    caps = gst_sample_get_caps(sample);
    capsStruct = gst_caps_get_structure(caps, 0 /*index*/);
    if (capsStruct == nullptr)
    {
        qWarning("Failed getting structure from caps.");
        return NULL;
    }

    // 获取宽高属性
    ret = gst_structure_get_int(capsStruct, "width", &arrivedWidth);
    if (ret == false)
    {
        qWarning("Failed getting width from structure.");
        return NULL;
    }

    ret = gst_structure_get_int(capsStruct, "height", &arrivedHeight);
    if (ret == false)
    {
        qWarning("Failed getting height from structure.");
        return NULL;
    }
	jpgBuf* pb = new jpgBuf;
	pb->width = arrivedWidth;
	pb->height = arrivedHeight;
	pb->size = -1;
    // 获取buff
    sampleBuffer = gst_sample_get_buffer(sample);
    if (sampleBuffer != nullptr)
    {
        // buff_map
        gst_buffer_map(sampleBuffer, &mapInfo, GST_MAP_READ);
		pb->size = mapInfo.size;
		pb->jpg=(char*)malloc(sizeof(char)*(pb->size));
		memcpy(pb->jpg,mapInfo.data, pb->size);
        gst_buffer_unmap(sampleBuffer, &mapInfo);
    }
    gst_sample_unref(sample);
	
	return pb;
}



CRtsp2RtmpPipe::CRtsp2RtmpPipe()
{
}
CRtsp2RtmpPipe::~CRtsp2RtmpPipe()
{
}
// 开始流媒体管道
bool CRtsp2RtmpPipe::startPipe()
{
}
// 结束流媒体管道
bool CRtsp2RtmpPipe::stopPipe()
{
}
