#ifndef CVIDEOPIPE_H
#define CVIDEOPIPE_H
#include <QString>
#include <QWidget>
#include <QObject>
#include <gst/gst.h>
#include <gst/video/videooverlay.h>
#include <gst/app/gstappsink.h>
#include <common.h>
#include <QMutex>

typedef struct _jpgBuf {
	char *jpg;
	int size;
	int width;
	int height;
	
}jpgBuf;



class CVideoPipe : public QObject
{
    // Q_OBJECT
public:
    CVideoPipe();
    virtual ~CVideoPipe();

    // 开始流媒体管道
    virtual bool startPipe() = 0;
    // 结束流媒体管道
    virtual bool stopPipe() = 0;

    // 结束流媒体管道
    virtual bool pausePipe();

    // 结束流媒体管道
    virtual bool playPipe();

    // 设置管道源信息
    virtual bool setSource(QString strSrc);

    // 消息回调
    static gboolean postGstMessage(GstBus *bus, GstMessage *message, gpointer user_data);

    // 是否已经开始
    bool isStarted() { return m_bStarted; }

    // 获取当前启动字段
    QString getSource() { return m_strSource; }

protected:
    // 是否已经开始
    bool m_bStarted;

    // 启动源字段
    QString m_strLaunch;

    // 源
    QString m_strSource;

    //源信息
    GstElement *m_gstPipe;

    QMutex m_mtxPipe;
};

// rtsp主源 显示 记录管道
class CRtsp2RecordPipe : public CVideoPipe
{
public:
    CRtsp2RecordPipe();
    virtual ~CRtsp2RecordPipe();

    // 开始流媒体管道
    virtual bool startPipe();

    // 结束流媒体管道
    virtual bool stopPipe();

    // 设置源
    virtual bool setSource(QString strSrc);

    // 设置文件保存路径
    virtual bool setFileSavePath(QString path_name);

    // 保存到新文件
    virtual bool save2NewFile(QString path_name);

    // 设置视频现实窗口的wid
    bool setWID(DWORD WID);

    bool snapshot(QString path_name);
	jpgBuf *snapshotString();
    // 检查并恢复pipe状态
    void checkPipeState();

protected:
    // 消息回调
    static gboolean postGstMessage(GstBus *bus, GstMessage *message, gpointer user_data);

protected:
    WId m_CamWID;
    GstElement *m_vSink;
    GstElement *m_fileSink;
    GstAppSink *m_snapshotSink;

    int m_pauseCnt;
    QString m_curFileName;
    QString m_strCurLaunch;
};

// 文件到jt808管道
class CFile2RtmpPipe : public CVideoPipe
{
public:
    CFile2RtmpPipe();
    ~CFile2RtmpPipe();
    // 开始流媒体管道
    virtual bool startPipe();
    // 结束流媒体管道
    virtual bool stopPipe();

protected:
};

// rstp到jt808管道
class CRtsp2RtmpPipe : public CVideoPipe
{
public:
    CRtsp2RtmpPipe();
    ~CRtsp2RtmpPipe();
    // 开始流媒体管道
    virtual bool startPipe();
    // 结束流媒体管道
    virtual bool stopPipe();

protected:
};
#endif // CVIDEOPIPE_H
