#include "Ckafkaproducer.h"
#include <sys/time.h>
#include <ctime>
#include <chrono>
#include <time.h>
#include <sstream>
CKafkaProducer::CKafkaProducer()
{
    m_kafka_handle = NULL;
    m_kafka_topic = NULL;
    m_kafka_conf = NULL;
    m_kafka_topic_conf = NULL;
    m_kafka_topic_partition_list = NULL;
 
    m_partition = RD_KAFKA_PARTITION_UA;
}

CKafkaProducer::~CKafkaProducer()
{
    rd_kafka_flush(m_kafka_handle, 10*1000); //wait for max 10 seconds
    
    rd_kafka_topic_destroy(m_kafka_topic);
    rd_kafka_destroy(m_kafka_handle);
    rd_kafka_topic_partition_list_destroy(m_kafka_topic_partition_list);
	terminateTaskThread();
}

void CKafkaProducer::terminateTaskThread()
{
	if (m_taskThread != nullptr) {
        m_taskThread->detach();
        delete m_taskThread;
        m_taskThread = nullptr;
    }
	
}
static void
dr_msg_cb(rd_kafka_t *rk, const rd_kafka_message_t *rkmessage, void *opaque) {
        if (rkmessage->err)
                fprintf(stderr, "%% Message delivery failed: %s\n",
                        rd_kafka_err2str(rkmessage->err));
        else{
                fprintf(stderr,
                        "%% Message delivered (%zd bytes, "
                        "partition %" PRId32 ")\n",
                        rkmessage->len, rkmessage->partition);
						
				char *str = (char*)malloc(rkmessage->len);
				memcpy(str, rkmessage->payload, rkmessage->len);
				fprintf(stderr,"buff is %s\n", str);
				free(str);
		}		

        /* The rkmessage is destroyed automatically by librdkafka */
}
static int
stats_cb(rd_kafka_t *rk,  char *json,size_t json_len,void *opaque)
{
	 char err[json_len];
	 memcpy(err,json,json_len);
	 fprintf(stderr, "%% stats_cb: %s: %s\n",err );
	 return 0;
}
static void
error_cb(rd_kafka_t *rk, int err, const char *reason, void *opaque) {
        rd_kafka_resp_err_t orig_err;
        char errstr[512];

        fprintf(stderr, "%% Error: %s: %s\n", rd_kafka_err2name((rd_kafka_resp_err_t)err), reason);

        if (err != RD_KAFKA_RESP_ERR__FATAL)
                return;

        //orig_err = rd_kafka_fatal_error(rk, errstr, sizeof(errstr));
        //fprintf(stderr, "%% FATAL ERROR: %s: %s\n", rd_kafka_err2name(orig_err),
         //       errstr);

        /* Clean termination to get delivery results (from rd_kafka_flush())
         * for all outstanding/in-transit/queued messages. */
        fprintf(stderr, "%% Terminating on fatal error\n");
      
}

static void
logger(const rd_kafka_t *rk, int level, const char *fac, const char *buf) {
        struct timeval tv;
        gettimeofday(&tv, NULL);
        fprintf(stderr, "%u.%03u RDKAFKA-%i-%s: %s: %s\n", (int)tv.tv_sec,
                (int)(tv.tv_usec / 1000), level, fac,
                rk ? rd_kafka_name(rk) : NULL, buf);
}


int CKafkaProducer::init(char *topic, char *brokers, int partition)
{
	int ret = 0;
    rd_kafka_conf_res_t ret_conf = RD_KAFKA_CONF_OK;
	char errstr[512] = {0};
	
	m_kafka_conf = rd_kafka_conf_new();
	
	if (rd_kafka_conf_set(m_kafka_conf, "bootstrap.servers", brokers, errstr,
                              sizeof(errstr)) != RD_KAFKA_CONF_OK) {
		printf( "kafka set brokers faild [%s]\n", errstr);
		rd_kafka_conf_destroy(m_kafka_conf);
		return -1;
    }
	/*
	if (rd_kafka_conf_set(m_kafka_conf, "enable.idempotence", "1", errstr,
                              sizeof(errstr)) != RD_KAFKA_CONF_OK) {
		printf("%s\n", errstr);
		rd_kafka_conf_destroy(m_kafka_conf);
		return -1;
   }
*/

    
	//启用异步模式时，producer缓存消息的时间。我们设置成1000时，它会缓存1s的数据再一次发送出去，这样可以极大的增加broker吞吐量，但也会造成时效性的降低。
	ret_conf = rd_kafka_conf_set(m_kafka_conf, "queue.buffering.max.ms", "0", errstr, sizeof(errstr));
    if(ret_conf != RD_KAFKA_CONF_OK){printf("Error: rd_kafka_conf_set() failed 1; ret_conf=%d; errstr:%s\n", ret_conf, errstr); return -1;}
	
	//启用异步模式时，producer缓存队列里最大缓存的消息数量，如果超过这个值，producer就会阻塞或者丢掉消息。
    ret_conf = rd_kafka_conf_set(m_kafka_conf, "queue.buffering.max.messages", "2", errstr, sizeof(errstr));
    if(ret_conf != RD_KAFKA_CONF_OK){printf("Error: rd_kafka_conf_set() failed 1; ret_conf=%d; errstr:%s\n", ret_conf, errstr); return -1;}
	//启用异步模式时，一个batch缓存的消息数量。达到这个数值时，producer才会发送消息。（每次批量发送的数量）
	
	ret_conf = rd_kafka_conf_set(m_kafka_conf, "batch.num.messages", "1", errstr, sizeof(errstr));
    if(ret_conf != RD_KAFKA_CONF_OK){printf("Error: rd_kafka_conf_set() failed 1; ret_conf=%d; errstr:%s\n", ret_conf, errstr); return -1;}

	
	ret_conf = rd_kafka_conf_set(m_kafka_conf, "message.timeout.ms", "5 * 1000", errstr, sizeof(errstr));
    if(ret_conf != RD_KAFKA_CONF_OK){printf("Error: rd_kafka_conf_set() failed 1; ret_conf=%d; errstr:%s\n", ret_conf, errstr); return -1;}
 
	ret_conf = rd_kafka_conf_set(m_kafka_conf, "queue.buffering.max.ms", "5.0", errstr, sizeof(errstr));
    if(ret_conf != RD_KAFKA_CONF_OK){printf("Error: rd_kafka_conf_set() failed 1; ret_conf=%d; errstr:%s\n", ret_conf, errstr); return -1;}
 
	ret_conf = rd_kafka_conf_set(m_kafka_conf, "linger.ms", "0", errstr, sizeof(errstr));
    if(ret_conf != RD_KAFKA_CONF_OK){printf("Error: rd_kafka_conf_set() failed 1; ret_conf=%d; errstr:%s\n", ret_conf, errstr); return -1;}

	
    ret_conf = rd_kafka_conf_set(m_kafka_conf, "message.send.max.retries", "3", errstr, sizeof(errstr));
    if(ret_conf != RD_KAFKA_CONF_OK){printf("Error: rd_kafka_conf_set() failed 2; ret_conf=%d; errstr:%s\n", ret_conf, errstr); return -1;}

 
 
	ret_conf = rd_kafka_conf_set(m_kafka_conf, "acks", "1", errstr, sizeof(errstr));
	if(ret_conf != RD_KAFKA_CONF_OK){printf("Error: rd_kafka_conf_set() failed 3; ret_conf=%d; errstr:%s\n", ret_conf, errstr); return -1;}
	
	 ret_conf = rd_kafka_conf_set(m_kafka_conf, "max.in.flight.requests.per.connection", "1", errstr, sizeof(errstr));
	if(ret_conf != RD_KAFKA_CONF_OK){printf("Error: rd_kafka_conf_set() failed 3; ret_conf=%d; errstr:%s\n", ret_conf, errstr); return -1;}
	
 
    ret_conf = rd_kafka_conf_set(m_kafka_conf, "retry.backoff.ms", "800", errstr, sizeof(errstr));
    if(ret_conf != RD_KAFKA_CONF_OK){printf("Error: rd_kafka_conf_set() failed 3; ret_conf=%d; errstr:%s\n", ret_conf, errstr); return -1;}
 
	rd_kafka_conf_set_dr_msg_cb(m_kafka_conf, dr_msg_cb);
	rd_kafka_conf_set_error_cb(m_kafka_conf, error_cb);
	rd_kafka_conf_set_log_cb(m_kafka_conf, logger);
	rd_kafka_conf_set_opaque(m_kafka_conf, this);
	rd_kafka_conf_set_stats_cb(m_kafka_conf, stats_cb);
	m_kafka_topic_conf = rd_kafka_topic_conf_new();
	 ret_conf = rd_kafka_topic_conf_set(m_kafka_topic_conf, "auto.offset.reset", "earliest", errstr, sizeof(errstr));
    if(ret_conf != RD_KAFKA_CONF_OK){printf("Error: rd_kafka_conf_set() failed 4; ret_conf=%d; errstr:%s\n", ret_conf, errstr); return -1;}
 
    m_kafka_topic_partition_list = rd_kafka_topic_partition_list_new(1);
 
    rd_kafka_topic_partition_list_add(m_kafka_topic_partition_list, topic, partition); 
	m_partition = partition;
	m_kafka_handle = rd_kafka_new(RD_KAFKA_PRODUCER, m_kafka_conf, errstr, sizeof(errstr));
	if (!m_kafka_handle) {
		printf("%% Failed to create new producer: %s\n",
				errstr);
		return -1;
	}
	if(brokers && rd_kafka_brokers_add(m_kafka_handle, brokers) < 1)
    {
        printf("Error: No valid brokers specified\n");
        return -1;
    }
 
    m_kafka_topic = rd_kafka_topic_new(m_kafka_handle, topic, m_kafka_topic_conf); 
	m_taskThread = new std::thread(&CKafkaProducer::msgTask,this);
	return 0;
	
}
std::string Int_to_String(time_t n)
{

	std::ostringstream stream;

	stream<<n;  //n为int类型
	std::string res = "/home/hncy_data/" + stream.str() +".txt";
	return res;

}
void CKafkaProducer::msgTask() {
	while(1) {
		if(m_msg_queue.length() >0) {
			kafkaMsg* pmsg;
			m_msg_queue.pop(pmsg);
			fprintf(stderr, "send before\n");
			int ret = sendMessage(pmsg->msg, pmsg->size);
			fprintf(stderr, "send %d\n",
                        ret);
			if(ret)
			{
				printf("+++send faled");
				auto now = std::chrono::system_clock::now();
				time_t tt =std::chrono::system_clock::to_time_t(now);
				std::string rr = Int_to_String(tt);
				FILE *of = fopen(rr.c_str(), "wb");
				if (of == NULL)
				{
					//            fprintf(stderr, "File %s could not be created\n", outfile);
				}
				else
				{
					if (fwrite(pmsg->msg, 1, pmsg->size, of) != pmsg->size)
					{
						//                fprintf(stderr, "Could not write to file %s\n", outfile);
					}
					fclose(of);
				}
			}
			delete pmsg->msg;
			delete pmsg;
		}
		usleep(500);
	}



}


int CKafkaProducer::sendMessage(char *str, int len)
{
	
	rd_kafka_resp_err_t err;
	char * topic = m_kafka_topic_partition_list->elems[0].topic;
    int partition = m_kafka_topic_partition_list->elems[0].partition;
	 err = rd_kafka_producev(
                    m_kafka_handle, RD_KAFKA_V_TOPIC(topic),
                    RD_KAFKA_V_VALUE(str, len),
                    /* Copy the message payload so the `buf` can
                     * be reused for the next message. */
                    RD_KAFKA_V_MSGFLAGS(RD_KAFKA_MSG_F_COPY|RD_KAFKA_MSG_F_FREE ), RD_KAFKA_V_END);
	
	if (err) {
		
		if (err == RD_KAFKA_RESP_ERR__QUEUE_FULL) {
			rd_kafka_poll(m_kafka_handle,1000 /*block for max 1000ms*/);
			printf("%% kafka msg queue is full  %s: %s\n", topic,rd_kafka_err2str(err));
			err = rd_kafka_producev(
                    m_kafka_handle, RD_KAFKA_V_TOPIC(topic),
                    RD_KAFKA_V_VALUE(str, len),
                    /* Copy the message payload so the `buf` can
                     * be reused for the next message. */
                    RD_KAFKA_V_MSGFLAGS(RD_KAFKA_MSG_F_COPY|RD_KAFKA_MSG_F_FREE ), RD_KAFKA_V_END);
			if (err)
				printf("%% tryagain Failed to produce to topic %s: %s\n", topic,rd_kafka_err2str(err));
		} else {
			
			printf("%% Failed to produce to topic %s: %s\n", topic,rd_kafka_err2str(err));
		}
			
	}
	int r = rd_kafka_poll(m_kafka_handle, 0 /*non-blocking*/);
	//fprintf(stderr, "rd_kafka_poll %d\n",r);
    //rd_kafka_flush(m_kafka_handle, 10 * 1000 /* wait for max 10 seconds */);
	if (err)
		return -1;
	else
		return 0;
}

int CKafkaProducer::messagePushback(char* json_result,int json_len, char * jpg_char,int jpg_len)
{
	kafkaMsg* pmsg = (kafkaMsg*)malloc(sizeof(kafkaMsg));
	pmsg->size = json_len+jpg_len+1;
	pmsg->msg = (char*)malloc(pmsg->size);
	memcpy(pmsg->msg, json_result, json_len);
	pmsg->msg[json_len] = '$';
	memcpy(pmsg->msg+(json_len+1),jpg_char, jpg_len );
	
	m_msg_queue.push(pmsg);
	return 0;
}

int CKafkaProducer::messagejsonPushback(char* json_result,int json_len)
{
	kafkaMsg* pmsg = (kafkaMsg*)malloc(sizeof(kafkaMsg));
	pmsg->size = json_len;
	pmsg->msg = (char*)malloc(pmsg->size);
	memcpy(pmsg->msg, json_result, json_len);
	
	m_msg_queue.push(pmsg);
	return 0;
}


