#include "WebCamFinder.h"
#include <list>
#include <QDebug>
#include <string>
#include <assert.h>
#include <cstring>
#include <QHostAddress>
#include <QNetworkInterface>
#include <stdarg.h>

using namespace std;

#ifdef WIN32
#include <winsock2.h>
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <netdb.h>
#endif

#define MAXBUF 16*1024
#define PUERTO 3702
#define GRUPO "239.255.255.250"

extern QStringList getLocalIPs();

int OutputDebugString(string format, ...)
{
    //格式:count代表参数个数, ...代表n个参数
    char output[512] = {0};
    memset (output, 0, sizeof(output));

    va_list ap;
    va_start(ap, format); //获得可变参数列表
    vsprintf(output, format.c_str(), ap);
    va_end(ap); //释放资源

//    qDebug()<<QString(output);
    return 0;
}


static bool ValidateIPString(char* pszIP)
{
    if(pszIP == NULL || strlen(pszIP) < 7)
        return false;

    int a1, a2, a3, a4;
    int nParams = sscanf(pszIP, "%d.%d.%d.%d", &a1, &a2, &a3, &a4);
    if(nParams != 4)
        return false;

    sprintf(pszIP, "%d.%d.%d.%d", a1, a2, a3, a4);
    return true;
}


int HttpUrlGetIpPort(char *pUrl, char *pIp, unsigned short *pPort, char *pName)
{
    UINT   dwIP;
    char    *pChar,*pStreamName;
    char    str[256];
    char    tempIP[64] = {0};

    if( strlen( pUrl ) < 16 )    return -1;
    strncpy(str, pUrl,255);

    if(qstrnicmp( str, "http://", 7 ) )
        return -2;

    pChar = strchr( str+7, '/' );
    //    if( pChar == NULL )    return -3;
    if( pChar == NULL )
    {
        pStreamName = NULL;
    }
    else
    {
        pStreamName = pChar + 1;
        *pChar = 0;
    }

    pChar = strchr( str+7, ':' );
    if( pChar )
    {
        *pChar = 0;
        pChar ++;
        *pPort = (unsigned short)strtoul( pChar, NULL, 10 );
    }
    else
    {
        *pPort = 80;
    }

    strcpy(tempIP, str+7);
    ValidateIPString(tempIP); //去除冗余字符

    dwIP = inet_addr( tempIP );
    if( dwIP == INADDR_NONE )
        return -4;

    strcpy(pIp, tempIP);

    if( pStreamName )
    {
        strncpy( pName, pStreamName , 127);
        //sprintf_s( pUrl, 256, "rtsp://%d.%d.%d.%d/%s", (*pIp)&0xFF, ((*pIp)>>8)&0xFF, ((*pIp)>>16)&0xFF, (*pIp)>>24, pStreamName );
    }
    else
    {
        pName[0] = 0;
        //sprintf_s( pUrl, 256, "rtsp://%d.%d.%d.%d/stream", (*pIp)&0xFF, ((*pIp)>>8)&0xFF, ((*pIp)>>16)&0xFF, (*pIp)>>24 );
    }

    return 0;
}


CWebCamFinder::CWebCamFinder(void)
    :m_fd(0),
    m_bIsReceiving(false),
    m_lpCallback(nullptr),
    m_lpContext(nullptr)
{
}

CWebCamFinder::~CWebCamFinder(void)
{
    m_vctFd.clear();
}

void CWebCamFinder::SetDiscoveryInfoCB(DeviceDiscoverCB lpCallback, void* lpContext)
{
      m_lpCallback = lpCallback;
      m_lpContext = lpContext;
}

int CWebCamFinder::probeSocketIni(int *fd, string szLocalIP)
{
    struct ip_mreq mreq;
    struct sockaddr_in  cli;
    memset( &mreq, 0, sizeof(struct ip_mreq) );
    memset( &cli, 0, sizeof(struct sockaddr_in));

    //创建UDP Socket
    if( (*fd = socket( AF_INET, SOCK_DGRAM, IPPROTO_UDP) ) < 0 )
    {
        OutputDebugString("socket failed \n");
        close(*fd);
        return -1;
    }

    //if (inet_aton(GRUPO, &mreq.imr_multiaddr) < 0) {
    //    perror("inet_aton");
    //    close(*fd);
    //    return -1;
    //}

    //当多网卡或多IP存在时，绑定第一个IP
    //hostent* thisHost = ::gethostbyaddr();
    //inet_ntoa(*(struct in_addr*)*thisHost->h_addr_list);

    //绑定端口
    cli.sin_family = AF_INET;
    cli.sin_port = htons(/*PUERTO*/0); //让系统随机分配端口
    cli.sin_addr.s_addr = /*htonl(INADDR_ANY)*/inet_addr(szLocalIP.c_str());
    if( bind(*fd, (struct sockaddr *)&cli, sizeof(cli)) < 0 ){
        OutputDebugString("bind socket failed \n");
        return -1;
    }

    //加入到组播组
    mreq.imr_multiaddr.s_addr = inet_addr(GRUPO);
    mreq.imr_interface.s_addr = htonl(INADDR_ANY);
    if( setsockopt(*fd, IPPROTO_IP, IP_ADD_MEMBERSHIP, (const char*)&mreq, sizeof(mreq)) < 0 ){
        OutputDebugString("setsockopt IP_ADD_MEMBERSHIP failed \n");
        close(*fd);
        return -1;
    }

    //设置TTL
    int nTTL = 100;
    if(setsockopt(*fd, IPPROTO_IP, IP_MULTICAST_TTL, (const char*)&nTTL, sizeof(int)) < 0)
    {
        OutputDebugString("setsockopt IP_MULTICAST_TTL failed \n");
    }
    //设置不接收本机发出去的组播回声包
    int nLoopBack = 0;
    if(setsockopt(*fd, IPPROTO_IP, IP_MULTICAST_LOOP, (const char*)&nLoopBack, sizeof(int)) < 0)
    {
        OutputDebugString("setsockopt IP_MULTICAST_LOOP failed \n");
    }

    //设置接收缓冲区大小
    int nRecvBuf = 512*1024; 
    setsockopt( *fd, SOL_SOCKET,SO_RCVBUF, (const char*)&nRecvBuf, sizeof(int));
    return 0;
}

void CWebCamFinder::probeSocketEnd(int *fd)
{
    if(*fd != 0)
    {
        close(*fd);
        *fd = 0;
    }
}

int CWebCamFinder::sendProbe(int fd,const char *uuid,const char *types)
{
    int r;
    struct sockaddr_in srv;
    memset( &srv, 0, sizeof(struct sockaddr_in) );
    srv.sin_family = AF_INET;
    srv.sin_port = htons(PUERTO);
    srv.sin_addr.s_addr = inet_addr(GRUPO);
    //if( inet_aton(GRUPO, &srv.sin_addr ) < 0 ) {
    //    perror("inet_aton");
    //    close(fd);
    //    return -1;
    //}
    char Probe[4096];
    sprintf(Probe,"<?xml version=\"1.0\" encoding=\"utf-8\"?>"\
            "<Envelope xmlns:dn=\"http://www.onvif.org/ver10/network/wsdl\" xmlns=\"http://www.w3.org/2003/05/soap-envelope\">"\
            "<Header>"\
            "<wsa:MessageID xmlns:wsa=\"http://schemas.xmlsoap.org/ws/2004/08/addressing\">%s</wsa:MessageID>"\
            "<wsa:To xmlns:wsa=\"http://schemas.xmlsoap.org/ws/2004/08/addressing\">urn:schemas-xmlsoap-org:ws:2005:04:discovery</wsa:To>"\
            "<wsa:Action xmlns:wsa=\"http://schemas.xmlsoap.org/ws/2004/08/addressing\">http://schemas.xmlsoap.org/ws/2005/04/discovery/Probe</wsa:Action>"\
            "</Header>"\
            "<Body>"\
            "<Probe xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns=\"http://schemas.xmlsoap.org/ws/2005/04/discovery\">"\
            "<Types>%s</Types>"\
            "<Scopes />"\
            "</Probe>"\
            "</Body>"\
            "</Envelope>\r\n",uuid,types);

    if( (r = sendto(fd, Probe, strlen(Probe), 0, (struct sockaddr *)&srv, sizeof(srv))) < 0 )
    {
        OutputDebugString("sendto failed \n");
        close(fd);
        return -1;
    }

    return 0;
}

// 当前搜索是否空闲
bool CWebCamFinder::isSearchIdel()
{
    if(m_vctFd.size() > 0)
    {
        return false;
    }
    return true;
}

int CWebCamFinder::StartFind(void)
{
    if (!isSearchIdel()) 
    {
        return -2;
    }

    int nRet = 0;
    m_bIsReceiving = true;
    QStringList list = getLocalIPs();
    m_mtxFd.lock();
    for (int i = 0; i < list.size(); i++)
    {
        int fd;
        /* code */
        nRet = probeSocketIni(&fd, list[i].toStdString());
        if(nRet < 0)
        {
            m_mtxFd.unlock();
            return nRet;
        }

        nRet = sendProbe(fd,"uuid:e6153fce-af7a-4b48-94f7-05562e09b4f0","dn:NetworkVideoTransmitter");
        if(nRet < 0)
        {
            m_mtxFd.unlock();
            assert(0);
            fd = 0;
            return -1;
        }
        m_vctFd.push_back(fd);
    }
    m_mtxFd.unlock();
    
    //UDP接收
    ReceivingLoop();

    return true;
}

//UDP接收
void CWebCamFinder::ReceivingLoop(void)
{
    m_mtxFd.lock();
    for (int i = 0; i < m_vctFd.size(); i++)
    {
        recvProbeMatches(m_vctFd[i], 2000);
        probeSocketEnd(&m_vctFd[i]);
    }
    m_mtxFd.unlock();
}

void  CWebCamFinder::StopFind()
{
    if (!isShutdown() )
    {
        m_mtxFd.lock();
        for (int i = 0; i < m_vctFd.size(); i++)
        {
            /* code */
            probeSocketEnd(&m_vctFd[i]);
        }
        m_vctFd.clear();
        m_mtxFd.unlock();
        m_bIsReceiving = false;
    }
}

int CWebCamFinder::recvProbeMatches(int fd,int timeout_interval)
{
    int r;
    char buf[MAXBUF];
    unsigned long before= GetTickCount();

    std::list<DEVICE_IPC_INFO*> lstIpcs;
    
    char address[1024]; //内存空间要分配得大一点，否则可能会发生缓冲区溢出
    char hardwareID[128] = {0};
    char serial[128] = {0};

    char ProbeMatches[1024];
    char    *p_str_begin    = NULL;
    char    *p_str_end      = NULL;

     //设置接收超时时间
    //int timeout = timeout;
    struct timeval timeout={int(timeout_interval/1000),(timeout_interval%1000) * 1000};
    if (setsockopt(fd, SOL_SOCKET, SO_RCVTIMEO, (char*)&timeout, sizeof(timeout)) == SOCKET_ERROR)
    {
        qDebug()<<errno;
        OutputDebugString("Error: Set UDP Recv timeout \n");
    }

    sockaddr_in addr;
    memset(&addr, 0, sizeof(addr));
    addr.sin_family = AF_INET;
    addr.sin_addr.s_addr = htonl(INADDR_ANY);
    int time_cost = 0;
    while(m_bIsReceiving &&  time_cost < 6000)
    {
        time_cost = GetTickCount() - before;
        memset(buf,0,sizeof(buf));
        socklen_t nAddrLen = sizeof(addr);
        if( (r = recvfrom(fd, buf, MAXBUF, 0, (sockaddr*)&addr , &nAddrLen)) <= 0 )
        {
            OutputDebugString("recvfrom failed:errno:%d \n", errno);
            close(fd);
            return -1;
        }
        else
        {
            buf[r] = 0;
            OutputDebugString("IP: [%s]\n", inet_ntoa(addr.sin_addr));
            OutputDebugString("\r\n");

            DEVICE_IPC_INFO * pIPC = new DEVICE_IPC_INFO();
            //memset(pIPC, 0, sizeof(DEVICE_IPC_INFO));

            memset(ProbeMatches,0,sizeof(ProbeMatches));

            int nPrefixType = -1;

            p_str_begin = strstr(buf,"<tns:ProbeMatches");
            p_str_end   = strstr(buf,"</tns:ProbeMatches>");
            if(p_str_begin != NULL && p_str_end != NULL)
            {
                nPrefixType = 1;
            }
            else
            {
                p_str_begin = strstr(buf,"<d:ProbeMatches");
                p_str_end   = strstr(buf,"</d:ProbeMatches>");
                if(p_str_begin != NULL && p_str_end != NULL)
                {
                    nPrefixType = 2;
                }
            }
    
           if(nPrefixType == -1)
           {
               delete pIPC;
               continue;
           }

            memcpy(ProbeMatches, p_str_begin, p_str_end-p_str_begin);
            // printf("%s \n",ProbeMatches);
            memset(address,0,sizeof(address));

            int prefixLen = 0;
            if(nPrefixType == 1)
            {
               prefixLen = strlen("<tns:XAddrs>");
               p_str_begin = strstr(ProbeMatches, "<tns:XAddrs>");
               p_str_end   = strstr(ProbeMatches, "</tns:XAddrs>"); 
            }
            else if(nPrefixType == 2)
            {
               prefixLen = strlen("<d:XAddrs>");
               p_str_begin = strstr(ProbeMatches, "<d:XAddrs>");
               p_str_end   = strstr(ProbeMatches, "</d:XAddrs>"); 
            }

            if(p_str_begin != NULL && p_str_end != NULL) 
            {
                assert( p_str_end-p_str_begin-prefixLen < (int)sizeof(address));

                if(p_str_end-p_str_begin-prefixLen >= (int)sizeof(address))
                {
                    delete pIPC;
                    continue;
                }

                memcpy(address, p_str_begin+prefixLen, p_str_end-p_str_begin-prefixLen); //获取设备查询URL
                OutputDebugString("Addr: %s \n",address);

                unsigned short uPort = 0;
                char onvifAddrName[128] = {0};
                int result = HttpUrlGetIpPort(address, pIPC->IP, &uPort, onvifAddrName);
                assert(result == 0);

                pIPC->Port = uPort;
            }

            p_str_begin = strstr(buf,"onvif://www.onvif.org/hardware/");
           
            if(p_str_begin != NULL )
            {
                p_str_end   = strstr(p_str_begin, " ");

                if(p_str_end != NULL)
                {
                    int prefixLen = strlen("onvif://www.onvif.org/hardware/");
                    memcpy(hardwareID, p_str_begin+prefixLen, p_str_end-p_str_begin-prefixLen); //获取硬件型号

                    strcpy(pIPC->hardware_type, hardwareID);
                }
            }

            p_str_begin = strstr(buf, "onvif://www.onvif.org/serial/");
           
            if(p_str_begin != NULL )
            {
                p_str_end   = strstr(p_str_begin, " ");

                if(p_str_end != NULL)
                {
                    int prefixLen = strlen("onvif://www.onvif.org/serial/");
                    memcpy(serial, p_str_begin+prefixLen, p_str_end-p_str_begin-prefixLen); //获取序列号

                    //strcpy(pIPC->uuid, serial);
                    strcpy(pIPC->DeviceID, serial);
                }
            }

            //if(strlen(pIPC->DeviceID) == 0)
            //{
            //    strcpy(pIPC->DeviceID, CreateUuid()); //随机分配一个ID
            //}
            
           if(m_lpCallback)
              m_lpCallback(pIPC, m_lpContext); //调用回调函数传递数据给界面

            lstIpcs.push_back(pIPC);
        }
    }

    OutputDebugString("Onvif found ipc number: %d \n", (int)lstIpcs.size());
    //std::list<DEVICE_IPC_INFO*>::iterator it;
    //for(it = lstIpcs.begin(); it != lstIpcs.end(); it++)
    //{
    //     DEVICE_IPC_INFO * pDev = *it;
    //}
    lstIpcs.clear();

    //退出组播组
    struct ip_mreq mreq;
    memset( &mreq, 0, sizeof(struct ip_mreq) );
    mreq.imr_multiaddr.s_addr = inet_addr(GRUPO);
    mreq.imr_interface.s_addr = htonl(INADDR_ANY);
    if( setsockopt(fd, IPPROTO_IP, IP_DROP_MEMBERSHIP, (const char*)&mreq, sizeof(mreq)) < 0 ){
        OutputDebugString("IP_DROP_MEMBERSHIP failed \n");
        close(fd);
    }

    return 0;
}
