#include "CConfigManager.h"
#include <QDir>
#include "message.h"
#include "socket/TcpModule.h"
#include "base_module/factory.h"
#include "socket/CWebSocketClient.h"
#include <assert.h>
#include "libboot.h"
#include "device/CDeviceManager.h"

//RTSP_CAMERA_MAP CConfigManager::m_mapOnlineCam;
//QMutex          CConfigManager::m_mtxOnlineCam;

extern bool makePath(QString strPath);

const QString MAIN_DATABASE_DIR = "../dbms/";

// 最大500M文件大小sss
const int MAX_VIDEO_SAVE_SIZE = 500;
// 最大200分钟的保存记录
const int MAX_VIDEO_SAVE_INTERVAL = 200;

extern QString getMainMacAddress();

REGISTER_MODULE(CConfigManager, "CConfigManager")

CDatebase::CDatebase(QString name)
    : m_dbName(name),
      m_bLoaded(false),
      m_bLocked(false)
{
}

// 注册条目
bool CDatebase::regKey(QString key, QVariant def_val)
{
    auto it = m_mapDatebase.find(key);
    if(it == m_mapDatebase.end())
    {
        DATEBASE_ITEM item;
        item.def_value = def_val;
        item.value = def_val;
        item.name = key;
        m_mapDatebase[key] = item;
    }
    else
    {
        it.value().def_value = def_val;
    }

    return true;
}

bool CDatebase::getValidValue(QString key, QVariant &value)
{
    auto it = m_mapDatebase.find(key);
    if (it != m_mapDatebase.end())
    {
        value = it.value().value;
        return true;
    }
    else
    {
        return false;
    }
}

// 获取值
QVariant CDatebase::getValue(QString key)
{
    auto it = m_mapDatebase.find(key);
    if (it != m_mapDatebase.end())
    {
        return it.value().value;
    }
    else
    {
        return QVariant();
    }
}

// 设置值
bool CDatebase::setValue(QString key, QVariant val)
{
    auto it = m_mapDatebase.find(key);
    if (it != m_mapDatebase.end())
    {
        it.value().value = val;
    }
    else
    {
        regKey(key, val);
        //return false;
    }
    return true;
}

// 恢复出厂值
bool CDatebase::restoreFactory()
{
    // 先加锁
    m_bLocked = true;
    //打开或创建文件
    QFile file(MAIN_DATABASE_DIR + name() + ".db");     //相对路径、绝对路径、资源路径都可以
    if (!file.open(QFile::WriteOnly | QFile::Truncate)) //可以用QIODevice，Truncate表示清空原来的内容
        return false;

    QDomDocument doc;
    //写入xml头部
    QDomProcessingInstruction instruction; //添加处理命令
    instruction = doc.createProcessingInstruction("xml", "version=\"1.0\" encoding=\"UTF-8\"");
    doc.appendChild(instruction);
    //添加根节点
    QDomElement root = doc.createElement(name());
    doc.appendChild(root);
    QDomText text;

    foreach (auto item, m_mapDatebase)
    {
        QDomElement xml_item = doc.createElement(item.name);
        text = doc.createTextNode(item.def_value.toString());
        xml_item.appendChild(text);
        xml_item.setAttribute("default", item.def_value.toString());
        root.appendChild(xml_item);
    }

    //输出到文件
    QTextStream out_stream(&file);
    doc.save(out_stream, 4);
    file.close();

    return true;
}

// 数据库名称
QString CDatebase::name()
{
    return m_dbName;
}

// 保存
bool CDatebase::save()
{
    // 在有锁的情况下直接返回
    if (m_bLocked)
    {
        return false;
    }
    //打开或创建文件
    QFile file(MAIN_DATABASE_DIR + name() + ".db");     //相对路径、绝对路径、资源路径都可以
    if (!file.open(QFile::WriteOnly | QFile::Truncate)) //可以用QIODevice，Truncate表示清空原来的内容
        return false;

    QDomDocument doc;
    //写入xml头部
    QDomProcessingInstruction instruction; //添加处理命令
    instruction = doc.createProcessingInstruction("xml", "version=\"1.0\" encoding=\"UTF-8\"");
    doc.appendChild(instruction);
    //添加根节点
    QDomElement root = doc.createElement(name());
    doc.appendChild(root);
    QDomText text;

    foreach (auto item, m_mapDatebase)
    {
        QDomElement xml_item = doc.createElement(item.name);
        text = doc.createTextNode(item.value.toString());
        xml_item.appendChild(text);
        xml_item.setAttribute("default", item.def_value.toString());
        root.appendChild(xml_item);
    }

    //输出到文件
    QTextStream out_stream(&file);
    doc.save(out_stream, 4);
    file.close();
    m_bLoaded = true;
    return true;
}

// 加载数据
bool CDatebase::load()
{
    //打开或创建文件
    QFile file(MAIN_DATABASE_DIR + name() + ".db"); //相对路径、绝对路径、资源路径都可以
    if (!file.open(QFile::ReadOnly))
        return false;

    QDomDocument doc;
    if (!doc.setContent(&file))
    {
        file.close();
        return false;
    }
    file.close();

    QDomElement root = doc.documentElement(); //返回根节点
    qDebug() << root.nodeName();
    QDomNodeList list = root.childNodes();
    for (int i = 0; i < list.count(); i++) //遍历子元素，count和size都可以用,可用于标签数计数
    {
        QDomNode n = list.at(i);
        if (n.isElement())
        {
            qDebug() << n.nodeName() << ":" << n.toElement().text();

            DATEBASE_ITEM item;
            item.name = n.nodeName();
            item.value = n.toElement().text();
            item.def_value = n.toElement().attributeNode("default").value();
            m_mapDatebase[item.name] = item;
        }
    }

    m_bLoaded = true;
    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////

CConfigManager::CConfigManager(QObject *parent)
    : QObject(parent),
      m_pTcpModule(nullptr),
      m_pWebSock(nullptr),
      m_pDbList(nullptr)
{
    makePath(MAIN_DATABASE_DIR);
    //    makePath(SUB_DATABASE_DIR);

    // 载入数据库列表
    loadDbmsList();
    // 恢复数据库到列表
    restoreDB2List();
}

CConfigManager::~CConfigManager()
{
    std::cout << name() << std::endl;
}

CConfigManager::~CConfigManager();

//初始化操作
bool CConfigManager::Initialize()
{
    STARTUP_TASK(TaskMain);
    return true;
}

//模块注册接口
bool CConfigManager::RegistMessage()
{
    m_pTcpModule = dynamic_cast<ITcpModule *>(QueryPlugin("TcpModule"));
    if(m_pTcpModule)
    {
        m_pTcpModule->RegistMessageCallback(this, MODULE_CONFIG_MNG);
    }
    return true;
}

//主消息任务，主要用于和其他模块之间的消息通讯
void CConfigManager::TaskMain(void *pBuff)
{
    (void)pBuff;
    setCurrentTaskName("tskCfgMain");
    MODULE_MESSAGE *pMsg = nullptr;
    RegistMessage();
    waitForStart();

    m_pWebSock = dynamic_cast<CWebSocketClient *>(QueryPlugin("CWebSocketClient"));

    while (!isShutdown())
    {
        if (GetModuleMessage(pMsg)) //get msg from message queue
        {
            switch (pMsg->iCmd)
            {
            case CMD_CONFIG_RESTORE_FACTORY:
                ProcRestoreFactory(pMsg);
                break;
            default:
                break;
            }

            SAFE_DELETE(pMsg);
        }
    }
}

// 处理恢复到出厂值
void CConfigManager::ProcRestoreFactory(MODULE_MESSAGE *pMsg)
{
    bool bSuccess = true;
    int type = 0;
    memcpy(&type, pMsg->szBuff, sizeof(int));

    // 恢复出厂值
    bSuccess = restoreFactory(type);
    // 回消息处理完毕
    onRestorefactorysettings(type, bSuccess);
}

// 返回初始化结果
bool CConfigManager::onRestorefactorysettings(int type, int result)
{
    assert(m_pWebSock);
    bool bNeedReset = false;
    QVariantMap prop_arr;
    prop_arr.insert("msgName", "onRestorefactorysettings");
    // type	number	是否清除数据。1表示清除数据并恢复出厂设置； 2表示不清除数据，只恢复出厂设置； 3表示只清除数据，不恢复出厂设置
    prop_arr.insert("type", type);
    // result	String	表示type代表的操作的执行结果，1表示成功； 0表示失败
    prop_arr.insert("result", result);
    // info
    // 只有全清和清除配置的情况下，才需要清库
    if (type == RESTORE_FACTORY_ALL || type == RESTORE_FACTORY_DATABASE)
    {
        prop_arr.insert("info", "系统已经清库,将在2秒后重启...");
        bNeedReset = true;
    }
    else
    {
        prop_arr.insert("info", "数据清理已经完成.");
    }

    if (!m_pWebSock->sendMessage(prop_arr))
    {
        return false;
    }

    if (bNeedReset)
    {
        mSleep(2000);
        CDeviceManager *pDevice = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
        if (pDevice)
        {
            pDevice->closeApp();
        }
    }

    return true;
}

// 注册数据库
CDatebase *CConfigManager::regDatabase(MODULE_ID id, QString name)
{
    // 先通过id和名称查找数据库
    CDatebase *pDatebase = getDatabase(id, name);
    // 不存在的时候，创建添加
    if (!pDatebase)
    {
        {
            QMutexLocker locker(&m_mtxDBMS);
            pDatebase = new CDatebase(name);
            m_mapDBMS.insert(id, pDatebase);
            m_pDbList->regKey(name, id);
        }
        // 这里要保存下列表
        saveDbmsList();
    }
    // 否则导入数据库
    else
    {
        pDatebase->load();
    }
    return pDatebase;
}

// 注册清库回调函数
bool CConfigManager::regRestoreCallback(RESTORE_CALLBACK call_back, void *pThis)
{
    QMutexLocker locker(&m_mtxCallback);
    RESTORE_CALLBACK_INFO info;
    info.pFunc = call_back;
    info.pThis = pThis;
    m_mapRestoreCallback.append(info);
}

// 数据重置恢复出厂值
bool CConfigManager::restoreFactory(int type)
{
    // 只有全清和清除配置的情况下，才需要清库
    if (type == RESTORE_FACTORY_ALL || type == RESTORE_FACTORY_DATABASE)
    {
        QMutexLocker locker(&m_mtxDBMS);
        foreach (auto db, m_mapDBMS)
        {
            db->restoreFactory();
        }
    }

    // 文件或其他的恢复出厂操作，交给具有应用模块回调处理
    {
        QMutexLocker locker(&m_mtxCallback);
        foreach (auto cb, m_mapRestoreCallback)
        {
            RESTORE_CALLBACK func = cb.pFunc;
            (*func)(cb.pThis, type);
        }
    }
    return true;
}

// 保存配置
bool CConfigManager::saveDbmsList()
{
    return m_pDbList->save();
}

// 读取配置
bool CConfigManager::loadDbmsList()
{
    m_pDbList = new CDatebase("db_list");
    m_mtxDBMS.lock();
    m_mapDBMS.insert(MODULE_CONFIG_MNG, m_pDbList);
    m_pDbList->regKey("db_list", MODULE_CONFIG_MNG);
    m_mtxDBMS.unlock();

    m_pDbList->load();
   return true; 
}

// 这里相当于预先根据列表文件创建空的数据库，数据库使用时，通过应用程序载入数据
bool CConfigManager::restoreDB2List()
{
    QMutexLocker locker(&m_mtxDBMS);
    foreach (auto item, m_pDbList->m_mapDatebase)
    {
        CDatebase *pDatabase = new CDatebase(item.name);
        m_mapDBMS.insert(item.value.toInt(), pDatabase);
    }
    return true;
}

// 获取数据库实例
CDatebase *CConfigManager::getDatabase(MODULE_ID id, QString name)
{
    m_mtxDBMS.lock();
    QList<CDatebase *> list = m_mapDBMS.values(id);
    m_mtxDBMS.unlock();

    foreach (auto db, list)
    {
        if (db->name() == name)
        {
            if (!db->m_bLoaded)
            {
                db->load();
            }
            return db;
        }
    }

    return nullptr;
}
