#ifndef CCONFIG_MNG_H
#define CCONFIG_MNG_H
#include "common.h"
#include "message.h"
#include "base_module.h"
#include <QObject>
#include <QHostInfo>
#include <QMutex>
#include <QtXml>

class CWebSocketClient;
class ITcpModule;

// 数据库条目，
struct DATEBASE_ITEM
{
    // 名称键值
    QString name;
    // 默认值
    QVariant def_value;
    // 当前值
    QVariant value;
};

typedef QMap<QString, DATEBASE_ITEM> DATA_BASE_MAP;

// 单个数据库
class CDatebase
{
    friend class CConfigManager;

public:
    CDatebase(QString name);
    // 注册条目
    bool regKey(QString key, QVariant def_val);
    // 获取值
    QVariant getValue(QString key);

    // 获取有效值
    bool getValidValue(QString key, QVariant &value);

    // 设置值
    bool setValue(QString key, QVariant val);
    // 恢复出厂值
    bool restoreFactory();
    // 数据库名称
    QString name();
    // 存库
    bool save();
    // 载入
    bool load();

protected:
    // 数据map表
    DATA_BASE_MAP m_mapDatebase;
    // 数据库名称
    QString m_dbName;
    // 是否已经加载
    bool m_bLoaded;
    // 是否已经锁定
    bool m_bLocked;
};

// 模块为索引的数据库map表
typedef QMultiMap<int, CDatebase *> DATA_MNG_MAP;
// 恢复出厂值的回调函数定义
typedef bool (*RESTORE_CALLBACK)(void *, int);

// 恢复出厂值的回调定义
struct RESTORE_CALLBACK_INFO
{
    /* data */
    void *pThis;
    RESTORE_CALLBACK pFunc;
};

// 回调函数列表
typedef QVector<RESTORE_CALLBACK_INFO> RESTORE_CALLBACK_VCT;

// 配置管理类
class CConfigManager : public QObject, public IModule
{
    Q_OBJECT
public:
    MODULE_NAME(CConfigManager)
    explicit CConfigManager(QObject *parent = nullptr);
    ~CConfigManager();

    // 注册数据库
    CDatebase *regDatabase(MODULE_ID id, QString name);
    // 注册清库回调
    bool regRestoreCallback(RESTORE_CALLBACK call_back, void *pThis);
    // 获取数据库实例
    CDatebase *getDatabase(MODULE_ID id, QString name);

protected:
    // 处理恢复出厂值消息
    void ProcRestoreFactory(MODULE_MESSAGE *pMsg);
    // 数据重置恢复出厂
    bool restoreFactory(int type);
    // 保存数据库管理列表
    bool saveDbmsList();
    // 读取数据库列表
    bool loadDbmsList();
    // 恢复数据库到列表
    bool restoreDB2List();

    //初始化操作
    virtual bool Initialize();
    //模块注册接口
    bool RegistMessage();

    //主消息任务，主要用于和其他模块之间的消息通讯
    THREAD_DEFINITION(CConfigManager, TaskMain)

    //主消息任务，主要用于和其他模块之间的消息通讯
    virtual void TaskMain(void *pBuff);

    // 返回初始化结果
    bool onRestorefactorysettings(int type, int result);

private:
    // 依赖模块
    ITcpModule *m_pTcpModule;
    CWebSocketClient *m_pWebSock;
    // 数据库管理map
    DATA_MNG_MAP m_mapDBMS;
    // 数据库相关操作信号量
    QMutex m_mtxDBMS;

    // 恢复出厂回调
    RESTORE_CALLBACK_VCT m_mapRestoreCallback;
    // 回调保护信号量
    QMutex m_mtxCallback;

    // 数据库列表
    CDatebase *m_pDbList;
};

#endif // CCONFIG_MNG_H
