#include "CDeviceManager.h"
#include "../socket/TcpServer.h"
#include "../base_module/factory.h"
#include "socket/CWebSocketClient.h"
#include "timer/CTimerManager.h"
#include "swdl/CSWDLManager.h"
#include "gps/CGPSManager.h"
#include "file/CFileManager.h"
#include <QTimer>
#include <assert.h>

// 获取主mac地址用作设备ID
extern QString getMainMacAddress();

REGISTER_MODULE(CDeviceManager, "CDeviceManager")

const QString DEVICE_DB_NAME = "dev_db";
LOG_DEFINITION(CDeviceManager)
const QString BACKUP_DIR_NAME = "backup/";
//
CDeviceManager::CDeviceManager(QObject *parent)
    : QObject(parent),
      IModule(),
      m_pTcpModule(nullptr),
      m_pConfigMng(nullptr),
      m_pWebSock(nullptr),
      m_pTimerMng(nullptr),
      m_pSwdlMng(nullptr),
      m_pSensorMng(nullptr),
      m_pFileMng(nullptr),
      m_pDatebaseDev(nullptr),
      m_bGetCfgFromServ(false),
      m_bLogined(false),
      m_iVehicleColor(1),
      m_strManufactor("HNCY_AI"),
      m_strTerminalType("HNCY_AI_00001"),
      m_strTerminalID("HNCY_AI_0000100001")
{
    LOG_CREATE(CDeviceManager, "device");
	m_pFileMng = dynamic_cast<CFileManager *>(QueryPlugin("CFileManager"));
	if (m_pFileMng)
    {
        makePath(m_pFileMng->getDataPath() + BACKUP_DIR_NAME);
    }
    m_pConfigMng = dynamic_cast<CConfigManager *>(QueryPlugin("CConfigManager"));
    if (m_pConfigMng)
    {
        m_pDatebaseDev = m_pConfigMng->getDatabase(MODULE_DEVICE_MNG, DEVICE_DB_NAME);
        if (!m_pDatebaseDev)
        {
            createDatebase();
        }
        else
        {
            restoreFromDB(m_pDatebaseDev);
        }
    }
}

//
CDeviceManager::~CDeviceManager()
{
    std::cout << name() << std::endl;
}

void CDeviceManager::setNetworkStatus(bool bAlarm)
{
    emit notifyNetworkAlarm(bAlarm);
}

// 创建数据库
bool CDeviceManager::createDatebase()
{
    m_pDatebaseDev = m_pConfigMng->regDatabase(MODULE_DEVICE_MNG, DEVICE_DB_NAME);
    regDBKey();
    m_pDatebaseDev->save();
    return true;
}

// 注册数据库默认键值
bool CDeviceManager::regDBKey()
{
    m_pDatebaseDev->regKey("commonStatusFrq", 300);
    m_pDatebaseDev->regKey("pingFreq", 60);
    m_pDatebaseDev->regKey("uploadSpeedFreq", 60);
    m_pDatebaseDev->regKey("parkingAlarmEnable", true);
    m_pDatebaseDev->regKey("parkingAlarmInterval", 600);
    m_pDatebaseDev->regKey("videoFrameSplittingInterval", 1000); // ms

    m_pDatebaseDev->regKey("vehicle_no", "川A66666");
    m_pDatebaseDev->regKey("phone_num", "18888888888");

    m_pDatebaseDev->regKey("call_serv_ip", "127.0.0.1");
    m_pDatebaseDev->regKey("call_serv_port", "9800");
    m_pDatebaseDev->regKey("call_user_name", "admin");
    m_pDatebaseDev->regKey("call_password", "123456");
    m_pDatebaseDev->regKey("call_channel", "1");

    m_pDatebaseDev->regKey("ws_servaddr", "ws://119.3.148.141:58006/integratedDevice/websocket/");
    m_pDatebaseDev->regKey("ws_username", "wisdomTest");
    m_pDatebaseDev->regKey("ws_password", "123456");

    m_pDatebaseDev->regKey("cam_view_cnt", 4);

    m_pDatebaseDev->regKey("city_id", 1);
    m_pDatebaseDev->regKey("province_id", 1);
	m_pDatebaseDev->regKey("device_id", "12345678");
	LOG_TRACE(CDeviceManager, "new");
    return true;
}

// 从数据库恢复数据
bool CDeviceManager::restoreFromDB(CDatebase *pDatebase)
{
    //regDBKey();
    pDatebase->load();
    return true;
}

// 保存配置
bool CDeviceManager::saveConfig()
{
    LOG_TRACE(CDeviceManager, "save config!");
    if (m_pDatebaseDev)
    {
        m_pDatebaseDev->save();
    }
    return true;
}

//初始化操作
bool CDeviceManager::Initialize()
{
    //起主消息处理线程
    STARTUP_TASK(TaskMain)

    return true;
}

// 关闭APP
void CDeviceManager::closeApp()
{
    emit notifyCloseApp();
}

// 给通用通信模块注册模块字
bool CDeviceManager::RegistMessage()
{
    m_pTcpModule = dynamic_cast<ITcpModule *>(QueryPlugin("TcpModule"));
    if (m_pTcpModule)
    {
        m_pTcpModule->RegistMessageCallback(this, MODULE_DEVICE_MNG);
    }
    return true;
}

//主消息任务，主要用于和其他模块之间的消息通讯
void CDeviceManager::TaskMain(void *pBuff)
{
    (void)pBuff;
    setCurrentTaskName("tskDevMain");
    MODULE_MESSAGE *pMsg = nullptr;
    RegistMessage();
    waitForStart();
    if (m_pDatebaseDev)
    {
        restoreFromDB(m_pDatebaseDev);
    }

    m_pWebSock = dynamic_cast<CWebSocketClient *>(QueryPlugin("CWebSocketClient"));
    m_pSensorMng = dynamic_cast<CGPSManager *>(QueryPlugin("CGPSManager"));

    //起主消息处理线程
    STARTUP_TASK(TaskSensorCol)
    connect2Slots();

    while (!isShutdown())
    {
        if (GetModuleMessage(pMsg)) // get msg from message queue
        {
            switch (pMsg->iCmd)
            {
            case TIMER_WS_STATUS_REPORT:
                sendCommonStatus();
                break;
            case TIMER_WS_GPS_REPORT:
                sendGpsSpeed();
                break;
            case TIMER_WS_HEART_BIT:
                ping();
                break;
            case CMD_QUERY_DEVICE_CALL_CFG:
                procQueryCallConfig(*pMsg);
                break;
            default:
                break;
            }

            SAFE_DELETE(pMsg);
        }
    }
}

// 查询呼叫配置
void CDeviceManager::procQueryCallConfig(MODULE_MESSAGE &msg)
{
    MODULE_MESSAGE send_msg;
    int idx = 0;
    memcpy(&send_msg.dst_type, msg.szBuff + idx, sizeof(int));
    idx += sizeof(int);
    memcpy(&send_msg.dst_id, msg.szBuff + idx, sizeof(int));
    idx += sizeof(int);
    memcpy(&send_msg.iCmd, msg.szBuff + idx, sizeof(int));
    idx += sizeof(int);

    idx = 0;
    QString serv_ip = getCallServIp().toString();
    memcpy(send_msg.szBuff + idx, (char *)serv_ip.toStdString().c_str(), min(16, (int)serv_ip.toStdString().length()));
    idx += 16;

    int port = getCallServPort().toInt();
    memcpy(send_msg.szBuff + idx, &port, sizeof(int));
    idx += sizeof(int);

    QString user_name = getCallUsername().toString();
    memcpy(send_msg.szBuff + idx, (char *)user_name.toStdString().c_str(), min(32, (int)user_name.toStdString().length()));
    idx += 32;

    QString password = getCallPassword().toString();
    memcpy(send_msg.szBuff + idx, (char *)password.toStdString().c_str(), min(16, (int)password.toStdString().length()));
    idx += 16;

    int channel = getCallChannel().toInt();
    memcpy(send_msg.szBuff + idx, &channel, sizeof(int));
    idx += sizeof(int);

    assert(m_pTcpModule);
    m_pTcpModule->SendMsg(send_msg, sizeof(send_msg));
}

// 获取设备省份代码
WORD CDeviceManager::getProvinceID()
{
    return m_pDatebaseDev->getValue("province_id").toInt();
}

// 设置设备省份代码
void CDeviceManager::setProvinceID(WORD id)
{
    m_pDatebaseDev->setValue("province_id", id);
}

// 获取设备市区代码
WORD CDeviceManager::getCityID()
{
    return m_pDatebaseDev->getValue("city_id").toInt();
}

// 设置设备市区代码
void CDeviceManager::setCityID(WORD city)
{
    m_pDatebaseDev->setValue("city_id", city);
}

// 获取设备生产商信息
QString CDeviceManager::getManufactor()
{
    return m_strManufactor;
}

// 获取设备类型
QString CDeviceManager::getDeviceType()
{
    return m_strTerminalType;
}

// 获取终端类型(适用车辆)
WORD CDeviceManager::getTerminalType()
{
    return 0xBF;
}

// 获取设备终端ID
QString CDeviceManager::getTerminalID()
{
    return m_strTerminalID;
}

// 获取车辆颜色类型
int CDeviceManager::getVehicleColor()
{
    return m_pDatebaseDev->getValue("vehicle_color").toInt();
}

// 获取车辆颜色类型
void CDeviceManager::setVehicleColor(int color)
{
    m_pDatebaseDev->setValue("vehicle_color", color);
}

// 信号槽简易信号处理
void CDeviceManager::connect2Slots()
{
    m_pConfigMng = dynamic_cast<CConfigManager *>(QueryPlugin("CConfigManager"));
    m_pSwdlMng = dynamic_cast<CSWDLManager *>(QueryPlugin("CSWDLManager"));
    m_pFileMng = dynamic_cast<CFileManager *>(QueryPlugin("CFileManager"));

    assert(m_pConfigMng);
    assert(m_pSwdlMng);

    // 定时器模块
    m_pTimerMng = dynamic_cast<CTimerManager *>(::QueryPlugin("CTimerManager"));
    if (m_pTimerMng)
    {
        m_pTimerMng->RegTimer(MODULE_DEVICE_MNG, TIMER_WS_STATUS_REPORT, 60);
        m_pTimerMng->RegTimer(MODULE_DEVICE_MNG, TIMER_WS_GPS_REPORT, 60);
        m_pTimerMng->RegTimer(MODULE_DEVICE_MNG, TIMER_WS_HEART_BIT, 60);
    }
}

//传感器收集任务, 用于定时轮询传感器相关的信息;
void CDeviceManager::TaskSensorCol(void *pBuff)
{
    (void)pBuff;
    while (this->isShutdown() == false)
    {
        mSleep(200);
    }
}

// 获取设备ID
QString CDeviceManager::getDeviceID()
{
	QString strDevCode = m_pDatebaseDev->getValue("device_id").toString();
	LOG_INFO(CDeviceManager, "%s%s", "id Boardcast=>", strDevCode.toStdString().c_str());
	if (strDevCode == "12345678")
	{
		strDevCode = getMainMacAddress();
		if (!strDevCode.isNull())
		{
			m_pDatebaseDev->setValue("device_id", strDevCode);
			m_pDatebaseDev->save();
			
		}
	}
	return strDevCode;
}

// 获取系统版本，这里要调用底层函数，这里先打桩
QString CDeviceManager::getSystemVersion()
{
    QString ver = "未知版本";
    if (m_pSwdlMng)
    {
        ver = m_pSwdlMng->getSystemVersion();
    }
    return ver;
}

// 获取硬件版本信息
QString CDeviceManager::getHardVersion()
{
    return "HARD_20210605_T01";
}

// 获取IMEI串号
QString CDeviceManager::getIMEI()
{
    return "IMEI_00001_T01";
}

// 终端SIM卡ICCID
QString CDeviceManager::getSimICCID()
{
    return "ICCID_00001_T01";
}

// GNSS模块属性(BYTE)
QString CDeviceManager::getFirmwareVersion()
{
    return "FIRMWARE_20210605_T01";
}

// GNSS模块属性(BYTE)
BYTE CDeviceManager::getGNSSProperty()
{
    // bit0 是否支持定位：0不支持；1支持
    // bit1 是否支持北斗定位：0不支持；1支持
    // bit2 是否支持GLONASS定位：0不支持；1支持
    // bit3 是否支持Galileo定位：0不支持；1支持
    return 0x07;
}

// 通讯模块属性(BYTE)
BYTE CDeviceManager::getComProperty()
{
    // bit0 是否支持GPRS：0不支持；1支持
    // bit1 是否支持CDMA：0不支持；1支持
    // bit2 是否支持TD-SCDMA：0不支持；1支持
    // bit3 是否支持WCDMA：0不支持；1支持
    // bit4 是否支持CDMA2000：0不支持；1支持
    // bit5 是否支持TD-LTE：0不支持；1支持
    // bit7 是否支持其他通讯方式：0不支持；1支持
    return 0x3F;
}

// 恢复出场值
void CDeviceManager::restoreFactory()
{
    // T01
    LOG_WARN(CDeviceManager, "=================数据库清除操作=================");
    m_pDatebaseDev->restoreFactory();
}

// 获取软件版本
QString CDeviceManager::getSoftwareVersion()
{
    QString ver = "未知版本";
    if (m_pSwdlMng)
    {
        ver = m_pSwdlMng->getSoftwareVersion();
    }
    return ver;
}

// 制作信息
QString CDeviceManager::getManualFac()
{
    QString manual = "未知制造信息";
    if (m_pSwdlMng)
    {
        manual = m_pSwdlMng->getManualFac();
    }
    return manual;
}

// 新软件版本
QString CDeviceManager::getNewSoftVer()
{
    QString ver = "未知版本";
    if (m_pSwdlMng)
    {
        ver = m_pSwdlMng->getNewSoftVer();
    }
    return ver;
}

// 升级路径
QString CDeviceManager::getUpdatePath()
{
    QString path = "";
    if (m_pSwdlMng)
    {
        path = m_pSwdlMng->getUpdatePath();
    }
    return path;
}

// 获取硬盘类型 1表示SD卡，2表示硬盘，3表示SSD卡 先打桩
int CDeviceManager::getDisktype()
{
    if (m_pFileMng)
    {
        return m_pFileMng->getDisktype();
    }
    return 0;
}

// 获取磁盘空间状态。剩余空间，单位MB 先打桩
float CDeviceManager::getFreeSpaceMB()
{
    if (m_pFileMng)
    {
        return m_pFileMng->getFreeSpaceMB();
    }
    return 0;
}

// 获取磁盘空间。
float CDeviceManager::getDiskSpaceMB()
{
    if (m_pFileMng)
    {
        return m_pFileMng->getDiskSpaceMB();
    }
    return 0;
}

// 获取磁盘空间状态。剩余空间，单位% 先打桩
float CDeviceManager::getFreeSpacePrecent()
{
    if (m_pFileMng)
    {
        return m_pFileMng->getDiskSpaceMB();
    }
    return 0;
}

// 获取运行状态。0为正常,1为维修,2为停用；先打桩
int CDeviceManager::getStatus()
{
    return 0;
}

// 获取里程；单位：米。
int CDeviceManager::getMileage()
{
    if (m_pSensorMng)
    {
        m_pSensorMng->getMileageInfo();
    }
    return 0;
}

// 获取网络类型 。0为3G,1为4G,2为5G，3为WIFI，4为有线；
int CDeviceManager::getNetwork()
{
    return 1;
}

// 获取网络流量消耗；单位为MB
int CDeviceManager::getNetworkTraffic()
{
    return 1000;
}

//  获取网络状态 连通/断开/重连
int CDeviceManager::getNetworkStatus()
{
    return NETWORK_STATUS_CONNECTED;
}

// 以下为设备相关的接口
bool CDeviceManager::login(QString usr_name, QString password)
{
    assert(m_pWebSock);
    QVariantMap prop_arr;
    prop_arr.insert("msgName", "login");
    prop_arr.insert("account", usr_name);
    prop_arr.insert("password", password);
    prop_arr.insert("deviceId", getDeviceID());

    if (!m_pWebSock->sendMessage(prop_arr))
    {
        return false;
    }
    return true;
}

// 从服务器获取配置
bool CDeviceManager::getParams()
{
    assert(m_pWebSock);

    QVariantMap prop_arr;
    prop_arr.insert("msgName", "getParams");
    if (!m_pWebSock->sendMessage(prop_arr))
    {
        return false;
    }
    return true;
}

// 心跳命令
bool CDeviceManager::ping()
{
    assert(m_pWebSock);

    QVariantMap prop_arr;
    prop_arr.insert("msgName", "ping");
    if (!m_pWebSock->sendMessage(prop_arr))
    {
        return false;
    }

    return true;
}

// 发送状态信息
bool CDeviceManager::sendCommonStatus()
{
    assert(m_pWebSock);
    QVariantMap prop_arr;
    prop_arr.insert("msgName", "sendCommonStatus");
    //    deviceId	String	32字符串
    prop_arr.insert("deviceId", getDeviceID());
    //    SystemVersion	String	系统版本信息。
    prop_arr.insert("SystemVersion", getSystemVersion());
    //    SoftwareVersion	String	软件版本信息。
    prop_arr.insert("SoftwareVersion", getSoftwareVersion());
    //    dt	number	硬盘类型 1表示SD卡，2表示硬盘，3表示SSD卡
    prop_arr.insert("dt", getDisktype());
    //    spaceLeft	number	磁盘空间状态。剩余空间，单位MB
    prop_arr.insert("spaceLeft", getFreeSpaceMB());
    //    spaceLeftPercentage	number	磁盘空间状态。剩余空间，单位%
    prop_arr.insert("spaceLeftPercentage", getFreeSpacePrecent());
    //    status	String	运行状态。0为正常,1为维修,2为停用；
    prop_arr.insert("status", getStatus());
    //    mileage	number	里程；单位：米。
    prop_arr.insert("mileage", getMileage());
    //    network	number	网络类型 。0为3G,1为4G,2为5G，3为WIFI，4为有线；
    prop_arr.insert("network", getNetwork());
    //    networkTraffic	number	网络流量消耗；单位为MB
    prop_arr.insert("networkTraffic", getNetworkTraffic());

    if (!m_pWebSock->sendMessage(prop_arr))
    {
        return false;
    }
    return true;
}

// 返回系统更新结果
bool CDeviceManager::upgradeSystemResult(QString sys_ver, int sys_ret, QString soft_ver, int soft_ret)
{
    assert(m_pWebSock);
    QVariantMap prop_arr;

    prop_arr.insert("msgName", "upgradeSystemResult");
    // SystemVersion	String	当前要更新的系统版本
    prop_arr.insert("SystemVersion", sys_ver);

    // SystemUpgradeResult	String	返回信息；1表示成功；0表示失败
    prop_arr.insert("SystemUpgradeResult", sys_ret);

    // SoftwareVersion	String	当前要更新的软件版本
    prop_arr.insert("SoftwareVersion", soft_ver);

    // SoftwareUpgradeResult	String	返回信息；1表示成功；0表示失败
    prop_arr.insert("SoftwareUpgradeResult", soft_ret);

    if (!m_pWebSock->sendMessage(prop_arr))
    {
        return false;
    }
    return true;
}

// 上报GPS 速度信息
bool CDeviceManager::sendGpsSpeed()
{
    assert(m_pWebSock);

    if (m_pSensorMng)
    {
        QVariantMap prop_arr;
        GPS_INFO gps_info;
        if (!m_pSensorMng->getCurGPSInfo(gps_info))
        {
            return false;
        }

        prop_arr.insert("msgName", "sendGpsSpeed");
        // lng	number	经度；格式是WGS84大地坐标；如果设备定位无效，值为0。
        prop_arr.insert("lng", QString("%1").arg(QString::number(gps_info.longitude, '.', 8)));

        // lat	number	纬度；格式是WGS84大地坐标；如果设备定位无效，值为0。
        prop_arr.insert("lat", QString("%1").arg(QString::number(gps_info.latitude, '.', 8)));

        // speed	number	速度。 km/h
        prop_arr.insert("speed", QString("%1").arg(gps_info.speed));
		prop_arr.insert("angle", QString("%1").arg(gps_info.dir));

        // gt	String	GPS上传时间
        QDateTime dt = QDateTime::currentDateTime();
        if (gps_info.time_tick != 0)
        {
            dt.fromTime_t(gps_info.time_tick);
        }

        prop_arr.insert("gt", dt.toString("yyyy-MM-dd hh:mm:ss"));

        // parkTime	number	停车时长;单位: 秒。
        prop_arr.insert("parkTime", QString("%1").arg(m_pSensorMng->getTotalStopTime()));

        // mileage	number	里程；单位：米。
        prop_arr.insert("mileage", m_pSensorMng->getMileageInfo());

        if (!m_pWebSock->sendMessage(prop_arr))
        {
            return false;
        }
    }
    return true;
}

// 上报停车报警
bool CDeviceManager::sendParkingAlarm()
{
    assert(m_pWebSock);
    QVariantMap prop_arr;

    prop_arr.insert("msgName", "sendParkingAlarm");
    // beginTime	String	开始时间；开始时间不得大于结束时间。
    prop_arr.insert("beginTime", "1");

    // endTime	String	结束时间。
    prop_arr.insert("endTime", "1");

    // parkTime	String	停车时长；单位: 秒
    prop_arr.insert("parkTime", "1");

    // startPositionLng	String	开始停车的经度
    prop_arr.insert("startPositionLng", "1");

    // startPositionLat	String	开始停车的纬度
    prop_arr.insert("startPositionLat", "1");

    if (!m_pWebSock->sendMessage(prop_arr))
    {
        return false;
    }
    return true;
}

// 登陆响应
bool CDeviceManager::onLogin(QVariantMap &prop_arr)
{
    // message	string	返回信息；1表示成功；0表示失败
    QString str_ret = prop_arr["message"].toString();
    int iRet = str_ret.toInt();
    m_bLogined = (bool)iRet;
    emit notifyLogin(iRet);
    return true;
}

// 获取参数响应
// 输入参数使用json方式；在onMessage中接收；平台端可能通过此接口主动修改配置参数，一体机需要针对更新参数进行相应的流程操作；当某参数为空时，表示不涉及该参数的改变
bool CDeviceManager::onGetParams(QVariantMap &prop_arr)
{
    if (m_pDatebaseDev)
    {
        // pingFreq	string	一体机发送心跳包的频率，单位是秒；默认频率是60秒
        if (prop_arr.find("pingFreq") != prop_arr.end())
        {
            int pingFreq = (int)prop_arr["pingFreq"].toFloat();
            setHeartBitFrequency(pingFreq);
        }

        // commonStatusFrq	string	一体机发送通用参数的频率，单位是秒；默认是300秒
        if (prop_arr.find("commonStatusFrq") != prop_arr.end())
        {
            int commonStatusFrq = (int)prop_arr["commonStatusFrq"].toFloat();
            setReportFrequency(commonStatusFrq);
        }

        // uploadSpeedFreq	string	一体机上报gps,车速的频率；单位是秒，默认是60秒
        if (prop_arr.find("uploadSpeedFreq") != prop_arr.end())
        {
            int uploadSpeedFreq = (int)prop_arr["uploadSpeedFreq"].toFloat();
            setUploadSpeedFreq(uploadSpeedFreq);
        }

        // parkingAlarmEnable	string	是否开启停车报警；yes是开启
        if (prop_arr.find("parkingAlarmEnable") != prop_arr.end())
        {
            QString parkingAlarmEnable = prop_arr["parkingAlarmEnable"].toString();
            if (parkingAlarmEnable == "yes")
            {
                setParkingAlarmEnable(true);
            }
            else
            {
                setParkingAlarmEnable(false);
            }
        }

        // parkingAlarmInterval	string	停车报警的时长，单位是秒
        if (prop_arr.find("parkingAlarmInterval") != prop_arr.end())
        {
            setParkingAlarmInterval((int)prop_arr["parkingAlarmInterval"].toFloat());
        }

        // videoFrameSplittingInterval	string	视频拆帧的时间间隔，单位是毫秒，表示每多少毫秒拆帧一次
        if (prop_arr.find("videoFrameSplittingInterval") != prop_arr.end())
        {
            int videoFrameSplittingInterval = (int)prop_arr["videoFrameSplittingInterval"].toFloat();
            setFrameSplittingInterval(videoFrameSplittingInterval);
        }
    }
    // 重置属性 发送数据
    QVariantMap prop_send;
    prop_send.insert("msgName", "getParamsResult");
    // 直接返回成功
    prop_send.insert("result", 1);

    if (!m_pWebSock->sendMessage(prop_send))
    {
        return false;
    }
    return true;
}

// 更新系统
bool CDeviceManager::upgradeSystem(QVariantMap &prop_arr)
{
    // SystemVersion	String	系统版本信息。为空表示不需要更新
    QString SystemVersion = prop_arr["SystemVersion"].toString();
    // SystemVersionUrl	String	系统版本下载地址。为空表示不需要更新
    QString SystemVersionUrl = prop_arr["SystemVersionUrl"].toString();
    // SoftwareVersion	String	软件版本信息。为空表示不需要更新
    QString SoftwareVersion = prop_arr["SoftwareVersion"].toString();
    // SoftwareVersionUrl	String	软件版本下载地址。为空表示不需要更新
    QString SoftwareVersionUrl = prop_arr["SoftwareVersionUrl"].toString();

    if (m_pSwdlMng)
    {
        m_pSwdlMng->updateSystem(SystemVersion, SystemVersionUrl, SoftwareVersion, SoftwareVersionUrl);
    }

    return true;
}

// 设置车辆信息
void CDeviceManager::setVehicleNo(QVariant VehicleInfo)
{
    m_pDatebaseDev->setValue("vehicle_no", VehicleInfo);
}

// 获取车牌号
QVariant CDeviceManager::getVehicleNo()
{
    return m_pDatebaseDev->getValue("vehicle_no");
}

// 设置车辆电话号码
void CDeviceManager::setPhoneNumber(QVariant phone_num)
{
    m_pDatebaseDev->setValue("phone_num", phone_num);
}

// 获取车辆电话号码
QVariant CDeviceManager::getPhoneNumber()
{
    return m_pDatebaseDev->getValue("phone_num");
}

// 设置心跳频率 单位为分
void CDeviceManager::setHeartBitFrequency(QVariant freq)
{
    int iFreq = freq.toInt();
    if (iFreq > 0)
    {
        m_pDatebaseDev->setValue("pingFreq", iFreq);
        if (m_pTimerMng)
        {
            m_pTimerMng->ChangeInterval(TIMER_WS_HEART_BIT, iFreq);
        }
    }
}

// 获取心跳频率
QVariant CDeviceManager::getHeartBitFrequency()
{
    return m_pDatebaseDev->getValue("pingFreq");
}

// 设置上报频率 单位为分
void CDeviceManager::setReportFrequency(QVariant freq)
{
    int iFreq = freq.toInt();
    if (iFreq > 0)
    {
        m_pDatebaseDev->setValue("commonStatusFrq", iFreq);
        if (m_pTimerMng)
        {
            m_pTimerMng->ChangeInterval(TIMER_WS_STATUS_REPORT, iFreq);
        }
    }
}

// 获取上报频率
QVariant CDeviceManager::getReportFrequency()
{
    return m_pDatebaseDev->getValue("commonStatusFrq");
}

// 速度上报频率
void CDeviceManager::setUploadSpeedFreq(QVariant uploadSpeedFreq)
{
    int iUploadSpeedFreq = uploadSpeedFreq.toInt();
    if (iUploadSpeedFreq > 0)
    {
        m_pDatebaseDev->setValue("uploadSpeedFreq", iUploadSpeedFreq);
        if (m_pTimerMng)
        {
            m_pTimerMng->ChangeInterval(TIMER_WS_GPS_REPORT, iUploadSpeedFreq);
        }
    }
}

// 速度上报频率
QVariant CDeviceManager::getUploadSpeedFreq()
{
    return m_pDatebaseDev->getValue("uploadSpeedFreq");
}

// 停车告警使能状态
void CDeviceManager::setParkingAlarmEnable(QVariant Enable)
{
    bool bEnable = Enable.toBool();
    m_pDatebaseDev->setValue("parkingAlarmEnable", bEnable);
}

// 停车告警使能状态
QVariant CDeviceManager::getParkingAlarmEnable()
{
    return m_pDatebaseDev->getValue("parkingAlarmEnable");
}

// 设置停车告警门限
void CDeviceManager::setParkingAlarmInterval(QVariant ParkingAlarmInterval)
{
    int iParkingAlarmInterval = ParkingAlarmInterval.toInt();
    if (iParkingAlarmInterval != 0)
    {
        m_pDatebaseDev->setValue("parkingAlarmInterval", iParkingAlarmInterval);
    }
}

// 获取停车告警门限
QVariant CDeviceManager::getParkingAlarmInterval()
{
    return m_pDatebaseDev->getValue("parkingAlarmInterval");
}

// 设置拆帧间隔时间
void CDeviceManager::setFrameSplittingInterval(QVariant frameInverval)
{
    int iFrameInverval = frameInverval.toInt();
    if (iFrameInverval)
    {
        m_pDatebaseDev->setValue("videoFrameSplittingInterval", iFrameInverval);
    }
}

// 设置拆帧间隔时间
QVariant CDeviceManager::getFrameSplittingInterval()
{
    return m_pDatebaseDev->getValue("videoFrameSplittingInterval");
}

// 设置呼叫服务器ip
void CDeviceManager::setCallServIp(QVariant serv_ip)
{
    m_pDatebaseDev->setValue("call_serv_ip", serv_ip);
}

// 获取呼叫服务器ip
QVariant CDeviceManager::getCallServIp()
{
    return m_pDatebaseDev->getValue("call_serv_ip");
}

// 设置呼叫服务器端口
void CDeviceManager::setCallServPort(QVariant serv_port)
{
    m_pDatebaseDev->setValue("call_serv_port", serv_port);
}

// 获取呼叫服务器端口
QVariant CDeviceManager::getCallServPort()
{
    return m_pDatebaseDev->getValue("call_serv_port");
}

// 设置呼叫服务器用户名
void CDeviceManager::setCallUsername(QVariant user_name)
{
    m_pDatebaseDev->setValue("call_user_name", user_name);
}

// 获取呼叫服务器用户名
QVariant CDeviceManager::getCallUsername()
{
    return m_pDatebaseDev->getValue("call_user_name");
}

// 设置呼叫服务器密码
void CDeviceManager::setCallPassword(QVariant password)
{
    m_pDatebaseDev->setValue("call_password", password);
}

// 获取呼叫服务器密码
QVariant CDeviceManager::getCallPassword()
{
    return m_pDatebaseDev->getValue("call_password");
}

// 设置呼叫服务器通道
void CDeviceManager::setCallChannel(QVariant channel)
{
    m_pDatebaseDev->setValue("call_channel", channel);
}

// 获取呼叫服务器通道
QVariant CDeviceManager::getCallChannel()
{
    return m_pDatebaseDev->getValue("call_channel");
}

// 设置web socket服务器地址
void CDeviceManager::setWsServaddr(QVariant servaddr)
{
    m_pDatebaseDev->setValue("ws_servaddr", servaddr);
}

// 获取web socket服务器地址
QVariant CDeviceManager::getWsServaddr()
{
    return m_pDatebaseDev->getValue("ws_servaddr");
}

// 设置web socket用户名
void CDeviceManager::setWsUsername(QVariant username)
{
    m_pDatebaseDev->setValue("ws_username", username);
}

// 获取web socket用户名
QVariant CDeviceManager::getWsUsername()
{
    return m_pDatebaseDev->getValue("ws_username");
}

// 设置web socket密码
void CDeviceManager::setWsPassword(QVariant password)
{
    m_pDatebaseDev->setValue("ws_password", password);
}

// 获取web socket密码
QVariant CDeviceManager::getWsPassword()
{
    return m_pDatebaseDev->getValue("ws_password");
}

// 设置摄像头窗口数目
void CDeviceManager::setCamViewCount(QVariant cam_count)
{
    m_pDatebaseDev->setValue("cam_view_cnt", cam_count);
}

// 获取摄像头窗口数目
QVariant CDeviceManager::getCamViewCount()
{
    return m_pDatebaseDev->getValue("cam_view_cnt");
}
