#ifndef CDEVICEMANAGER_H
#define CDEVICEMANAGER_H
#include <QObject>
#include <QMap>
#include "base_module.h"
#include "config/CConfigManager.h"
#include "camera/WebCamFinder.h"
#include "log/CLogManager.h"

class ITcpModule;
class CWebSocketClient;
class CDeviceManager;
class CTimerManager;
class CSWDLManager;
class CGPSManager;
class CFileManager;

struct CallConfig
{
    QString serv_ip;
    QString serv_port;
    QString user_name;
    QString password;
    QString channel;
};

// 升级模块暂时打桩
class CSWDLMng : public QObject
{
    Q_OBJECT
public:
    // 制作信息
    QString getManualFac() { return m_strManualFac; }
    // 新软件版本
    QString getNewSoftVer() { return m_strNewSoftVer; }
    // 升级路径
    QString getUpdatePath() { return m_strUpdatePath; }

protected:
    // 制作信息
    QString m_strManualFac;
    // 新软件版本
    QString m_strNewSoftVer;
    // 升级路径
    QString m_strUpdatePath;
};

class CDeviceManager : public QObject, public IModule
{
    Q_OBJECT
    // 心态频率
    Q_PROPERTY(QVariant pingFreq READ getHeartBitFrequency WRITE setHeartBitFrequency)
    // 上报频率
    Q_PROPERTY(QVariant commonStatusFrq READ getReportFrequency WRITE setReportFrequency)
    // 速度上报频率
    Q_PROPERTY(QVariant uploadSpeedFreq READ getUploadSpeedFreq WRITE setUploadSpeedFreq)
    // 拆帧频率
    Q_PROPERTY(QVariant videoFrameSplittingInterval READ getFrameSplittingInterval WRITE setFrameSplittingInterval)
    // 停车告警上报间隔
    Q_PROPERTY(QVariant parkingAlarmInterval READ getParkingAlarmInterval WRITE setParkingAlarmInterval)
    // 停车告警使能标记
    Q_PROPERTY(QVariant parkingAlarmEnable READ getParkingAlarmEnable WRITE setParkingAlarmEnable)
    // 车辆信息
    Q_PROPERTY(QVariant vehicle_no READ getVehicleNo WRITE setVehicleNo)
    // 电话号码
    Q_PROPERTY(QVariant phone_num READ getPhoneNumber WRITE setPhoneNumber)

    //
    Q_PROPERTY(QVariant call_serv_ip READ getCallServIp WRITE setCallServIp)
    Q_PROPERTY(QVariant call_serv_port READ getCallServPort WRITE setCallServPort)
    Q_PROPERTY(QVariant call_user_name READ getCallUsername WRITE setCallUsername)
    Q_PROPERTY(QVariant call_password READ getCallPassword WRITE setCallPassword)
    Q_PROPERTY(QVariant call_channel READ getCallChannel WRITE setCallChannel)

    // ws 相关配置
    Q_PROPERTY(QVariant ws_servaddr READ getWsServaddr WRITE setWsServaddr)
    Q_PROPERTY(QVariant ws_username READ getWsUsername WRITE setWsUsername)
    Q_PROPERTY(QVariant ws_password READ getWsPassword WRITE setWsPassword)

    // 摄像头窗口数目
    Q_PROPERTY(QVariant cam_view_cnt READ getCamViewCount WRITE setCamViewCount)

    friend class DlgMainSetting;

public:
    MODULE_NAME(CDeviceManager)
    LOG_DECLARE()
    explicit CDeviceManager(QObject *parent = nullptr);
    ~CDeviceManager();

    // 关闭APP
    void closeApp();

    // 是否已经登录
    bool isLogined() { return m_bLogined; }

    // 获取设备省份代码
    WORD getProvinceID();

    // 设置设备省份代码
    void setProvinceID(WORD id);

    // 获取设备市区代码
    WORD getCityID();

    // 设置设备市区代码
    void setCityID(WORD city);

    // 获取设备生产商信息
    QString getManufactor();

    // 获取设备类型
    QString getDeviceType();

    // 获取终端类型(适用车辆)
    WORD getTerminalType();

    // 获取设备终端ID
    QString getTerminalID();

    // 获取车辆颜色类型
    int getVehicleColor();

    // 设置车辆颜色类型
    void setVehicleColor(int color);

    // 获取IMEI串号
    QString getIMEI();

    // 获取软件版本
    QString getSoftwareVersion();

    // 获取硬件版本信息
    QString getHardVersion();

    // 终端SIM卡ICCID
    QString getSimICCID();

    // GNSS模块属性(BYTE)
    QString getFirmwareVersion();

    // GNSS模块属性(BYTE)
    BYTE getGNSSProperty();

    // 通讯模块属性(BYTE)
    BYTE getComProperty();

    // 恢复出场值
    void restoreFactory();

    // 保存配置
    bool saveConfig();

    void setNetworkStatus(bool bAlarm);

    // 设置呼叫服务器ip
    void setCallServIp(QVariant serv_ip);
    // 获取呼叫服务器ip
    QVariant getCallServIp();

    // 设置呼叫服务器端口
    void setCallServPort(QVariant serv_port);
    // 获取呼叫服务器端口
    QVariant getCallServPort();

    // 设置呼叫服务器用户名
    void setCallUsername(QVariant user_name);
    // 获取呼叫服务器用户名
    QVariant getCallUsername();

    // 设置呼叫服务器密码
    void setCallPassword(QVariant password);
    // 获取呼叫服务器密码
    QVariant getCallPassword();

    // 设置呼叫服务器通道
    void setCallChannel(QVariant channel);
    // 获取呼叫服务器通道
    QVariant getCallChannel();

    // 设置车辆信息
    void setVehicleNo(QVariant code);
    // 获取车牌号
    QVariant getVehicleNo();

    // 设置车辆电话信息
    void setPhoneNumber(QVariant phone_num);
    // 获取车辆电话号码
    QVariant getPhoneNumber();

    // 设置web socket服务器地址
    void setWsServaddr(QVariant servaddr);
    // 获取web socket服务器地址
    QVariant getWsServaddr();

    // 设置web socket用户名
    void setWsUsername(QVariant username);
    // 获取web socket用户名
    QVariant getWsUsername();

    // 设置web socket密码
    void setWsPassword(QVariant password);
    // 获取web socket密码
    QVariant getWsPassword();

    // 设置摄像头窗口数目
    void setCamViewCount(QVariant cam_count);
    // 获取摄像头窗口数目
    QVariant getCamViewCount();
	// 获取设备ID
    QString getDeviceID();

protected:
    //初始化操作
    virtual bool
    Initialize();
    //模块注册接口
    bool RegistMessage();

    

    // 获取系统版本，这里要调用底层函数，这里先打桩
    QString getSystemVersion();

    // 获取硬盘类型 1表示SD卡，2表示硬盘，3表示SSD卡 先打桩
    int getDisktype();

    // 获取磁盘空间状态。剩余空间，单位MB 先打桩
    float getFreeSpaceMB();

    // 获取磁盘空间。
    float getDiskSpaceMB();

    // 获取磁盘空间状态。剩余空间，单位% 先打桩
    float getFreeSpacePrecent();

    // 获取运行状态。0为正常,1为维修,2为停用；先打桩
    int getStatus();

    // 获取里程；单位：米。
    int getMileage();

    // 获取网络类型 。0为3G,1为4G,2为5G，3为WIFI，4为有线；
    int getNetwork();

    // 获取网络流量消耗；单位为MB
    int getNetworkTraffic();

    //  获取网络状态 连通/断开/重连
    int getNetworkStatus();

    // 获取心跳频率
    QVariant getHeartBitFrequency();

    // 设置心跳频率 单位为分
    void setHeartBitFrequency(QVariant freq);

    // 制作信息
    QString getManualFac();

    // 新软件版本
    QString getNewSoftVer();

    // 升级路径
    QString getUpdatePath();

    // 获取上报频率
    QVariant getReportFrequency();

    // 设置上报频率 单位为秒
    void setReportFrequency(QVariant freq);

    // 设置速度上报频率
    void setUploadSpeedFreq(QVariant uploadSpeedFreq);
    // 获取gps/速度上报频率
    QVariant getUploadSpeedFreq();

    // 停车告警使能状态
    void setParkingAlarmEnable(QVariant Enable);
    // 停车告警使能状态
    QVariant getParkingAlarmEnable();

    // 设置停车告警门限
    void setParkingAlarmInterval(QVariant ParkingAlarmInterval);

    // 获取停车告警门限
    QVariant getParkingAlarmInterval();

    // 设置拆帧间隔时间
    void setFrameSplittingInterval(QVariant frameInverval);

    // 设置拆帧间隔时间
    QVariant getFrameSplittingInterval();

    // 获取服务器地址
    QString getServHostess() { return m_servAddress; }

    // 设置服务器地址
    void setServAddress(QString addr) { m_servAddress = addr; }

public:
    // 登陆响应
    bool onLogin(QVariantMap &prop_arr);

    // 获取参数响应
    bool onGetParams(QVariantMap &prop_arr);

    // 更新系统
    bool upgradeSystem(QVariantMap &prop_arr);

Q_SIGNALS:

    // 通知登陆状态信息
    void notifyLogin(bool);

    void notifyShowLogin();

    void notifyCloseApp();

    // 通知网络状态
    void notifyNetworkAlarm(bool bAlarm);

public Q_SLOTS:

    // 以下为设备相关的接口
    // 登陆
    bool login(QString usr_name, QString password);

    // 从服务器获取配置
    bool getParams();

    // 返回系统更新结果
    bool upgradeSystemResult(QString sys_ver, int sys_ret, QString soft_ver, int soft_ret);

    // 上报停车报警
    bool sendParkingAlarm();

    // 心跳命令
    bool ping();

    // 发送状态信息
    bool sendCommonStatus();

    // 上报GPS 速度信息
    bool sendGpsSpeed();

    // 信号槽简易信号处理
    void connect2Slots();

    // 查询呼叫配置
    void procQueryCallConfig(MODULE_MESSAGE &msg);

protected:
    //主消息任务，主要用于和其他模块之间的消息通讯
    THREAD_DEFINITION(CDeviceManager, TaskMain)
    //传感器收集任务, 用于定时轮询传感器相关的信息;
    THREAD_DEFINITION(CDeviceManager, TaskSensorCol)

    //主消息任务，主要用于和其他模块之间的消息通讯
    virtual void TaskMain(void *pBuff);
    //传感器收集任务, 用于定时轮询传感器相关的信息;
    virtual void TaskSensorCol(void *pBuff);

    // 获取默认磁盘
    QString getDefDisk();
    // 枚举磁盘设备
    QStringList enumDisks();
    // 获取存储设备信息
    bool getStorageInfo(QString device, QStorageInfo &st);

    // 从数据库恢复数据
    bool restoreFromDB(CDatebase *pDatebase);
    // 创建新的数据库
    bool createDatebase();
    // 注册数据库默认键值
    bool regDBKey();

protected:
    // 依赖项模块
    ITcpModule *m_pTcpModule;
    CSWDLMng m_swdlMng;
    CConfigManager *m_pConfigMng;
    CWebSocketClient *m_pWebSock;
    CTimerManager *m_pTimerMng;
    CSWDLManager *m_pSwdlMng;
    CGPSManager *m_pSensorMng;
    CFileManager *m_pFileMng;
    // 设备管理数据库
    CDatebase *m_pDatebaseDev;

    QMap<QString, QHostInfo> m_onlineList;

    // 是否获取到服务器配置
    bool m_bGetCfgFromServ;
    // 服务器地址
    QString m_servAddress;

    // 是否已经登录
    bool m_bLogined;

    // 车辆颜色信息
    int m_iVehicleColor;

    QString m_strManufactor;
    QString m_strTerminalType;
    QString m_strTerminalID;
};

#endif // CDEVICEMANAGER_H
