#include "CFileManager.h"
#include "../socket/TcpServer.h"
#include "../base_module/factory.h"
#include "socket/CWebSocketClient.h"
#include <QtXml>
#include <assert.h>

const QString VIDEO_LIST_NAME = "video_list.lst";
const QString IMAGE_LIST_NAME = "image_list.lst";

const QString DB_NAME_FILE_MNG = "file_mng";

// 分离文件路径
extern QStringList sepratePathName(QString path_name);
// 生成dir
extern bool makePath(QString strPath);

CCamFileNode::CCamFileNode()
    : bFileLocked(false)
{
}

CCamFileNode::CCamFileNode(QString path_name)
    : bFileLocked(false)
{
    this->fileName = path_name;
    QStringList path_lst = sepratePathName(path_name);

    QStringList lst = path_lst[1].split("_");
    if (lst.size() == 3)
    {
        QDate date = QDate::fromString(lst[0], "yyyy-MM-dd");
        QTime time = QTime::fromString(lst[1], "hh-mm");
        tmBegin.setDate(date);
        tmBegin.setTime(time);

        QString strTmp = lst[2].replace("Cam", "");
        QString Cam_idx = strTmp.left(strTmp.indexOf("."));
        channel = Cam_idx.toInt();
        media_type = 2;
        stream_code = 1;
        storage_type = 1;
        alarms = 0;
    }
}

// 导出到string
QString CCamFileNode::toString()
{
    QString strAlarms = QString("%1").arg(alarms);
    return fileName +
           "#" + QVariant(channel).toString() +
           "#" + QVariant(media_type).toString() +
           "#" + QVariant(stream_code).toString() +
           "#" + QVariant(storage_type).toString() +
           "#" + tmBegin.toString("yyyy-MM-dd_hh-mm-ss") + "#" + tmEnd.toString("yyyy-MM-dd_hh-mm-ss") + "#" + strAlarms + "\n";
}

// 从string导入
bool CCamFileNode::fromString(QString info)
{
    QStringList list = info.split('#');
    if (list.size() == 8)
    {
        fileName = list[0];
        // 通道号
        channel = (BYTE)list[1].toInt();
        // 媒体类型
        media_type = (BYTE)list[2].toInt();
        // 码流
        stream_code = (BYTE)list[3].toInt();
        // 存储介质
        storage_type = (BYTE)list[4].toInt();

        // 开始时间
        tmBegin = QDateTime::fromString(list[5], "yyyy-MM-dd_hh-mm-ss");
        // 结束时间(预计)
        tmEnd = QDateTime::fromString(list[6], "yyyy-MM-dd_hh-mm-ss");
        // 告警
        alarms = list[7].toLongLong();
        // 文件大小,不做处理,等需要的时候现场查询
        file_size = 0;

        return true;
    }
    return false;
}

// 是否有效，主要通过文件名和开时时间作判断，这两个信息都存在文件名的格式里面
bool CCamFileNode::isValid()
{
    if (tmBegin.isValid() && !fileName.isEmpty())
    {
        return true;
    }
    return false;
}

// 文件节点管理类
CFileNodeMangaer::CFileNodeMangaer(QString strListName, QString strFliter)
    : m_strListName(strListName),
      m_strFliter(strFliter),
      m_bRestorFactoryLocked(false)
{
}

// 获取视频列表
QString CFileNodeMangaer::getFileList(QDateTime &tmBegin, QDateTime &tmEnd)
{
    QString files;
    uint begin = tmBegin.toTime_t();
    uint end = tmEnd.toTime_t();

    // 这里主要是为了防止错误输入结束时间比开始时间早导致结束条件不能满足而做无效遍历,直接返回
    if (begin > end)
    {
        return files;
    }

    bool bFind = false;
    m_mtxFileList.lock();
    FILE_LIST_BY_TIME::Iterator it_begin = m_mapFlieListByTime.lowerBound(begin);
    FILE_LIST_BY_TIME::Iterator it_end = m_mapFlieListByTime.upperBound(end);
    FILE_LIST_BY_TIME::Iterator it;
    for (it = it_begin; it != it_end && it != m_mapFlieListByTime.end(); it++)
    {
        CCamFileNode *pNode = dynamic_cast<CCamFileNode *>(it.value());
        if (pNode)
        {
            files += pNode->fileName + ',';
        }
        else
        {
            break;
        }
    }
    m_mtxFileList.unlock();

    return files;
}

bool CFileNodeMangaer::queryFileList(FILE_QUERY_INFO query, VCT_FILE_LIST &file_lst)
{
    QString files;
    bool bFind = false;
    m_mtxFileList.lock();
    FILE_LIST_BY_TIME::Iterator it_begin = m_mapFlieListByTime.lowerBound(query.tmBegin);
    FILE_LIST_BY_TIME::Iterator it_end = m_mapFlieListByTime.upperBound(query.tmEnd);
    FILE_LIST_BY_TIME::Iterator it;
    if (query.tmBegin == 0)
    {
        it_begin = m_mapFlieListByTime.begin();
    }

    if (query.tmEnd == 0)
    {
        it_end = m_mapFlieListByTime.end();
    }

    for (it = it_begin; it != it_end && it != m_mapFlieListByTime.end(); it++)
    {
        CCamFileNode *pNode = dynamic_cast<CCamFileNode *>(it.value());
        if (pNode)
        {
            // 通道查询条件不为空或通道不相等跳过
            if (query.channel != 0 && query.channel != pNode->channel)
            {
                continue;
            }
            // 告警查询条件不为空或告警条件不相等跳过
            if (query.alarm_status != 0 && query.alarm_status != pNode->alarms)
            {
                continue;
            }

            // 非视频资源跳过
            if (query.media_type != 0 && query.media_type != 2 && query.media_type != 3)
            {
                continue;
            }

            // 码流查询条件不为空或码流条件不相等跳过
            if (query.stream_code != 0 && query.stream_code != pNode->stream_code)
            {
                continue;
            }

            // 存储介质查询条件不为空或存储介质不相等跳过
            if (query.storage_type != 0 && query.storage_type != pNode->storage_type)
            {
                continue;
            }

            // 获取文件大小
            pNode->file_size = getFileSize(*pNode);

            // 到这里所有的查询条件都能满足了,加入队列
            file_lst.append(*it.value());
        }
        else
        {
            break;
        }
    }
    m_mtxFileList.unlock();
    return true;
}

// 查询文件大小
int CFileNodeMangaer::getFileSize(CCamFileNode &node)
{
    QString path_name = getDataPath();
    QFile file(path_name + node.fileName);

    return file.size();
}

// 添加文件节点
void CFileNodeMangaer::addFileNode(CCamFileNode &file_node)
{
    CCamFileNode *pNode;
    FILE_LIST_BY_NAME::Iterator it = m_mapFlieListByName.find(file_node.fileName);
    if (m_mapFlieListByName.end() == it)
    {
        pNode = new CCamFileNode(file_node);
        m_mtxFileList.lock();
        m_mapFlieListByName[pNode->fileName] = pNode;
        m_mapFlieListByTime.insert(pNode->tmBegin.toTime_t(), pNode);
        m_mtxFileList.unlock();
    }
}

// 删除文件节点
bool CFileNodeMangaer::delFileNode(QString file_name)
{
    bool bSuccess = true;
    // file name
    QString path_name = getDataPath();
    m_mtxFileList.lock();
    FILE_LIST_BY_NAME::Iterator it = m_mapFlieListByName.find(file_name);
    if (it != m_mapFlieListByName.end())
    {
        CCamFileNode *pNode = it.value();
        m_mapFlieListByTime.remove(pNode->tmBegin.toTime_t(), pNode);
        m_mapFlieListByName.remove(file_name);
        delete pNode;
    }
    else
    {
        bSuccess = false;
    }

    m_mtxFileList.unlock();

    QFile file_rm(path_name + file_name);
    if (!file_rm.remove())
    {
        bSuccess = false;
        file_rm.deleteLater();
    }

    return bSuccess;
}

// 恢复出厂设置
void CFileNodeMangaer::restoreFactory()
{
    QMutexLocker locker(&m_mtxFileList);
    m_bRestorFactoryLocked = true;
    m_mapFlieListByTime.clear();
    m_mapFlieListByName.clear();
    QString strPath = getDataPath();
    QFile file(strPath + m_strListName); //相对路径、绝对路径、资源路径都行
    file.remove();
}

// 保存文件节点
bool CFileNodeMangaer::saveFileList()
{
    if (m_bRestorFactoryLocked)
    {
        return false;
    }

    //打开或创建文件
    QString strPath = getDataPath();
    QFile file(strPath + m_strListName); //相对路径、绝对路径、资源路径都行
    if (file.open(QIODevice::ReadWrite | QIODevice::Text))
    {
        m_mtxFileList.lock();
        foreach (CCamFileNode *pnode, m_mapFlieListByName)
        {
            if (pnode->isValid())
            {
                file.write(pnode->toString().toStdString().c_str());
            }
        }
        m_mtxFileList.unlock();

        file.flush();
        file.close();
    }
}

// 导入文件节点,包含重建列表
bool CFileNodeMangaer::loadFileList()
{
    //打开或创建文件
    QFile file(getDataPath() + m_strListName); //相对路径、绝对路径、资源路径都行
    if (file.open(QFile::ReadOnly))
    {
        CCamFileNode file_node;
        while (!file.atEnd())
        {
            QByteArray line = file.readLine();
            if (file_node.fromString(line))
            {
                addFileNode(file_node);
            }
        }
        file.close();
    }

    bool bRenewList = checkFileList();
    bRenewList += rebuildFileList();
    if (bRenewList)
    {
        saveFileList();
    }

    return true;
}

// 重建文件列表
bool CFileNodeMangaer::rebuildFileList()
{
    bool bRenewList = false;
    QString dirpath = getDataPath();
    //设置文件过滤器
    QStringList nameFilters;

    //设置文件过滤格式
    nameFilters = m_strFliter.split(";");
    QStringList dir_list;
    dir_list << "video/"
             << "image/";

    QMap<QDateTime, QDateTime> tm_map;

    foreach (QString str_dir, dir_list)
    {
        //设置要遍历的目录
        QDir dir(dirpath + str_dir);
        //将过滤后的文件名称存入到files列表中
        QStringList files = dir.entryList(nameFilters, QDir::Files | QDir::Readable, QDir::Name);

        QDateTime last_begin_time;

        foreach (QString file_name, files)
        {
            QString local_path = str_dir + file_name;
            FILE_LIST_BY_NAME::Iterator it = m_mapFlieListByName.find(local_path);
            if (it == m_mapFlieListByName.end())
            {
                CCamFileNode node(local_path);
                if (!last_begin_time.isValid())
                {
                    last_begin_time = node.tmBegin;
                    tm_map[last_begin_time] = last_begin_time;
                }

                else if (last_begin_time != node.tmBegin)
                {
                    auto it = tm_map.find(last_begin_time);
                    if (it != tm_map.end())
                    {
                        it.value() = node.tmBegin;
                    }
                }

                if (node.isValid())
                {
                    addFileNode(node);
                    bRenewList = true;
                }
                else
                {
                    QString path_name = getDataPath() + local_path;
                    QFile(path_name).remove();
                }
            }
        }
    }

    // 写结束时间
    for (auto it = m_mapFlieListByName.begin(); it != m_mapFlieListByName.end(); it++)
    {
        it.value()->tmEnd = it.value()->tmBegin.addSecs(3600);
    }

    return bRenewList;
}

// 获取第一个有效的文件节点
CCamFileNode *CFileNodeMangaer::getFirstNodeForDel()
{
    CCamFileNode *pNode = nullptr;
    QMutexLocker lock(&m_mtxFileList);
    FILE_LIST_BY_TIME::Iterator it = m_mapFlieListByTime.begin();
    while (!pNode && it != m_mapFlieListByTime.end())
    {
        if (!it.value()->bFileLocked)
        {
            pNode = it.value();
            break;
        }
    }
    return pNode;
}

// 检查列表文件
bool CFileNodeMangaer::checkFileList()
{
    bool bRenewList = false;
    QString strPath = getDataPath();
    FILE_LIST_BY_NAME::Iterator it = m_mapFlieListByName.begin();
    m_mtxFileList.lock();
    for (; it != m_mapFlieListByName.end();)
    {
        CCamFileNode *pNode = it.value();
        QFile file(strPath + pNode->fileName);
        if (file.size() == 0)
        {
            file.remove();
        }

        if (!file.exists())
        {
            //指针移至下一个位置
            it = m_mapFlieListByName.erase(it);
            m_mapFlieListByTime.remove(pNode->tmBegin.toTime_t(), pNode);
            delete (pNode);
            pNode = nullptr;
            bRenewList = true;
        }
        else
        {
            it++;
        }
    }
    m_mtxFileList.unlock();
    return bRenewList;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////               CFileManager              //////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////

REGISTER_MODULE(CFileManager, "CFileManager")

CFileManager::CFileManager(QObject *parent)
    : QObject(parent),
      IModule(),
      m_pTcpModule(nullptr),
      m_pWebSock(nullptr),
      m_pConfigMng(nullptr),
      m_fMinFreeSpaceRate(30.0),
      m_pDbFile(nullptr),
      m_vidFileMng(VIDEO_LIST_NAME, "*.mp4"),
      m_imgFileMng(IMAGE_LIST_NAME, "*.jpg;*.jpeg"),
      m_bRestoreFactoryLocked(false)
{

    m_pConfigMng = dynamic_cast<CConfigManager *>(QueryPlugin("CConfigManager"));
    if (m_pConfigMng)
    {
        m_pConfigMng->regRestoreCallback(restoreFactoryCallback, this);
        m_pDbFile = m_pConfigMng->getDatabase(MODULE_FILE_MNG, DB_NAME_FILE_MNG);
        if (!m_pDbFile)
        {
            createDatebase();
        }

        restoreFromDB();
    }

    makePaths();

    // 导入视频列表
    m_vidFileMng.setDatePath(m_strDatapath);
    m_vidFileMng.loadFileList();

    // 导入图像列表
    m_imgFileMng.setDatePath(m_strDatapath);
    m_imgFileMng.loadFileList();
}

CFileManager::~CFileManager()
{
    std::cout << name() << std::endl;
}

bool CFileManager::queryFileList(FILE_QUERY_INFO query, VCT_FILE_LIST &file_lst)
{
    return m_vidFileMng.queryFileList(query, file_lst);
}

// 恢复出厂配置回调函数
bool CFileManager::restoreFactoryCallback(void *This, int type)
{
    CFileManager *pThis = (CFileManager *)This;
    return pThis->restoreFactory(type);
}

// 恢复出厂值
bool CFileManager::restoreFactory(int type)
{
    m_bRestoreFactoryLocked = true;
    if (type == RESTORE_FACTORY_ALL || type == RESTORE_FACTORY_FILE)
    {
        QString dirpath = getDataPath();
        //设置文件过滤器
        QStringList nameFilters;
        nameFilters << "*.*";
        QStringList dir_list;
        dir_list << "video/"
                 << "image/"
                 << "";

        foreach (QString str_dir, dir_list)
        {
            //设置要遍历的目录
            QDir dir(dirpath + str_dir);
            //将过滤后的文件名称存入到files列表中
            QStringList files = dir.entryList(nameFilters, QDir::Files | QDir::Readable, QDir::Name);

            foreach (QString file_name, files)
            {
                QString path_name = getDataPath() + str_dir + file_name;
                QFile(path_name).remove();
            }
        }

        m_vidFileMng.restoreFactory();
        m_imgFileMng.restoreFactory();
    }

    return true;
}

//初始化操作
bool CFileManager::Initialize()
{
    //起主消息处理线程
    STARTUP_TASK(TaskMain)

    return true;
}

// 创建数据库
bool CFileManager::createDatebase()
{
    m_pDbFile = m_pConfigMng->regDatabase(MODULE_FILE_MNG, DB_NAME_FILE_MNG);
    m_pDbFile->regKey("data_path", getDataPath());
    m_pDbFile->save();
}

// 恢复数据库，没有的字段需要保存默认值
bool CFileManager::restoreFromDB()
{
    bool bSaveDB = false;
    m_pDbFile->regKey("data_path", getDataPath());
    QString strDisk = m_pDbFile->getValue("disk_root").toString();
    // 检查路径合法性
    if (strDisk.isEmpty())
    {
        strDisk = getDefDisk();
        m_pDbFile->regKey("disk_root", strDisk);
        bSaveDB = true;
    }

    m_strDatapath = m_pDbFile->getValue("data_path").toString();

    // 检查路径合法性
    if (m_strDatapath.isEmpty() || m_strDatapath.indexOf(strDisk) < 0)
    {
        m_strDatapath = getDataPath();
        m_pDbFile->regKey("data_path", m_strDatapath);
        bSaveDB = true;
    }

    // 获取数据磁盘信息
    getStorageInfo(strDisk, m_dataStorage);

    if (bSaveDB)
    {
        m_pDbFile->save();
    }

    return true;
}

bool CFileManager::RegistMessage()
{
    m_pTcpModule = dynamic_cast<ITcpModule *>(QueryPlugin("TcpModule"));
    if (m_pTcpModule)
    {
        m_pTcpModule->RegistMessageCallback(this, MODULE_FILE_MNG);
    }
    return true;
}

//获取默认磁盘信息
QString CFileManager::getDefDisk()
{
    QString strDskPath = "/";
    qint64 max_freesize = 0;
    foreach (const QStorageInfo &storage, QStorageInfo::mountedVolumes())
    {
        if (storage.isValid() && storage.isReady())
        {
            if (!storage.isReadOnly() && storage.isValid() && storage.device().left(4) == "/dev")
            {
                qDebug() << storage.device();
                if (max_freesize < storage.bytesFree())
                {
                    strDskPath = storage.device();
                    max_freesize = storage.bytesFree();
                }
            }
        }
    }

    return strDskPath;
}

// 获取存储设备信息
bool CFileManager::getStorageInfo(QString device, QStorageInfo &st)
{
    foreach (const QStorageInfo &storage, QStorageInfo::mountedVolumes())
    {
        if (storage.isValid() && storage.isReady())
        {
            if (storage.device() == device)
            {
                st = storage;
                return true;
            }
        }
    }
    return false;
}

// 获取数据路径
QString CFileManager::getDataPath()
{
    if (m_strDatapath.isEmpty())
    {
        QString strRootpath = getDefDisk();
        if (getStorageInfo(strRootpath, m_dataStorage))
        {
            m_strDatapath = m_dataStorage.rootPath() + "/hncy_data/";
            return m_strDatapath;
        }
        else
        {
            return "./hncy_data/";
        }
    }

    return m_strDatapath;
}

// 获取硬盘类型 1表示SD卡，2表示硬盘，3表示SSD卡 先打桩
int CFileManager::getDisktype()
{
    int dsk_type = 0;
    QString strDev = m_dataStorage.device();
    // 获取硬盘类型 1表示SD卡，2表示硬盘，3表示SSD卡 先打桩
    if (strDev.indexOf("sda") >= 0)
    {
        dsk_type = 2;
    }
    else if (strDev.indexOf("mmc") >= 0)
    {
        dsk_type = 1;
    }
    else
    {
        dsk_type = 3;
    }
    return dsk_type;
}

// 获取磁盘空间状态。剩余空间，单位MB 先打桩
float CFileManager::getFreeSpaceMB()
{
    m_dataStorage.refresh();
    return m_dataStorage.bytesFree() / 1000000;
}

// 获取磁盘空间。
float CFileManager::getDiskSpaceMB()
{
    return m_dataStorage.bytesTotal() / 1000000;
}

// 获取磁盘空间状态。剩余空间，单位% 先打桩
float CFileManager::getFreeSpacePrecent()
{
    m_dataStorage.refresh();
    return (m_dataStorage.bytesFree() * 100) / m_dataStorage.bytesTotal();
}

void CFileManager::makePaths()
{
    makePath(m_strDatapath);
}

// 动态管理，只删除视频文件
void CFileManager::managerDiskSpace()
{
    while (getFreeSpacePrecent() < m_fMinFreeSpaceRate)
    {
        CCamFileNode *pNode = m_vidFileMng.getFirstNodeForDel();
        if (pNode)
        {
            m_vidFileMng.delFileNode(pNode->fileName);
        }
    }
}

//主消息任务，主要用于和其他模块之间的消息通讯
void CFileManager::TaskMain(void *pBuff)
{
    (void)pBuff;
    setCurrentTaskName("tskAlarmMain");
    MODULE_MESSAGE *pMsg = nullptr;
    RegistMessage();
    // 列表重建任务，不用等到系统模块加载完毕后运行，内部处理任务。
    STARTUP_TASK(TaskRebuildFileList)
    waitForStart();
    m_pWebSock = dynamic_cast<CWebSocketClient *>(QueryPlugin("CWebSocketClient"));

    while (!isShutdown())
    {
        if (GetModuleMessage(pMsg)) // get msg from message queue
        {
            switch (pMsg->iCmd)
            {
            case CMD_FILE_MNG_DEL_FILES:
                ProcDelFiles(pMsg);
                break;
            case CMD_ADD_FILE_TO_LIST:
                addFile2List(pMsg);
                break;
            default:
                break;
            }

            SAFE_DELETE(pMsg);
        }
    }
}

//列表重建任务，系统启动过程中运行;
void CFileManager::TaskRebuildFileList(void *pBuff)
{

    return;
}

// 获取文件类型
FILE_TYPE CFileManager::getFileTYPE(QString strname)
{
    QString strExt = strname.mid(strname.lastIndexOf('.') + 1);
    if (strExt == "mp4" || strExt == "MP4")
    {
        return FILE_TYPE_VIDEO;
    }
    else if (strExt == "jpg" || strExt == "jpeg")
    {
        return FILE_TYPE_IMAGE;
    }
    else
    {
        return FILE_TYPE_UNKNOWN;
    }
}

// 将文件添加到list
bool CFileManager::addFile2List(MODULE_MESSAGE *pMsg)
{
    int idx = 0;
    QDateTime tmBegin;
    QDateTime tmEnd;
    BYTE channel;
    BYTE media_type;
    BYTE stream_code;
    BYTE storage_type;

    // 通道号 (索引+1)
    channel = pMsg->szBuff[idx++];
    // 资源类型
    media_type = pMsg->szBuff[idx++];
    // 码流类型
    stream_code = pMsg->szBuff[idx++];
    // 存储器类型
    storage_type = pMsg->szBuff[idx++];

    uint64_t tm;
    memcpy(&tm, pMsg->szBuff, sizeof(tm));
    idx += sizeof(tm);
    tmBegin.fromTime_t(tm);
    memcpy(&tm, pMsg->szBuff, sizeof(tm));
    idx += sizeof(tm);
    tmEnd.fromTime_t(tm);

    QString path_name = (char *)(pMsg->szBuff + idx);

    QFile file(path_name);
    QString data_path = getDataPath();
    int path_idx = path_name.indexOf(data_path);
    if (file.exists() && path_idx >= 0)
    {
        CCamFileNode file_node;
        file_node.bFileLocked = false;
        file_node.fileName = path_name.replace(data_path, "");
        file_node.tmBegin = tmBegin;
        file_node.tmEnd = tmEnd;
        file_node.channel = channel;
        file_node.media_type = media_type;
        file_node.stream_code = stream_code;
        file_node.storage_type = storage_type;
        file_node.alarms = 0;

        FILE_TYPE type = getFileTYPE(file_node.fileName);
        switch (type)
        {
        case FILE_TYPE_VIDEO:
            m_vidFileMng.addFileNode(file_node);
            // 保存文件列表
            m_vidFileMng.saveFileList();
            break;
        case FILE_TYPE_IMAGE:
            m_imgFileMng.addFileNode(file_node);
            // 保存文件列表
            m_imgFileMng.saveFileList();
            break;
        case FILE_TYPE_UNKNOWN:
            break;
        }
    }

    // 消息过来的场景下，一般都是视频转储产生，需要检查剩余空间
    managerDiskSpace();
}

// 删除文件
bool CFileManager::delFileNodeByName(QString file_name)
{
    // file name
    QString path_name = getDataPath();
    bool bSuccess = false;

    FILE_TYPE type = getFileTYPE(file_name);
    switch (type)
    {
    case FILE_TYPE_VIDEO:
        bSuccess = m_vidFileMng.delFileNode(file_name);
        break;
    case FILE_TYPE_IMAGE:
        bSuccess = m_imgFileMng.delFileNode(file_name);
        break;
    case FILE_TYPE_UNKNOWN:
        break;
    }

    return bSuccess;
}

// 删除文件操作
void CFileManager::ProcDelFiles(MODULE_MESSAGE *pMsg)
{
    // 锁定文件操作
    // 检查扩展长度，如果文件名过长，文件名内容会保存在扩展长度中
    // 取出文件名
    // 遍历需要删除的文件/文件夹
    // 删除文件/文件夹
    // 解除文件操作锁定
    // 回消息反馈操作结果
}

// 服务器要求获取录像文件的文件列表
bool CFileManager::getVideoRecordList(QVariantMap &prop_arr)
{
    QDateTime tmBegin, tmEnd;
    // beginTime	String	开始时间；时间戳，精确到毫秒
    QString beginTime = prop_arr["beginTime"].toString();
    // endTime	String	结束时间；时间戳，精确到毫秒
    QString endTime = prop_arr["endTime"].toString();
    tmBegin = QDateTime::fromString(beginTime, "yyyy-MM-dd hh:mm:ss");
    tmEnd = QDateTime::fromString(endTime, "yyyy-MM-dd hh:mm:ss");

    QString strList = m_vidFileMng.getFileList(tmBegin, tmEnd);
    prop_arr["msgName"] = "onVideoRecordList";
    prop_arr["fileList"] = strList;

    if (m_pWebSock)
    {
        m_pWebSock->sendMessage(prop_arr);
    }
}

// 删除文件
bool CFileManager::fileDelete(QVariantMap &prop_arr)
{
    bool bSuccess = true;
    // filepath	Array	需要删除的文件的列表
    QString filepath_arr = prop_arr["filepath"].toString();
    QStringList file_lst = filepath_arr.split(',');

    // delete file
    foreach (QString file_path, file_lst)
    {
        // 特殊处理,避免根目录或输入错误
        if (file_path.at(0) == '/')
        {
            file_path = file_path.mid(1);
        }
        bSuccess = delFileNodeByName(file_path);
    }

    onFileDelete(prop_arr, (int)bSuccess);

    return bSuccess;
}

// 返回删除文件结果
bool CFileManager::onFileDelete(QVariantMap &prop_arr, int result)
{
    assert(m_pWebSock);
    QVariantMap::Iterator it = prop_arr.find("msgName");
    if (it != prop_arr.end())
    {
        prop_arr["msgName"] = "onFileDelete";
    }
    else
    {
        prop_arr.insert("msgName", "onFileDelete");
    }

    // result	String	表示文件删除操作的执行结果，1表示成功； 0表示失败
    prop_arr.insert("result", result);

    if (!m_pWebSock->sendMessage(prop_arr))
    {
        return false;
    }
    return true;
}
