#ifndef CGB28181CLIENT_H
#define CGB28181CLIENT_H
#include "CSipEventHandle.h"
#include "CSipMessageHandle.h"
#include "CSipDevice.h"
#include <QQueue>
#include <QMutex>

namespace GB28181
{
    class CGB28181Client : public CSipEventHandle
    {
    public:
        CGB28181Client();
        virtual ~CGB28181Client();

        bool Initialize();

        /*******************************************
        函数功能:加载配置项，虚函数，待子类实现
        入口参数: void
        出口参数: true 成功
        备    注:
        *******************************************/
        virtual bool loadConfig() = 0;

    protected:
        /*******************************************
        函数功能:通知视频模块开始推流
        入口参数: int cam_idx,  相机索引
                string ip,  对端IP
                int port,   对端端口
                string ssrc ssrc码
        出口参数: 
        备    注:
        *******************************************/
        virtual void notifyStartVideo(int cam_idx, string ip, int port, string ssrc) = 0;

        /*******************************************
        函数功能:通知视频模块停止推流
        入口参数: 
        出口参数: 
        备    注:
        *******************************************/
        virtual void notifyStopVideo(int cam_idx) = 0;

    protected:
        /*******************************************
        函数功能:注册成功处理
        入口参数: rid 注册ID
        出口参数: 0 成功
        备    注:
        *******************************************/
        virtual int OnRegisterSuccess(int rid);

        /*******************************************
        函数功能:PTZ控制
        入口参数: control_cmd_t &ctrlcmd,    控制字
                 string &devid              设备ID
        出口参数: 0 成功
        备    注:
        *******************************************/
        virtual int HandlePTZControl(DEVICE_CONTROL_CMD &ctrlcmd, string &devid);

        /*******************************************
        函数功能:设备控制
        入口参数: rid 注册ID
        出口参数: 0 成功
        备    注:
        *******************************************/
        virtual int HandleDeviceControl(MANSCDP_DEV_CTRL_CMD cmd, string &devid, MANSCDP_SWITCH_STATUS onoff);

        /*******************************************
        函数功能:PTZ控制
        入口参数: control_cmd_t &ctrlcmd,    控制字
                 string &devid              设备ID
        出口参数: 0 成功
        备    注:
        *******************************************/
        virtual int onMsgResponse(string &reqid, int status_code);

        /*******************************************
        函数功能:创建sip消息实例
        入口参数: 
        出口参数: osip_message_t * 消息指针
        备    注:
        *******************************************/
        virtual osip_message_t *createMsg();

        /*******************************************
        函数功能:发送请求
        入口参数:string context_type,  类型
                string msg            消息内容
        出口参数: 0 成功
        备    注:
        *******************************************/
        virtual int sendRequest(string context_type, string message);

        /*******************************************
        函数功能:向sip终端发送消息/响应
        入口参数: eXosip_event_t *exevent, 
                string context_type,  类型
                string msg            消息内容
        出口参数: 0 成功
        备    注:
        *******************************************/
        virtual int sendResponse(eXosip_event_t *exevent, string context_type, string msg);

        /*******************************************
        函数功能:向sip终端发送简单响应
        入口参数: rid 注册ID
        出口参数: 0 成功
        备    注:
        *******************************************/
        int sendResponseOk(eXosip_event_t *exevent);

        /*******************************************
        函数功能:发送保活消息
        入口参数:
        出口参数: 0 成功
        备    注:
        *******************************************/
        virtual int keepAlive();

        /*******************************************
        函数功能:超时检查,主要是心跳超时检查
        入口参数:
        出口参数: 0 成功
        备    注:
        *******************************************/
        virtual int checkTimeOut();

        /*******************************************
        函数功能:注册在线信息
        入口参数:
        出口参数: 0 成功
        备    注:
        *******************************************/
        int RegisterOnline();

    protected:
        /* INVITE related events within calls */
        virtual int onExosipCall_invite(const SIP_ENENT_PTR &event);

        //结束视频通话
        virtual int onExosipCall_closed(const SIP_ENENT_PTR &event);

    protected:
        /*******************************************
        函数功能:开始SIP侦听
        入口参数: rid 注册ID
        出口参数: 0 成功
        备    注: 需要等待初始化完成
        *******************************************/
        bool startListen(const std::string &user_agent);

        /*******************************************
        函数功能:接收sip消息事件函数
        入口参数:
        出口参数: 成功返回0
        备    注:
        *******************************************/
        int recvEvents();

        /*******************************************
        函数功能:处理sip事件
        入口参数:
        出口参数:成功返回0
        备    注:和接收函数处在两个不同的任务防止接收任务被堵塞
        *******************************************/
        int processEvents();

        /*******************************************
        函数功能: 创建事件节点
        入口参数:eXosip_t *exosip_context,  会话实体指针
                eXosip_event_t *exosip_event 事件指针
        出口参数:SIP_ENENT_PTR 事件指针
        备    注: 在接收函数中封装的事件节点信息，方便和处理函数通信
        *******************************************/
        SIP_ENENT_PTR createEvent(eXosip_t *exosip_context, eXosip_event_t *exosip_event);

        /*******************************************
        函数功能: 获取消息SN号
        入口参数:
        出口参数:SN Number
        备    注: 
        *******************************************/
        int getIncNumber() { return m_incNumber++; }

    protected:
        eXosip_t *m_excontext;
        uint64_t m_eventId;

        std::string m_remoteSip;
        std::string m_localSip;
        QQueue<SIP_ENENT_PTR> m_eventQueue;
        QMutex m_mtxQue;

        bool m_bRegok;
        int m_incNumber;
    };
}
#endif // CGB28181CLIENT_H
