#include "CGB28181Module.h"
#include "../socket/TcpServer.h"
#include "../base_module/factory.h"
#include "timer/CTimerManager.h"
#include "config/CConfigManager.h"
#include <assert.h>

const QString GB28181_DB_NAME = "gb28181";

REGISTER_MODULE(CGB28181Module, "CGB28181Module")

LOG_DEFINITION(CGB28181Module)

CGB28181Module::CGB28181Module(QObject *parent)
    : m_pDbGB28181(nullptr)
{
    LOG_CREATE(CGB28181Module, "GB28181");
}

CGB28181Module::~CGB28181Module()
{
}

bool CGB28181Module::loadConfig()
{
    m_pConfigMng = dynamic_cast<CConfigManager *>(QueryPlugin("CConfigManager"));
    if (m_pConfigMng)
    {
        m_pDbGB28181 = m_pConfigMng->getDatabase(MODULE_GB28181_MNG, GB28181_DB_NAME);
        if (!m_pDbGB28181)
        {
            createDatebase();
        }

        restoreFromDB(m_pDbGB28181);
    }
    return true;
}

// 创建数据库
bool CGB28181Module::createDatebase()
{
    LOG_TRACE(CGB28181Module, "m_pDbGB28181 is not exist, create Datebase!");
    if (m_pConfigMng)
    {
        m_pDbGB28181 = m_pConfigMng->regDatabase(MODULE_GB28181_MNG, GB28181_DB_NAME);
        m_pDbGB28181->regKey("serv_id", m_ServerID);
        m_pDbGB28181->regKey("serv_ip", m_serverIP);
        m_pDbGB28181->regKey("serv_port", m_serverPort);
        m_pDbGB28181->regKey("serv_pwd", m_ServerPwd);

        m_pDbGB28181->regKey("local_id", m_DeviceID);
        m_pDbGB28181->regKey("local_port", m_localPort);
        m_pDbGB28181->regKey("reg_interval", m_regInterval);

        m_pDbGB28181->regKey("cam1_id", m_cam1ItemID);
        m_pDbGB28181->regKey("cam2_id", m_cam2ItemID);
        m_pDbGB28181->regKey("cam3_id", m_cam3ItemID);
        m_pDbGB28181->regKey("cam4_id", m_cam4ItemID);

        m_pDbGB28181->regKey("stream_code", m_streamCode);

        m_pDbGB28181->save();
    }
    else
    {
        // log_err();
    }
    return true;
}

// 从数据库恢复
bool CGB28181Module::restoreFromDB(CDatebase *pDatebase)
{
    LOG_TRACE(CGB28181Module, "restore Datebase!");
    m_serverIP = pDatebase->getValue("serv_ip").toString();
    m_ServerID = pDatebase->getValue("serv_id").toString();
    m_serverPort = pDatebase->getValue("serv_port").toString();
    m_ServerPwd = pDatebase->getValue("serv_pwd").toString();

    m_DeviceID = pDatebase->getValue("local_id").toString();
    m_localPort = pDatebase->getValue("local_port").toString();
    m_regInterval = pDatebase->getValue("reg_interval").toString();

    m_cam1ItemID = pDatebase->getValue("cam1_id").toString();
    m_cam2ItemID = pDatebase->getValue("cam2_id").toString();
    m_cam3ItemID = pDatebase->getValue("cam3_id").toString();
    m_cam4ItemID = pDatebase->getValue("cam4_id").toString();
    m_streamCode = pDatebase->getValue("stream_code").toInt();

    return true;
}

//初始化操作
bool CGB28181Module::Initialize()
{
    LOG_TRACE(CGB28181Module, "Initialize!");
    // 客户端初始化
    if (!CGB28181Client::Initialize())
    {
        LOG_ERROR(CGB28181Module, "CGB28181Client::Initialize FAILED!");
        return false;
    }

    //起主消息处理线程
    STARTUP_TASK(TaskMain)

    return true;
}

bool CGB28181Module::RegistMessage()
{
    m_pTcpModule = dynamic_cast<ITcpModule *>(QueryPlugin("TcpModule"));
    if (m_pTcpModule)
    {
        m_pTcpModule->RegistMessageCallback(this, MODULE_GB28181_MNG);
    }
    return true;
}

// reg timer
void CGB28181Module::registTimer()
{
    // 定时器模块
    m_pTimerMng = dynamic_cast<CTimerManager *>(::QueryPlugin("CTimerManager"));
    if (m_pTimerMng)
    {
        m_pTimerMng->RegTimer(MODULE_GB28181_MNG, TIMER_GB28181_HEART_BEAT, 50);
        m_pTimerMng->RegTimer(MODULE_GB28181_MNG, TIMER_GB28181_TIMEOUT_CHK, 50);
    }
    else
    {
        LOG_ERROR(CGB28181Module, "m_pTimerMng is nullptr!");
    }
}

void CGB28181Module::saveConfig()
{
    LOG_TRACE(CGB28181Module, "save config!");
    if (m_pDbGB28181)
    {
        m_pDbGB28181->setValue("serv_id", m_ServerID);
        m_pDbGB28181->setValue("serv_ip", m_serverIP);
        m_pDbGB28181->setValue("serv_port", m_serverPort);
        m_pDbGB28181->setValue("serv_pwd", m_ServerPwd);

        m_pDbGB28181->setValue("local_id", m_DeviceID);
        m_pDbGB28181->setValue("local_port", m_localPort);
        m_pDbGB28181->setValue("reg_interval", m_regInterval);

        m_pDbGB28181->setValue("cam1_id", m_cam1ItemID);
        m_pDbGB28181->setValue("cam2_id", m_cam2ItemID);
        m_pDbGB28181->setValue("cam3_id", m_cam3ItemID);
        m_pDbGB28181->setValue("cam4_id", m_cam4ItemID);

        m_pDbGB28181->setValue("stream_code", m_streamCode);

        m_pDbGB28181->save();
    }
}

void CGB28181Module::TaskMain(void *pBuff)
{
    (void)pBuff;
    setCurrentTaskName("tskGB28181");
    MODULE_MESSAGE *pMsg = nullptr;
    RegistMessage();
    registTimer();
    waitForStart();

    STARTUP_TASK(tskReceiveEvents);
    STARTUP_TASK(tskProcessEvents);

    while (!this->isShutdown())
    {
        if (GetModuleMessage(pMsg)) // get msg from message queue
        {
            switch (pMsg->iCmd)
            {
            case TIMER_GB28181_HEART_BEAT:
                procHeartBeat(pMsg);
                break;
            case TIMER_GB28181_TIMEOUT_CHK:
                checkTimeOut();
                break;
            case CMD_CAM_ONLINE_OFFLINE:
                procCameraOnline(pMsg);
                break;
            default:
                break;
            }

            SAFE_DELETE(pMsg);
        }
    }
}

// SIP事件接收任务
void CGB28181Module::tskReceiveEvents(void *pBuff)
{
    while (!isShutdown())
    {
        recvEvents();
        mSleep(100);
    }
}
// SIP事件处理任务
void CGB28181Module::tskProcessEvents(void *pBuff)
{
    while (!isShutdown())
    {
        processEvents();
        mSleep(100);
    }
}

// 处理摄像头上线/离线事件
void CGB28181Module::procCameraOnline(MODULE_MESSAGE *pMsg)
{
    int idx = 0;
    int online = 0;
    int cam_idx = 0;
    memcpy(&online, pMsg->szBuff, sizeof(bool));
    idx += sizeof(bool);
    memcpy(&cam_idx, pMsg->szBuff + idx, sizeof(int));
    idx += sizeof(int);
    setItemOnline(online, cam_idx);
}

void CGB28181Module::procHeartBeat(MODULE_MESSAGE *pMsg)
{
    (void)pMsg;
    static int timeoutcnt = 0; ///<超时3次未回复 重新注册

    if (m_bRegok)
    {
        if (0 != keepAlive())
        {
            timeoutcnt++;
        }

        if (timeoutcnt >= 3)
        {
            m_bRegok = false;
        }
    }
    else
    {
        RegisterOnline();
    }
}

/*******************************************
函数功能:通知视频模块停止推流
入口参数:
出口参数:
备    注:
*******************************************/
void CGB28181Module::notifyStopVideo(int cam_idx)
{
    if (m_pTcpModule)
    {
        //给应用层发送消息,同步传感器信息
        MODULE_MESSAGE msg;
        msg.dst_id = MODULE_CAMERA_MNG;
        msg.iCmd = CMD_STOP_GB28181_VIDEO;

        memcpy(msg.szBuff, &cam_idx, sizeof(int));

        //模块间发送消息
        m_pTcpModule->SendMsg(msg, sizeof(msg));
    }
}
/*******************************************
函数功能:通知视频模块开始推流
入口参数: int cam_idx,  相机索引
        string ip,  对端IP
        int port,   对端端口
        string ssrc ssrc码
出口参数:
备    注:
*******************************************/
void CGB28181Module::notifyStartVideo(int cam_idx, string rtp_ip, int port, string ssrc)
{
    if (m_pTcpModule)
    {
        //给应用层发送消息,同步传感器信息
        MODULE_MESSAGE msg;
        msg.dst_id = MODULE_CAMERA_MNG;
        msg.iCmd = CMD_START_GB28181_VIDEO;
        int idx = 0;
        // 通信类型 int
        memcpy(msg.szBuff + idx, &cam_idx, sizeof(int));
        idx += sizeof(int);

        // 码流 int
        memcpy(msg.szBuff + idx, &m_streamCode, sizeof(int));
        idx += sizeof(int);

        // 端口 int
        memcpy(msg.szBuff + idx, &port, sizeof(int));
        idx += sizeof(int);

        // 对端IP 20
        strncpy((char *)(msg.szBuff + idx), rtp_ip.c_str(), 20);
        idx += 20;

        // SSRC
        strncpy((char *)(msg.szBuff + idx), rtp_ip.c_str(), 64);
        idx += 64;

        //模块间发送消息
        m_pTcpModule->SendMsg(msg, sizeof(msg));
    }
}
