#ifndef CGB28181MODULE_H
#define CGB28181MODULE_H
#include "base_module.h"
#include <QObject>
#include "CGB28181Client.h"
#include "./log/CLogManager.h"

using namespace GB28181;
class ITcpModule;
class CTimerManager;
class CDatebase;
class CConfigManager;

class CGB28181Module : public CGB28181Client, public IModule
{
    Q_OBJECT
public:
    LOG_DECLARE()
    MODULE_NAME(CGB28181Module)
    CGB28181Module(QObject *parent = nullptr);
    ~CGB28181Module();

    //初始化操作
    virtual bool Initialize();
    //模块注册接口
    bool RegistMessage();
    // reg timer
    void registTimer();

    void saveConfig();

protected:
    //主消息任务，主要用于和其他模块之间的消息通讯
    THREAD_DEFINITION(CGB28181Module, TaskMain)
    // SIP 事件接收任务
    THREAD_DEFINITION(CGB28181Module, tskReceiveEvents)
    // SIP 事件处理任务
    THREAD_DEFINITION(CGB28181Module, tskProcessEvents)

    //主消息任务，主要用于和其他模块之间的消息通讯
    virtual void TaskMain(void *pBuff);
    // SIP事件接收任务
    virtual void tskReceiveEvents(void *pBuff);
    // SIP事件处理任务
    virtual void tskProcessEvents(void *pBuff);

    // 导入配置
    virtual bool loadConfig();

    // 处理心跳事件
    void procHeartBeat(MODULE_MESSAGE *pMsg);

    // 处理摄像头上线/离线事件
    void procCameraOnline(MODULE_MESSAGE *pMsg);

    // 保存配置文件
    bool restoreFromDB(CDatebase *pDatebase);

    // 导入配置文件
    bool createDatebase();

    /*******************************************
    函数功能:通知视频模块开始推流
    入口参数: int cam_idx,  相机索引
            string ip,  对端IP
            int port,   对端端口
            string ssrc ssrc码
    出口参数: 
    备    注:
    *******************************************/
    virtual void notifyStartVideo(int cam_idx, string ip, int port, string ssrc);

    /*******************************************
    函数功能:通知视频模块停止推流
    入口参数: 
    出口参数: 
    备    注:
    *******************************************/
    virtual void notifyStopVideo(int cam_idx);

protected:
    ITcpModule *m_pTcpModule;
    CTimerManager *m_pTimerMng;

    // 本地用户数据库
    CDatebase *m_pDbGB28181;
    CConfigManager *m_pConfigMng;
};

#endif // CGB28181MODULE_H
