#ifndef CSIPDEFS_H
#define CSIPDEFS_H
#include "eXosip2/eXosip.h"
#include "eXosip2/eX_setup.h"
#include <vector>
#include <string.h>
#include <functional>
#include <memory>

namespace GB28181
{
    struct SIP_ENENT;
    typedef std::shared_ptr<SIP_ENENT> SIP_ENENT_PTR;
    typedef std::function<int(const SIP_ENENT_PTR &)> event_proc;

    struct exosip_locker
    {
        explicit exosip_locker(eXosip_t *e)
        {
            m_excontext = e;
            eXosip_lock(m_excontext);
        }
        ~exosip_locker()
        {
            eXosip_unlock(m_excontext);
        }
        eXosip_t *m_excontext;
    };

    struct SIP_ENENT
    {
        int value;
        const char *name;
        event_proc proc;
        eXosip_t *excontext;
        eXosip_event_t *exevent;
        uint64_t id;

        SIP_ENENT() = default;
    };

    typedef struct
    {
        const char *name;
        event_proc proc;
    } EventNameProcPair;

    enum MANSCDP_CMD_CATEGORY
    {
        MANSCDP_CMD_CATEGORY_CONTROL,
        MANSCDP_CMD_CATEGORY_QUERY,
        MANSCDP_CMD_CATEGORY_NOTIFY,
        MANSCDP_CMD_CATEGORY_RESPONSE,

        MANSCDP_CMD_CATEGORY_MAX,
        MANSCDP_CMD_CATEGORY_UNKNOWN = MANSCDP_CMD_CATEGORY_MAX
    };

    enum MANSCDP_CMD_TYPE
    {
        //< Control
        MANSCDP_CONTROL_CMD_DEVICE_CONTROL, ///<设备控制
        MANSCDP_CONTROL_CMD_DEVICE_CONFIG,  ///<设备配置

        //< Query
        MANSCDP_QUERY_CMD_DEVICE_STATUS,   ///<设备控制
        MANSCDP_QUERY_CMD_CATALOG,         ///<设备目录查询
        MANSCDP_QUERY_CMD_DEVICE_INFO,     ///<设备信息查询
        MANSCDP_QUERY_CMD_RECORD_INFO,     ///<文件目录检索
        MANSCDP_QUERY_CMD_ALARM,           ///<报警查询
        MANSCDP_QUERY_CMD_CONFIG_DOWNLOAD, ///<设备配置查询
        MANSCDP_QUERY_CMD_PRESET_QUERY,    ///<预置位查询
        MANSCDP_QUERY_CMD_MOBILE_POSITION, ///<移动设备位置数据查询

        //< Notify
        MANSCDP_NOTIFY_CMD_KEEPALIVE,       ///<设备状态信息报送，保活
        MANSCDP_NOTIFY_CMD_ALARM,           ///<报警通知
        MANSCDP_NOTIFY_CMD_MEDIA_STATUS,    ///<媒体通知
        MANSCDP_NOTIFY_CMD_BROADCASE,       ///<语音广播通知
        MANSCDP_NOTIFY_CMD_MOBILE_POSITION, ///<移动设备位置通知

        MANSCDP_CMD_TYPE_MAX,
        MANSCDP_CMD_TYPE_UNKNOWN = MANSCDP_CMD_TYPE_MAX
    };

    enum MANSCDP_DEV_CTRL_CMD
    {
        PTZCmd = 1,
        TeleBoot,
        RecordCmd,
        GuardCmd,
        AlarmCmd = 5,
        IFrameCmd,
        DragZoomIn,
        DragZoomOut,
        HomePosition
    };

    enum MANSCDP_SWITCH_STATUS
    {
        SWITCH_ON, ///< 状态切换  开关
        SWITCH_OFF
    };

    enum MANSCDP_DEV_CFG_CMD
    {
        BasicParam = 1,
        SVACEncodeConfig,
        SVACDecodeConfig
    };

    typedef std::vector<MANSCDP_DEV_CTRL_CMD> VCT_DEV_CTRL_CMD;
    typedef std::vector<MANSCDP_DEV_CFG_CMD> VCT_DEV_CFG_CMD;
    typedef std::vector<std::string> VCT_CFG_DLOAD_CMD;

    ///< MANSCDP xml消息 必选项如下
    /*
    * <?xml version="1.0"?>
    * <Query>
    * <CmdType>Catalog</CmdType>
    * <SN>27</SN>
    * <DeviceID>1234</DeviceID>
    * </Query>
    */
    struct MANSCDP_MSG_HEADER
    {
        MANSCDP_CMD_CATEGORY cmd_category;
        MANSCDP_CMD_TYPE cmd_type;
        std::string sn;
        std::string devid;
        VCT_DEV_CFG_CMD devcfg_subcmd;
        VCT_DEV_CTRL_CMD devctl_subcmd;
        VCT_CFG_DLOAD_CMD cfgdownload_subcmd;
    };

    ///< 云台 镜头变倍
    struct PTZ_CMD_ZOOM
    {
        enum PTZ_CTRL_TYPE
        {
            ZOOM_UNKNOWN = 0,
            ZOOM_OUT,
            ZOOM_IN,
        };
        PTZ_CTRL_TYPE cmdtype;
        uint8_t speed;
    };

    ///< 云台垂直方向控制
    struct PTZ_CMD_VERTICAL
    {
        enum PTZ_CTRL_TYPE
        {
            TILT_UNKNOWN = 0,
            TILT_UP,
            TILT_DOWN,
        };
        PTZ_CTRL_TYPE cmdtype;
        uint8_t speed;
    };

    ///< 云台水平控制方向
    struct PTZ_CMD_HORIZONTAL
    {
        enum PTZ_CTRL_TYPE
        {
            PAN_UNKNOWN = 0,
            PAN_LEFT,
            PAN_RIGHT,
        };
        PTZ_CTRL_TYPE cmdtype;
        uint8_t speed;
    };

    ///< FI指令 光圈
    struct FI_CMD_APERTURE
    {
        enum PTZ_CTRL_TYPE
        {
            IFIS_UNKNOWN = 0,
            IFIS_SHRINK,        ///<缩小
            IFIS_AMPLIFICATION, ///<放大
        };
        PTZ_CTRL_TYPE cmdtype;
        uint8_t speed;
    };

    ///< FI指令 聚焦 光圈
    struct FI_CMD_FOCUS
    {
        enum PTZ_CTRL_TYPE
        {
            FOCUS_UNKNOWN = 0,
            FOCUS_NEAR,
            FOCUS_FAR,
        };
        PTZ_CTRL_TYPE cmdtype;
        uint8_t speed;
    };

    ///< 预置位指令
    struct DEVICE_PRESET_CMD
    {
        enum PTZ_CTRL_TYPE
        {
            PRESET_UNKNOWN = 0,
            PRESET_SET,
            PRESET_CALL,
            PRESET_DELE
        };
        PTZ_CTRL_TYPE cmdtype;
        uint8_t index;
    };

    ///< 巡航指令
    struct DEVICE_PATROL_CMD
    {
        enum PTZ_CTRL_TYPE
        {
            PATROL_UNKNOWN = 0,
            PATROL_ADD,
            PATROL_DELE,
            PATROL_SET_SPEED,
            PATROL_SET_TIME, ///<设置停留时间
            PATROL_START,    ///<开始巡航
            PATROL_STOP
        };
        PTZ_CTRL_TYPE cmdtype;
        uint8_t patrol_id; ///<巡航组号
        uint8_t preset_id; ///<预置位号
        uint16_t value;    ///<数据，速度和停留时间使用
    };

    ///< 自动扫描指令
    struct DEVICE_SCAN_CMD
    {
        enum PTZ_CTRL_TYPE
        {
            SCAN_UNKNOWN = 0,
            SCAN_START,
            SCAN_STOP,
            SCAN_SET_LEFT_BOADER,
            SCAN_SET_RIGHT_BOADER,
            SCAN_SET_SPEED
        };
        PTZ_CTRL_TYPE cmdtype;
        uint8_t scan_id;
        uint16_t speed; ///< 设置scan速度使用
    };

    enum DEVICE_CONTROL_TYPE
    {
        CTRL_CMD_UNKNOWN = 0,
        PTZ_TYPE,    ///< PTZ控制
        FI_TYPE,     ///<光圈、聚焦控制
        PRESET_TYPE, ///<预置位
        PATROL_TYPE, ///<巡航
        SCAN_TYPE,   ///<扫描
        AUX_TYPE,    ///<辅助开关

        CONTROL_STOP ///<停止控制
    };

    struct DEVICE_CONTROL_CMD
    {
        uint8_t first_byte; ///< A5H
        uint8_t version;    ///< 版本号
        uint8_t check;      ///< 校验位

        DEVICE_CONTROL_TYPE ctrltype;

        struct
        {
            PTZ_CMD_HORIZONTAL ptz_pan;
            PTZ_CMD_VERTICAL ptz_tilt;
            PTZ_CMD_ZOOM ptz_zoom;
        };

        struct
        {
            FI_CMD_FOCUS fi_focus;
            FI_CMD_APERTURE fi_iris;
        };

        DEVICE_PRESET_CMD preset;

        DEVICE_PATROL_CMD patrol;

        DEVICE_SCAN_CMD autoscan;
    };
}
#endif // CSIPDEFS_H
