#include "CSipDevice.h"
#include <QDebug>
#include <gb28181/CGB28181Module.h>

namespace GB28181
{
    CSipDevice::CSipDevice()
        : m_localPort("5066"),
          m_serverPort("15060"),
          m_localIP("127.0.0.1"),
          m_serverIP("192.168.2.199"),
          m_DeviceID("34020000000000003001"),
          m_ServerID("34020000002000000001"),
          m_ServerPwd("12345678"),
          m_regInterval("3600"),
          m_streamCode(1)
    {
        // 测试代码,正式代码中需要注销
        // int cam_idx = 0;
        // SipItem item;
        // item.bOnline = true;
        // item.name = getNameByIndex(cam_idx);
        // item.cam_idx = cam_idx;
        // item.parent_id = m_ServerID;
        // item.code = getCode(m_DeviceID);
        // item.item_id = getItemidByIndex(cam_idx);
        // m_mapItems[item.item_id] = item;
        m_camItemID[0] = &m_cam1ItemID;
        m_camItemID[1] = &m_cam2ItemID;
        m_camItemID[2] = &m_cam3ItemID;
        m_camItemID[3] = &m_cam4ItemID;

        for (int i = 0; i < MAX_CAM_COUNT; i++)
        {
            *m_camItemID[i] = getItemidByIndex(i).c_str();
        }
    }

    QString CSipDevice::getItemID(int idx)
    {
        LOG_TRACE(CGB28181Module, "index=%d", idx);
        if (idx >= 0 && idx < MAX_CAM_COUNT)
        {
            return *m_camItemID[idx];
        }
        else
        {
            return "";
        }
    }

    //std::string CSipDevice::getServDomain()
    //{
    //    return getCode(m_ServerID);
    //}

    //void CSipDevice::setLocalIP(std::string ip)
    //{

    //}

    //void CSipDevice::setServIP(std::string ip)
    //{

    //}

    //void CSipDevice::setLocalPort(std::string port)
    //{

    //}

    //void CSipDevice::setServPort(std::string port)
    //{

    //}

    //void CSipDevice::setLocalID(std::string id)
    //{

    //}

    //void CSipDevice::setServID(std::string id)
    //{

    //}

    //void CSipDevice::setServPwd(std::string pwd)
    //{

    //}

    //void CSipDevice::setRegInterval(std::string interval)
    //{

    //}

    std::string CSipDevice::getCode(std::string id)
    {
        LOG_TRACE(CGB28181Module, "id=%s", id.c_str());
        char code[SIP_CODE_LEN + 1];
        memset(code, 0, sizeof(code));
        strncpy(code, id.c_str(), SIP_CODE_LEN);
        return code;
    }

    std::string CSipDevice::getNameByIndex(int cam_idx)
    {
        QString name;
        name.sprintf("Camera %02d", cam_idx + 1);
        return name.toStdString();
    }

    int CSipDevice::getIndexByName(std::string name)
    {
        int cam_idx = 0;
        QString cam_name = name.c_str();
        QStringList lst = cam_name.split(" ");

        // 合法解析取idx 非法的不做处理,直接返回默认的0号摄像头
        if (lst.size() == 2)
        {
            cam_idx = lst[1].toInt() - 1;
            if (cam_idx < 0 || cam_idx > 3)
            {
                cam_idx = 0;
            }
        }

        return cam_idx;
    }

    // 摄像头ID转索引
    int CSipDevice::getCamIndexByID(std::string id)
    {
        int cam_idx = 0;
        auto it = m_mapItems.find(id);
        if (it != m_mapItems.end())
        {
            cam_idx = it.value().cam_idx;
        }
        return cam_idx;
    }

    std::string CSipDevice::getItemidByIndex(int cam_idx)
    {
        char item_id[SIP_ID_LEN + 1] = {0};
        strncpy(item_id, m_DeviceID.toStdString().c_str(), SIP_ID_LEN);
        int idx = 0;
        item_id[SIP_CODE_LEN + idx++] = char('9');
        item_id[SIP_CODE_LEN + idx++] = char('9');
        item_id[SIP_CODE_LEN + idx++] = char('1' + cam_idx);
        return item_id;
    }

    void CSipDevice::getItemsInfo(std::stringstream &ss)
    {
        // 没有ITEM的情况下直接返回,不填充设备列表
        if (m_mapItems.size() == 0)
        {
            return;
        }

        ss << "<SumNum>" << m_mapItems.size() << "</SumNum>\n";
        ss << "<DeviceList Num='" << m_mapItems.size() << "'>\n";
        foreach (SipItem item, m_mapItems)
        {
            ss << "<Item>\n";
            ss << "<DeviceID>" << item.item_id << "</DeviceID>\n";
            ss << "<Name>" << item.name << "</Name>\n";
            ss << "<Manufacturer>Hikvision</Manufacturer>\n";
            ss << "<Model>IP Camera</Model>\n";
            ss << "<Owner>Owner</Owner>\n";
            ss << "<CivilCode>" << item.code << "</CivilCode>\n";
            ss << "<Address>Address</Address>\n";
            ss << "<Parental>0</Parental>\n";
            ss << "<ParentID>" << item.parent_id << "</ParentID>\n";
            ss << "<SafetyWay>0</SafetyWay>\n";
            ss << "<RegisterWay>1</RegisterWay>\n";
            ss << "<Secrecy>0</Secrecy>\n";
            std::string status = item.bOnline ? "ON" : "OFF";
            ss << "<Status>" << status << "</Status>\n";
            ss << "</Item>\n";
        }
        ss << "</DeviceList>\n";
    }

    void CSipDevice::setItemOnline(bool online, int cam_idx)
    {
        std::string item_id = getItemidByIndex(cam_idx);
        // 先查找
        auto it = m_mapItems.find(item_id);

        // 找到的情况下,只需要设置状态
        if (it != m_mapItems.end())
        {
            it.value().bOnline = online;
        }
        // 没有找到则需要创建新的对象
        else
        {
            SipItem item;
            item.bOnline = online;
            item.name = getNameByIndex(cam_idx);
            item.cam_idx = cam_idx;
            item.parent_id = m_ServerID.toStdString();
            item.code = getCode(m_DeviceID.toStdString());
            item.item_id = getItemidByIndex(cam_idx);
            m_mapItems[item.item_id] = item;
        }
    }

    std::string CSipDevice::getCatalog(std::string sn)
    {
        std::stringstream ss;
        ss << "<?xml version='1.0' encoding='GB2312'?>\n";
        ss << "<Response>\r\n";
        ss << "<CmdType>Catalog</CmdType>\r\n";
        ss << "<SN>" << sn << "</SN>\r\n";
        ss << "<DeviceID>" << m_DeviceID.toStdString() << "</DeviceID>\r\n";
        getItemsInfo(ss);
        ss << "</Response>\n";
        qDebug() << ss.str().c_str();
        return ss.str();
    }

}
