#ifndef CCLIENTDEVICE_H
#define CCLIENTDEVICE_H
#include <string>
#include <sstream>
#include <QMap>
#include <QString>
#include <QObject>

namespace GB28181
{

#define SIP_ID_LEN 20
#define SIP_CODE_LEN 10
#define MAX_CAM_COUNT 4

    struct SipItem
    {
        SipItem() {}
        bool bOnline;
        int cam_idx;
        std::string name;
        std::string item_id;
        std::string code;
        std::string parent_id;
    };

    typedef QMap<std::string, SipItem> SIP_ITEM_MAP;
    class CSipDevice : public QObject
    {
        Q_OBJECT
        Q_PROPERTY(QString local_ip MEMBER m_localIP)
        Q_PROPERTY(QString serv_ip MEMBER m_serverIP)
        Q_PROPERTY(QString local_port MEMBER m_localPort)
        Q_PROPERTY(QString serv_port MEMBER m_serverPort)
        Q_PROPERTY(QString local_id MEMBER m_DeviceID)
        Q_PROPERTY(QString serv_id MEMBER m_ServerID)
        Q_PROPERTY(QString domain MEMBER m_Domain)
        Q_PROPERTY(QString serv_pwd MEMBER m_ServerPwd)
        Q_PROPERTY(QString reg_interval MEMBER m_regInterval)

        Q_PROPERTY(QString cam1_id MEMBER m_cam1ItemID)
        Q_PROPERTY(QString cam2_id MEMBER m_cam2ItemID)
        Q_PROPERTY(QString cam3_id MEMBER m_cam3ItemID)
        Q_PROPERTY(QString cam4_id MEMBER m_cam4ItemID)
        Q_PROPERTY(int stream_code MEMBER m_streamCode)

    public:
        CSipDevice();

        //获取编码信息
        std::string getCode(std::string id);

        // 获取item ID
        QString getItemID(int idx);

    protected:
        //
        void setItemOnline(bool online, int cam_idx);

    protected:
        //    std::string getDeviceID(){return m_DeviceID;}

        std::string getCatalog(std::string sn);

        void getItemsInfo(std::stringstream &ss);

        //获取ITEM ID信息
        std::string getItemidByIndex(int cam_idx);

        // 摄像头索引转摄像头名称
        std::string getNameByIndex(int cam_idx);

        // 摄像头名称转索引
        int getIndexByName(std::string name);

        // 摄像头ID转索引
        int getCamIndexByID(std::string id);

    protected:
        QString m_localIP;
        QString m_serverIP;
        QString m_localPort;
        QString m_serverPort;
        QString m_DeviceID;
        QString m_ServerID;
        QString m_Domain;
        QString m_ServerPwd;
        QString m_regInterval;

        QString m_cam1ItemID;
        QString m_cam2ItemID;
        QString m_cam3ItemID;
        QString m_cam4ItemID;

        int m_streamCode;

        QString *m_camItemID[MAX_CAM_COUNT];

        SIP_ITEM_MAP m_mapItems;
    };
}
#endif // CCLIENTDEVICE_H
