#include "CSipEventHandle.h"
#include <string.h>
#include <QDebug>

namespace GB28181
{
#define CALLBACK_TEMPLATE(F) (std::bind(&CSipEventHandle::F, this, std::placeholders::_1))

    CSipEventHandle::CSipEventHandle()
    {
        ///初始化Map表
        EventNameProcPair eventProcTable[EXOSIP_EVENT_COUNT] = {
            {"EXOSIP_REGISTRATION_SUCCESS", CALLBACK_TEMPLATE(onExosipRegistration_success)},
            {"EXOSIP_REGISTRATION_FAILURE", CALLBACK_TEMPLATE(onExosipRegistration_failure)},
            {"EXOSIP_CALL_INVITE", CALLBACK_TEMPLATE(onExosipCall_invite)},
            {"EXOSIP_CALL_REINVITE", CALLBACK_TEMPLATE(onExosipCall_reinvite)},
            {"EXOSIP_CALL_NOANSWER", CALLBACK_TEMPLATE(onExosipCall_noanswer)},
            {"EXOSIP_CALL_PROCEEDING", CALLBACK_TEMPLATE(onExosipCall_proceeding)},
            {"EXOSIP_CALL_RINGING", CALLBACK_TEMPLATE(onExosipCall_ringing)},
            {"EXOSIP_CALL_ANSWERED", CALLBACK_TEMPLATE(onExosipCall_answered)},
            {"EXOSIP_CALL_REDIRECTED", CALLBACK_TEMPLATE(onExosipCall_redirected)},
            {"EXOSIP_CALL_REQUESTFAILURE", CALLBACK_TEMPLATE(onExosipCall_requestfailure)},
            {"EXOSIP_CALL_SERVERFAILURE", CALLBACK_TEMPLATE(onExosipCall_serverfailure)},
            {"EXOSIP_CALL_GLOBALFAILURE", CALLBACK_TEMPLATE(onExosipCall_globalfailure)},
            {"EXOSIP_CALL_ACK", CALLBACK_TEMPLATE(onExosipCall_ack)},
            {"EXOSIP_CALL_CANCELLED", CALLBACK_TEMPLATE(onExosipCall_cancelled)},
            {"EXOSIP_CALL_MESSAGE_NEW", CALLBACK_TEMPLATE(onExosipCall_message_new)},
            {"EXOSIP_CALL_MESSAGE_PROCEEDING", CALLBACK_TEMPLATE(onExosipCall_message_proceeding)},
            {"EXOSIP_CALL_MESSAGE_ANSWERED", CALLBACK_TEMPLATE(onExosipCall_message_answered)},
            {"EXOSIP_CALL_MESSAGE_REDIRECTED", CALLBACK_TEMPLATE(onExosipCall_message_redirected)},
            {"EXOSIP_CALL_MESSAGE_REQUESTFAILURE", CALLBACK_TEMPLATE(onExosipCall_message_requestfailure)},
            {"EXOSIP_CALL_MESSAGE_SERVERFAILURE", CALLBACK_TEMPLATE(onExosipCall_message_serverfailure)},
            {"EXOSIP_CALL_MESSAGE_GLOBALFAILURE", CALLBACK_TEMPLATE(onExosipCall_message_globalfailure)},
            {"EXOSIP_CALL_CLOSED", CALLBACK_TEMPLATE(onExosipCall_closed)},
            {"EXOSIP_CALL_RELEASED", CALLBACK_TEMPLATE(onExosipCall_released)},
            {"EXOSIP_MESSAGE_NEW", CALLBACK_TEMPLATE(onExosipMessage_new)},
            {"EXOSIP_MESSAGE_PROCEEDING", CALLBACK_TEMPLATE(onExosipMessage_proceeding)},
            {"EXOSIP_MESSAGE_ANSWERED", CALLBACK_TEMPLATE(onExosipMessage_answered)},
            {"EXOSIP_MESSAGE_REDIRECTED", CALLBACK_TEMPLATE(onExosipMessage_redirected)},
            {"EXOSIP_MESSAGE_REQUESTFAILURE", CALLBACK_TEMPLATE(onExosipMessage_requestfailure)},
            {"EXOSIP_MESSAGE_SERVERFAILURE", CALLBACK_TEMPLATE(onExosipMessage_serverfailure)},
            {"EXOSIP_MESSAGE_GLOBALFAILURE", CALLBACK_TEMPLATE(onExosipMessage_globalfailure)},
            {"EXOSIP_SUBSCRIPTION_NOANSWER", CALLBACK_TEMPLATE(onExosipSubscription_noanswer)},
            {"EXOSIP_SUBSCRIPTION_PROCEEDING", CALLBACK_TEMPLATE(onExosipSubscription_proceeding)},
            {"EXOSIP_SUBSCRIPTION_ANSWERED", CALLBACK_TEMPLATE(onExosipSubscription_answered)},
            {"EXOSIP_SUBSCRIPTION_REDIRECTED", CALLBACK_TEMPLATE(onExosipSubscription_redirected)},
            {"EXOSIP_SUBSCRIPTION_REQUESTFAILURE", CALLBACK_TEMPLATE(onExosipSubscription_requestfailure)},
            {"EXOSIP_SUBSCRIPTION_SERVERFAILURE", CALLBACK_TEMPLATE(onExosipSubscription_serverfailure)},
            {"EXOSIP_SUBSCRIPTION_GLOBALFAILURE", CALLBACK_TEMPLATE(onExosipSubscription_globalfailure)},
            {"EXOSIP_SUBSCRIPTION_NOTIFY", CALLBACK_TEMPLATE(onExosipSubscription_notify)},
            {"EXOSIP_IN_SUBSCRIPTION_NEW", CALLBACK_TEMPLATE(onExosipIn_subscription_new)},
            {"EXOSIP_NOTIFICATION_NOANSWER", CALLBACK_TEMPLATE(onExosipNotification_noanswer)},
            {"EXOSIP_NOTIFICATION_PROCEEDING", CALLBACK_TEMPLATE(onExosipNotification_proceeding)},
            {"EXOSIP_NOTIFICATION_ANSWERED", CALLBACK_TEMPLATE(onExosipNotification_answered)},
            {"EXOSIP_NOTIFICATION_REDIRECTED", CALLBACK_TEMPLATE(onExosipNotification_redirected)},
            {"EXOSIP_NOTIFICATION_REQUESTFAILURE", CALLBACK_TEMPLATE(onExosipNotification_requestfailure)},
            {"EXOSIP_NOTIFICATION_SERVERFAILURE", CALLBACK_TEMPLATE(onExosipNotification_serverfailure)},
            {"EXOSIP_NOTIFICATION_GLOBALFAILURE", CALLBACK_TEMPLATE(onExosipNotification_globalfailure)}};

        for (uint32_t i = 0; i < EXOSIP_EVENT_COUNT; ++i)
        {
            m_eventProcMap.insert(std::make_pair(i, eventProcTable[i]));
        }
    }

    CSipEventHandle::~CSipEventHandle()
    {
    }

    EventNameProcPair CSipEventHandle::getEventProc(eXosip_event_type_t type)
    {
        EventNameProcPair pair = {"", nullptr};
        if (type > EXOSIP_EVENT_COUNT)
        {
            //LOG_INFO << "Event Type: " << type << " don't exist!";
            return pair;
        }

        auto value = m_eventProcMap.find(type);
        if (value == m_eventProcMap.end())
        {
            return pair;
        }
        else
        {
            return value->second;
        }
    }

    /* REGISTER related events */
    int CSipEventHandle::onExosipRegistration_success(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipRegistration_success";
        //TODO 通知注册成功
        OnRegisterSuccess(event->exevent->rid);
        return 0;
    }

    int CSipEventHandle::onExosipRegistration_failure(const SIP_ENENT_PTR &event)
    {
        // qDebug() << "onExosipRegistration_failure";
        //TODO REGISTER send twice, this is question!
        //LOG_WARN << "register id: " << event->exevent->rid << " need authentication!";
        eXosip_lock(event->excontext);
        eXosip_automatic_action(event->excontext);
        eXosip_unlock(event->excontext);
        return 0;
    }

    /* INVITE related events within calls */
    int CSipEventHandle::onExosipCall_invite(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_invite";
        return 0;
    }

    int CSipEventHandle::onExosipCall_reinvite(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_reinvite";
        return 0;
    }

    int CSipEventHandle::onExosipCall_noanswer(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_noanswer";
        return 0;
    }

    int CSipEventHandle::onExosipCall_proceeding(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_proceeding";
        return 0;
    }

    int CSipEventHandle::onExosipCall_ringing(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_ringing";
        return 0;
    }

    int CSipEventHandle::onExosipCall_answered(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_answered";
        return 0;
    }

    int CSipEventHandle::onExosipCall_redirected(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_redirected";
        return 0;
    }

    int CSipEventHandle::onExosipCall_requestfailure(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_requestfailure";
        return 0;
    }

    int CSipEventHandle::onExosipCall_serverfailure(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_serverfailure";
        return 0;
    }

    int CSipEventHandle::onExosipCall_globalfailure(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_globalfailure";
        return 0;
    }

    int CSipEventHandle::onExosipCall_ack(const SIP_ENENT_PTR &event)
    {
        // 正常情况下需要起线程,推流,这里只需要给相机模块发消息,让gstreamer推出RTP流即可
        qDebug() << "onExosipCall_ack";
        return 0;
    }

    int CSipEventHandle::onExosipCall_cancelled(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_cancelled";
        return 0;
    }

    /* request related events within calls (except INVITE) */
    int CSipEventHandle::onExosipCall_message_new(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_message_new";
        eXosip_event_t *exosip_event = event->exevent;

        if (!strncmp(exosip_event->request->sip_method, "MESSAGE", strlen("MESSAGE")))
        {
            procMessage(event);
        }
        else if (!strncmp(exosip_event->request->sip_method, "BYE", strlen("BYE")))
        {
            onExosipCall_closed(event);
        }

        return 0;
    }

    int CSipEventHandle::onExosipCall_message_proceeding(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_message_proceeding";
        return 0;
    }

    int CSipEventHandle::onExosipCall_message_answered(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_message_answered";
        return 0;
    }

    int CSipEventHandle::onExosipCall_message_redirected(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_message_redirected";
        return 0;
    }

    int CSipEventHandle::onExosipCall_message_requestfailure(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_message_requestfailure";
        return 0;
    }

    int CSipEventHandle::onExosipCall_message_serverfailure(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_message_serverfailure";
        return 0;
    }

    int CSipEventHandle::onExosipCall_message_globalfailure(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_message_globalfailure";
        return 0;
    }

    int CSipEventHandle::onExosipCall_closed(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_closed";
        return 0;
    }

    /* for both UAS & UAC events */
    int CSipEventHandle::onExosipCall_released(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipCall_released";
        return 0;
    }

    /* events received for request outside calls */
    int CSipEventHandle::onExosipMessage_new(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipMessage_new";
        eXosip_event_t *exosip_event = event->exevent;

        if (!strncmp(exosip_event->request->sip_method, "REGISTER", strlen("REGISTER")))
        {
            //        m_RegisterHandler->HandleRequest(event);
        }
        else if (!strncmp(exosip_event->request->sip_method, "MESSAGE", strlen("MESSAGE")))
        {
            procMessage(event);
        }
        return 0;
    }

    int CSipEventHandle::onExosipMessage_proceeding(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipMessage_proceeding";
        return 0;
    }

    int CSipEventHandle::onExosipMessage_answered(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipMessage_answered";
        respMessage(event);
        return 0;
    }

    int CSipEventHandle::onExosipMessage_redirected(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipMessage_redirected";
        return 0;
    }

    int CSipEventHandle::onExosipMessage_requestfailure(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipMessage_requestfailure";
        return 0;
    }

    int CSipEventHandle::onExosipMessage_serverfailure(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipMessage_serverfailure";
        return 0;
    }

    int CSipEventHandle::onExosipMessage_globalfailure(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipMessage_globalfailure";
        return 0;
    }

    /* Presence and Instant Messaging */
    int CSipEventHandle::onExosipSubscription_noanswer(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipSubscription_noanswer";
        return 0;
    }

    int CSipEventHandle::onExosipSubscription_proceeding(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipSubscription_proceeding";
        return 0;
    }

    int CSipEventHandle::onExosipSubscription_answered(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipSubscription_answered";
        return 0;
    }

    int CSipEventHandle::onExosipSubscription_redirected(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipSubscription_redirected";
        return 0;
    }

    int CSipEventHandle::onExosipSubscription_requestfailure(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipSubscription_requestfailure";
        return 0;
    }

    int CSipEventHandle::onExosipSubscription_serverfailure(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipSubscription_serverfailure";
        return 0;
    }

    int CSipEventHandle::onExosipSubscription_globalfailure(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipSubscription_globalfailure";
        return 0;
    }

    int CSipEventHandle::onExosipSubscription_notify(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipSubscription_notify";
        procMessage(event);
        return 0;
    }

    int CSipEventHandle::onExosipIn_subscription_new(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipIn_subscription_new";
        return 0;
    }

    int CSipEventHandle::onExosipNotification_noanswer(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipNotification_noanswer";
        return 0;
    }

    int CSipEventHandle::onExosipNotification_proceeding(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipNotification_proceeding";
        return 0;
    }

    int CSipEventHandle::onExosipNotification_answered(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipNotification_answered";
        return 0;
    }

    int CSipEventHandle::onExosipNotification_redirected(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipNotification_redirected";
        return 0;
    }

    int CSipEventHandle::onExosipNotification_requestfailure(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipNotification_requestfailure";
        return 0;
    }

    int CSipEventHandle::onExosipNotification_serverfailure(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipNotification_serverfailure";
        return 0;
    }

    int CSipEventHandle::onExosipNotification_globalfailure(const SIP_ENENT_PTR &event)
    {
        qDebug() << "onExosipNotification_globalfailure";
        return 0;
    }

}
