#ifndef CSIPEVENTHANDLE_H
#define CSIPEVENTHANDLE_H
#include "CSipMessageHandle.h"
#include <map>

namespace GB28181
{
    class CSipEventHandle : public CSipMessageHandle
    {
    public:
        CSipEventHandle();
        ~CSipEventHandle();

    public:
        /* REGISTER related events */
        int onExosipRegistration_success(const SIP_ENENT_PTR &event);

        int onExosipRegistration_failure(const SIP_ENENT_PTR &event);

    public:
        EventNameProcPair getEventProc(eXosip_event_type_t type);
        /* INVITE related events within calls */
        virtual int onExosipCall_invite(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_reinvite(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_noanswer(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_proceeding(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_ringing(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_answered(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_redirected(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_requestfailure(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_serverfailure(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_globalfailure(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_ack(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_cancelled(const SIP_ENENT_PTR &event);

    public:
        /* request related events within calls (except INVITE) */
        virtual int onExosipCall_message_new(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_message_proceeding(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_message_answered(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_message_redirected(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_message_requestfailure(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_message_serverfailure(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_message_globalfailure(const SIP_ENENT_PTR &event);

        virtual int onExosipCall_closed(const SIP_ENENT_PTR &event);

    public:
        /* for both UAS & UAC events */
        virtual int onExosipCall_released(const SIP_ENENT_PTR &event);

    public:
        /* events received for request outside calls */
        virtual int onExosipMessage_new(const SIP_ENENT_PTR &event);

        virtual int onExosipMessage_proceeding(const SIP_ENENT_PTR &event);

        virtual int onExosipMessage_answered(const SIP_ENENT_PTR &event);

        virtual int onExosipMessage_redirected(const SIP_ENENT_PTR &event);

        virtual int onExosipMessage_requestfailure(const SIP_ENENT_PTR &event);

        virtual int onExosipMessage_serverfailure(const SIP_ENENT_PTR &event);

        virtual int onExosipMessage_globalfailure(const SIP_ENENT_PTR &event);

    public:
        /* Presence and Instant Messaging */
        virtual int onExosipSubscription_noanswer(const SIP_ENENT_PTR &event);

        virtual int onExosipSubscription_proceeding(const SIP_ENENT_PTR &event);

        virtual int onExosipSubscription_answered(const SIP_ENENT_PTR &event);

        virtual int onExosipSubscription_redirected(const SIP_ENENT_PTR &event);

        virtual int onExosipSubscription_requestfailure(const SIP_ENENT_PTR &event);

        virtual int onExosipSubscription_serverfailure(const SIP_ENENT_PTR &event);

        virtual int onExosipSubscription_globalfailure(const SIP_ENENT_PTR &event);

        virtual int onExosipSubscription_notify(const SIP_ENENT_PTR &event);

    public:
        virtual int onExosipIn_subscription_new(const SIP_ENENT_PTR &event);

    public:
        virtual int onExosipNotification_noanswer(const SIP_ENENT_PTR &event);

        virtual int onExosipNotification_proceeding(const SIP_ENENT_PTR &event);

        virtual int onExosipNotification_answered(const SIP_ENENT_PTR &event);

        virtual int onExosipNotification_redirected(const SIP_ENENT_PTR &event);

        virtual int onExosipNotification_requestfailure(const SIP_ENENT_PTR &event);

        virtual int onExosipNotification_serverfailure(const SIP_ENENT_PTR &event);

        virtual int onExosipNotification_globalfailure(const SIP_ENENT_PTR &event);

    protected:
        /*******************************************
        函数功能:注册成功处理
        入口参数: rid 注册ID
        出口参数: 0 成功
        备    注:
        *******************************************/
        virtual int OnRegisterSuccess(int rid) = 0;

    private:
        std::map<uint32_t, EventNameProcPair> m_eventProcMap;
    };
}
#endif // CSIPEVENTHANDLE_H
