#ifndef CSIPMESSAGEHANDLE_H
#define CSIPMESSAGEHANDLE_H
#include "CSipDefs.h"
#include <string.h>
#include <map>
#include "utils/CPtzParser.h"
#include "utils/CXmlParser.h"
#include "CSipDevice.h"

using namespace std;

namespace GB28181
{
    typedef std::function<int(const SIP_ENENT_PTR &, tinyxml_doc_t &, MANSCDP_MSG_HEADER &)> msg_event_proc;
    class CSipMessageHandle : public CSipDevice
    {
    public:
        CSipMessageHandle();

    public:
        virtual int procMessage(const SIP_ENENT_PTR &e);
        ///< handle response of outgoing request
        virtual int respMessage(const SIP_ENENT_PTR &e);
        //
        int HandleResponseFailure(const SIP_ENENT_PTR &e);

    protected:
        /*******************************************
        函数功能:收到响应类型的消息处理,主要是刷新心跳超时
        入口参数: control_cmd_t &ctrlcmd,    控制字
                 string &devid              设备ID
        出口参数: 0 成功
        备    注:
        *******************************************/
        virtual int onMsgResponse(string &reqid, int status_code) = 0;

        /*******************************************
        函数功能:PTZ控制
        入口参数: control_cmd_t &ctrlcmd,    控制字
                 string &devid              设备ID
        出口参数: 0 成功
        备    注:
        *******************************************/
        virtual int HandlePTZControl(DEVICE_CONTROL_CMD &ctrlcmd, string &devid) = 0;
        /*******************************************
        函数功能:设备控制
        入口参数: rid 注册ID
        出口参数: 0 成功
        备    注:
        *******************************************/
        virtual int HandleDeviceControl(MANSCDP_DEV_CTRL_CMD cmd, string &devid, MANSCDP_SWITCH_STATUS onoff) = 0;

        /*******************************************
        函数功能:向sip终端发送消息/响应
        入口参数: rid 注册ID
        出口参数: 0 成功
        备    注:
        *******************************************/
        virtual int sendResponse(eXosip_event_t *exevent, string context_type, string msg) = 0;

        /*******************************************
        函数功能:发送请求
        入口参数: osip_message_t * request,
        出口参数: 0 成功
        备    注:
        *******************************************/
        virtual int sendRequest(string context_type, string message) = 0;

        /*******************************************
        函数功能:向sip终端发送简单响应
        入口参数: rid 注册ID
        出口参数: 0 成功
        备    注:
        *******************************************/
        virtual int sendResponseOk(eXosip_event_t *exevent) = 0;

        /*******************************************
        函数功能:创建sip消息实例
        入口参数:
        出口参数: osip_message_t * 消息指针
        备    注:
        *******************************************/
        virtual osip_message_t *createMsg() = 0;

    private:
        int handle_incoming_req_control(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int handle_incoming_req_query(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int handle_incoming_req_notify(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int handle_incoming_req_response(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);

    private:
        int on_devctrl_ptzcmd(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int on_devctrl_teleboot(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int on_devctrl_recordcmd(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int on_devctrl_guardcmd(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int on_devctrl_alarmcmd(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int on_devctrl_iframecmd(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int on_devctrl_dragzoomin(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int on_devctrl_dragzoomout(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int on_devctrl_homeposition(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);

    private:
        int on_devcfg_basicparam(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int on_devcfg_svacencodeconfig(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int on_devcfg_svacdeconfigconfig(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);

        int devctrl_subcmd(const SIP_ENENT_PTR &e, string &devid, MANSCDP_DEV_CTRL_CMD subcmd, MANSCDP_SWITCH_STATUS onoff);

    private:
        int on_query_device_status(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int on_query_catalog(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int on_query_device_info(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int on_query_record_info(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int on_query_alarm(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int on_query_config_download(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int on_query_preset_query(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);
        int on_query_mobile_position(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh);

        int getStatcodeFromResp(osip_message_t *resp);

        const char *getMsgIdFromReq(osip_message_t *req);

        int sendSimplyResp(const char *uname, eXosip_t *excontext, int tid, int status);

    protected:
        ///< control type
        map<MANSCDP_DEV_CTRL_CMD, msg_event_proc> m_devctlproc;
        map<MANSCDP_DEV_CFG_CMD, msg_event_proc> m_devcfgproc;
        ///< query type
        map<MANSCDP_CMD_TYPE, msg_event_proc> m_queryproc;
        CXmlParser m_xmlparser;
        CPtzParser m_ptzparser;
    };
}
#endif // CSIPMESSAGEHANDLE_H
