#ifndef CGPSMANAGER_H
#define CGPSMANAGER_H
#include <QObject>
#include "log/CLogManager.h"
#include "common.h"
#include <QIODevice>
#include "device/CDeviceManager.h"

#include <QtSerialPort/QSerialPort>
#include <QtSerialPort/QSerialPortInfo>
class ITcpModule;
class CDeviceManager;

class CGPSManager : public QObject, public IModule
{
    Q_OBJECT
public:
    LOG_DECLARE()
    MODULE_NAME(CGPSManager)
    CGPSManager(QObject *parent = nullptr);
    ~CGPSManager();

    //初始化操作
    virtual bool Initialize();
    //模块注册接口
    bool RegistMessage();

    // 获取速度信息
    float getSpeedInfo();
	float getAngleInfo();
	QString getLatitude();
	QString getLongitude();
    // 获取速度信息
    uint getMileageInfo();
	bool isLegal();
    // 获取当前GPS信息
    bool getCurGPSInfo(GPS_INFO& gps);
    // 获取总停车时间
    uint getTotalStopTime() { return m_totalStopTime; }

Q_SIGNALS:
    // 通知传感器数据更新
    void notifySensorUpdate();
    // GPS异常通知
    void notifyGpsMasked(bool);
    // 速度异常通知
    void notifySpeedAlarm(bool);

    // 通知打开GPS上报
    void notifyOpenGpsAutoReoprt();
	void notifyaiStatus(int);

public Q_SLOTS:

public slots:
    void onReadUartData();
    
    // 初始化gps UART
    void InitGpsUART();

protected:
    //主消息任务，主要用于和其他模块之间的消息通讯
    THREAD_DEFINITION(CGPSManager, TaskMain)
    //传感器收集任务, 用于定时轮询传感器相关的信息;
    THREAD_DEFINITION(CGPSManager, TaskGPSCol)

    //主消息任务，主要用于和其他模块之间的消息通讯
    virtual void TaskMain(void *pBuff);
    //传感器收集任务, 用于定时轮询传感器相关的信息;
    virtual void TaskGPSCol(void *pBuff);
 
	void ProcSTATUSModeRequest(MODULE_MESSAGE &msg);
	
	void ProcaiTimerRequest(MODULE_MESSAGE &msg);
    // 关闭gps UART
    bool closeGpsUART();

    // 使能GPS定位 AT+QGPS=1
    bool enableGPSLOC();

    // 错误处理
    void praseUartError(QString str_err);

    //获取GPS信息
    QString getGPSinfo();

    // 解析GPS信息
    bool  parseGpsRMC(QString str_info, GPS_INFO &info);
    bool  parseGpsQGPSLoc(QString str_info, GPS_INFO &info);
    bool  parseGpsGGA(QString str_info, GPS_INFO &info);

    // 检查gps是否有效
    bool isValidGps(GPS_INFO& gps_info);

    // 发送gps查询请求
    void queryGpsByUart();

    // 读取uart数据
    bool readUart(QString &info);

    // 处理收到的串口GPS数据;
    void procGpsData(QString &gps_string);

    // 
    
    bool openGpsReport();
    void openGpsUart();

    bool openUart(QSerialPort &uart, QString port, QIODevice::OpenMode mode = QIODevice::ReadWrite);
protected:
    // 依赖模块
    ITcpModule *m_pTcpModule;
	CTimerManager *m_pTimerMng;
	CDeviceManager *m_pdeviceMng;
    // 保护信号量
    QMutex m_mtxUart;
    // uart 句柄
    int m_fd;
    // 总里程
    float m_totalMileage;
    // 总停车时间
    int m_totalStopTime;
    // gps更新失败次数, 如果连续30次没有更新则上报gps告警
    int m_iGpsErrorTimes;
	int m_aiStatus;
    // 当前GPS
    GPS_INFO m_lastGps;
	time_t last_ai_report_time;

    QSerialPort m_UartPort;
};

#endif // CGPSMANAGER_H
