#include "dmmManager.h"
#include "../socket/TcpServer.h"
#include "../base_module/factory.h"
#include "jt808/CJT808Terminal.h"
#include "timer/CTimerManager.h"
#include <string.h>

REGISTER_MODULE(CdmmManager, "CdmmManager")

//LOG_DEFINITION(CdmmManager)

CdmmManager::CdmmManager(QObject *parent)
    : QObject(parent),
      IModule(),
      m_pTcpModule(nullptr),
      m_usbStatus(0),
	  m_cardStatus(0),
	  m_hardnet(0),
	  m_softnet(0),
      m_iStatusRespCnt(0)
	  
{
}

CdmmManager::~CdmmManager()
{
    std::cout << name() << std::endl;
}

//初始化操作
bool CdmmManager::Initialize()
{
    //起主消息处理线程
    printf("---------CdmmManager::Initialize-----------\n");
    STARTUP_TASK(TaskMain)

    return true;
}





bool CdmmManager::RegistMessage()
{
	printf("---------CdmmManager::RegistMessage-----------\n");
    m_pTcpModule = dynamic_cast<ITcpModule *>(QueryPlugin("TcpModule"));
    if (m_pTcpModule)
    {
        m_pTcpModule->RegistMessageCallback(this, MODULE_DEV_CHECK);
    }

    // 定时器模块
    CTimerManager *pTimerMng = dynamic_cast<CTimerManager *>(::QueryPlugin("CTimerManager"));
    if (pTimerMng)
    {
        pTimerMng->RegTimer(MODULE_DEV_CHECK, TIMER_CHECK_GET_USB, 10);
		//mSleep(0.1);
		//pTimerMng->RegTimer(MODULE_DEV_CHECK, TIMER_CHECK_GET_CARD, 10);
		//mSleep(0.1);
		//pTimerMng->RegTimer(MODULE_DEV_CHECK, TIMER_CHECK_GET_HARD, 10);
		//mSleep(0.1);
		//pTimerMng->RegTimer(MODULE_DEV_CHECK, TIMER_CHECK_GET_NET, 10);
		//mSleep(0.1);
    }

    return true;
}

//主消息任务，主要用于和其他模块之间的消息通讯
void CdmmManager::TaskMain(void *pBuff)
{
    (void)pBuff;
    setCurrentTaskName("tskCdmmManager");
    MODULE_MESSAGE *pMsg = nullptr;
    RegistMessage();
    waitForStart();

	printf("---------CdmmManager::TaskMain-----------\n");
    //emit notifyAudioStatus(m_curAudioStatus);

    while (!isShutdown())
    {
        if (GetModuleMessage(pMsg)) // get msg from message queue
        {
        	printf("---------CdmmManager::msg %d-----------\n",pMsg->iCmd);
            switch (pMsg->iCmd)
            {
            case TIMER_CHECK_GET_USB:
			{
				ProcGet4Gtatus(*pMsg);
				break;
			}
			case TIMER_CHECK_GET_CARD:
			{
				ProcGetcardtatus(*pMsg);
				break;
			}
			case TIMER_CHECK_GET_HARD:
			{
				ProcGethardtatus(*pMsg);
				break;
			}
			case TIMER_CHECK_GET_NET:
			{
				ProcGetsofttatus(*pMsg);
				break;
			}
			case CMD_CHECK_REP_4G:
			{
				ProcRep4GStatus(*pMsg);
				break;
			}
			case CMD_CHECK_REP_CARD:
			{
				ProcRepcardStatus(*pMsg);
				break;
			}
			case CMD_CHECK_REP_HARD:
			{
				ProcRephardStatus(*pMsg);
				break;
			}
			case CMD_CHECK_REP_SOFT:
			{
				ProcRepsoftStatus(*pMsg);
				break;
			}
            default:
            	printf("default");
                break;
            }

            SAFE_DELETE(pMsg);
        }
    }
}

int getDistSafe()
{
	QStorageInfo ROM = QStorageInfo::root();
    ROM.refresh();  //获得最新磁盘信息
    int total_gb = ROM.bytesTotal()/1024/1024/1024;
	int nouse_gb = ROM.bytesAvailable()/1024/1024/1024;
	printf("total %d GB: available %d GB\n", total_gb, nouse_gb);
	float r = (float)(nouse_gb*100 / total_gb)  ;
	printf("usage %.2f%%\n", r);
	int ret = 0;
	if (r <= 10.0 )
	{
		ret = 1;
	}

	return ret;
    
}
/*
void executeCMD(const char *cmd, char *result)   
{   
    char buf_ps[1024];   
    char ps[1024]={0};   
    FILE *ptr;   
    strcpy(ps, cmd);   
    if((ptr=popen(ps, "r"))!=NULL)   
    {   
        while(fgets(buf_ps, 1024, ptr)!=NULL)   
        {   
           strcat(result, buf_ps);   
           if(strlen(result)>1024)   
               break;   
        }   
        pclose(ptr);   
        ptr = NULL;  
		
    }   
    else  
    {   
        printf("popen %s error\n", ps);   
    }   
}  

*/
void CdmmManager::ProcRep4GStatus(MODULE_MESSAGE &msg)
{
	printf("ProcRep4GStatus\n");
	memcpy(&m_usbStatus, msg.szBuff, sizeof(int));
	emit notifyusbStatus(m_usbStatus);
	printf("----m_usbStatus %d ------\n", m_usbStatus);
	
	memcpy(&m_cardStatus,(msg.szBuff+sizeof(int)), sizeof(int));

	
	emit notifycardStatus(m_cardStatus);
	printf("----m_cardStatus %d ------\n", m_cardStatus);
	memcpy(&m_hardnet,(msg.szBuff+2*sizeof(int)), sizeof(int));
	
		
	emit notifyhardStatus(m_hardnet);
	printf("----m_hardnet %d ------\n", m_hardnet);
	memcpy(&m_softnet,(msg.szBuff+3*sizeof(int)), sizeof(int));
	
		
	emit notifysoftStatus(m_softnet);
	printf("----m_softnet %d ------\n", m_softnet);

	int distStat = getDistSafe();
	emit notifydisk(distStat);
/*
	char error[1024];
	executeCMD("docker ps --format  \'{{ .Names }}\'| grep hncy | awk \'{print $1}\'", error);
	printf("error is %s len is %d\n", error, strlen(error));
	if (strlen(error) == 1)
		{
			emit notifyaiStatus(1);
	}
	else
		{
		emit notifyaiStatus(0);
	}
	*/
	m_cardStatus = 0; 
	m_usbStatus = 0;
	m_hardnet = 0;
	m_softnet = 0;
}
void CdmmManager::ProcRepcardStatus(MODULE_MESSAGE &msg)
{
	memcpy(&m_cardStatus, msg.szBuff, sizeof(int));
	printf("ProcRepcardStatus\n");
	emit notifycardStatus(m_cardStatus);
	m_cardStatus = 0; 
}
void CdmmManager::ProcRephardStatus(MODULE_MESSAGE &msg)
{
	memcpy(&m_hardnet, msg.szBuff, sizeof(int));
	printf("ProcRephardStatus\n");
	emit notifyhardStatus(m_hardnet);
	m_hardnet = 0;
}
void CdmmManager::ProcRepsoftStatus(MODULE_MESSAGE &msg)
{
	memcpy(&m_softnet, msg.szBuff, sizeof(int));
	printf("ProcGetsofttatus\n");
	emit notifysoftStatus(m_softnet);
	m_softnet = 0;
}

void CdmmManager::ProcGet4Gtatus(MODULE_MESSAGE &recv_msg)
{
	//给应用层发送消息,同步传感器信息
    MODULE_MESSAGE msg;
    msg.dst_id = MODULE_DEV_SERVICE;
    msg.dst_type = APP_TYPE_CHECK;
    msg.iCmd = CMD_CHECK_GET_4G;
	//printf("ProcGet4Gtatus");
	//模块间发送消息
    int ret = m_pTcpModule->SendMsg(msg, sizeof(msg));
	printf("ProcGet4Gtatus %d ,%d, %d\n", ret, msg.iCmd, ret);
}
void CdmmManager::ProcGetcardtatus(MODULE_MESSAGE &recv_msg)
{
	//给应用层发送消息,同步传感器信息
    MODULE_MESSAGE msg;
    msg.dst_id = MODULE_DEV_SERVICE;
    msg.dst_type = APP_TYPE_CHECK;
    msg.iCmd = CMD_CHECK_GET_CARD;
	//printf("ProcGetcardtatus");
	//模块间发送消息
    int ret = m_pTcpModule->SendMsg(msg, sizeof(msg));
	printf("ProcGetcardtatus %d ,%d\n", ret, msg.iCmd);
}
void CdmmManager::ProcGethardtatus(MODULE_MESSAGE &recv_msg)
{
	//给应用层发送消息,同步传感器信息
    MODULE_MESSAGE msg;
    msg.dst_id = MODULE_DEV_SERVICE;
    msg.dst_type = APP_TYPE_CHECK;
    msg.iCmd = CMD_CHECK_GET_HARD;
	//printf("ProcGethardtatus");
	//模块间发送消息
    int ret = m_pTcpModule->SendMsg(msg, sizeof(msg));
	printf("ProcGethardtatus %d ,%d\n", ret, msg.iCmd);
}
void CdmmManager::ProcGetsofttatus(MODULE_MESSAGE &recv_msg)
{
	//给应用层发送消息,同步传感器信息
    MODULE_MESSAGE msg;
    msg.dst_id = MODULE_DEV_SERVICE;
    msg.dst_type = APP_TYPE_CHECK;
    msg.iCmd = CMD_CHECK_GET_SOFT;
	//printf("ProcGetsofttatus");
	//模块间发送消息
    int ret = m_pTcpModule->SendMsg(msg, sizeof(msg));
	printf("ProcGetsofttatus %d ,%d\n", ret, msg.iCmd);
}
