﻿
#include <QImage>
#include <QPixmap>
#include <QDebug>
#include <time.h>
#include <QNetworkInterface>
#include <QList>
#include "common.h"
#include <QDir>
#include <algorithm>
#include <math.h>
#include <zlib.h>
#include "libboot.h"

#ifdef WIN32
#include <windows.h>
#else
#include <unistd.h>
#endif

static QString MAIN_BOOT_PATH;
static QString BACKUP_BOOT_PATH;
static int APP_LAUNCH_MODE = 0;

// 获取主区启动路径
extern QString getMasterBootPath()
{
    return MAIN_BOOT_PATH;
}

extern QString getSlaveBootPath()
{
    return MAIN_BOOT_PATH;
}

extern int getLaunchMode()
{
    return APP_LAUNCH_MODE;
}

// 毫秒延时
extern void mSleep(int ms)
{
#ifdef WIN32
    Sleep(ms);
#else
    usleep(1000 * ms);
#endif
}

extern std::string getOSType()
{
#if defined Q_OS_WIN
    return "windows";
#elif defined Q_OS_LINUX
    return "linux";
#elif defined Q_OS_MAC
    return "osx";
#elif defined Q_OS_ANDROID
    return "android";
#elif defined Q_OS_IOS
    return "ios";
#endif
}

int64_t eight_char_to_int64(BYTE *data)
{
    int64_c temp64;
    temp64.cr[7] = data[0];
    temp64.cr[6] = data[1];
    temp64.cr[5] = data[2];
    temp64.cr[4] = data[3];
    temp64.cr[3] = data[4];
    temp64.cr[2] = data[5];
    temp64.cr[1] = data[6];
    temp64.cr[0] = data[7];
    return temp64.value;
}

/* 4字节转化为Long */
DWORD four_char_to_long(BYTE *data)
{
    long_c long_temp;
    long_temp.ln = 0;

    long_temp.cr[0] = data[3];
    long_temp.cr[1] = data[2];
    long_temp.cr[2] = data[1];
    long_temp.cr[3] = data[0];
    return long_temp.ln;
}

/* 2字节转化为short */
WORD two_char_to_short(BYTE *data)
{
    short_c short_temp;

    short_temp.cr[0] = data[1];
    short_temp.cr[1] = data[0];
    return short_temp.st;
}

BYTE *long_to_four_char(DWORD data, BYTE *str)
{
    long_c long_temp;

    long_temp.ln = data;
    str[0] = long_temp.cr[3];
    str[1] = long_temp.cr[2];
    str[2] = long_temp.cr[1];
    str[3] = long_temp.cr[0];
    return str;
}

BYTE *int64_to_eight_char(int64_t data, BYTE *str)
{
    int64_c temp64;
    temp64.value = data;
    str[0] = temp64.cr[7];
    str[1] = temp64.cr[6];
    str[2] = temp64.cr[5];
    str[3] = temp64.cr[4];
    str[4] = temp64.cr[3];
    str[5] = temp64.cr[2];
    str[6] = temp64.cr[1];
    str[7] = temp64.cr[0];
    return str;
}

BYTE *short_to_two_char(WORD data, BYTE *str)
{
    short_c short_temp;

    short_temp.st = data;
    str[0] = short_temp.cr[1];
    str[1] = short_temp.cr[0];
    return str;
}

// 删除目录下的文件
bool removeDir(QString dirName)
{
    QDir directory(dirName);
    if (!directory.exists())
    {
        return true;
    }

    QString srcPath = QDir::toNativeSeparators(dirName);
    if (!srcPath.endsWith(QDir::separator()))
        srcPath += QDir::separator();

    QStringList fileNames = directory.entryList(QDir::AllEntries | QDir::NoDotAndDotDot | QDir::Hidden);
    bool error = false;
    for (QStringList::size_type i = 0; i != fileNames.size(); ++i)
    {
        QString filePath = srcPath + fileNames.at(i);
        QFileInfo fileInfo(filePath);
        if (fileInfo.isFile() || fileInfo.isSymLink())
        {
            QFile::setPermissions(filePath, QFile::WriteOwner);
            if (!QFile::remove(filePath))
            {
                error = true;
            }
        }
        else if (fileInfo.isDir())
        {
            if (!removeDir(fileInfo.fileName()))
            {
                error = true;
            }
        }
    }

    if (!directory.rmdir(QDir::toNativeSeparators(directory.path())))
    {
        error = true;
    }

    return !error;
}

// 生成文件夹路径
bool makePath(QString strPath)
{
    bool bSuccess = true;
    // 先检查目录是否存在
    QDir dir(strPath);
    if (!dir.exists())
    {
        bSuccess = dir.mkpath(".");
    }
    return bSuccess;
}

// 分离文件名和路径
QStringList sepratePathName(QString path_name)
{
    QStringList list;
    int sep_idx = std::max(path_name.lastIndexOf("/"), path_name.lastIndexOf("\\"));
    if (sep_idx >= 0)
    {
        list.append(path_name.left(sep_idx + 1));
        list.append(path_name.mid(sep_idx + 1));
    }
    else
    {
        list.append("");
        list.append(path_name);
    }
    return list;
}

// 计算弧度
double rad(double d)
{
    const double PI = 3.1415926535898;
    return d * PI / 180.0;
}

// 从两个gps坐标点（经纬度）获得两点的直线距离，单位是米
double CalcDistance(double lat1, double long1, double lat2, double long2)
{
    const float EARTH_RADIUS = 6378.137;
    double radLat1 = rad(lat1);
    double radLat2 = rad(lat2);
    double a = radLat1 - radLat2;
    double b = rad(long1) - rad(long2);
    double s = 2 * asin(sqrt(pow(sin(a / 2), 2) + cos(radLat1) * cos(radLat2) * pow(sin(b / 2), 2)));
    s = s * EARTH_RADIUS;
    s = (int)(s * 10000000) / 10000;
    return s;
}

// 获取两个gps之间的距离
float getDistanceByGPS(GPS_INFO &gps1, GPS_INFO &gps2)
{
    return (float)CalcDistance(gps1.latitude, gps1.longitude, gps2.latitude, gps2.longitude);
}

// 获取主mac地址作为设备ID
QString strMacQAddr;

QString getMainMacAddress()
{
	if (!strMacQAddr.isNull())
		return strMacQAddr;
	
    QList<QNetworkInterface> nets = QNetworkInterface::allInterfaces(); // 获取所有网络接口列表
    int nCnt = nets.count();
    QString strMacAddr = "12345678";
	int times = 3;
	while(times >= 0)
	{
		for (int i = 0; i < nCnt; i++)
	    {
	        // 只取eth0的mac地址
	        if (nets[i].name() == "eth0")
	        {
	            strMacAddr = nets[i].hardwareAddress();
				strMacQAddr = strMacAddr.replace(':', "");
	            break;
	        }
	    }
		if ( strMacAddr != "12345678")
		{
			break;
			
		}
		else
		{
			times -=1;
			continue;
		}
	}
    
    return strMacQAddr;
}

// 获取多个本地IP地址信息
QStringList getLocalIPs()
{
    QStringList strIpList;
    QList<QHostAddress> list = QNetworkInterface::allAddresses();
    foreach (QHostAddress address, list)
    {
        // 主机地址为空，排除
        if (address.isNull())
            continue;
        QAbstractSocket::NetworkLayerProtocol nProtocol = address.protocol();
        QString strAddress = address.toString();
        bool bLoopback = address.isLoopback();

        // 如果是IPv4,且非环回地址
        if (nProtocol == QAbstractSocket::IPv4Protocol && bLoopback == false)
        {
            bool bOk = false;
            address.toIPv4Address(&bOk);
            if (bOk)
            {
                QStringList lst = strAddress.split('.');
                if (lst.length() != 4)
                {
                    continue;
                }

                // 192网段的非路由地址
                if (lst[0].toInt() == 192 && lst[3].toInt() != 1)
                {
                    strIpList.push_back(strAddress);
                }
            }
        }
        // 如果是IPv6，直接排除
        else
        {
            continue;
        }
    }

    return strIpList;
}

// 获取本地ip地址
QString getLocalMainIP()
{
    QString strIp;
    // 枚举所有IP地址
    QList<QHostAddress> list = QNetworkInterface::allAddresses();

    foreach (QHostAddress address, list)
    {
        // 主机地址为空，排除
        if (address.isNull())
            continue;
        QAbstractSocket::NetworkLayerProtocol nProtocol = address.protocol();
        QString strAddress = address.toString();
        bool bLoopback = address.isLoopback();

        // 如果是IPv4,且非环回地址
        if (nProtocol == QAbstractSocket::IPv4Protocol && bLoopback == false)
        {
            bool bOk = false;
            (void)address.toIPv4Address(&bOk);
            if (bOk)
            {
                QStringList lst = strAddress.split('.');
                if (lst.length() != 4)
                    continue;

                // 192网段的非路由地址
                if (lst[0].toInt() == 192 && lst[3].toInt() != 1)
                {
                    return strAddress;
                }
            }
        }
        // 如果是IPv6，直接排除
        else
        {
            continue;
        }
    }

    return strIp;
}

// 返回自系统开机以来的毫秒数（tick）
unsigned long GetTickCount()
{
    struct timespec ts;

    clock_gettime(CLOCK_MONOTONIC, &ts);

    return (ts.tv_sec * 1000 + ts.tv_nsec / 1000000);
}

//拷贝文件夹：
bool copyDirectoryFiles(const QString &fromDir, const QString &toDir, bool coverFileIfExist)
{
    QDir sourceDir(fromDir);
    QDir targetDir(toDir);
    /**< 如果目标目录不存在，则进行创建 */
    if (!targetDir.exists())
    {

        if (!targetDir.mkdir(targetDir.absolutePath()))

            return false;
    }

    QFileInfoList fileInfoList = sourceDir.entryInfoList();

    foreach (QFileInfo fileInfo, fileInfoList)
    {

        if (fileInfo.fileName() == "." || fileInfo.fileName() == "..")
        {
            continue;
        }

        /**< 当为目录时，递归的进行copy */
        if (fileInfo.isDir())
        {

            if (!copyDirectoryFiles(fileInfo.filePath(), targetDir.filePath(fileInfo.fileName()), coverFileIfExist))
            {
                return false;
            }
        }
        /**< 当允许覆盖操作时，将旧文件进行删除操作 */
        else
        {
            if (coverFileIfExist && targetDir.exists(fileInfo.fileName()))
            {
                targetDir.remove(fileInfo.fileName());
            }

            /// 进行文件copy
            if (!QFile::copy(fileInfo.filePath(), targetDir.filePath(fileInfo.fileName())))
            {
                return false;
            }
        }
    }

    return true;
}

#include <QTextCodec>

int utf8ToGb2312(char *strUtf8, char *strGbk, int size)
{
    QTextCodec *utf8Codec = QTextCodec::codecForName("utf-8");
    QTextCodec *gb2312Codec = QTextCodec::codecForName("gb2312");

    QString strUnicode = utf8Codec->toUnicode(strUtf8);
    QByteArray ByteGb2312 = gb2312Codec->fromUnicode(strUnicode);

    strncpy(strGbk, ByteGb2312.data(), size);
    return ByteGb2312.size();
}

#ifdef USING_OPENCV
#include <opencv2/imgproc/imgproc.hpp>
#include <opencv2/highgui.hpp>
#include <opencv2/imgproc/types_c.h>
#include "opencv2/core.hpp"

using namespace cv;
extern QImage cvMat_to_QImage(const cv::Mat &mat);
extern cv::Mat QImage2cvMat(QImage image);

// QImage转Mat
cv::Mat QImage2cvMat(QImage image)
{
    cv::Mat mat;

    switch (image.format())
    {
    case QImage::Format_ARGB32:
    case QImage::Format_RGB32:
    case QImage::Format_ARGB32_Premultiplied:
    {
        mat = cv::Mat(image.height(), image.width(),
                      CV_8UC4,
                      (void *)image.constBits(),
                      image.bytesPerLine());

        // drop the all-white alpha channel
        cvtColor(mat, mat, COLOR_BGRA2BGR);
        break;
    }
    case QImage::Format_RGB888:
        mat = cv::Mat(image.height(), image.width(), CV_8UC3, (void *)image.constBits(), image.bytesPerLine());
        cvtColor(mat, mat, CV_BGR2RGB);
        break;
    case QImage::Format_Indexed8:
    default:
        mat = cv::Mat(image.height(), image.width(), CV_8UC1, (void *)image.constBits(), image.bytesPerLine());
        break;
    }
    return mat;
}

// mat转QImage
QImage cvMat2QImage(const cv::Mat &mat)
{
    // 8-bits unsigned, NO. OF CHANNELS = 1
    if (mat.type() == CV_8UC1)
    {
        QImage image(mat.cols, mat.rows, QImage::Format_Indexed8);
        // Set the color table (used to translate colour indexes to qRgb values)
        image.setColorCount(256);
        for (int i = 0; i < 256; i++)
        {
            image.setColor(i, qRgb(i, i, i));
        }
        // Copy input cv::Mat
        uchar *pSrc = mat.data;
        for (int row = 0; row < mat.rows; row++)
        {
            uchar *pDest = image.scanLine(row);
            memcpy(pDest, pSrc, mat.cols);
            pSrc += mat.step;
        }
        return image;
    }
    // 8-bits unsigned, NO. OF CHANNELS = 3
    else if (mat.type() == CV_8UC3)
    {
        // Copy input cv::Mat
        cv::Mat mat_tmp;
        cvtColor(mat, mat_tmp, CV_BGR2RGB);
        const uchar *pSrc = (const uchar *)mat_tmp.data;
        // Create QImage with same dimensions as input cv::Mat
        QImage image(pSrc, mat.cols, mat.rows, (int)mat_tmp.step, QImage::Format_RGB888);
        return image.copy();
    }
    else if (mat.type() == CV_8UC4)
    {
        qDebug() << "CV_8UC4";
        // Copy input cv::Mat
        const uchar *pSrc = (const uchar *)mat.data;
        // Create QImage with same dimensions as input cv::Mat
        QImage image(pSrc, mat.cols, mat.rows, (int)mat.step, QImage::Format_ARGB32);
        return image.copy();
    }
    else
    {
        qDebug() << "ERROR: cv::Mat could not be converted to QImage.";
        return QImage();
    }
}

// Mat转QPixmap
QPixmap cvMat2QPixmap(const cv::Mat &mat)
{
    QImage img = cvMat2QImage(mat);
    return QPixmap::fromImage(img);
}
#endif
