﻿#ifndef COMMON_H
#define COMMON_H
#include <stdint.h>
#include <string>
#include <QString>

typedef unsigned long DWORD;
typedef unsigned int UINT;

#ifdef WIN32
typedef unsigned __int64 SOCKET;
#define SCHED_RR 0
#else
typedef int SOCKET;
#define SOCKET_ERROR -1
#endif

#ifndef WORD
typedef unsigned short WORD;
#endif

#ifndef BYTE
typedef unsigned char BYTE;
#endif

#ifndef DWORD
typedef unsigned long DWORD;
#endif

#define SOFTWARE_VERSION "V1.0.21.4.22"

#define MAX_PATH 260

// 指针安全删除
#define SAFE_DELETE(p)  \
    {                   \
        if (p)          \
        {               \
            delete (p); \
            (p) = NULL; \
        }               \
    }

// 数组安全删除
#define SAFE_DELETE_ARRAY(p) \
    {                        \
        if (p)               \
        {                    \
            delete[](p);     \
            (p) = NULL;      \
        }                    \
    }

// 安全释放
#define SAFE_RELEASE(p)     \
    {                       \
        if (p)              \
        {                   \
            (p)->Release(); \
            (p) = NULL;     \
        }                   \
    }

// 最大最小宏定义
#ifndef MAX
#define MAX(a, b) (((a) > (b)) ? (a) : (b))
#endif

#ifndef MIN
#define MIN(a, b) (((a) < (b)) ? (a) : (b))
#endif

// 0为3G,1为4G,2为5G，3为WIFI，4为有线；
enum NETWORK_TYPE
{
    NETWORK_TYPE_3G = 0,
    NETWORK_TYPE_4G,
    NETWORK_TYPE_5G,
    NETWORK_TYPE_WIFI,
    NETWORK_TYPE_WIRELESS,
};

enum NETWORK_STATUS
{
    NETWORK_STATUS_BREAK = 0,
    NETWORK_STATUS_CONNECTED,
    NETWORK_STATUS_RECONNECTING,
};

//是否清除数据。1表示清除数据并恢复出厂设置； 2表示不清除数据，只恢复出厂设置； 3表示只清除数据，不恢复出厂设置
enum RESTORE_FACTORY_TYPE
{
    RESTORE_FACTORY_ALL = 1,
    RESTORE_FACTORY_DATABASE,
    RESTORE_FACTORY_FILE
};

// 获取硬盘类型 1表示SD卡，2表示硬盘，3表示SSD卡 先打桩
enum DISK_TYPE
{
    DISK_TYPE_SDCARD = 1,
    DISK_TYPE_M2,
    DISK_TYPE_SSD,
};

// 获取运行状态。0为正常,1为维修,2为停用；先打桩
enum DEVICE_STATUS
{
    DEVICE_STATUS_RUNNING = 0,
    DEVICE_STATUS_MAINTAIN,
    DEVICE_STATUS_STOP,
};

//陀螺仪信息
struct GYRO_INFO
{
    //X轴加速度,
    float x_a;
    //Y轴加速度,
    float y_a;
    //Z轴加速度,
    float z_a;
    //X轴角加速度,
    float x_theta;
    //Y轴角加速度,
    float y_theta;
    //Z轴角加速度,
    float z_theta;
    //滚转角，
    float roll;
    //俯仰角，
    float pitch;
    //偏航角，
    float Yaw;

    //X轴磁场，
    float x_mag;
    //Y轴磁场，
    float y_mag;
    //Z轴磁场，
    float z_mag;

    //温度
    float temperature;
};

//GPS信息
struct GPS_INFO
{
    uint time_tick;
    //定位数据状态（A：有效定位，V无效定位），
    bool bValid;
    //纬度ddmm.mmmmm（度分），
    double latitude;
    //纬度半球N（北半球）或S（南半球），
    bool bNorth;
    //经度dddmm.mmmmm（度分），
    double longitude;
    //经度半球E（东经）或W（西经，
    bool bEast;
    // 速度
    float speed;

    //hdop
    float hdop;
    // 高度
    float high;
    // 方向
    float dir;
    // 卫星数目
    int sat_num;
};

// 相机参数
struct CAMERA_PARAMETER
{
    //相机索引号
    int index;
    //采集模式
    int mode;

    //成像参数
    int cent_x;
    int cent_y;
    int radio;

    //外参
    int offset_x;
    int offset_y;
    double rotate;
};

typedef union
{
    WORD st;
    BYTE cr[2];
} short_c;

typedef union
{
    uint32_t ln;
    WORD st[2];
    BYTE cr[4];
} long_c;

typedef union
{
    int64_t value;
    BYTE cr[8];
} int64_c;

// 毫秒延时
extern void mSleep(int ms);

// 最大支持的相机个数
#define MAX_CAMERA_COUNT 4

// tickcount
extern unsigned long GetTickCount();

// 获取两次gps数据之间的距离
extern float getDistanceByGPS(GPS_INFO &gps1, GPS_INFO &gps2);

extern std::string getOSType();

// 删除目录下的文件
extern bool removeDir(QString dirName);
// 创建目录
extern bool makePath(QString strPath);
// 分离文件名和路径
extern QStringList sepratePathName(QString path_name);

// 获取启动模式
extern int getLaunchMode();

// 压缩
extern int gzCompress(BYTE *in_buff, int lenth, BYTE *out_buff, int max_len);

//拷贝文件夹：
extern bool copyDirectoryFiles(const QString &fromDir, const QString &toDir, bool coverFileIfExist = true);

//utf8转ge2312
extern int utf8ToGb2312(char *strUtf8, char *strGbk, int size);

extern int64_t eight_char_to_int64(BYTE *data);
extern DWORD four_char_to_long(BYTE *data);
extern WORD two_char_to_short(BYTE *data);
extern BYTE *int64_to_eight_char(int64_t data, BYTE *str);
extern BYTE *long_to_four_char(DWORD data, BYTE *str);
extern BYTE *short_to_two_char(WORD data, BYTE *str);

#endif // COMMON_H
