﻿#ifndef MESSAGE_H
#define MESSAGE_H
#pragma once
#include <iostream>
#include <string.h>

enum APP_TYPE
{
    APP_TYPE_NONE = 0,  // 未定义
    APP_TYPE_ALL = 1,   // 全部类型,主要用于发送
    APP_TYPE_MAIN,      // 主程序
    APP_TYPE_AI,        // 算法服务程序
    APP_TYPE_SERV,      // 客户显示终端
    APP_TYPE_CLIENT,    // 其他客户端类型
    APP_TYPE_MICROPHONE, // 手咪终端
    APP_TYPE_CHECK,        //设备检查终端
};

enum MODULE_ID
{
    HEARTBEAT_MODULE = 100,
    MODULE_UI_ADPT,
    SRV_TCP_SERVER_MODULE = 0x500, // TCP模块
    MODULE_WEB_SOCKET_MNG = 0x501, // WebSocket模块
    MODULE_CAMERA_MNG = 0x502,     // 视频管理模块
    MODULE_DEVICE_MNG = 0x503,     // 设备管理模块
    MODULE_ALARM_MNG = 0x504,      // 告警管理模块
    MODULE_SENSOR_MNG = 0x505,     // 传感器管理模块
    MODULE_CONFIG_MNG = 0x506,     // 配置管理类
    MODULE_SWDL_MNG = 0x507,       // 软件升级管理类
    MODULE_FILE_MNG = 0x508,       // FILE SYSTEM MNG
    MODULE_URAT_ADPT = 0x509,      // uart module
    MODULE_JT808_MNG = 0x50A,      // jt808 module
    MODULE_GB28181_MNG = 0x50B,    // GB28181
    MODULE_TIMER_MNG = 0x50C,      // 定时器管理类
    MODULE_LOG_MNG = 0x50D,        // 日志管理类
    MODULE_MC_ADPT = 0x50E,        // 手咪适配模块模块
    MODULE_DEV_CHECK= 0x50F,       // 设备检测模块

    MODULE_APP_COMM = 0x1000,       // AI通讯模块
    MODULE_AI_SENSORS = 0x1000 + 1, // AI传感器模块
    MODULE_AI_VIDEO = 0x1000 + 2,   // 后台结果分析
    MODULE_MC_MOD = 0x1000 + 3,     // 手咪模块
	MODULE_DEV_SERVICE = 0x1000 + 4,  //设备检测模块服务
    
    MODULE_TCP_CLIENT = 0x2000 + 3, //通用终端通信模块
};

//消息命令字
enum MODULE_COMMAND_ID
{
    CMD_NULL_MESSAGE = 0,         // 空消息
    CMD_NOTIFY_APP_TYPE = 100,    // APP握手消息，通知APP类型
    CMD_ASK_FOR_APPTYPE = 101,    // APP握手消息，请求获取APP类型
    CMD_ASKFOR_DEVICE_INFO = 102, // APP握手消息，请求设备信息
    CMD_NOTIFY_DEVICE_INFO = 103, // APP握手消息，通知设备信息
    CMD_HEATBIT_MESSAGE = 106,    // APP心跳消息
    //程序间接口，每一个都要有独立定义
    CMD_WEBSOCKER_MNG_BASE = 500,  // WEBSOCK接口消息基址
    CMD_WEBSOCKER_CONNECT_TO_SERV, // 连接服务器
    CMD_SEND_TEXT_TO_SERV,         // WEBSOCK接口发送下一条消息到服务器

    // 视频管理模块
    CMD_VIDEO_MNG_BASE = 1000,          // 视频管理模块消息基址
    CMD_VIDEO_SET_CAM_CFG = 1001,       // 视频管理模块设置相机配置
    CMD_ADD_FILE_TO_LIST = 1002,        // 视频管理模块添加文件到列表
    CMD_START_GB28181_VIDEO = 1003,     // 开始GB28181推流
    CMD_STOP_GB28181_VIDEO = 1004,      // 结束GB28181推流
    CMD_CAM_ONLINE_OFFLINE = 1005,      // 广播相机上线/离线
    CMD_START_JT808_RT_VIDEO = 1006,    // 开始jt808实时推流
    CMD_JT808_RT_VIDEO_CONTROL = 1007,  // jt808实时推流控制
    CMD_START_JT808_HIS_VIDEO = 1008,   // jt808文件推流
    CMD_JT808_HIS_VIDEO_CONTROL = 1009, // jt808文件推流控制

    // 手咪音频管理模块
    CMD_MC_GET_CONFIG = 1101, // 获取手咪配置 (空)
    CMD_MC_SET_CONFIG = 1102, // 设置手咪配置 (int 文本长度/string 配置文本)
    CMD_MC_GET_STATUS = 1103, // 获取手咪状态 (空)
    CMD_MC_REP_STATUS = 1104, // 上报手咪状态 (int 状态)

    // 设备管理模块
    CMD_DEVICE_MNG_BASE = 1500,        // 设备管理基址
    CMD_QUERY_DEVICE_CALL_CFG = 1501,  // 查询呼叫配置
    CMD_NOTIFY_DEVICE_CALL_CFG = 1502, // 通知配置信息
    CMD_SWDL_PACKAGE_UPDATE = 1503,    // 软件包升级
    CMD_SWDL_UPDATE_RESULT = 1504,     // 软件包升级结果

    // 告警管理模块
    CMD_ALARM_MNG_BASE = 2000, //告警模块基址

    // 传感器模块
    CMD_SENSOR_MNG_BASE = 2500,    // 传感器模块基址
    CMD_UPDATE_SENSOR_INFO = 2501, // 进程通讯消息，需要指定消息值
    CMD_SENSOR_MNG_STATUS = 2502,

    // CONFIG
    CMD_CONFIG_MNG_BASE = 3000, // 配置模块基址
    CMD_CONFIG_RESTORE_FACTORY, // 配置模块恢复出场值
    CMD_CONFIG_RESTORE_RESULT,  // 恢复出场值结果

    // FILE
    CMD_FILE_MNG_BASE = 3500,         // 文件管理基址
    CMD_FILE_MNG_DEL_FILES = 3501,    // 删除文件
    CMD_JT808_ASKFOR_FILELIST = 3502, // JT808文件列表请求

    // JT808模块
    CMD_JT808_DOWN_CMD = 4000, // JT808 下发消息

    // TIMER MODULE
    TIMER_BASE = 5000,                // 定时器基址
    TIMER_WS_STATUS_REPORT = 5001,    // WS通用状态上报
    TIMER_WS_GPS_REPORT = 5002,       // WS GPS信息上报
    TIMER_WS_HEART_BIT = 5003,        // WS 心跳上报
    TIMER_FIND_CAM_DEVICE = 5004,     // 查找摄像头设备
    TIMER_SAVE_TO_NEW_VIDEO = 5005,   // 保存新视频
    TIMER_JT808_HEART_BEAT = 5006,    // jt808 心跳定时器
    TIMER_JT808_GPS_REPORT = 5007,    // jt808 临时GPS跟踪定时器
    TIMER_GB28181_HEART_BEAT = 5008,  // GB28181 心跳
    TIMER_GB28181_TIMEOUT_CHK = 5009, // GB28181 心跳
    TIMER_UPDATE_SUCCESSED = 5010,    // 升级成功定时器 5m内没有异常则认为升级成功
    TIMER_GET_MC_STATUS = 5011,       // 升级成功定时器 5m内没有异常则认为升级成功
    TIMER_CHECK_GET_USB = 5012,       //获取 设备usb 4G gps 模块消息
    TIMER_CHECK_GET_CARD = 5013,      //获取 设备卡状态消息
    TIMER_CHECK_GET_HARD = 5014,      //获取 设备4g网卡是否存在消息
    TIMER_CHECK_GET_NET = 5015,       //获取 设备ping baidu状态消息
	TIMER_CHECK_AI_REPORT_TIMER = 5016, //
	CMD_TIMER_SAVE_TO_NEW_VIDEO_TEST = 5017,
	CMD_TIMER_DEVICE_UPDATE = 5018,
	CMD_TIMER_PHOTO_UPLOAD = 5019,
	TIMER_GPS_RESEND = 5020,
	
	//设备检测服务
	CMD_CHECK_GET_4G = 6000,
	CMD_CHECK_GET_CARD = 6001,
	CMD_CHECK_GET_HARD = 6002,
	CMD_CHECK_GET_SOFT = 6003,
	CMD_CHECK_REP_4G = 6004,
	CMD_CHECK_REP_CARD = 6005,
	CMD_CHECK_REP_HARD = 6006,
	CMD_CHECK_REP_SOFT = 6007,

    CMD_APP_QUIT = 0xFFFF,
};

enum MESSAGE_ERROR_CODE
{
    ERR_SUCCESS = 0,
    ERR_PROJECT_NOT_STARTED = 100,
    ERR_LACK_INFOMATION,
    ERR_USER_CANCELED,
    ERR_SEND_FAILED,
};

#define MAX_BUFF_SIZE 1000

// 模块目标地址信息
struct MODULE_POST_INFO
{
    MODULE_POST_INFO()
    {
        dst_type = APP_TYPE_MAIN;
        dst_id = HEARTBEAT_MODULE;
        iCmd = CMD_NULL_MESSAGE;
    }

    APP_TYPE dst_type;
    MODULE_ID dst_id;
    MODULE_COMMAND_ID iCmd;
};

#define MAX_MESSAGE_BUFF_SIZE 1000

#pragma pack(1)
// 模块消息
struct MODULE_MESSAGE
{
    MODULE_MESSAGE()
    {
        dst_type = APP_TYPE_MAIN;
        dst_id = HEARTBEAT_MODULE;
        iCmd = CMD_NULL_MESSAGE;
        memset(szBuff, 0, sizeof(szBuff));
        ext_length = 0;
        pszExtBuff = nullptr;
        //        lenth = 0;
    }
    ~MODULE_MESSAGE()
    {
        if (pszExtBuff)
        {
            delete[] pszExtBuff;
            pszExtBuff = nullptr;
        }
    }

    APP_TYPE dst_type;
    MODULE_ID dst_id;
    MODULE_COMMAND_ID iCmd;
    int ext_length;
    char *pszExtBuff;
    unsigned char szBuff[MAX_MESSAGE_BUFF_SIZE];
    // int lenth;
};
#pragma pack()

#endif // MESSAGE_H
