#include "CJT808CmdProc.h"
#include "jt808/CJT808Terminal.h"
#include <QDebug>
#include <assert.h>

CJT808CmdProc::CJT808CmdProc()
    : m_HeartbeatInterval(60),
      m_ReportInterval(60),
      m_pTerminal(nullptr)
{
    //    m_mapFuncProc[DOWN_REGISTERRESPONSE] = &CJT808CmdProc::PraseRegist;
    m_almStatus.value = 0;
    m_Status_stu.value = 0;
    m_Status_stu.reg.acc_on = 1;
    m_Status_stu.reg.gps_fix = 1;
    m_Status_stu.reg.gps_ns = 0;
    m_Status_stu.reg.gps_ew = 0;
    m_Status_stu.reg.op_status = 0;
    m_Status_stu.reg.gps_enc = 1;
    m_Status_stu.reg.car_state = 11;
    m_Status_stu.reg.oilway = 0;
    m_Status_stu.reg.power = 0;
    m_Status_stu.reg.door_lock = 1;
    m_Status_stu.reg.gps_used = 1;
    m_Status_stu.reg.beidou_used = 1;
    m_Status_stu.reg.glonass_used = 0;
    m_Status_stu.reg.galileo_used = 0;
}

CJT808CmdProc::~CJT808CmdProc()
{
}

bool CJT808CmdProc::Initialize()
{
    m_pTerminal = dynamic_cast<CJT808Terminal *>(QueryPlugin("CJT808Terminal"));
    assert(m_pTerminal);
    bool bSuccess = CJt808Config::Initialize();
    return bSuccess;
}

bool CJT808CmdProc::registJt808Msg()
{
#define CALLBACK_TEMPLATE(F) (std::bind(&CJT808CmdProc::F, this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3))

    JT808_MSG_PROC_NODE reg_array[] = {
        {SERVER_NORMAL_ANSWER, CALLBACK_TEMPLATE(procServCommonAnswer)},
        {SERVER_REGIST_ANSWER, CALLBACK_TEMPLATE(procServRegistAnswer)},
        {SERVER_SERV_TIME_ANSWER, CALLBACK_TEMPLATE(procServTimeAnswer)},
        {SERVER_SEP_PACKET_ASK, CALLBACK_TEMPLATE(procSepPacketAsk)},
        {SERVER_SET_TERMINAL_CFG, CALLBACK_TEMPLATE(procSetParameter)},
        {SERVER_READ_TERMINAL_CFG, CALLBACK_TEMPLATE(procQueryTerminalCfg)},
        {SERVER_CTRL_TERMINAL, CALLBACK_TEMPLATE(procTerminalControl)},
        {SERVER_READ_SOME_CFG, CALLBACK_TEMPLATE(procQueryTerminalCfgItem)},
        {SERVER_PROPERTY_ASK, CALLBACK_TEMPLATE(procQueryParameter)},
        {SERVER_UPDATE_ASK, CALLBACK_TEMPLATE(procSoftwareUpdate)},
        {SERVER_READ_GPS_DATA, CALLBACK_TEMPLATE(procQueryGpsData)},
        {SERVER_TEMPORARY_LOC_CTRL, CALLBACK_TEMPLATE(procGpsReportTemp)},
        //{SERVER_CONFIRM_ALARM, CALLBACK_TEMPLATE()},
        //{SERVER_CONNECTION_CHECK, CALLBACK_TEMPLATE()},
        {SERVER_SEND_TEXT, CALLBACK_TEMPLATE(procSendText)},
        {SERVER_EVENT_SET, CALLBACK_TEMPLATE(procSetEvent)},
        {SERVER_SEND_ASK, CALLBACK_TEMPLATE(procAskQuestion)},
        {SERVER_SEND_ODER, CALLBACK_TEMPLATE(procSetOrderMenu)},
        {SERVER_SEND_MSG, CALLBACK_TEMPLATE(procInfoServ)},
        {SERVER_CALL_BACK, CALLBACK_TEMPLATE(procSetCallBack)},
        {SERVER_SET_PHONE_BOOK, CALLBACK_TEMPLATE(procSetPhoneNum)},
        {SERVER_VEHICLE_CTRL, CALLBACK_TEMPLATE(procCtrlVehicle)},
        {SERVER_SET_AREA_ROUND, CALLBACK_TEMPLATE(procSetAreaRound)},
        {SERVER_DEL_AREA_ROUND, CALLBACK_TEMPLATE(procDelAreaRound)},
        {SERVER_SET_AREA_RECT, CALLBACK_TEMPLATE(procSetAreaRect)},
        {SERVER_DEL_AREA_RECT, CALLBACK_TEMPLATE(procDelAreaRect)},
        {SERVER_SET_AREA_POLYGON, CALLBACK_TEMPLATE(procSetAreaPolygon)},
        {SERVER_DEL_AREA_POLYGON, CALLBACK_TEMPLATE(procDelAreaPolygon)},
        {SERVER_SET_AREA_ROUTE, CALLBACK_TEMPLATE(procSetAreaRoute)},
        {SERVER_DEL_AREA_ROUTE, CALLBACK_TEMPLATE(procDleAreaRoute)},
        {SERVER_QUERY_AERA_INFO, CALLBACK_TEMPLATE(procQueryAreaInfo)},
        {SERVER_READ_RECODE, CALLBACK_TEMPLATE(procReadRecord)},
        {SERVER_SET_RECODE, CALLBACK_TEMPLATE(procSetRecordPara)},
        {SERVER_REQUEST_IDCARD, CALLBACK_TEMPLATE(procRequestIdcard)},
        {SERVER_MEDIA_RETURN, CALLBACK_TEMPLATE(procMediaReq)},
        {SERVER_TAKE_PICTURE, CALLBACK_TEMPLATE(procTakePicture)},
        {SERVER_MEDIA_CHECK, CALLBACK_TEMPLATE(procCheckMedia)},
        {SERVER_READ_MEDIA, CALLBACK_TEMPLATE(procReadMedia)},
        {SERVER_RECORD_SOUND, CALLBACK_TEMPLATE(procStartRecordSound)},
        {SERVER_UPLOAD_MEDIADATA, CALLBACK_TEMPLATE(procUploadMediadata)},
        {SERVER_TRANSMISSION, CALLBACK_TEMPLATE(procMsgTransMission)},
        {SERVER_RSA, CALLBACK_TEMPLATE(procSetServRSA)},
        {SERVER_QUERY_MEDIA_PROPERTY, CALLBACK_TEMPLATE(procQueryMediaProperty)},
        {SERVER_RT_MEDIA_TRANS, CALLBACK_TEMPLATE(procRealtimeMediaTrans)},
        {SERVER_RT_MEDIA_CTRL, CALLBACK_TEMPLATE(procRealtimeMediaControl)},
        {SERVER_MEDIA_STATUS_NOTIFY, CALLBACK_TEMPLATE(procMediaStatusNotify)},
        {SERVER_QUERY_MEIDA_RESOURCE, CALLBACK_TEMPLATE(procQueryMediaResource)},
        {SERVER_VIDEO_FILE_PLAY, CALLBACK_TEMPLATE(procVideoPlay)},
        {SERVER_VIDEO_FILE_CTRL, CALLBACK_TEMPLATE(procVideoControl)},
        {SERVER_FILE_UPLOAD, CALLBACK_TEMPLATE(procFileUpload)},
        {SERVER_FILE_UPLOAD_CTRL, CALLBACK_TEMPLATE(procFileUploadControl)},
        {SERVER_PTZ_ROTATAL, CALLBACK_TEMPLATE(procPTZControlDot)},
        {SERVER_PTZ_FOCUS, CALLBACK_TEMPLATE(procPTZControlDot)},
        {SERVER_PTZ_APERTURE, CALLBACK_TEMPLATE(procPTZControlDot)},
        {SERVER_PTZ_WIPER, CALLBACK_TEMPLATE(procPTZControlDot)},
        {SERVER_PTZ_INFRARED_LIGHT, CALLBACK_TEMPLATE(procPTZControlDot)},
        {SERVER_PTZ_ZOOM, CALLBACK_TEMPLATE(procPTZControlDot)},
    };

    if (m_pTerminal)
    {
        m_pTerminal->registJt808Messages(reg_array, sizeof(reg_array) / sizeof(JT808_MSG_PROC_NODE));
    }
    else
    {
        assert(0);
    }

    return true;
}

/*******************************************
函数功能: 组织位置信息
入口参数:
出口参数:
备    注:
*******************************************/
WORD CJT808CmdProc::fillLocationInfo(BYTE *payload, GPS_INFO &gps)
{
    return CJt808Location::fillLocationInfo(payload, gps);
}

// unsigned int CJT808CmdProc::getHeartbeatInterval()
// {
//     return m_HeartbeatInterval;
// }

// unsigned int CJT808CmdProc::getGpsReportInterval()
// {
//     return m_ReportInterval;
// }

void CJT808CmdProc::setTcpSocket(QTcpSocket *pSocket)
{
    if (pSocket->isValid())
    {
        m_pSocket = pSocket;
    }
}

void CJT808CmdProc::HeartBeat()
{
}

bool CJT808CmdProc::sendRegistInfo()
{
    return true;
}

bool CJT808CmdProc::sendAuthInfo()
{
}

bool CJT808CmdProc::praseCmd(void *pPara)
{
    int cmd = 0;
    MAP_JT808_FUNC_PROC::Iterator it = m_mapFuncProc.find(cmd);
    if (it != m_mapFuncProc.end())
    {
        JT808_COMMAND_PROC_FUNC func = it.value();
        (this->*func)(pPara);
    }
    return true;
}

bool CJT808CmdProc::PraseRegist(void *pPara)
{
}

void CJT808CmdProc::sendMsg2Serv(void *pBuff, int len)
{
}

void CJT808CmdProc::sendMsg2Module(MODULE_MESSAGE *pMsg)
{
}

void CJT808CmdProc::connectProc()
{
    // emit notifyJt808Reconnect();
}

/*******************************************
函数功能: 服务器注册应答
入口参数: data,  消息体指针
          lenth, 消息体长度
出口参数:
备    注: 测试通过
*******************************************/
void CJT808CmdProc::procServCommonAnswer(int inc_num, BYTE *payload, WORD lenth)
{
    (void)inc_num;

    int idx = 0;
    WORD cycle_num = two_char_to_short(payload + idx);
    idx += 2;
    WORD cmd_id = two_char_to_short(payload + idx);
    idx += 2;
    BYTE result = two_char_to_short(payload + idx);
    idx++;

    //    // 检查是否存在多包缓存,这里如果存在多包缓存需要释放消息
    //    if (m_pTerminal)
    //    {
    //        m_pTerminal->releaseSendMultiMsg(cycle_num);
    //    }

    switch (cmd_id)
    {
    case TERMINAL_HEARTBEAT:
        qDebug() << "<<--心跳应答:" << result;
        break;
    case TERMINAL_REGIST:
        qDebug() << "<<--终端注册应答:" << result;
        break;
    case TERMINAL_DEREGIST:
        qDebug() << "<<--终端注消应答:" << result;
        break;
    case TERMINAL_AUTHENTICATION:
        if (result == 0)
        {
            qDebug() << "<<--设备鉴权应答成功:" << result;
            if (m_pTerminal->getServCtrlStep() == CTRL_STEP_LOGIN_SERVER_WAIT)
            {
                m_pTerminal->setServCtrlStatus(SERVER_CONNECT_AUTHENOK);
            }
        }
        else
        {
            qDebug() << "<<--设备鉴权应答失败:" << result;
        }
        break;
    case TERMINAL_UP_GPS_DATA:
        qDebug() << "<<--位置信息上报应答:" << result;
        break;
    default:
        qDebug("<<--平台通用应答:ID=0x%x, 结果:%d", cmd_id, result);
        break;
    }
}

/*******************************************
函数功能: 查询服务器时间应答
入口参数: data, 消息体指针
        lenth, 消息体长度
出口参数:
备    注: 测试通过
*******************************************/
void CJT808CmdProc::procServTimeAnswer(int cycle_num, BYTE *payload, WORD lenth)
{
    QDateTime dt;
    // 年月日
    int year, month, day;
    // 时分秒
    int h, m, s;
    int idx = 0;
    // 年需要补齐千年号
    year = bcd2value(payload[idx++]) + 2000;
    month = bcd2value(payload[idx++]);
    day = bcd2value(payload[idx++]);

    // 时需要补上时区
    h = bcd2value(payload[idx++]);
    m = bcd2value(payload[idx++]);
    s = bcd2value(payload[idx++]);

    dt.setDate(QDate(year, month, day));
    dt.setTime(QTime(h, m, s));
    qDebug() << "当前服务器时间:" << dt.toString("yy年MM月dd日 hh:mm:ss");
}

/*******************************************
函数功能: 服务器注册应答
入口参数: data, 消息体指针
          lenth, 消息体长度
出口参数:
备    注: 测试通过
*******************************************/
void CJT808CmdProc::procServRegistAnswer(int cycle_num, BYTE *pPayload, WORD pay_size)
{
    switch (pPayload[2])
    {
    case 0:
    {
        // 终端注册返回尽荷消息格式为:应答流水(WORD),应答结果(BYTE),鉴权码（STRING）
        DEVICE_INFO &device = m_pTerminal->getDeviceInfo();
        if (pay_size - 3 < REGIST_CODE_LENTH)
        {
            memset(device.regist_code, 0, sizeof(device.regist_code));
            strncpy((char *)device.regist_code, (char *)&pPayload[3], REGIST_CODE_LENTH - 1);
            qDebug() << "<<--终端注册成功:" << (char *)device.regist_code;
            m_pTerminal->setServCtrlStatus(SERVER_CONNECT_REGOK);
        }
        else
        {
            qDebug() << "<<--鉴权码太长" << pay_size - 3;
        }
        break;
    }
    case 1:
        qDebug() << "<<--车辆已被注册";
        break;
    case 2:
        qDebug() << "<<--数据库无此车辆";
        break;
    case 3:
        qDebug() << "<<--终端已被注册";
        break;
    case 4:
        qDebug() << "<<--数据库无此终端";
        break;
    default:
        qDebug() << "<<--未知注册状态:%d" << pPayload[2];
        break;
    }
}

/*******************************************
函数功能: 下行透传
入口参数: data 数据包
        lenth 长度
出口参数:
备    注:
*******************************************/
WORD CJT808CmdProc::procMsgTransMission(int cycle_num, BYTE *payload, WORD lenth)
{
    qDebug() << "<<--下行透传请求";
    int idx = 0;
    BYTE type = payload[idx];
    switch (type)
    {
    case 0: /* gnss模块详细定位数据 */
        break;

    case 0x0B: /* 道路运输证IC卡信息穿透时间为30s超时不重发 */
        break;

    case 0x41: /* 串口1穿透数据下发 */
        break;

    case 0x42: /* 串口2穿透数据下发 */
        break;

    default:
        break;
    }

    qDebug() << "-->>下行透传应答";
    sendCommonAnswer(cycle_num, SERVER_TRANSMISSION, 0);
}

/*******************************************
函数功能: 发送通用应答
入口参数:
出口参数:
备    注:
*******************************************/
WORD CJT808CmdProc::sendCommonAnswer(WORD cycle_num, WORD id, BYTE result)
{
    QByteArray ba(100, 0);
    BYTE *send_data = (BYTE *)ba.data();
    WORD s_lenth;
    BYTE err;
    qDebug("-->>发送通用应答,流水号:%d, ID:0x%x, 结果:%d", cycle_num, id, result);

    /* 应答流水号 */
    send_data[0] = (BYTE)(cycle_num >> 8);
    send_data[1] = (BYTE)(cycle_num >> 0);
    /* 应答ID */
    send_data[2] = (BYTE)(id >> 8);
    send_data[3] = (BYTE)(id >> 0);
    /* 结果码 */
    send_data[4] = result;
    s_lenth = 5;

    return sendPayload2Jt808Serv(TERMINAL_NORMAL_ANSWER, send_data, s_lenth);
}

/*******************************************
函数功能: 服务端RSA密钥
入口参数: data 数据包
        lenth 长度
出口参数:
备    注:
*******************************************/
WORD CJT808CmdProc::procSetServRSA(int cycle_num, BYTE *payload, WORD lenth)
{
    qDebug() << "<<--设置服务端RSA密钥";
    int idx = 0;
    m_ServRsaCode.dwCodeE = four_char_to_long(&payload[idx]);
    idx += 4;
    memcpy(m_ServRsaCode.pbyCodeN, &payload[idx], sizeof(m_ServRsaCode.pbyCodeN));

    qDebug() << "-->>服务端RSA密钥应答";
    sendCommonAnswer(cycle_num, SERVER_RSA, 0);

    // T01 TEST
    mSleep(500);
    sendLocalRSA();
}

/*******************************************
函数功能: 本地RSA密钥上报
入口参数: data 数据包
        lenth 长度
出口参数:
备    注:
*******************************************/
WORD CJT808CmdProc::sendLocalRSA()
{
    QByteArray ba(1000, 0);
    BYTE *send_data = (BYTE *)ba.data();
    int idx = 0;
    long_to_four_char(m_LocalRsaCode.dwCodeE, &send_data[idx]);
    idx += 4;
    memcpy(&send_data[idx], m_LocalRsaCode.pbyCodeN, sizeof(m_LocalRsaCode.pbyCodeN));
    idx += sizeof(m_LocalRsaCode.pbyCodeN);
    qDebug() << "-->>本地RSA密钥上报";
    sendPayload2Jt808Serv(TERMINAL_RSA, send_data, idx);
}

/*******************************************
函数功能: 发送数据到JT08服务器
入口参数: WORD cmd,       命令字
        BYTE *payload,  净荷
        WORD lenth      净荷长度
出口参数:
备    注:
*******************************************/
WORD CJT808CmdProc::sendPayload2Jt808Serv(WORD cmd, BYTE *payload, WORD lenth)
{
    if (m_pTerminal)
    {
        m_pTerminal->sendPayload2Jt808Serv(cmd, payload, lenth);
    }
}

/*******************************************
函数功能: 服务器补发分包请求
入口参数: WORD cmd,       命令字
        BYTE *payload,  净荷
        WORD lenth      净荷长度
出口参数:
备    注:
*******************************************/
WORD CJT808CmdProc::procSepPacketAsk(int cycle_num, BYTE *payload, WORD lenth)
{
    if (m_pTerminal)
    {
        return m_pTerminal->procSepPacketAsk(cycle_num, payload, lenth);
    }
    return 0;
}
