#ifndef CJT808CMDPROC_H
#define CJT808CMDPROC_H
#include "message.h"
#include "common.h"
#include <QTcpSocket>
#include <QMap>
#include <QObject>
#include "jt808_msg.h"
#include "jt808_common.h"
#include <QMutex>

#include "CJt808Config.h"
#include "CJt808Location.h"
#include "CJt808AreaRoute.h"
#include "CJt808Information.h"
#include "CJt808Device.h"
#include "CJt808Media.h"
#include "common.h"

class CJT808CmdProc;
class CJT808Terminal;
class CJT808Terminal;

//
typedef bool (CJT808CmdProc::*JT808_COMMAND_PROC_FUNC)(void *pPara);
typedef QMap<int, JT808_COMMAND_PROC_FUNC> MAP_JT808_FUNC_PROC;

struct RSA_CODE
{
        DWORD dwCodeE;
        BYTE pbyCodeN[128];
};

class CJT808CmdProc : public CJt808Config,
                      public CJt808Location,
                      public CJt808AreaRoute,
                      public CJt808Information,
                      public CJt808Device,
                      public CJt808Media
{
public:
        explicit CJT808CmdProc();
        ~CJT808CmdProc();
        //
        void setTcpSocket(QTcpSocket *pSocket);
        //
        void HeartBeat();
        //
        bool sendRegistInfo();
        //
        bool sendAuthInfo();
        //
        bool regMessageModule(int msg, MODULE_ID id);

        // //
        // unsigned int getHeartbeatInterval();
        // //
        // unsigned int getGpsReportInterval();

        //注册808消息处理
        bool registJt808Msg();

        /*******************************************
        函数功能: 初始化
        入口参数:
        出口参数:初始化状态
        备    注:
        *******************************************/
        bool Initialize();

        /*******************************************
        函数功能: 组织位置信息
        入口参数:
        出口参数:
        备    注:
        *******************************************/
        virtual WORD fillLocationInfo(BYTE *payload, GPS_INFO &gps);

        /*******************************************
        函数功能: 发送通用应答
        入口参数:
        出口参数:
        备    注:
        *******************************************/
        virtual WORD sendCommonAnswer(WORD cycle_num, WORD id, BYTE result);

protected:
        //
        bool praseCmd(void *pPara);
        //
        void sendMsg2Serv(void *pBuff, int len);
        //
        void sendMsg2Module(MODULE_MESSAGE *pMsg);
        //
        void connectProc();
        //
        bool PraseRegist(void *pPara);

        /*******************************************
        函数功能: 服务器通用应答
        入口参数: data,  消息体指针
                lenth, 消息体长度
        出口参数:
        备    注: 测试通过
        *******************************************/
        void procServCommonAnswer(int cycle_num, BYTE *payload, WORD lenth);

        /*******************************************
        函数功能: 服务器应答时间
        入口参数: data,  消息体指针
                lenth, 消息体长度
        出口参数:
        备    注: 测试通过
        *******************************************/
        void procServTimeAnswer(int cycle_num, BYTE *payload, WORD lenth);

        /*******************************************
        函数功能: 服务器注册应答
        入口参数: data, 消息体指针
                lenth, 消息体长度
        出口参数:
        备    注: 测试通过
        *******************************************/
        void procServRegistAnswer(int cycle_num, BYTE *payload, WORD lenth);

        /*******************************************
        函数功能: 下行透传
        入口参数: data 数据包
                 lenth 长度
        出口参数:
        备    注:
        *******************************************/
        virtual WORD procMsgTransMission(int cycle_num, BYTE *payload, WORD lenth);

        /*******************************************
        函数功能: 服务器补发分包请求
        入口参数: WORD cmd,       命令字
                BYTE *payload,  净荷
                WORD lenth      净荷长度
        出口参数:
        备    注:
        *******************************************/
        virtual WORD procSepPacketAsk(int cycle_num, BYTE *payload, WORD lenth);

        /*******************************************
        函数功能: RSA 密钥
        入口参数: data 数据包
                 lenth 长度
        出口参数:
        备    注:
        *******************************************/
        virtual WORD procSetServRSA(int cycle_num, BYTE *payload, WORD lenth);

        /*******************************************
        函数功能: 本地RSA密钥上报
        入口参数: data 数据包
                lenth 长度
        出口参数:
        备    注:
        *******************************************/
        virtual WORD sendLocalRSA();

        /*******************************************
        函数功能: 发送数据到JT08服务器
        入口参数: WORD cmd,       命令字
                BYTE *payload,  净荷
                WORD lenth      净荷长度
        出口参数:
        备    注:
        *******************************************/
        virtual WORD sendPayload2Jt808Serv(WORD cmd, BYTE *payload, WORD lenth);

        /*******************************************
        函数功能: 获取jt808告警状态
        入口参数:
        出口参数:
        备    注:
        *******************************************/
        virtual ALARM_STATUS &getAlarmStatus() { return m_almStatus; }

        /*******************************************
        函数功能: 获取jt808通用状态
        入口参数:
        出口参数:
        备    注:
        *******************************************/
        virtual UNIT_STATUS &getCommonStatus() { return m_Status_stu; }

protected:
        QTcpSocket *m_pSocket;
        CJT808Terminal *m_pTerminal;

        // 处理函数标
        MAP_JT808_FUNC_PROC m_mapFuncProc;
        unsigned int m_HeartbeatInterval;
        unsigned int m_ReportInterval;

        // 服务器端RSA编码
        RSA_CODE m_ServRsaCode;
        // 本地客户的RSA编码
        RSA_CODE m_LocalRsaCode;
        // 告警状态
        ALARM_STATUS m_almStatus;
        // 通用状态
        UNIT_STATUS m_Status_stu;

        int end = 0x55aa;
};

#endif // CJT808CMDPROC_H
