#include "CJT808Terminal.h"
#include "socket/TcpServer.h"
#include "base_module/factory.h"
#include "factory.h"
#include "timer/CTimerManager.h"
#include "jt808/CJT808Terminal.h"
#include "device/CDeviceManager.h"
#include "assert.h"
#include <string>
#include <QTimeZone>
#include <QDateTime>
#include <zlib.h>

REGISTER_MODULE(CJT808Terminal, "CJT808Terminal")
LOG_DEFINITION(CJT808Terminal)
const QString JT808_DB_NAME = "jt808";

CJT808Terminal::CJT808Terminal(QObject *parent)
    : QObject(parent),
      m_pTcpModule(nullptr),
      m_pDeviceMng(nullptr),
      m_Jt808Sock(this), //创建TCP套接字对象
      m_Jt808Socket(-1),
      m_pDbJT808(nullptr),
      m_jt808MsgSem(0)
{
    LOG_CREATE(CJT808Terminal, "JT808");
    m_pCmdProc = new CJT808CmdProc;
    loadConfig();
}

CJT808Terminal::~CJT808Terminal()
{
    m_mapJt808Func.clear();
    m_jt808MsgSem.release();
    SAFE_DELETE(m_pCmdProc);
    // 离线注销
    // sendUnregistData();
    std::cout
        << name() << std::endl;
}

bool CJT808Terminal::loadConfig()
{
    m_pConfigMng = dynamic_cast<CConfigManager *>(QueryPlugin("CConfigManager"));
    if (m_pConfigMng)
    {
        m_pDbJT808 = m_pConfigMng->getDatabase(MODULE_JT808_MNG, JT808_DB_NAME);
        if (!m_pDbJT808)
        {
            createDatebase();
        }

        restoreFromDB(m_pDbJT808);
    }
    return true;
}

// 创建数据库
bool CJT808Terminal::createDatebase()
{
    LOG_TRACE(CJT808Terminal, "m_pDbJT808 is not exist, create Datebase!");
    if (m_pConfigMng)
    {
        m_pDbJT808 = m_pConfigMng->regDatabase(MODULE_JT808_MNG, JT808_DB_NAME);
        m_pDbJT808->regKey("main_ip", "47.100.112.218");
        m_pDbJT808->regKey("slave_ip", "47.100.112.218");
        m_pDbJT808->regKey("tcp_port", "8808");
        m_pDbJT808->regKey("udp_port", "8809");
        m_pDbJT808->regKey("heart_beat_interval", 60);
        m_pDbJT808->save();
    }
    else
    {
        // log_err();
    }
    return true;
}

// 从数据库恢复
bool CJT808Terminal::restoreFromDB(CDatebase *pDatebase)
{
    LOG_TRACE(CJT808Terminal, "restore Datebase!");
    return true;
}

void CJT808Terminal::saveConfig()
{
    LOG_TRACE(CJT808Terminal, "save config!");
    if (m_pDbJT808)
    {
        m_pDbJT808->save();
    }

    if (m_pDeviceMng)
    {
        m_pDeviceMng->saveConfig();
    }
}

//初始化操作
bool CJT808Terminal::Initialize()
{
    m_pCmdProc->Initialize();
    m_pCmdProc->registJt808Msg();

#define CALLBACK_TEMPLATE_SET(F) (std::bind(&CJT808Terminal::F, this, std::placeholders::_1))
#define CALLBACK_TEMPLATE_GET(F) (std::bind(&CJT808Terminal::F, this))

    /*-------------------------T808配置ID--------------------------*/
    TERMINAL_CONFIG_ITEM config_arr[] =
        {
            {SET_HEARTBEAT_TIME, VALUE_TYPE_DWORD, 60, CALLBACK_TEMPLATE_SET(setHeartBeatTime), CALLBACK_TEMPLATE_GET(getHeartBeatTime)},  // 心跳时间间隔 秒
            {SET_PROVINCE_ID, VALUE_TYPE_WORD, 0, CALLBACK_TEMPLATE_SET(setProvinceID), CALLBACK_TEMPLATE_GET(getHeartBeatTime)},          // 车辆所在省ID
            {SET_CITY_ID, VALUE_TYPE_WORD, 0, CALLBACK_TEMPLATE_SET(setCityID), CALLBACK_TEMPLATE_GET(getHeartBeatTime)},                  // 车辆所在市ID
            {SET_VEHICLE_ID, VALUE_TYPE_STRING, "", CALLBACK_TEMPLATE_SET(setVehicleNo), CALLBACK_TEMPLATE_GET(getHeartBeatTime)},         // 车牌号
            {SET_VEHICLE_COLOR, VALUE_TYPE_BYTE, 0, CALLBACK_TEMPLATE_SET(setVehicleColor), CALLBACK_TEMPLATE_GET(getHeartBeatTime)},      // 车辆颜色
            {SET_MAIN_SERVER_ADDR, VALUE_TYPE_STRING, 0, CALLBACK_TEMPLATE_SET(setMasterServIP), CALLBACK_TEMPLATE_GET(getMasterServIP)},  // 主服务器地址
            {SET_BACK_SERVER_ADDR, VALUE_TYPE_STRING, 0, CALLBACK_TEMPLATE_SET(setSlaveServIP), CALLBACK_TEMPLATE_GET(getSlaveServIP)},    // 备份服务器地址
            {SET_MAIN_SERVER_TCP_PORT, VALUE_TYPE_DWORD, 0, CALLBACK_TEMPLATE_SET(setServTcpPort), CALLBACK_TEMPLATE_GET(getServTcpPort)}, // 服务器TCP端口
            {SET_MAIN_SERVER_UDP_PORT, VALUE_TYPE_DWORD, 0, CALLBACK_TEMPLATE_SET(setServUdpPort), CALLBACK_TEMPLATE_GET(getServUdpPort)}, // 服务器UDP端口
        };

    m_pCmdProc->registConfigProc(config_arr, sizeof(config_arr) / sizeof(TERMINAL_CONFIG_ITEM));

    //起主消息处理线程
    STARTUP_TASK(TaskMain)

    return true;
}

bool CJT808Terminal::RegistMessage()
{
    m_pTcpModule = dynamic_cast<ITcpModule *>(QueryPlugin("TcpModule"));
    if (m_pTcpModule)
    {
        m_pTcpModule->RegistMessageCallback(this, MODULE_JT808_MNG);
    }
    return true;
}

//主消息任务，主要用于和其他模块之间的消息通讯
void CJT808Terminal::TaskMain(void *pBuff)
{
    (void)pBuff;
    setCurrentTaskName("tskJt808Main");
    MODULE_MESSAGE *pMsg = nullptr;
    RegistMessage();
    registTimer();
    waitForStart();

    //起主消息处理线程
    STARTUP_TASK(TaskClientRecv)
    STARTUP_TASK(TskServCtrl)
    STARTUP_TASK(TskServCmdProc)
    STARTUP_TASK(TskAlarmCheck)

    while (!isShutdown())
    {
        if (GetModuleMessage(pMsg)) // get msg from message queue
        {
            switch (pMsg->iCmd)
            {
            case TIMER_JT808_HEART_BEAT:
            {
                m_pCmdProc->HeartBeat();
                break;
            }
            case TIMER_JT808_GPS_REPORT:
            {
                ReportPosition();
                break;
            }
            case CMD_SWDL_UPDATE_RESULT:
            {
                procUpdateResult(*pMsg);
                break;
            }
            default:
                break;
            }
            SAFE_DELETE(pMsg);
        }
    }
}
// reg timer
void CJT808Terminal::registTimer()
{
    // 定时器模块
    m_pTimerMng = dynamic_cast<CTimerManager *>(::QueryPlugin("CTimerManager"));
    if (m_pTimerMng)
    {
        int interval = 60;
        if (m_pDbJT808)
        {
            interval = m_pDbJT808->getValue("heart_beat_interval").toInt();
            if (interval == 0)
            {
                interval = 60;
                m_pDbJT808->setValue("heart_beat_interval", 60);
                m_pDbJT808->save();
            }
        }

        m_pTimerMng->RegTimer(MODULE_JT808_MNG, TIMER_JT808_HEART_BEAT, interval);
    }
}

// 接收客户端消息处理线程
void CJT808Terminal::TaskClientRecv(void *pBuff)
{
    (void)pBuff;
    BYTE *pbyBuff = new BYTE[MAX_JT808_RECV_LEN_V1];
    while (!isShutdown())
    {
        if (!m_bConnected)
        {
            mSleep(200);
            continue;
        }

        memset(pbyBuff, 0, MAX_JT808_RECV_LEN_V1);
        int iLen = recv(m_Jt808Socket, pbyBuff, MAX_JT808_RECV_LEN_V1, 0); // 接收服务器端消息
        if (iLen <= 0)
        {
            mSleep(500);
            m_bConnected = false;
            m_ServerPara.ctrl_step = CTRL_STEP_WAIT_REG;
            continue;
        }
        else
        {
            QString strOut = "接收jt808消息：";
            for (int i = 0; i < iLen; i++)
            {
                QString str;
                strOut += str.sprintf("%02x ", pbyBuff[i]);
            }
            qDebug() << "=================接收消息====================";
            qDebug() << strOut;
            parseJt808RecvData(pbyBuff, iLen);
        }
    }
}

//注册jt808消息处理
bool CJT808Terminal::connectToServ()
{
    // TODO: 在此添加控件通知处理程序代码
    m_Jt808Socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP); // 指定为tcp方式

    sockaddr_in nServeraddr;
    memset(&nServeraddr, 0, sizeof(nServeraddr));
    nServeraddr.sin_family = AF_INET;
    nServeraddr.sin_port = htons(getServTcpPort().toInt());                           // htons()作用是将端口号由主机字节序转换为网络字节序的整数值。
    nServeraddr.sin_addr.s_addr = inet_addr(getMasterServIP().toStdString().c_str()); // inet_addr()作用是将一个IP字符串转化为一个网络字节序的整数值，

    if (SOCKET_ERROR == ::connect(m_Jt808Socket, (sockaddr *)&nServeraddr, sizeof(nServeraddr)))
    {
        m_bConnected = false;
        return false;
    }

    m_bConnected = true;
    return true;
}

// 处理应用层发回的升级结果消息
void CJT808Terminal::procUpdateResult(MODULE_MESSAGE &msg)
{
    int pkg_type = 0;
    bool bSuccess = false;
    int idx = 0;
    memcpy(&pkg_type, msg.szBuff + idx, sizeof(int));
    idx += sizeof(int);

    memcpy(&bSuccess, msg.szBuff + idx, sizeof(bool));
    idx += sizeof(bool);

    m_pCmdProc->answerUpdateResult(pkg_type, bSuccess);
}

//注册jt808消息处理
bool CJT808Terminal::registJt808Messages(JT808_MSG_PROC_NODE *pMsgIdxArray, int count)
{
    // 参数校验
    if (pMsgIdxArray == nullptr)
    {
        return false;
    }

    for (int i = 0; i < count; i++)
    {
        auto it = m_mapJt808Func.find(pMsgIdxArray[i].cmd);
        if (it != m_mapJt808Func.end())
        {
            // 提示错误并覆盖函数
            assert(1);
            m_mapJt808Func[pMsgIdxArray[i].cmd] = pMsgIdxArray[i].func;
        }
        else
        {
            m_mapJt808Func[pMsgIdxArray[i].cmd] = pMsgIdxArray[i].func;
        }
    }
    return true;
}

// 调试命令，打印已经注册的消息
void CJT808Terminal::displayRegistedMessage()
{
}

// 连接到服务器
bool CJT808Terminal::connectToJt808Serv(QString strIp, int port)
{
    connectToServ();
    return true;
}

//客户端槽函数
void CJT808Terminal::connected()
{
    qDebug() << "CJT808Terminal::connected  successfully";
    // 连接成功的情况下，主动发起鉴权
}

//客户端槽函数
void CJT808Terminal::disConnected()
{
    qDebug() << "CJT808Terminal::disConnected!";
}

//客户端槽函数
void CJT808Terminal::ReadData()
{
    qDebug() << "CJT808Terminal::readyRead " << m_Jt808Sock.readAll();
}

// 读取错误处理
void CJT808Terminal::ReadError(QAbstractSocket::SocketError socketError)
{
    qDebug() << "CJT808Terminal::error " << socketError;

    //非连接状态将重置连接
    if (QAbstractSocket::ConnectingState != m_Jt808Sock.state())
    {
        //　取消原有连接
        m_Jt808Sock.abort(); //取消原有连接
        //　与服务器端建立连接
        emit notifyConnect2Serv();
    }
}

/*******************************************
函数名称: TskServCtrl
函数功能: 服务器连接控制
入口参数:
出口参数:
备    注:
*******************************************/
void CJT808Terminal::TskServCmdProc(void *p_arg)
{
    while (!isShutdown())
    {
        // 取信号量
        m_jt808MsgSem.acquire();
        if (m_queueExecMsg.size() > 0)
        {
            // 取消息
            Jt808Msg *pMsg = m_queueExecMsg.front();
            if (pMsg)
            {
                // 分发消息
                dispachJt808Msg(*pMsg);

                SAFE_DELETE(pMsg);
            }
            m_queueExecMsg.pop_front();
        }
    }
}

/*******************************************
函数名称: TskAlarmCheck
函数功能: 告警检查任务
入口参数:
出口参数:
备    注:
*******************************************/
void CJT808Terminal::TskAlarmCheck(void *p_arg)
{
    while (!isShutdown())
    {
        // 电子围栏检查
        m_pCmdProc->eFenceCheck();
        // 速度越限检查
        m_pCmdProc->speedExceedCheck();
        // 其他检查
        mSleep(1000);
    }
}

void CJT808Terminal::dispachJt808Msg(Jt808Msg &msg)
{
    WORD cmd = msg.getMsgID();
    auto it = m_mapJt808Func.find(cmd);
    if (it != m_mapJt808Func.end())
    {
        QByteArray ba = msg.getPayload();
        WORD cycle_num = msg.getCycle();
        it.value()(cycle_num, (BYTE *)ba.data(), ba.length());
    }
    else
    {
        // 提示没有找到消息处理
    }
}

/*******************************************
函数名称: TskServCtrl
函数功能: 服务器连接控制
入口参数:
出口参数:
备    注:
*******************************************/
void CJT808Terminal::TskServCtrl(void *p_arg)
{
    const WORD step_time = 100;
    const DWORD wait_one_sec = 1000 / step_time;
    const DWORD server_relink_time = 30 * wait_one_sec;
    DWORD ctrl_time_add = 0; // 200MS
    BYTE login_times = 0;

    p_arg = p_arg;
    qDebug() << ("--->系统启动");
    m_ServerPara.ctrl_step = CTRL_STEP_WAIT_REG;

    if (!m_pDeviceMng)
    {
        m_pDeviceMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
    }

    while (!isShutdown())
    {
        mSleep(100);
        switch (m_ServerPara.ctrl_step)
        {
        /* 等待向服务器注册 */
        case CTRL_STEP_WAIT_REG:
            // qDebug() << ("-->>CTRL_STEP_WAIT_REG");
            m_ServerPara.state = SERVER_NO_CONNECT;
            m_ServerPara.ctrl_step = CTRL_STEP_CONNECT_SERVER;
            ctrl_time_add = 0;
            m_Device.regist_code[0] = '\0';
            break;

        /* 向服务器发起链接 */
        case CTRL_STEP_CONNECT_SERVER:
            // qDebug() << ("-->>CTRL_STEP_CONNECT_SERVER");
            // connect
            connectToServ();
            m_ServerPara.ctrl_step = CTRL_STEP_CONNECT_SERVER_WAIT;
            ctrl_time_add = 0;
            break;

        case CTRL_STEP_CONNECT_SERVER_WAIT:
            // qDebug() << ("-->>CTRL_STEP_CONNECT_SERVER_WAIT");
            if (!m_bConnected)
            {
                if (++ctrl_time_add > server_relink_time)
                {
                    qDebug() << ("808服务器连接超时");
                    m_ServerPara.ctrl_step = CTRL_STEP_CONNECT_SERVER; //重新发起链接
                    ctrl_time_add = 0;
                }
                mSleep(1000);
            }
            else
            {
                mSleep(100);
                login_times = 0;
                m_ServerPara.ctrl_step = CTRL_STEP_BEFORE_REGIST_DEVICE;
            }
            break;

        case CTRL_STEP_BEFORE_REGIST_DEVICE:
            qDebug() << ("-->>CTRL_STEP_BEFORE_REGIST_DEVICE");
            /* 查看鉴权码是否为空 */
            if (m_Device.regist_code[0])
            {
                m_ServerPara.ctrl_step = CTRL_STEP_LOGIN_SERVER;
                m_ServerPara.err_times = 0;
            }
            else
            {
                m_ServerPara.ctrl_step = CTRL_STEP_REGIST_DEVICE;
            }
            break;

        case CTRL_STEP_REGIST_DEVICE:
            qDebug() << ("-->>发送设备注册");
            sendRegistData();
            m_ServerPara.ctrl_step = CTRL_STEP_REGIST_DEVICE_WAIT;
            ctrl_time_add = 0;
            break;

        case CTRL_STEP_REGIST_DEVICE_WAIT:
            if (m_ServerPara.state == SERVER_CONNECT_REGOK)
            {
                m_ServerPara.ctrl_step = CTRL_STEP_LOGIN_SERVER;
            }
            else if (++ctrl_time_add >= 30 * wait_one_sec)
            {
                ctrl_time_add = 0;
                m_ServerPara.ctrl_step = CTRL_STEP_REGIST_DEVICE;
            }
            break;

        case CTRL_STEP_LOGIN_SERVER:
            qDebug() << "-->>发送设备鉴权:" << (char *)m_Device.regist_code;
            sendLoginData();
            m_ServerPara.ctrl_step = CTRL_STEP_LOGIN_SERVER_WAIT;
            ctrl_time_add = 0;
            break;

        /* 等待鉴权响应 */
        case CTRL_STEP_LOGIN_SERVER_WAIT:
            if (m_ServerPara.state == SERVER_CONNECT_AUTHENOK)
            {
                ctrl_time_add = 0;
                m_ServerPara.ctrl_step = CTRL_STEP_STAND_BY;
            }
            else if (++ctrl_time_add >= 30 * wait_one_sec)
            {
                if (++login_times >= 3)
                {
                    m_Device.regist_code[0] = '\0';
                    login_times = 0;
                }
                ctrl_time_add = 0;
                m_ServerPara.ctrl_step = CTRL_STEP_BEFORE_REGIST_DEVICE;
            }
            break;

        case CTRL_STEP_STAND_BY:
            location_info_send_pro(step_time);
            // heartbeat_send_pro(step_time);
            break;

        default:
            m_ServerPara.ctrl_step = CTRL_STEP_WAIT_REG;
            break;
        }
    }
}

/*******************************************
函数名称: location_info_send_pro
函数功能: 位置信息发送Pro
入口参数: step_time,调用时间间隔
出口参数:
备    注:
*******************************************/
void CJT808Terminal::location_info_send_pro(WORD step_time)
{
    static DWORD wait_time_add = 600;
    DWORD send_interva = 0;

    send_interva = m_Device.send_time_inteval_default;

    if (send_interva < 1 || send_interva > 900)
    {
        send_interva = 30;
    }

    if (++wait_time_add >= send_interva * 1000 / step_time)
    {
        ReportPosition();
        wait_time_add = 0;
    }
}

/*******************************************
函数名称: heartbeat_send_pro
函数功能: 心跳发送Pro
入口参数: step_time,调用时间间隔
出口参数:
备    注:
*******************************************/
void CJT808Terminal::heartbeat_send_pro(WORD step_time)
{
    static DWORD wait_time_add = 20;
    DWORD send_interva = 60;

    if (++wait_time_add >= send_interva * 1000 / step_time)
    {
        sendHeatBeat();
        wait_time_add = 0;
    }
}

#include <stdio.h>
/*******************************************
函数功能: 发送数据
入口参数:
出口参数:
备    注:
*******************************************/
WORD CJT808Terminal::sendJt808Msg2Serv(BYTE *pbyBuff, WORD lenth)
{
#ifdef WIN32
    int con_para = 0;
#else
    int con_para = MSG_NOSIGNAL;
#endif
    QString strOut = "发送jt808消息：";
    for (int i = 0; i < lenth; i++)
    {
        QString str;
        strOut += str.sprintf("%02x ", pbyBuff[i]);
    }
    qDebug() << "=================发送消息====================";
    qDebug() << strOut;

    QMutexLocker locker(&m_mtxSend);
    int ret = send(m_Jt808Socket, pbyBuff, lenth, con_para);
    if (ret < 0)
    {
        int nError = -1;
        cout << ("发送失败:") << nError << endl;
        m_bConnected = false;
        m_ServerPara.ctrl_step = CTRL_STEP_WAIT_REG;
        return -1;
    }

    return ret;
}

/*******************************************
函数功能: 发送数据到JT08服务器
入口参数: BYTE *pbyBuff,  净荷
        WORD lenth      净荷长度
出口参数:
备    注:
*******************************************/
WORD CJT808Terminal::sendPayload2Jt808Serv(WORD cmd, BYTE *send_buff, WORD lenth)
{
    // 每个包最多1023个长度,如果超过1023长度则统一每个包1000长度发送
    if (lenth < 1023)
    {
        Jt808MsgNode msg_node;
        msg_node.setPayload(send_buff, lenth);
        int send_len = msg_node.generateMsg(this->getPhoneNum().toStdString(), cmd, lenth);
        sendJt808Msg2Serv(msg_node.m_pbyBuffer, send_len);
        msg_node.release();
    }
    else // 多包发送需要缓存,等待受到消息完成后或者超时的情况下,删除缓冲
    {
        int left_len = lenth;
        WORD total_pkt = lenth / 1000 + 1;
        WORD cur_pkt = 1;
        WORD cycle_num = INVALID_CYCLE_NUM;
        int buff_idx = 0;
        while (left_len > 0)
        {
            Jt808MsgNode msg_node;
            int data_len = min(1000, left_len);
            int copy_len = msg_node.setPayload(send_buff + buff_idx, data_len, total_pkt, cur_pkt);
            buff_idx += copy_len;
            int send_len = msg_node.generateMsg(this->getPhoneNum().toStdString(), cmd, copy_len, true, cycle_num);

            sendJt808Msg2Serv(msg_node.m_pbyBuffer, send_len);

            if (cur_pkt == 1)
            {
                cycle_num = msg_node.getCycle();

                // m_mapSendingMsg[cycle_num] = new Jt808Msg(msg_node);
            }
            else
            {
                // m_mapSendingMsg[cycle_num]->addMessageNode(msg_node);
            }

            left_len = left_len - buff_idx;
            cur_pkt++;
        };
    }
}

// 消息解析
bool CJT808Terminal::procMessage(Jt808MsgNode &msg_node)
{
    if (msg_node.isMultiPack())
    {
        // 获取流水号
        WORD incNum = msg_node.getCycle();
        // 多包消息中查找流水号
        auto it = m_mapPrepareMsg.find(incNum);
        if (it != m_mapPrepareMsg.end())
        {
            // 查找到了直接加入到对应的消息节点
            Jt808Msg *pMsg = it.value();
            if (pMsg)
            {
                pMsg->addMessageNode(msg_node);
            }
            else
            {
                assert(0);
            }
        }
        else
        {
            // 没有找到则新建消息
            m_mapPrepareMsg[incNum] = new Jt808Msg();
            m_mapPrepareMsg[incNum]->addMessageNode(msg_node);
            it = m_mapPrepareMsg.find(incNum);
        }

        // 消息完整的情况下,将消息加入处理队列
        if (it != m_mapPrepareMsg.end())
        {
            Jt808Msg *pMsg = it.value();
            if (pMsg)
            {
                if (pMsg->isMsgComplete())
                {
                    m_queueExecMsg.push_back(it.value());
                    m_mapPrepareMsg.remove(it.key());
                    m_jt808MsgSem.release();
                }
            }
            else
            {
                assert(0);
            }
        }
    }
    else
    {
        Jt808Msg *pMsg = new Jt808Msg(msg_node);
        m_queueExecMsg.push_back(pMsg);
        m_jt808MsgSem.release();
    }
    return true;
}
/*******************************************
函数功能: 服务器数据解析
入口参数:
出口参数:
备    注:
*******************************************/
BYTE CJT808Terminal::parseJt808RecvData(BYTE *rdata, WORD lenth)
{
    BYTE *data_ptr = rdata;
    int len_left = lenth;
    // 这里防止多个消息包粘包，需要逐字解析
    do
    {
        Jt808MsgNode msg_node;
        // 构造jt808msg 剩余长度不为空
        data_ptr = msg_node.rebuildMsg(data_ptr, len_left);

        // 消息解析
        procMessage(msg_node);

        // 刷新剩余len
        if (data_ptr)
        {
            len_left = len_left - (data_ptr - rdata);
        }
        else
        {
            len_left = 0;
        }

    } while (data_ptr && len_left > JT808_HEAD_LENGTH); //余数据指针不为null

    return 0;
}

void CJT808Terminal::jt808test()
{
    // TERMINAL_SERV_TIME_ASK
    sendQueryServTime();
    // heat_beat
    sendHeatBeat();
    // 注销
    // sendUnregistData();
}

/*******************************************
函数功能: 文本信息下发
入口参数: data, 消息体指针
          lenth, 消息体长度
出口参数:
备    注: 测试通过
*******************************************/
void CJT808Terminal::gprs_msg_dsp(BYTE *data, WORD lenth)
{
    BYTE *msg = data + 1;
    WORD msg_lenth = lenth - 1;
    WORD i;

    data[lenth] = 0;
    qDebug() << ("<<--文本信息下发:%0.2X,%s", data[0], &data[1]);

    /* 检验头和尾 */
    if (!stringIsSame((char *)msg, "*zl,") && msg[msg_lenth - 1] != '#')
    {
        return;
    }

    /* 向指定手机号发送终端信息 |*zl,info,13335197169# */
    if (stringIsSame((char *)msg + 4, "info"))
    {
        if (check_para_num(msg) != 2)
        {
            return;
        }

        if ((msg_lenth - strlen("*zl,info,") - 1) != 11)
        {
            return;
        }

        for (i = 0; i < 11; i++)
        {
            if (!isdigit(msg[msg_lenth - 12 + i]))
            {
                return;
            }
        }
        msg[msg_lenth - 1] = '\0';
        sendSMS((char *)&msg[msg_lenth - 12],
                "ID:%s\r\n版本:%s@\r\n服务器:%d.%d.%d.%d:%d",
                m_Device.phone_num, "1.0",
                (BYTE)(m_Device.server_ip_int >> 24), (BYTE)(m_Device.server_ip_int >> 16),
                (BYTE)(m_Device.server_ip_int >> 8), (BYTE)(m_Device.server_ip_int >> 0),
                m_Device.server_port);
    }

    /* 向指定手机号发送报警手机号 |*zl,anum,13335197169# */
    else if (stringIsSame((char *)msg + 4, "anum"))
    {
        if (check_para_num(msg) != 2)
        {
            return;
        }

        if ((msg_lenth - strlen("*zl,anum,") - 1) != 11)
        {
            return;
        }

        for (i = 0; i < 11; i++)
        {
            if (!isdigit(msg[msg_lenth - 12 + i]))
            {
                return;
            }
        }
        msg[msg_lenth - 1] = '\0';
        sendSMS((char *)&msg[msg_lenth - 12], "ID:%s\r\n报警号1:%s\r\n报警号2:%s",
                m_Device.phone_num, m_Device.alarm_sms_phone[0], m_Device.alarm_sms_phone[1]);
    }
}

/* 添加DWORD参数 */
WORD CJT808Terminal::add_long_cfg(BYTE *ptr, BYTE *num, DWORD id, DWORD cfg)
{
    long_to_four_char(id, ptr);
    ptr[4] = 4;
    long_to_four_char(cfg, ptr + 5);
    (*num)++;
    return 9;
}

/* 添加STRING参数 */
WORD CJT808Terminal::add_string_cfg(BYTE *ptr, BYTE *num, DWORD id, BYTE *cfg)
{
    long_to_four_char(id, ptr);
    ptr[4] = 4;
    strcpy((char *)ptr + 5, (char *)cfg);
    (*num)++;
    return (strlen((char *)cfg) + 5);
}

/*******************************************
函数功能: 发送注销信息
入口参数:
出口参数:
备    注:
*******************************************/
void CJT808Terminal::sendUnregistData(void)
{
    qDebug() << "-->>发送注销信息";
    sendPayload2Jt808Serv(TERMINAL_DEREGIST, nullptr, 0);
}

/*******************************************
函数功能: 发送注册信息
入口参数:
出口参数:
备    注:
*******************************************/
void CJT808Terminal::sendRegistData(void)
{
    QByteArray ba(4000, 0);
    BYTE *send_data = (BYTE *)ba.data();

    WORD province = 0x007E;
    WORD city = 0x007D;
    QString manufactor = 0;
    QString terminal_type = "HNCY_AI";
    QString terminal_id = "HNCY_AI_00001";
    BYTE vehicle_color = 1;
    QString vehicle_no;

    assert(m_pDeviceMng);
    province = m_pDeviceMng->getProvinceID();
    city = m_pDeviceMng->getCityID();
    manufactor = m_pDeviceMng->getManufactor();
    terminal_type = m_pDeviceMng->getTerminalType();
    terminal_id = m_pDeviceMng->getTerminalID();
    vehicle_color = m_pDeviceMng->getVehicleColor();
    vehicle_no = m_pDeviceMng->getVehicleNo().toString();

    // 为了防止大小头不一样，这里不能用memcpy的方式对值作拷贝
    /* 省 */
    int idx = 0;
    short_to_two_char(province, send_data + idx);
    idx += 2;

    /* 市 */
    short_to_two_char(city, send_data + idx);
    idx += 2;

    /* 制造商ID 11个字符*/
    strncpy((char *)&send_data[idx], manufactor.toStdString().c_str(), 10);
    idx += 11;

    /* 终端型号 */
    memset(&send_data[idx], 0, 30);
    strncpy((char *)&send_data[idx], terminal_type.toStdString().c_str(), 29);
    idx += 30;

    /* 设备ID */
    memset(&send_data[idx], 0, 30);
    strncpy((char *)&send_data[idx], terminal_id.toStdString().c_str(), 29);
    idx += 30;

    /* 车牌颜色 */
    send_data[idx++] = vehicle_color;

    /* 车牌号 */
    int len = utf8ToGb2312((char *)vehicle_no.toStdString().c_str(), (char *)&send_data[idx], 20);
    //    int len = vehicle_no.toLocal8Bit().length();
    //    strncpy((char *)&send_data[idx], vehicle_no.toLocal8Bit().data(), len);
    idx += len + 1;

    sendPayload2Jt808Serv(TERMINAL_REGIST, send_data, idx);
}

/*******************************************
函数功能: 发送鉴权信息
入口参数:
出口参数:
备    注: 2019版本鉴权消息格式：鉴权码长度（BYTE）鉴权码内容（string）终端IMEI号（BYTE[15]）软件版本号[BYTE[20]]
*******************************************/
void CJT808Terminal::sendLoginData(void)
{
    QByteArray ba(4000, 0);
    BYTE *send_data = (BYTE *)ba.data();
    assert(m_pDeviceMng);

    int idx = 0;
    int str_len = strlen((char *)m_Device.regist_code);
    // 鉴权码长度（BYTE）
    send_data[idx++] = (BYTE)str_len;
    // 鉴权码内容（string）
    memcpy(&send_data[idx], (char *)m_Device.regist_code, str_len);
    idx += str_len;
    // 终端IMEI号（BYTE[15]）
    strncpy((char *)&send_data[idx], (char *)m_pDeviceMng->getIMEI().toStdString().c_str(), 15);
    idx += 15;
    // 软件版本号[BYTE[20]]
    strncpy((char *)&send_data[idx], (char *)m_pDeviceMng->getSoftwareVersion().toStdString().c_str(), 20);
    idx += 20;

    sendPayload2Jt808Serv(TERMINAL_AUTHENTICATION, send_data, idx);
}

/*******************************************
函数功能: 发送心跳
入口参数:
出口参数:
备    注:
*******************************************/
void CJT808Terminal::sendHeatBeat(void)
{
    qDebug() << "-->>发送心跳上报";
    sendPayload2Jt808Serv(TERMINAL_HEARTBEAT, nullptr, 0);
}

/*******************************************
函数功能: 查询服务器时间
入口参数:
出口参数:
备    注:
*******************************************/
void CJT808Terminal::sendQueryServTime(void)
{
    qDebug() << "-->>查询服务器时间";
    sendPayload2Jt808Serv(TERMINAL_SERV_TIME_ASK, nullptr, 0);
}

/*******************************************
函数功能: 上报位置信息，涉及到安全问题，这里需要等待鉴权成功后，才能上报，所以在终端类作预处理
入口参数:
出口参数:
备    注:
*******************************************/
void CJT808Terminal::ReportPosition(bool bTempRep)
{
    if (m_ServerPara.state == SERVER_CONNECT_AUTHENOK)
    {
        m_pCmdProc->reportLocationInfo(bTempRep);
    }
}

/*******************************************
函数功能: 上行透传
入口参数: type 透传类型 GNSS(0)  IC卡信息(0xb) 串口1(0x41) 串口2(0x42)
        data 数据包
        lenth 长度
出口参数:
备    注:
*******************************************/
WORD CJT808Terminal::sendTransMission(int cycle_num, BYTE type, BYTE *data, WORD lenth)
{
    QByteArray ba(lenth + 100, 0);
    BYTE *send_data = (BYTE *)ba.data();
    int idx = 0;
    send_data[idx++] = type;
    memcpy(&send_data[idx], data, lenth);
    idx += lenth;

    qDebug() << "-->>上行透传上报";
    sendPayload2Jt808Serv(TERMINAL_TRANSMISSION, send_data, idx);
    return 0;
}

/*******************************************
函数功能: 压缩上报
入口参数: data 数据包
         lenth 长度
出口参数:
备    注:
*******************************************/
WORD CJT808Terminal::sendZipData(int cycle_num, BYTE *data, WORD lenth)
{
    QByteArray ba(lenth + 100, 0);
    BYTE *send_data = (BYTE *)ba.data();
    int idx = 4;
    //    //zip buff
    //    int zip_len = gzCompress(data, lenth, &send_data[idx], lenth + 100);
    //    idx += zip_len;
    //    long_to_four_char(zip_len, &send_data[0]);

    qDebug() << "-->>压缩上报";
    return sendPayload2Jt808Serv(TERMINAL_UP_ZIP, send_data, idx);
}

/*******************************************
函数功能: 获取电话号码
入口参数:
出口参数:QString 电话号码
备    注:
*******************************************/
QString CJT808Terminal::getPhoneNum()
{
    QString phone_num;
    if (m_pDeviceMng)
    {
        phone_num = m_pDeviceMng->getPhoneNumber().toString();
    }
    return phone_num;
}

/*******************************************
函数功能: 设置电话号码
入口参数: QString phone_num 电话号码
出口参数:
备    注:
*******************************************/
void CJT808Terminal::setPhoneNum(QString phone_num)
{
    if (m_pDeviceMng)
    {
        m_pDeviceMng->setPhoneNumber(phone_num);
    }
}

/*******************************************
函数功能: 服务器补发分包请求
入口参数: WORD cmd,       命令字
        BYTE *payload,  净荷
        WORD lenth      净荷长度
出口参数:
备    注:
*******************************************/
WORD CJT808Terminal::procSepPacketAsk(WORD cmd, BYTE *payload, WORD lenth)
{
    int idx = 0;
    WORD cycle_num = two_char_to_short(payload + idx);
    idx += 2;
    WORD pkt_cnt = two_char_to_short(payload + idx);
    idx += 2;

    auto it = m_mapSendingMsg.find((int)cycle_num);
    if (it != m_mapSendingMsg.end())
    {
        Jt808Msg *pMsg = it.value();
        if (pMsg)
        {
            int total_cnt = pMsg->getCycle();
            for (int i = 0; i < pkt_cnt; i++)
            {
                WORD id = two_char_to_short(payload + idx);
                idx += 2;
                Jt808MsgNode *pNode = pMsg->getMsgNode(id);
                if (pNode)
                {
                    sendJt808Msg2Serv(pNode->m_pbyBuffer, pNode->m_ilenth);
                }
            }
        }
        else
        {
            assert(0);
        }
    }
}

/*******************************************
函数功能: 释放发送完毕的多包消息
入口参数:
出口参数:
备    注:在处理完消息或多次发送重传超时后操作
*******************************************/
void CJT808Terminal::releaseSendMultiMsg(WORD cycle_num)
{
    auto it = m_mapSendingMsg.find((int)cycle_num);
    if (it != m_mapSendingMsg.end())
    {
        Jt808Msg *pMsg = it.value();
        SAFE_DELETE(pMsg);
    }
    m_mapSendingMsg.remove(cycle_num);
}

/*******************************************
函数功能: 设置心跳时间间隔
入口参数:
出口参数:
备    注:
*******************************************/
void CJT808Terminal::setHeartBeatTime(QString value)
{
    int interval = value.toInt();
    m_pTimerMng->ChangeInterval(TIMER_JT808_HEART_BEAT, interval);
    m_pDbJT808->setValue("heart_beat_interval", value);
}

/*******************************************
函数功能: 获取心跳时间间隔
入口参数:
出口参数:
备    注:
*******************************************/
QString CJT808Terminal::getHeartBeatTime()
{
    return m_pDbJT808->getValue("heart_beat_interval").toString();
}

/*******************************************
函数功能: 设置城市ID
入口参数:
出口参数:
备    注:
*******************************************/
void CJT808Terminal::setCityID(QString value)
{
    if (m_pDeviceMng)
    {
        m_pDeviceMng->setCityID(value.toInt());
    }
}

/*******************************************
函数功能: 获取城市ID
入口参数:
出口参数:
备    注:
*******************************************/
QString CJT808Terminal::getCityID()
{
    QString city_id = "0";
    if (m_pDeviceMng)
    {
        city_id = QVariant(m_pDeviceMng->getCityID()).toString();
    }
    return city_id;
}

/*******************************************
函数功能: 获取省份ID
入口参数:
出口参数:
备    注:
*******************************************/
QString CJT808Terminal::getProvinceID()
{
    QString province_id = "0";
    if (m_pDeviceMng)
    {
        province_id = QVariant(m_pDeviceMng->getProvinceID()).toString();
    }
    return province_id;
}

/*******************************************
函数功能: 设置省份ID
入口参数:
出口参数:
备    注:
*******************************************/
void CJT808Terminal::setProvinceID(QString value)
{
    if (m_pDeviceMng)
    {
        m_pDeviceMng->setProvinceID(value.toInt());
    }
}
/*******************************************
函数功能: 设置车辆运营类型(颜色 蓝牌 黄牌)
入口参数:
出口参数:
备    注:
*******************************************/
QString CJT808Terminal::getVehicleColor()
{
    if (m_pDeviceMng)
    {
        return QVariant(m_pDeviceMng->getVehicleColor()).toString();
    }
    return 0;
}

/*******************************************
函数功能: 设置车辆运营类型(颜色 蓝牌 黄牌)
入口参数:
出口参数:
备    注:
*******************************************/
void CJT808Terminal::setVehicleColor(QString color)
{
    if (m_pDeviceMng)
    {
        m_pDeviceMng->setVehicleColor(color.toInt());
    }
}

/*******************************************
函数功能: 获取车牌号码
入口参数:
出口参数: QString  车牌号码
备    注:
*******************************************/
QString CJT808Terminal::getVehicleNo()
{
    QString vehicle_no;
    if (m_pDeviceMng)
    {
        vehicle_no = m_pDeviceMng->getVehicleNo().toString();
    }
    return vehicle_no;
}

/*******************************************
函数功能: 设置车牌号码
入口参数: QString 车牌号码
出口参数:
备    注:
*******************************************/
void CJT808Terminal::setVehicleNo(QString vehicle_no)
{
    if (m_pDeviceMng)
    {
        m_pDeviceMng->setVehicleNo(vehicle_no);
    }
}

/*******************************************
函数功能: 设置主服务器IP地址
入口参数: QString  ip
出口参数:
备    注:
*******************************************/
void CJT808Terminal::setMasterServIP(QString serv_ip)
{
    assert(m_pDbJT808);
    if (m_pDbJT808->getValue("main_ip").toString() != serv_ip)
    {
        m_pDbJT808->setValue("main_ip", serv_ip);
        m_ServerPara.ctrl_step = CTRL_STEP_WAIT_REG;
        //更改IP需要重置注册码
        memset(m_Device.regist_code, 0, sizeof(m_Device.regist_code));
        saveConfig();
    }
}
/*******************************************
函数功能: 获取主服务器IP地址
入口参数:
出口参数: QString  IP
备    注:
*******************************************/
QString CJT808Terminal::getMasterServIP()
{
    assert(m_pDbJT808);
    QString ip = m_pDbJT808->getValue("main_ip").toString();
    //    QString port = m_pDbJT808->getValue("tcp_port").toString();
    return ip;
}

/*******************************************
函数功能: 设置备用服务器IP地址
入口参数: QString  IP
出口参数:
备    注:
*******************************************/
void CJT808Terminal::setSlaveServIP(QString serv_ip)
{
    assert(m_pDbJT808);
    if (m_pDbJT808->getValue("slave_ip").toString() != serv_ip)
    {
        m_pDbJT808->setValue("slave_ip", serv_ip);
        saveConfig();
    }
}

/*******************************************
函数功能: 设置备服务器IP地址
入口参数:
出口参数: QString  IP
备    注:
*******************************************/
QString CJT808Terminal::getSlaveServIP()
{
    assert(m_pDbJT808);

    QString ip = m_pDbJT808->getValue("slave_ip").toString();
    //    QString port = m_pDbJT808->getValue("tcp_port").toString();
    return ip;
}

/*******************************************
函数功能: 设置服务器TCP端口
入口参数: QString  端口
出口参数:
备    注:
*******************************************/
void CJT808Terminal::setServTcpPort(QString port)
{
    assert(m_pDbJT808);
    if (m_pDbJT808->getValue("tcp_port").toString() != port)
    {
        m_pDbJT808->setValue("tcp_port", port);
        m_Device.regist_code[0] = '\0';
        m_ServerPara.ctrl_step = CTRL_STEP_WAIT_REG;
        saveConfig();
    }
}
/*******************************************
函数功能: 获取服务器TCP端口
入口参数:
出口参数: QString  端口
备    注:
*******************************************/
QString CJT808Terminal::getServTcpPort()
{
    assert(m_pDbJT808);
    return m_pDbJT808->getValue("tcp_port").toString();
}

/*******************************************
函数功能: 设置服务器UDP端口
入口参数: QString  端口
出口参数:
备    注:
*******************************************/
void CJT808Terminal::setServUdpPort(QString port)
{
    assert(m_pDbJT808);
    if (m_pDbJT808->getValue("udp_port").toString() != port)
    {
        m_pDbJT808->setValue("udp_port", port);
        saveConfig();
    }
}
/*******************************************
函数功能: 获取服务器UDP端口
入口参数:
出口参数: QString  端口
备    注:
*******************************************/
QString CJT808Terminal::getServUdpPort()
{
    assert(m_pDbJT808);
    return m_pDbJT808->getValue("udp_port").toString();
}

/*******************************************
函数功能: 发送通用应答
入口参数:
出口参数:
备    注:
*******************************************/
WORD CJT808Terminal::sendCommonAnswer(WORD cycle_num, WORD id, BYTE result)
{
    assert(m_pCmdProc);
    return m_pCmdProc->sendCommonAnswer(cycle_num, id, result);
}
