#ifndef CJT808MANAGER_H
#define CJT808MANAGER_H
#include "base_module.h"
#include <QObject>
#include <QDateTime>
#include "jt808_msg.h"
#include "jt808_common.h"
#include <QTcpSocket>
#include "message.h"
#include "jt808_msg.h"
#include <QMutex>
#include <QQueue>
#include <QSemaphore>
#include <QMap>
#include "CJT808CmdProc.h"
#include "./log/CLogManager.h"
#include <QMap>
#include <functional>

class ITcpModule;
class CTimerManager;
class CDeviceManager;
class CDatebase;
class CConfigManager;

// JT808 消息队列
typedef QQueue<Jt808Msg *> JT808_MESSAGE_QUEUE;
// JT808 消息map主要用于多包消息中间保存
typedef QMap<int, Jt808Msg *> JT808_MESSAGE_MAP;

typedef std::function<void(WORD inc_num, BYTE *, WORD)> jt808_msg_proc;

// jt808下发消息字段没有被占用，直接使用原始字段信息下发
struct JT808_MSG_PROC_NODE
{
    int cmd;
    jt808_msg_proc func;
};

class CJT808Terminal : public QObject,
                       public IModule
{
    Q_OBJECT
    Q_PROPERTY(QString main_ip READ getMasterServIP WRITE setMasterServIP)
    Q_PROPERTY(QString slave_ip READ getSlaveServIP WRITE setSlaveServIP)
    Q_PROPERTY(QString tcp_port READ getServTcpPort WRITE setServTcpPort)
    Q_PROPERTY(QString udp_port READ getServUdpPort WRITE setServUdpPort)
    Q_PROPERTY(QString phone_num READ getPhoneNum WRITE setPhoneNum)
    Q_PROPERTY(QString vehicle_no READ getVehicleNo WRITE setVehicleNo)

public:
    LOG_DECLARE()
    MODULE_NAME(CJT808Terminal)
    explicit CJT808Terminal(QObject *parent = nullptr);
    ~CJT808Terminal();

    //注册jt808消息处理
    bool registJt808Messages(JT808_MSG_PROC_NODE *pMsgIdxArray, int count);

    void dispachJt808Msg(Jt808Msg &msg);

    // 连接到服务器
    bool connectToJt808Serv(QString strIp = "", int port = 0);

    void jt808test();

    // 保存配置信息
    virtual void saveConfig();
signals:
    //
    void notifyConnect2Serv();
public slots:
    //连接成功
    void connected();
    //连接断开
    void disConnected();
    //客户端槽函数
    void ReadData();
    // 读取错误处理
    void ReadError(QAbstractSocket::SocketError);

    /*******************************************
    函数名称: connectToServ
    函数功能: 连接服务器
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    bool connectToServ();

    /*******************************************
    函数功能: 设置服务器控制状态
    入口参数: SERVER_STATUS &status 控制状态
    出口参数:
    备    注:在处理完消息或多次发送重传超时后操作
    *******************************************/
    void setServCtrlStatus(SERVER_STATUS status) { m_ServerPara.state = status; }

    /*******************************************
    函数功能: 查询服务器控制步骤
    入口参数:
    出口参数: int 控制步骤
    备    注:在处理完消息或多次发送重传超时后操作
    *******************************************/
    int getServCtrlStep() { return m_ServerPara.ctrl_step; }

    /*******************************************
    函数功能: 释放发送完毕的多包消息
    入口参数:
    出口参数:
    备    注:在处理完消息或多次发送重传超时后操作
    *******************************************/
    void releaseSendMultiMsg(WORD cycle_num);

    /*******************************************
    函数功能: 查询设备信息
    入口参数:
    出口参数: 设备信息
    备    注:在处理完消息或多次发送重传超时后操作
    *******************************************/
    DEVICE_INFO &getDeviceInfo() { return m_Device; }

    /*******************************************
    函数功能: 发送数据
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    WORD sendJt808Msg2Serv(BYTE *pbyBuff, WORD lenth);

    /*******************************************
    函数功能: 发送数据到JT08服务器
    入口参数: BYTE *pbyBuff,  净荷
             WORD lenth      净荷长度
    出口参数:
    备    注:
    *******************************************/
    WORD sendPayload2Jt808Serv(WORD cmd, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 服务器补发分包请求
    入口参数: WORD cmd,       命令字
            BYTE *payload,  净荷
            WORD lenth      净荷长度
    出口参数:
    备    注:
    *******************************************/
    WORD procSepPacketAsk(WORD cmd, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 发送通用应答
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    WORD sendCommonAnswer(WORD cycle_num, WORD id, BYTE result);

protected:
    //主消息任务，主要用于和其他模块之间的消息通讯
    THREAD_DEFINITION(CJT808Terminal, TaskMain)
    // 接收客户端消息处理线程
    THREAD_DEFINITION(CJT808Terminal, TaskClientRecv)
    // 终端流程控制程序
    THREAD_DEFINITION(CJT808Terminal, TskServCtrl)
    // 服务命令处理函数
    THREAD_DEFINITION(CJT808Terminal, TskServCmdProc)
    // 告警检查函数
    THREAD_DEFINITION(CJT808Terminal, TskAlarmCheck)

    //主消息任务，主要用于和其他模块之间的消息通讯
    virtual void TaskMain(void *pBuff);
    // 接收客户端消息处理线程
    virtual void TaskClientRecv(void *pBuff);

    // 处理应用层发回的升级结果消息
    void procUpdateResult(MODULE_MESSAGE &msg);

    /*******************************************
    函数名称: Server_Ctr_task
    函数功能: 服务器连接控制
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    void TskServCtrl(void *p_arg);

    /*******************************************
    函数名称: Server_Ctr_task
    函数功能: 服务器消息处理任务
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    void TskServCmdProc(void *p_arg);

    /*******************************************
    函数名称: Server_Ctr_task
    函数功能: 告警检查任务
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    void TskAlarmCheck(void *p_arg);

    // 保存配置文件
    bool restoreFromDB(CDatebase *pDatebase);

    // 创建配置文件
    bool createDatebase();

    // 导入配置
    virtual bool loadConfig();

    /*******************************************
    函数功能: 获取电话号码
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    virtual QString getPhoneNum();

    /*******************************************
    函数功能: 设置电话号码
    入口参数: QString phone_num 电话号码
    出口参数:
    备    注:
    *******************************************/
    virtual void setPhoneNum(QString phone_num);

    //初始化操作
    virtual bool Initialize();

    //模块通信注册接口
    bool RegistMessage();

    // 调试命令，打印已经注册的消息
    void displayRegistedMessage();

    // reg timer
    void registTimer();

    // 消息解析
    bool procMessage(Jt808MsgNode &msg);

    // 注册jt808响应处理消息
    bool regJt808Cmd(int jt808_id, int module_id, int msg_id);

    /*******************************************
    函数名称: location_info_send_pro
    函数功能: 位置信息发送Pro
    入口参数: step_time,调用时间间隔
    出口参数:
    备    注:
    *******************************************/
    void location_info_send_pro(WORD step_time);

    /*******************************************
    函数名称: heartbeat_send_pro
    函数功能: 心跳发送Pro
    入口参数: step_time,调用时间间隔
    出口参数:
    备    注:
    *******************************************/
    void heartbeat_send_pro(WORD step_time);

    /*******************************************
    函数功能: 服务器数据解析
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    BYTE parseJt808RecvData(BYTE *rdata, WORD lenth);

    /*******************************************
    函数功能: 发送注册信息
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    void sendRegistData(void);

    /*******************************************
    函数功能: 发送注销信息
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    void sendUnregistData(void);

    /*******************************************
    函数功能: 发送鉴权信息
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    void sendLoginData(void);

    /*******************************************
    函数功能: 发送心跳
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    void sendHeatBeat(void);
    /*******************************************
    函数功能: 查询服务器时间
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    void sendQueryServTime(void);

    /*******************************************
    函数功能: 文本信息下发
    入口参数: data, 消息体指针
            lenth, 消息体长度
    出口参数:
    备    注: 测试通过
    *******************************************/
    void gprs_msg_dsp(BYTE *data, WORD lenth);

    /* 添加DWORD参数 */
    WORD add_long_cfg(BYTE *ptr, BYTE *num, DWORD id, DWORD cfg);

    /* 添加STRING参数 */
    WORD add_string_cfg(BYTE *ptr, BYTE *num, DWORD id, BYTE *cfg);

    /*******************************************
    函数功能: 上报位置信息，涉及到安全问题，这里需要等待鉴权成功后，才能上报，所以在终端类作预处理
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    void ReportPosition(bool bTempRep = false);

    /*******************************************
    函数功能: 上行透传
    入口参数: type 透传类型 GNSS(0)  IC卡信息(0xb) 串口1(0x41) 串口2(0x42)
             data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD sendTransMission(int cycle_num, BYTE type, BYTE *data, WORD lenth);

    /*******************************************
    函数功能: 压缩上报
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD sendZipData(int cycle_num, BYTE *data, WORD lenth);

    /*******************************************
    函数功能: 设置心跳时间间隔
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    void setHeartBeatTime(QString value);

    /*******************************************
    函数功能: 获取心跳时间间隔
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    QString getHeartBeatTime();

    /*******************************************
    函数功能: 获取车辆ID
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    virtual QString getVehicleNo();

    /*******************************************
    函数功能: 设置车辆ID
    入口参数: QString vehicle_no 车牌号
    出口参数:
    备    注:
    *******************************************/
    virtual void setVehicleNo(QString vehicle_no);

    /*******************************************
    函数功能: 设置车辆运营类型(颜色 蓝牌 黄牌)
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    void setVehicleColor(QString value);

    /*******************************************
    函数功能: 获取车辆运营类型(颜色 蓝牌 黄牌)
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    QString getVehicleColor();
    /*******************************************
    函数功能: 获取省份ID
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    QString getProvinceID();

    /*******************************************
    函数功能: 设置省份ID
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    void setProvinceID(QString value);

    /*******************************************
    函数功能: 设置城市ID
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    void setCityID(QString value);

    /*******************************************
    函数功能: 获取城市ID
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    QString getCityID();

protected:
    /*******************************************
    函数功能: 设置主服务器IP地址
    入口参数: QString  ip
    出口参数:
    备    注:
    *******************************************/
    void setMasterServIP(QString serv_ip);

    /*******************************************
    函数功能: 获取主服务器IP地址
    入口参数:
    出口参数: QString  ip
    备    注:
    *******************************************/
    QString getMasterServIP();

    /*******************************************
    函数功能: 设置备用服务器IP地址
    入口参数: QString  ip
    出口参数:
    备    注:
    *******************************************/
    void setSlaveServIP(QString serv_ip);

    /*******************************************
    函数功能: 设置备服务器IP地址
    入口参数:
    出口参数: QString  ip
    备    注:
    *******************************************/
    QString getSlaveServIP();

    /*******************************************
    函数功能: 设置服务器TCP端口
    入口参数: QString  端口
    出口参数:
    备    注:
    *******************************************/
    void setServTcpPort(QString port);
    /*******************************************
    函数功能: 获取服务器TCP端口
    入口参数:
    出口参数: QString  端口
    备    注:
    *******************************************/
    QString getServTcpPort();

    /*******************************************
    函数功能: 设置服务器UDP端口
    入口参数: QString  端口
    出口参数:
    备    注:
    *******************************************/
    void setServUdpPort(QString port);

    /*******************************************
    函数功能: 获取服务器UDP端口
    入口参数:
    出口参数: QString  端口
    备    注:
    *******************************************/
    QString getServUdpPort();

protected:
    // 本地消息分发模块
    ITcpModule *m_pTcpModule;
    CTimerManager *m_pTimerMng;
    CDeviceManager *m_pDeviceMng;
    // 本地用户数据库
    CDatebase *m_pDbJT808;
    CConfigManager *m_pConfigMng;

    // JT808通信sock
    QTcpSocket m_Jt808Sock;
    //本地客服端SOCKET
    SOCKET m_Jt808Socket;
    bool m_bConnected;

    //
    SERVER_STRUCT m_ServerPara;
    //
    DEVICE_INFO m_Device;
    //
    QMutex m_mtxSend;

    // 当前待执行消息队列(已经接受完毕的多包消息或单包消息)
    JT808_MESSAGE_QUEUE m_queueExecMsg;

    // 正在准备消息队列(多包未接受完毕消息)
    JT808_MESSAGE_MAP m_mapPrepareMsg;

    // 正在发送的多包队列(未接受完毕消息)
    JT808_MESSAGE_MAP m_mapSendingMsg;

    // 消息信号量
    QSemaphore m_jt808MsgSem;

    QMap<int, jt808_msg_proc> m_mapJt808Func;

    CJT808CmdProc *m_pCmdProc;
};

#endif // CJT808MANAGER_H
