#include "CJt808Config.h"
#include "jt808_common.h"
#include "device/CDeviceManager.h"
#include <assert.h>
#include "config/CConfigManager.h"
#include "jt808/CJT808Terminal.h"

const char *JT808_CONFIG_DB_NAME = "jt808cfg";

CJt808Config::CJt808Config()
    : m_pDbJt808Config(nullptr)
{
    // 从数据库中恢复数据
    CConfigManager *pConfigMng = dynamic_cast<CConfigManager *>(QueryPlugin("CConfigManager"));
    if (pConfigMng)
    {
        m_pDbJt808Config = pConfigMng->getDatabase(MODULE_JT808_MNG, JT808_CONFIG_DB_NAME);
        if (!m_pDbJt808Config)
        {
            createDatebase();
        }
        else
        {
            restoreFromDB(m_pDbJt808Config);
        }
    }
}

/*******************************************
函数功能: 默认设置函数,主要用于不支持的参数配置
入口参数: WORD lenth, 长度
            BYTE *data  buff首地址
出口参数: 
备    注:
*******************************************/
void CJt808Config::defaultSetFunc(int id, QString data)
{
    if (m_pDbJt808Config)
    {
        m_pDbJt808Config->setValue(QVariant(id).toString(), data);
    }
}

/*******************************************
函数功能: 默认获取函数,主要用于不支持的参数配置
入口参数: 
出口参数: 
备    注:
*******************************************/
QString CJt808Config::defaultGetFunc(int id)
{
    QString value;
    if (m_pDbJt808Config)
    {
        value = m_pDbJt808Config->getValue(QString("%1").arg(id)).toString();
    }

    return value;
}

/*******************************************
函数功能: 初始化函数，
入口参数:
出口参数:
备    注:
*******************************************/
bool CJt808Config::Initialize()
{
#define CALLBACK_TEMPLATE_SET(F) (std::bind(&CJt808Config::F, this, std::placeholders::_1))
#define CALLBACK_TEMPLATE_GET(F) (std::bind(&CJt808Config::F, this))
    /*-------------------------T808配置ID--------------------------*/
    TERMINAL_CONFIG_ITEM config_arr[] =
        {
            {SET_TCP_WAIT_TIME, VALUE_TYPE_DWORD, 10, nullptr, nullptr},              // TCP超时时间 秒
            {SET_TCP_RETRY_TIMES, VALUE_TYPE_DWORD, 1, nullptr, nullptr},             // TCP重传次数
            {SET_UDP_WAIT_TIME, VALUE_TYPE_DWORD, 10, nullptr, nullptr},              // UDP超时时间 秒
            {SET_UDP_RETRY_TIMES, VALUE_TYPE_DWORD, 0, nullptr, nullptr},             // UDP重传次数
            {SET_SMS_WAIT_TIME, VALUE_TYPE_DWORD, 60, nullptr, nullptr},              // 短消息应答超时 秒
            {SET_SMS_RETRY_TIMES, VALUE_TYPE_DWORD, 0, nullptr, nullptr},             // 短消息重传次数
            {SET_MAIN_SERVER_DIAL_APN, VALUE_TYPE_STRING, "", nullptr, nullptr},      // 主服务器APN
            {SET_MAIN_SERVER_DIAL_USER, VALUE_TYPE_STRING, "", nullptr, nullptr},     // 用户名
            {SET_MAIN_SERVER_DIAL_PWD, VALUE_TYPE_STRING, "", nullptr, nullptr},      // 密码
            {SET_BACK_SERVER_DIAL_APN, VALUE_TYPE_STRING, "", nullptr, nullptr},      // 备份服务器APN
            {SET_BACK_SERVER_DIAL_USER, VALUE_TYPE_STRING, "", nullptr, nullptr},     // 用户名
            {SET_BACK_SERVER_DIAL_PWD, VALUE_TYPE_STRING, "", nullptr, nullptr},      // 密码
            {SET_IC_SERVER_ADDR, VALUE_TYPE_STRING, "", nullptr, nullptr},            // IC卡认证服务器主地址
            {SET_IC_SERVER_TCP_PORT, VALUE_TYPE_DWORD, 0, nullptr, nullptr},          // IC卡认证服务器TCP端口
            {SET_IC_SERVER_UDP_PORT, VALUE_TYPE_DWORD, 0, nullptr, nullptr},          // IC卡认证服务器UDP端口
            {SET_IC_SERVER_ADDR_BACK, VALUE_TYPE_STRING, "", nullptr, nullptr},       // IC卡认证服务器备用地址
            {SET_UP_GPSDATA_TACTICS, VALUE_TYPE_DWORD, 0, nullptr, nullptr},          // 数据上报策略 0定时 1定距 2混合
            {SET_UP_GPSDATA_PLAN, VALUE_TYPE_DWORD, 0, nullptr, nullptr},             // 0:ACC 1 驾驶员登录状态
            {SET_UP_GPSDATA_TIME_NODRIV, VALUE_TYPE_DWORD, 0, nullptr, nullptr},      // 无驾驶员信息时的上报时间间隔 秒
            {SET_UP_SLAVE_SERV_APN, VALUE_TYPE_STRING, "", nullptr, nullptr},         // 从服务器APN
            {SET_UP_SERV_USERNAME, VALUE_TYPE_STRING, "", nullptr, nullptr},          // 从服务器无线通讯拨号用户名
            {SET_UP_SERV_PASSWORD, VALUE_TYPE_STRING, "", nullptr, nullptr},          // 从服务器无线通讯拨号密码
            {SET_UP_SERV_BAK_IP, VALUE_TYPE_STRING, "", nullptr, nullptr},            // 从服务器备用地址 ip或域名(端口用:分割，多个服务器用;分割)
            {SET_UP_REPORT_INTERVAL_SLEEP, VALUE_TYPE_DWORD, 300, nullptr, nullptr},  // 休眠时的上报时间间隔
            {SET_UP_GPSDATA_TIME_ALARM, VALUE_TYPE_DWORD, 30, nullptr, nullptr},      // 报警时的上报间隔
            {SET_UP_GPSDATA_TIME_DEFAULT, VALUE_TYPE_DWORD, 60, nullptr, nullptr},    // 正常上报时间间隔
            {SET_UP_GPSDATA_LENTH_DEFAULT, VALUE_TYPE_DWORD, 100, nullptr, nullptr},  // 正常上报距离间隔
            {SET_UP_GPSDATA_LENTH_NODRIV, VALUE_TYPE_DWORD, 100, nullptr, nullptr},   // 无驾驶员信息时的上报距离间隔 米
            {SET_UP_GPSDATA_LENTH_SLEEP, VALUE_TYPE_DWORD, 1000, nullptr, nullptr},   // 休眠时的上报时间间隔
            {SET_UP_GPSDATA_LENTH_ALARM, VALUE_TYPE_DWORD, 50, nullptr, nullptr},     // 报警时的上报间隔
            {SET_UP_GPSDATA_ANGLE, VALUE_TYPE_DWORD, 0, nullptr, nullptr},            // 拐点补传
            {SET_UP_ILLEGAL_MOVE, VALUE_TYPE_DWORD, 0, nullptr, nullptr},             // 非法位移阀值(米)
            {SET_UP_ILLEGAL_TIME, VALUE_TYPE_DWORD, 0, nullptr, nullptr},             // 违规行驶时间段范围精准到分，BYTE1 开始小时 BYTE2开始分钟数 BYTE3 结束小时 byte4 结束分钟
            {SET_SERVICE_TEL_NUM, VALUE_TYPE_STRING, "", nullptr, nullptr},           // 服务平台号码
            {SET_RESET_TEL_NUM, VALUE_TYPE_STRING, "", nullptr, nullptr},             // 复位号码
            {SET_SET_DEFAULT_TEL_NUM, VALUE_TYPE_STRING, "", nullptr, nullptr},       // 恢复出厂号码
            {SET_SERVICE_SMS_NUM, VALUE_TYPE_STRING, "", nullptr, nullptr},           // 服务平台号短息号码
            {SET_SERVICE_ALARM_NUM, VALUE_TYPE_STRING, "", nullptr, nullptr},         // 服务平台报警号短息号码
            {SET_TEL_ANSWER_TACTICS, VALUE_TYPE_DWORD, 0, nullptr, nullptr},          // 0自动接听 1 ACCON时自动接听
            {SET_MAX_TEL_TIME_ONCE, VALUE_TYPE_DWORD, 0xFFFFFFFF, nullptr, nullptr},  // 一次通话的最长时间 0为不允许 ,FFFFFFFF为无限制
            {SET_MAX_TEL_TIME_MONTH, VALUE_TYPE_DWORD, 0xFFFFFFFF, nullptr, nullptr}, // 一月最长通话时间      同上
            {SET_LISTEN_TEL_NUM, VALUE_TYPE_STRING, "", nullptr, nullptr},            // 监听号码
            {SET_PREROG_SMS_NUM, VALUE_TYPE_STRING, "", nullptr, nullptr},            // 特权短信号码
            {SET_ALARM_MASK, VALUE_TYPE_DWORD, 0, nullptr, nullptr},                  // 报警屏蔽字，和位置信息中的报警位对应，为1的发送相应报警
            {SET_ALARM_SMS_MASK, VALUE_TYPE_DWORD, 0, nullptr, nullptr},              // 短消息报警屏蔽字，和位置信息中的报警位对应，为1的发送相应报警
            {SET_CAM_MASK, VALUE_TYPE_DWORD, 0, nullptr, nullptr},                    // 拍照屏蔽字，同上
            {SET_CAM_SAVE_MASK, VALUE_TYPE_DWORD, 0, nullptr, nullptr},               // 拍照存储屏蔽字，为1存储同上
            {SET_CRUCIAL_ALARM_MASK, VALUE_TYPE_DWORD, 0, nullptr, nullptr},          // 关键报警屏蔽字为1
            {SET_OVER_SPEED, VALUE_TYPE_DWORD, 0, nullptr, nullptr},                  // 超速值
            {SET_OVER_SPEED_TIME, VALUE_TYPE_DWORD, 0, nullptr, nullptr},             // 超速持续时间
            {SET_OVER_TIME_ONCE, VALUE_TYPE_DWORD, 0, nullptr, nullptr},              // 单次连续驾驶超时 秒
            {SET_OVER_TIME_DAY, VALUE_TYPE_DWORD, 0, nullptr, nullptr},               // 一天连续驾驶超时
            {SET_MIN_REST_TIME, VALUE_TYPE_DWORD, 0, nullptr, nullptr},               // 最小休息时间
            {SET_MAX_STOP_TIME, VALUE_TYPE_DWORD, 0, nullptr, nullptr},               // 最长停车时间
            {SET_OVER_SPEED_NOTICE, VALUE_TYPE_WORD, 0, nullptr, nullptr},            // 超速报警预警差值
            {SET_OVER_TIME_NOTICE, VALUE_TYPE_WORD, 0, nullptr, nullptr},             // 超时报警预警差值
            {SET_COLLISION_ALARM, VALUE_TYPE_WORD, 0, nullptr, nullptr},              // 碰撞报警参数设置
            {SET_ROLLOVER_ALARM, VALUE_TYPE_WORD, 30, nullptr, nullptr},              // 侧翻报警参数设置
            {SET_FIXTIME_PHOTOGRAPH_CONTROL, VALUE_TYPE_DWORD, 0, nullptr, nullptr},  // 定时拍照控制
            {SET_FIXDIS_PHOTOGRAPH_CONTROL, VALUE_TYPE_DWORD, 0, nullptr, nullptr},   // 定距拍照控制
            {SET_MEDIA_ALITY, VALUE_TYPE_DWORD, 0, nullptr, nullptr},                 // 媒体质量 1-10 1最好
            {SET_BRIGHTNESS, VALUE_TYPE_DWORD, 0, nullptr, nullptr},                  // 亮度0-255
            {SET_CONTRAST, VALUE_TYPE_DWORD, 0, nullptr, nullptr},                    // 对比度 同上
            {SET_SATURATION, VALUE_TYPE_DWORD, 0, nullptr, nullptr},                  // 饱和度 同上
            {SET_CHROMA, VALUE_TYPE_DWORD, 0, nullptr, nullptr},                      // 色度
            {SET_ODOMETER, VALUE_TYPE_DWORD, 0, nullptr, nullptr},                    // 里程 0.1KM
            {SET_GNSS_MODE_TYPE, VALUE_TYPE_BYTE, 0x02, nullptr, nullptr},            // GNSS定位模式
            {SET_GNSS_BITRATE, VALUE_TYPE_BYTE, 0, nullptr, nullptr},                 // GNSS波特率设置
            {SET_GNSS_MESSAGE_FREQUENCY, VALUE_TYPE_BYTE, 0x1, nullptr, nullptr},     // GNSS定位数据输出频率
            {SET_GNSS_MESSAGE_COLLECTION, VALUE_TYPE_DWORD, 1, nullptr, nullptr},     // GNSS定位数据采集频率
            {SET_GNSS_MESSAGE_SEND_TYPE, VALUE_TYPE_BYTE, 0, nullptr, nullptr},       // GNSS定位数据上传频率
            {SET_GNSS_MESSAGE_SEND_SET, VALUE_TYPE_DWORD, 1, nullptr, nullptr},       // GNSS定位数据上传设置
            {SET_CAN1_COLLECTION_TIME, VALUE_TYPE_DWORD, 0, nullptr, nullptr},        // CAN通道1采集时间间隔
            {SET_CAN1_SEND_TIME, VALUE_TYPE_WORD, 0, nullptr, nullptr},               // CAN通道1发送时间间隔
            {SET_CAN2_COLLECTION_TIME, VALUE_TYPE_DWORD, 0, nullptr, nullptr},        // CAN通道2采集时间间隔
            {SET_CAN2_SEND_TIME, VALUE_TYPE_WORD, 0, nullptr, nullptr},               // CAN通道2发送时间间隔
            {SET_CAN_ID_COLLECTION, VALUE_TYPE_LONGLONG, 0, nullptr, nullptr},           // CAN总线ID0单独采集设置
            {SET_CAN_ID_COLLECTION1, VALUE_TYPE_LONGLONG, 0, nullptr, nullptr},          // CAN总线ID1单独采集设置
            {SET_CAN_ID_COLLECTION2, VALUE_TYPE_LONGLONG, 0, nullptr, nullptr},          // CAN总线ID2单独采集设置
            {SET_CAN_ID_COLLECTION3, VALUE_TYPE_LONGLONG, 0, nullptr, nullptr},          // CAN总线ID3单独采集设置
        };

    registConfigProc(config_arr, sizeof(config_arr) / sizeof(TERMINAL_CONFIG_ITEM));
}

// 配置回调
bool CJt808Config::registConfigProc(TERMINAL_CONFIG_ITEM *pConfigArray, int count)
{
    // 参数校验
    if (pConfigArray == nullptr)
    {
        return false;
    }

    for (int i = 0; i < count; i++)
    {
        int id = pConfigArray[i].id;
        auto it = m_mapConfig.find(id);
        if (it != m_mapConfig.end())
        {
            // 提示错误并覆盖函数
            assert(0);
            m_mapConfig[id] = pConfigArray[i];
        }
        else
        {
            m_mapConfig[id] = pConfigArray[i];
        }

        // 如果数据库没有找到字段,则注册字段
        QVariant value;
        if (!m_pDbJt808Config->getValidValue(QVariant(id).toString(), value))
        {
            m_pDbJt808Config->regKey(QVariant(id).toString(), pConfigArray[i].def_value);
        }
        else // 如果数据库找到字段,则需要恢复值到内存
        {
            if (pConfigArray[i].func_set)
            {
                pConfigArray[i].func_set(value.toString());
            }
        }
    }
    return true;
}

// 创建数据库
bool CJt808Config::createDatebase()
{
    //LOG_TRACE(CJT808Terminal, JT808_CONFIG_DB_NAME , " is not exist, create Datebase!");
    // 从数据库中恢复数据
    CConfigManager *pConfigMng = dynamic_cast<CConfigManager *>(QueryPlugin("CConfigManager"));
    if (pConfigMng)
    {
        m_pDbJt808Config = pConfigMng->regDatabase(MODULE_JT808_MNG, JT808_CONFIG_DB_NAME);
    }
    else
    {
        // log_err();
    }
    return true;
}

// 从数据库恢复, 这里不做处理,在注册时恢复
bool CJt808Config::restoreFromDB(CDatebase *pDatebase)
{
    return true;
}

/*******************************************
函数功能: 保存数据
入口参数:
出口参数:
备    注:
*******************************************/
void CJt808Config::saveConfig()
{
    if (m_pDbJt808Config)
    {
        m_pDbJt808Config->save();
    }
}

/*******************************************
函数功能: 终端参数查询
入口参数: data, 消息体指针
          lenth, 消息体长度
出口参数:
备    注:
*******************************************/
void CJt808Config::procQueryParameter(int cycle_num, BYTE *payload_down, WORD pkg_lenth)
{
    qDebug() << "<<--收到终端参数查询";
    QByteArray ba(4000, 0);
    BYTE *send_data = (BYTE *)ba.data();
    CDeviceManager *pDeviceMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
    assert(pDeviceMng);
    int idx = 0;
    // 终端类型(WORD)
    short_to_two_char(pDeviceMng->getTerminalType(), &send_data[idx]);
    idx += sizeof(WORD);
    // 制造商ID(BYTE[5])
    strncpy((char *)&send_data[idx], pDeviceMng->getManufactor().toStdString().c_str(), 5);
    idx += 5;
    // 终端类型(BYTE[30])
    strncpy((char *)&send_data[idx], pDeviceMng->getDeviceType().toStdString().c_str(), 30);
    idx += 30;
    // 终端ID(BYTE[30])
    strncpy((char *)&send_data[idx], pDeviceMng->getTerminalID().toStdString().c_str(), 30);
    idx += 30;
    // 终端SIM卡ICCID(BCD[10])
    memcpy((char *)&send_data[idx], pDeviceMng->getSimICCID().toStdString().c_str(), 10);
    idx += 10;

    // 终端硬件版本号长度(BYTE)
    string hard_ver = pDeviceMng->getHardVersion().toStdString();
    send_data[idx++] = hard_ver.size();
    // 终端硬件版本号(STRING)
    strncpy((char *)&send_data[idx], hard_ver.c_str(), (BYTE)hard_ver.size());
    idx += hard_ver.size();

    // 终端固件版本号长度(BYTE)
    string firmware_ver = pDeviceMng->getFirmwareVersion().toStdString();
    send_data[idx++] = firmware_ver.size();
    // 终端固件版本号(STRING)
    strncpy((char *)&send_data[idx], firmware_ver.c_str(), (BYTE)firmware_ver.size());
    idx += firmware_ver.size();

    // GNSS模块属性(BYTE)
    send_data[idx++] = pDeviceMng->getGNSSProperty();
    // 通讯模块属性(BYTE)
    send_data[idx++] = pDeviceMng->getComProperty();

    qDebug() << "-->>终端参数查询应答";
    sendPayload2Jt808Serv(TERMINAL_PROPERTY_ANSWER, send_data, idx);
}

/*******************************************
函数功能: 获取配制项值
入口参数: TERMINAL_CONFIG_ITEM &item, 配置项
出口参数: QString value
备    注:
*******************************************/
QString CJt808Config::getItemValue(TERMINAL_CONFIG_ITEM &item)
{
    if (item.func_get)
    {
        return item.func_get();
    }
    else
    {
        return defaultGetFunc(item.id);
    }
}

/*******************************************
函数功能: 填充终端配置
入口参数: TERMINAL_CONFIG_ITEM &item, 配置项
         BYTE *data Buff地址
出口参数: buff偏移长度
备    注:
*******************************************/
int CJt808Config::fillValue2Buff(TERMINAL_CONFIG_ITEM &item, BYTE *data)
{
    assert(m_pDbJt808Config != nullptr);

    QString value = getItemValue(item);
    int idx = 5;
    BYTE lenth = 0;

    long_to_four_char(item.id, data);

    switch (item.type)
    {
    // 字符串模式
    case VALUE_TYPE_BYTE:
        lenth = 1;
        data[idx] = (BYTE)value.toInt();
        break;
    case VALUE_TYPE_WORD:
        lenth = 2;
        short_to_two_char((WORD)value.toInt(), data + idx);
        break;
    case VALUE_TYPE_DWORD:
        lenth = 4;
        long_to_four_char((DWORD)value.toInt(), data + idx);
        break;
    case VALUE_TYPE_LONGLONG:
        lenth = 8;
        int64_to_eight_char((DWORD)value.toLongLong(), data + idx);
        break;
    case VALUE_TYPE_STRING:
        lenth = std::min(254, (int)value.toStdString().length());
        strncpy((char *)data + idx, value.toStdString().c_str(), lenth);
        break;
    default:
        break;
    }

    data[4] = lenth;
    //这里不能用sizeof(DWORD)在64位操作系统中这个长度是8和我们预期需要填入的不一致，务必注意
    return lenth + 5;
}
/*******************************************
函数功能: 查询终端配置
入口参数: data, 消息体指针
          lenth, 消息体长度
出口参数:
备    注:
*******************************************/
void CJt808Config::procQueryTerminalCfgItem(int cycle_num, BYTE *payload, WORD pkg_lenth)
{
    qDebug() << "<<--收到终端配置指定查询操作";
    QByteArray ba(4000, 0);
    BYTE *send_data = (BYTE *)ba.data();
    int idx = 0;
    int count = payload[idx++];

    int idx_rsp = 0;
    short_to_two_char(cycle_num, &send_data[idx_rsp]);
    idx_rsp += 2;
    idx_rsp++;
    int send_count = 0;

    for (int i = 0; i < count; i++)
    {
        DWORD dwID = four_char_to_long(&payload[idx]);
        //这里不能用sizeof(DWORD)在64位操作系统中这个长度是8和我们预期需要填入的不一致，务必注意
        idx += 4;
        TERMINAL_CONFIG_MAP::iterator it = m_mapConfig.find(dwID);
        if (it != m_mapConfig.end())
        {
            TERMINAL_CONFIG_ITEM &item = it.value();
            idx_rsp += fillValue2Buff(item, &send_data[idx_rsp]);
            send_count++;
        }
    }

    send_data[2] = send_count;
    qDebug() << "-->>终端参数查询应答";
    sendPayload2Jt808Serv(TERMINAL_ANSWER_SYS_CFG, send_data, idx_rsp);
}
/*******************************************
函数功能: 查询终端配置
入口参数: data, 消息体指针
          lenth, 消息体长度
出口参数:
备    注:
*******************************************/
void CJt808Config::procQueryTerminalCfg(int cycle_num, BYTE *payload, WORD pkg_lenth)
{
    qDebug() << "<<--收到终端配置全量查询操作";
    QByteArray ba(4000, 0);
    BYTE *send_data = (BYTE *)ba.data();
    int count = m_mapConfig.size();

    int idx_rsp = 0;
    short_to_two_char(cycle_num, &send_data[idx_rsp]);
    idx_rsp += 2;
    idx_rsp++;

    int send_count = 0;
    for (auto it = m_mapConfig.begin(); it != m_mapConfig.end(); it++)
    {
        TERMINAL_CONFIG_ITEM &item = it.value();
        if(item.id == 0 || item.id == 275)
        {
            int i =0;
        }
        idx_rsp += fillValue2Buff(item, &send_data[idx_rsp]);
        send_count++;
    }

    send_data[2] = send_count;

    qDebug() << "-->>终端参数查询应答";
    sendPayload2Jt808Serv(TERMINAL_ANSWER_SYS_CFG, send_data, idx_rsp);
}

/*******************************************
函数功能: 终端控制
入口参数: data, 消息体指针
          lenth, 消息体长度
出口参数:
备    注:
*******************************************/
void CJt808Config::procTerminalControl(int cycle_num, BYTE *payload, WORD pkg_lenth)
{
    qDebug() << "<<--收到终端控制操作";
    BYTE ctrl_code = payload[0];

    bool bReboot = false;
    BYTE ret_code = 0;

    switch (ctrl_code)
    {
    // 重定向服务器
    case TEMINAL_CONTROL_REDIFINE_SERT:
    {
        qDebug() << "重定向地址信息" << QString((char *)&payload[1]);
        QStringList lst = QString(payload[1]).split(";");
        break;
    }
    // 重启终端
    case TERMINAL_CONTROL_REBOOT:

        /* code */
        bReboot = true;
        break;
    // 终端恢复出场值
    case TERMINAL_CONTROL_RESET_FACTORY:
    {
        CDeviceManager *pDeviceMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
        assert(pDeviceMng);
        /* code */
        pDeviceMng->restoreFactory();
        break;
    }
    default:
        qDebug() << "不支持的控制操作" << QString((char *)&payload[1]);
        ret_code = 1;
        break;
    }

    qDebug() << "-->>终端控制操作响应:" << SERVER_CTRL_TERMINAL;
    sendCommonAnswer(cycle_num, SERVER_CTRL_TERMINAL, ret_code);

    if (bReboot)
    {
        system("reboot");
    }
}

/*******************************************
函数功能: 从buff设置参数
入口参数: BYTE *data Buff地址
出口参数: buff偏移长度
备    注:
*******************************************/
int CJt808Config::setValueByBuff(BYTE *data)
{
    QString value;
    int idx = 0;
    // 获取id
    DWORD id = four_char_to_long(data);
    idx += 4;
    // 获取长度
    int lenth = data[idx++];

    // 查找id
    auto it = m_mapConfig.find(id);
    // 根据类型赋值
    if (it != m_mapConfig.end())
    {
        TERMINAL_CONFIG_ITEM &item = it.value();
        QString value;
        switch (item.type)
        {
        // 字符串模式
        case VALUE_TYPE_BYTE:
            value = QString("%1").arg((int)data[idx]);
            break;
        case VALUE_TYPE_WORD:
            value = QString("%1").arg((int)two_char_to_short(data + idx));
            break;
        case VALUE_TYPE_DWORD:
            value = QString("%1").arg((int)four_char_to_long(data + idx));
            break;
        case VALUE_TYPE_LONGLONG:
            value = QString("%1").arg((int)eight_char_to_int64(data + idx));
            break;
        case VALUE_TYPE_STRING:
            value = QString("%1").arg((char *)(data + idx), lenth);
            break;
        default:
            break;
        }

        if (item.func_set)
        {
            item.func_set(value);
        }
        else
        {
            defaultSetFunc(item.id, value);
        }
    }

    return lenth + 5;
}

/*******************************************
函数功能: 终端参数设置
入口参数: data, 消息体指针
          lenth, 消息体长度
出口参数:
备    注:
*******************************************/
void CJt808Config::procSetParameter(int cycle_num, BYTE *payload, WORD pkg_lenth)
{
    qDebug() << ("<<--参数设置命令");

    // 先把结果发回去,避免因为修改IP导致结果无法上传
    qDebug() << ("-->>参数设置响应");
    sendCommonAnswer(cycle_num, SERVER_SET_TERMINAL_CFG, 0);

    int idx = 0;
    int count = payload[idx++];
    for (int i = 0; i < count && idx < pkg_lenth; i++)
    {
        idx += setValueByBuff(payload + idx);
    }

    this->saveConfig();
}
