#ifndef CTERMINALPROPERTY_H
#define CTERMINALPROPERTY_H
#include "common.h"
#include <QVariant>
#include <QMap>
#include "jt808_common.h"
#include "log/CLogManager.h"

class CDatebase;

enum TERMINAL_CONTROL_CODE
{
    // 重定向服务器
    TEMINAL_CONTROL_REDIFINE_SERT = 2,
    // 重启终端
    TERMINAL_CONTROL_REBOOT = 4,
    // 终端恢复出场值
    TERMINAL_CONTROL_RESET_FACTORY = 5,
};

enum TERMINAL_VALUE_TYPE
{
    VALUE_TYPE_BYTE = 0,
    VALUE_TYPE_WORD,
    VALUE_TYPE_DWORD,
    VALUE_TYPE_LONGLONG,
    VALUE_TYPE_BYTE_ARRAY,
    VALUE_TYPE_STRING,
};

typedef std::function<void(QString)> JT808_CFG_SET_PROC;
typedef std::function<QString()> JT808_CFG_GET_PROC;

struct TERMINAL_CONFIG_ITEM
{
    DWORD id;
    TERMINAL_VALUE_TYPE type;
    QVariant def_value; //默认值,只在注册时使用
    JT808_CFG_SET_PROC func_set;
    JT808_CFG_GET_PROC func_get;

    // 只对长度为0 1 2 4 8
    QString getValue()
    {
        return func_get();
    }
};

typedef QMap<DWORD, TERMINAL_CONFIG_ITEM> TERMINAL_CONFIG_MAP;

class CJt808Config
{
public:
    CJt808Config();
    virtual ~CJt808Config() {}

    /*******************************************
    函数功能: 初始化函数，
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    bool Initialize();

    /*******************************************
    函数功能: 注册配置变更处理函数
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    bool registConfigProc(TERMINAL_CONFIG_ITEM *pConfigArray, int count);

protected:
    /*******************************************
    函数功能: 保存数据
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    virtual void saveConfig();

    /*******************************************
    函数功能: 发送数据到JT08服务器
    入口参数: WORD cmd,       命令字
            BYTE *payload,  净荷
            WORD lenth      净荷长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD sendPayload2Jt808Serv(WORD cmd, BYTE *payload, WORD lenth) = 0;

    /*******************************************
    函数功能: 发送通用应答
    入口参数: cycle_num 流水号
             id  消息ID
             result 执行结果
    出口参数:
    备    注:
    *******************************************/
    virtual WORD sendCommonAnswer(WORD cycle_num, WORD id, BYTE result) = 0;

    /*******************************************
    函数功能: 终端参数查询
    入口参数: data, 消息体指针
            lenth, 消息体长度
    出口参数:
    备    注:
    *******************************************/
    void procQueryParameter(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 终端控制
    入口参数: data, 消息体指针
            lenth, 消息体长度
    出口参数:
    备    注:
    *******************************************/
    void procTerminalControl(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 查询终端配置
    入口参数: data, 消息体指针
            lenth, 消息体长度
    出口参数:
    备    注:
    *******************************************/
    void procQueryTerminalCfg(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 查询指定终端配置
    入口参数: data, 消息体指针
            lenth, 消息体长度
    出口参数:
    备    注:
    *******************************************/
    void procQueryTerminalCfgItem(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 终端参数设置
    入口参数: data, 消息体指针
            lenth, 消息体长度
    出口参数:
    备    注:
    *******************************************/
    void procSetParameter(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 填充终端配置
    入口参数: TERMINAL_CONFIG_ITEM &item, 配置项
             BYTE *data Buff地址
    出口参数: buff偏移长度
    备    注:
    *******************************************/
    int fillValue2Buff(TERMINAL_CONFIG_ITEM &item, BYTE *data);

    /*******************************************
    函数功能: 获取配制项值
    入口参数: TERMINAL_CONFIG_ITEM &item, 配置项
    出口参数: QString value
    备    注:
    *******************************************/
    QString getItemValue(TERMINAL_CONFIG_ITEM &item);

    /*******************************************
    函数功能: 默认设置函数,主要用于不支持的参数配置
    入口参数: int id, ID
             QVariant
    出口参数: 
    备    注:
    *******************************************/
    void defaultSetFunc(int id, QString data);

    /*******************************************
    函数功能: 获取设置函数,主要用于不支持的参数配置
    入口参数: int id, ID
    出口参数: 
    备    注:
    *******************************************/
    QString defaultGetFunc(int id);

    /*******************************************
    函数功能: 从buff设置参数
    入口参数: BYTE *data Buff地址
    出口参数: buff偏移长度
    备    注:
    *******************************************/
    int setValueByBuff(BYTE *data);

    // 保存配置文件
    bool restoreFromDB(CDatebase *pDatebase);

    // 导入配置文件
    bool createDatebase();

protected:
    // 终端配置变量
    TERMINAL_CONFIG_MAP m_mapConfig;
    CDatebase *m_pDbJt808Config;
};

#endif // CTERMINALPROPERTY_H
