#include "CJt808Device.h"
#include <QDebug>
#include <QDateTime>
#include "swdl/CSWDLManager.h"

CJt808Device::CJt808Device()
{
}

/*******************************************
函数功能: 电话回拨
入口参数: data 数据包
        lenth 长度
出口参数:
备    注:
*******************************************/
WORD CJt808Device::procSetCallBack(int cycle_num, BYTE *payload, WORD lenth)
{
    qDebug() << "<<--电话回拨下发";
    int idx = 0;

    BYTE type = payload[idx++]; // 0 普通通话； 1 监听
    std::string phone_num = (char *)payload[idx];
    // 因为终端不支持电话回拨功能，这里直接接收消息不做处理，返回成功

    qDebug() << "-->>电话回拨应答";
    sendCommonAnswer(cycle_num, SERVER_CALL_BACK, 0);
}

/*******************************************
函数功能: 设置电话本
入口参数: data 数据包
        lenth 长度
出口参数:
备    注:
*******************************************/
WORD CJt808Device::procSetPhoneNum(int cycle_num, BYTE *payload, WORD lenth)
{
    qDebug() << "<<--设置电话本下发";
    int idx = 0;
    // 0 删除全部联系人
    // 1 更新电话本，删除已有的，将新的加入
    // 2 追加联系人
    // 3 以联系人为索引修改电话本
    BYTE type = payload[idx++];
    if (type == 0 || type == 1)
    {
        m_mapPhoneBook.clear();
    }

    BYTE count = payload[idx++];
    BYTE phone_len = 0;
    BYTE name_len = 0;
    for (int i = 0; i < count; i++)
    {
        PhoneNode node;
        node.call_type = payload[idx++];
        phone_len = payload[idx++];
        QByteArray ba = QByteArray((char *)&payload[idx], phone_len);
        node.phone_num = QString(ba);
        idx += phone_len;

        name_len = payload[idx++];
        ba = QByteArray((char *)&payload[idx], name_len);
        node.user_name = QString(ba);
        idx += name_len;
        m_mapPhoneBook[node.user_name] = node;
    }

    qDebug() << "-->>设置电话本应答";
    sendCommonAnswer(cycle_num, SERVER_SET_PHONE_BOOK, 0);

    // 保存电话本
}

/*******************************************
函数功能: 控制车辆
入口参数: data 数据包
        lenth 长度
出口参数:
备    注:
*******************************************/
WORD CJt808Device::procCtrlVehicle(int cycle_num, BYTE *payload, WORD lenth)
{
    qDebug() << "<<--控制车辆下发";
    int idx = 0;
    // 控制数量
    WORD ctrl_cnt = two_char_to_short(&payload[idx]);
    idx += 2;
    for (int i = 0; i < ctrl_cnt; i++)
    {
        WORD ctrl_type = two_char_to_short(&payload[idx]);
        idx += 2;
        BYTE ctrl_code = payload[idx++];
        switch (ctrl_code)
        {
        case 1: //车门
            //控制车门函数
            break;
        default:
            break;
        }
    }

    qDebug() << "-->>控制车辆应答";
    QByteArray ba(100, 0);
    BYTE *send_data = (BYTE *)ba.data();
    idx = 0;
    short_to_two_char(cycle_num, &send_data[idx]);
    idx += 2;
    send_data[idx++] = 0;

    sendPayload2Jt808Serv(TERMINAL_VEHICLE_CTRL, send_data, idx);
}

/*******************************************
函数功能: 读取行驶记录
入口参数: data 数据包
        lenth 长度
出口参数:
备    注:SERVER_READ_RECODE
*******************************************/
WORD CJt808Device::procReadRecord(int cycle_num, BYTE *payload, WORD lenth)
{
    qDebug() << "<<--读取行驶记录";
    int idx = 0;
    BYTE cmd = payload[idx++];
    switch (cmd)
    {
    case GBT19056_GET_VERSION:         //采集记录仪执行标准版本
    case GBT19056_GET_CUR_DRIVER_INFO: //采集当前驾驶人信息
    case GBT19056_GET_CUR_TIME:        //采集记录仪的实时时间
    case GBT19056_GET_TOTAL_MILEAGE:   //采集累计行驶里程
    case GBT19056_GET_DATA_BLOCK:      //记录仪脉冲系数数据块格式
    case GBT19056_GET_VEHICLE_INFO:    //采集车辆信息
    case GBT19056_GET_STATUS_INFO:     //状态信号配置信息
    case GBT19056_GET_STATUS_INFO2:    //状态信号配置信息
    case GBT19056_GET_SPEED_RECORD:    //采集指定的行驶速度记录(08H)
    case GBT19056_GET_LOCATION_RECORD: //指定的位置信息记录(09H)
    case GBT19056_GET_ACCIDENT_RECORD: //采集指定的事故疑点记录(10H)
    case GBT19056_GET_OVERDUE_RECORD:  //采集指定的超时驾驶记录(11H)
    case GBT19056_GET_DRIVER_RECORD:   //采集指定的驾驶人身份记录(12H)
    case GBT19056_GET_POWER_SUPPLY:    //采集指定的记录仪外部供电记录(13H)
    case GBT19056_GET_MODIFY_RECORD:   //采集指定的记录仪参数修改记录(14H)
    case GBT19056_GET_SPEED_LOG:       //采集指定的速度状态日志(15H)
    default:
        break;
    }

    std::string str_info = "不支持行驶记录查询";
    QByteArray ba(str_info.length() + 100, 0);
    BYTE *send_data = (BYTE *)ba.data();
    idx = 0;
    short_to_two_char(cycle_num, &send_data[idx]);
    idx += 2;
    send_data[idx++] = cmd;
    strncpy((char *)&send_data[idx], str_info.c_str(), str_info.length() + 1);
    idx += str_info.length() + 1;

    qDebug() << "-->>读取行驶记录应答";
    sendPayload2Jt808Serv(TERMINAL_UP_RECODE, send_data, idx);
}

/*******************************************
函数功能: 记录参数下传设置
入口参数: data 数据包
        lenth 长度
出口参数:
备    注:
*******************************************/
WORD CJt808Device::procSetRecordPara(int cycle_num, BYTE *payload, WORD lenth)
{
    qDebug() << "<<--记录参数下传设置";
    int idx = 0;
    BYTE cmd = payload[idx++];
    switch (cmd)
    {
    case GBT19056_GET_VERSION:         //采集记录仪执行标准版本
    case GBT19056_GET_CUR_DRIVER_INFO: //采集当前驾驶人信息
    case GBT19056_GET_CUR_TIME:        //采集记录仪的实时时间
    case GBT19056_GET_TOTAL_MILEAGE:   //采集累计行驶里程
    case GBT19056_GET_DATA_BLOCK:      //记录仪脉冲系数数据块格式
    case GBT19056_GET_VEHICLE_INFO:    //采集车辆信息
    case GBT19056_GET_STATUS_INFO:     //状态信号配置信息
    case GBT19056_GET_STATUS_INFO2:    //状态信号配置信息
    case GBT19056_GET_SPEED_RECORD:    //采集指定的行驶速度记录(08H)
    case GBT19056_GET_LOCATION_RECORD: //指定的位置信息记录(09H)
    case GBT19056_GET_ACCIDENT_RECORD: //采集指定的事故疑点记录(10H)
    case GBT19056_GET_OVERDUE_RECORD:  //采集指定的超时驾驶记录(11H)
    case GBT19056_GET_DRIVER_RECORD:   //采集指定的驾驶人身份记录(12H)
    case GBT19056_GET_POWER_SUPPLY:    //采集指定的记录仪外部供电记录(13H)
    case GBT19056_GET_MODIFY_RECORD:   //采集指定的记录仪参数修改记录(14H)
    case GBT19056_GET_SPEED_LOG:       //采集指定的速度状态日志(15H)
    default:
        break;
    }

    qDebug() << "-->>记录参数下传设置应答";
    sendCommonAnswer(cycle_num, SERVER_SET_RECODE, 0);
}

/*******************************************
函数功能: 上报驾驶员身份信息请求
入口参数: data 数据包
        lenth 长度
出口参数:
备    注:
*******************************************/
WORD CJt808Device::procRequestIdcard(int cycle_num, BYTE *payload, WORD lenth)
{
    qDebug() << "<<--上报驾驶员身份信息请求";
    // 暂时不作处理，需要配合IC卡读卡器一起配合使用
    //emit getDriverInfo();

    qDebug() << "-->>上报驾驶员身份信息请求应答";
    sendCommonAnswer(cycle_num, SERVER_REQUEST_IDCARD, 0);

    // 下面是桩函数
    mSleep(500);

    DriverIccardInfo iccard_info;
    iccard_info.status = 1;
    QDateTime dt = QDateTime::currentDateTime();

    iccard_info.time[0] = dt.date().year() % 100;
    iccard_info.time[1] = dt.date().month();
    iccard_info.time[2] = dt.date().day();
    iccard_info.time[3] = dt.time().hour();
    iccard_info.time[4] = dt.time().minute();
    iccard_info.time[5] = dt.time().second();

    iccard_info.read_status = 0;
    iccard_info.driver_name = "合能创越01";
    iccard_info.license_num = "HNCY000000001";
    iccard_info.license_name = "合能创越_T01";
    iccard_info.valid_date[0] = 20;
    iccard_info.valid_date[1] = 25;
    iccard_info.valid_date[2] = 06;
    iccard_info.valid_date[3] = 18;
    iccard_info.driver_id = "300000000000000000X";
    sendDriverIdcard(iccard_info);
}

/*******************************************
函数功能: 上报驾驶员身份信息
入口参数: DriverIccardInfo &iccard_info IC卡信息
出口参数:
备    注:
*******************************************/
WORD CJt808Device::sendDriverIdcard(DriverIccardInfo &iccard_info)
{
    qDebug() << "<<--上报驾驶员身份信息请求";
    QByteArray ba(4000, 0);
    BYTE *send_data = (BYTE *)ba.data();
    int idx = 0;
    send_data[idx++] = iccard_info.status;
    time2bcd(iccard_info.time, &send_data[idx]);
    idx += 6;
    if (iccard_info.status == 1)
    {
        send_data[idx++] = iccard_info.read_status;
        if (0 == iccard_info.read_status)
        {
            send_data[idx++] = iccard_info.driver_name.length();
            strncpy((char *)&send_data[idx], iccard_info.driver_name.c_str(), iccard_info.driver_name.length());
            idx += iccard_info.driver_name.length();

            // 不足20补充0取20
            strncpy((char *)&send_data[idx], iccard_info.license_num.c_str(), iccard_info.license_num.length());
            idx += 20;

            send_data[idx++] = iccard_info.license_name.length();
            strncpy((char *)&send_data[idx], iccard_info.license_name.c_str(), iccard_info.license_name.length());
            idx += iccard_info.license_name.length();
            send_data[idx++] = hex2bcd(iccard_info.valid_date[0]);
            send_data[idx++] = hex2bcd(iccard_info.valid_date[1]);
            send_data[idx++] = hex2bcd(iccard_info.valid_date[2]);
            send_data[idx++] = hex2bcd(iccard_info.valid_date[3]);

            // 不足20补充0取20
            strncpy((char *)&send_data[idx], iccard_info.driver_id.c_str(), iccard_info.driver_id.length());
            idx += 20;
        }
    }

    qDebug() << "-->>上报驾驶员身份信息";
    sendPayload2Jt808Serv(TERMINAL_UP_DRIVER_INFO, send_data, idx);
}

/*******************************************
函数功能: 上报电子运单信息
入口参数: data 数据包
            lenth 长度
出口参数:
备    注:
*******************************************/
WORD CJt808Device::sendFreightNote(char *str_freight, int lenth)
{
    QByteArray ba(lenth + 100, 0);
    BYTE *send_data = (BYTE *)ba.data();
    int idx = 0;
    // 电子运单长度
    long_to_four_char(lenth, &send_data[idx]);
    idx += 4;
    // 运单内容
    memcpy(&send_data[idx], str_freight, lenth);
    idx += lenth;

    qDebug() << "-->>上报电子运单信息";
    sendPayload2Jt808Serv(TERMINAL_UP_FREIGHT_NOTE, send_data, idx);
}

/*******************************************
函数功能: 定位数据批量上传
入口参数: gps_info, GPS信息数组
        size gps信息数
        type 类型
出口参数:
备    注:TERMINAL_UP_MULTI_GPS_DATA
*******************************************/
WORD CJt808Device::sendGpsBatch(GPS_INFO *gps_info, WORD size, BYTE type)
{
    QByteArray ba(4000, 0);
    BYTE *send_data = (BYTE *)ba.data();
    int idx = 0;
    // 数据项个数
    short_to_two_char(size, &send_data[idx]);
    idx += 2;
    // 位置数据类型
    send_data[idx++] = type;
    // 位置数据项
    for (WORD i = 0; i < size; i++)
    {
        int _size = fillLocationInfo(&send_data[idx + 2], gps_info[i]);
        short_to_two_char(_size, &send_data[idx]);
        idx += 2 + _size;
    }

    qDebug() << "-->>定位数据批量上传";
    sendPayload2Jt808Serv(TERMINAL_UP_MULTI_GPS_DATA, send_data, idx);
}

/*******************************************
函数功能: CAN总线数据上传
入口参数: CAN_DATA *data_array, CAN数组
        WORD size 数组大小
出口参数:
备    注:TERMINAL_UP_CAN_DATA
*******************************************/
WORD CJt808Device::sendCanData(CAN_DATA *data_array, WORD size)
{
    QByteArray ba(4000, 0);
    BYTE *send_data = (BYTE *)ba.data();
    int idx = 0;
    // 数据项个数 WORD
    short_to_two_char(size, &send_data[idx]);
    // CAN总线接收时间
    QDateTime dt = QDateTime::currentDateTime();

    send_data[idx++] = hex2bcd(dt.time().hour());
    send_data[idx++] = hex2bcd(dt.time().minute());
    send_data[idx++] = hex2bcd(dt.time().second());
    short_to_two_char(dt.time().msec(), &send_data[idx]);
    idx += 2;
    // CAN数据项
    for (size_t i = 0; i < size; i++)
    {
        //CAN ID (DWORD)
        long_to_four_char(data_array[i].can_id, &send_data[idx]);
        idx += 4;
        // CAN DATA BYTE[8]
        memcpy(&send_data[idx], data_array[i].can_data, 8);
        idx += 8;
    }

    qDebug() << "-->>CAN总线数据上传";
    sendPayload2Jt808Serv(TERMINAL_UP_CAN_DATA, send_data, idx);
}

/*******************************************
函数功能: 软件升级请求
入口参数: int inc_num, 
        BYTE *payload, 
        WORD lenth
出口参数:
备    注:
*******************************************/
void CJt808Device::procSoftwareUpdate(int inc_num, BYTE *payload, WORD lenth)
{
    int idx = 0;
    int update_type = payload[idx++];

    BYTE manuFac_id[6] = {0};
    memcpy(manuFac_id, &payload[idx], 5);
    idx += 5;

    int ver_len = payload[idx++];

    QString ver = QString("%1").arg((char *)&payload[idx], ver_len);

    int data_len = four_char_to_long(&payload[idx]);
    idx += 4;

    // 写入数据到文件
    bool bSuccess = false;
    CSWDLManager *pSwdlMng = dynamic_cast<CSWDLManager *>(QueryPlugin("CSWDLManager"));
    if (pSwdlMng != nullptr)
    {
        QString file_name = pSwdlMng->getUpdatePkgPathname(ver);
        QFile file(file_name);
        if (!file.open(QFile::WriteOnly | QFile::Truncate)) //可以用QIODevice，Truncate表示清空原来的内容
        {
            file.write((char *)&payload[idx], data_len);
            file.close();

            // 通知swdl模块做压缩包升级
            bSuccess = pSwdlMng->updateByPkg(update_type, file_name);
        }
    }

    idx += data_len;

    // 响应消息包完成
    qDebug() << ("-->>升级包收发完成,等待升级中");
    sendCommonAnswer(inc_num, SERVER_SET_TERMINAL_CFG, 0);

    // 直接调用接口失败或无法调用包加载模块处理时,需要直接返回升级失败的结果,而当前成功的情况下,需要等具体升级完成之后返回升级结果
    if (bSuccess == false)
    {
        answerUpdateResult(update_type, false);
    }
}

/*******************************************
函数功能: 软件升级结果
入口参数: type 升级类型
        bSuccess 升级结果
出口参数:
备    注:
*******************************************/
void CJt808Device::answerUpdateResult(int type, bool bSuccess)
{
    QByteArray ba(100, 0);
    BYTE *send_data = (BYTE *)ba.data();
    qDebug() << "-->>发送软件升级结果:" << bSuccess;

    int idx = 0;
    /* 类型 */
    send_data[idx++] = (BYTE)type;
    /* 结果 */
    send_data[idx++] = (BYTE)bSuccess;

    sendPayload2Jt808Serv(TERMINAL_UPDATE_ANSWER, send_data, idx);
}

/*******************************************
函数功能: 云台控制桩函数,当前硬件没有云台控制功能,这里打桩处理,便于以后功能扩展
入口参数: int cycle_num,  流水号
        BYTE *payload,   净荷
        WORD lenth       长度
出口参数:
备    注:
*******************************************/
void CJt808Device::procPTZControlDot(int cycle_num, BYTE *payload, WORD lenth)
{
}
