#ifndef CJT808CONTROL_H
#define CJT808CONTROL_H
#include "common.h"
#include "jt808_msg.h"
#include "jt808_common.h"
#include <QString>
#include <QMap>

struct PhoneNode
{
    BYTE call_type;
    QString phone_num;
    QString user_name;
};

// GBT19056 命令字含义
enum GBT19056CMD
{
    GBT19056_GET_VERSION = 0,     //采集记录仪执行标准版本
    GBT19056_GET_CUR_DRIVER_INFO, //采集当前驾驶人信息
    GBT19056_GET_CUR_TIME,        //采集记录仪的实时时间
    GBT19056_GET_TOTAL_MILEAGE,   //采集累计行驶里程
    GBT19056_GET_DATA_BLOCK,      //记录仪脉冲系数数据块格式
    GBT19056_GET_VEHICLE_INFO,    //采集车辆信息
    GBT19056_GET_STATUS_INFO,     //状态信号配置信息
    GBT19056_GET_STATUS_INFO2,    //状态信号配置信息
    GBT19056_GET_SPEED_RECORD,    //采集指定的行驶速度记录(08H)
    GBT19056_GET_LOCATION_RECORD, //指定的位置信息记录(09H)
    GBT19056_GET_ACCIDENT_RECORD, //采集指定的事故疑点记录(10H)
    GBT19056_GET_OVERDUE_RECORD,  //采集指定的超时驾驶记录(11H)
    GBT19056_GET_DRIVER_RECORD,   //采集指定的驾驶人身份记录(12H)
    GBT19056_GET_POWER_SUPPLY,    //采集指定的记录仪外部供电记录(13H)
    GBT19056_GET_MODIFY_RECORD,   //采集指定的记录仪参数修改记录(14H)
    GBT19056_GET_SPEED_LOG,       //采集指定的速度状态日志(15H)
};

struct DriverIccardInfo
{
    BYTE status;              // 卡状态 插卡(1)/拔卡(2)
    BYTE time[6];             // 插卡/拔卡时间
    BYTE read_status;         // 读卡状态 成功(0) 卡片密钥认证失败(1) 卡片被锁定(2) 卡片被拔出(3) 数据校验失败(4)
    std::string driver_name;  // 驾驶员姓名
    std::string license_num;  // 发证机关
    std::string license_name; // 发证机关
    BYTE valid_date[4];       // 证件有效日期
    std::string driver_id;    // 身份证信息
};

struct CAN_DATA
{
    DWORD can_id;
    BYTE can_data[8];
};

typedef QMap<QString, PhoneNode> MAP_PHONE_BOOK;

class CJt808Device
{
public:
    CJt808Device();
    virtual ~CJt808Device() {}

    /*******************************************
    函数功能: 软件升级结果
    入口参数: type 升级类型
            bSuccess 升级结果
    出口参数:
    备    注:
    *******************************************/
    virtual void answerUpdateResult(int type, bool bSuccess);

protected:
    /*******************************************
    函数功能: 发送数据到JT08服务器
    入口参数: WORD cmd,       命令字
            BYTE *payload,  净荷
            WORD lenth      净荷长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD sendPayload2Jt808Serv(WORD cmd, BYTE *payload, WORD lenth) = 0;

    /*******************************************
    函数功能: 发送通用应答
    入口参数: cycle_num 流水号
             id  消息ID
             result 执行结果
    出口参数:
    备    注:
    *******************************************/
    virtual WORD sendCommonAnswer(WORD cycle_num, WORD id, BYTE result) = 0;

    /*******************************************
    函数功能: 组织位置信息
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    virtual WORD fillLocationInfo(BYTE *payload, GPS_INFO &gps) = 0;

    /*******************************************
    函数功能: 电话回拨
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procSetCallBack(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 设置电话本
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procSetPhoneNum(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 控制车辆
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procCtrlVehicle(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 读取行驶记录
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procReadRecord(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 记录参数下传设置
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procSetRecordPara(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 上报驾驶员身份信息请求
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procRequestIdcard(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 上报驾驶员身份信息
    入口参数: DriverIccardInfo &iccard_info IC卡信息
    出口参数:
    备    注:
    *******************************************/
    WORD sendDriverIdcard(DriverIccardInfo &iccard_info);

    /*******************************************
    函数功能: 上报电子运单信息
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    WORD sendFreightNote(char *str_freight, int lenth);

    /*******************************************
    函数功能: 定位数据批量上传
    入口参数: gps_info, GPS信息数组
            size gps信息数
            type 类型
    出口参数:
    备    注:TERMINAL_UP_MULTI_GPS_DATA
    *******************************************/
    WORD sendGpsBatch(GPS_INFO *gps_info, WORD size, BYTE type);

    /*******************************************
    函数功能: CAN总线数据上传
    入口参数: CAN_DATA *data_array, CAN数组
            WORD size 数组大小
    出口参数:
    备    注:TERMINAL_UP_CAN_DATA
    *******************************************/
    WORD sendCanData(CAN_DATA *data_array, WORD size);

    /*******************************************
    函数功能: 软件升级请求
    入口参数: int inc_num, 
            BYTE *payload, 
            WORD lenth
    出口参数:
    备    注:
    *******************************************/
    void procSoftwareUpdate(int inc_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 云台控制桩函数,当前硬件没有云台控制功能,这里打桩处理,便于以后功能扩展
    入口参数: int cycle_num,  流水号
            BYTE *payload,   净荷
            WORD lenth       长度
    出口参数:
    备    注:
    *******************************************/
    void procPTZControlDot(int cycle_num, BYTE *payload, WORD lenth);

protected:
    MAP_PHONE_BOOK m_mapPhoneBook;
};

#endif // CJT808CONTROL_H
