#include "CJt808Location.h"
#include <QDebug>
#include <QDateTime>
#include "timer/CTimerManager.h"
#include "gps/CGPSManager.h"

CJt808Location::CJt808Location()
    : m_iTempReportLeft(0),
      m_iTempReportInterval(0),
      m_pSensorMng(nullptr)
{
}

/*******************************************
函数功能: 发送位置信息
入口参数:是否临时跟踪数据
出口参数:
备    注:
*******************************************/
void CJt808Location::reportLocationInfo(bool bTempRep)
{
    if (m_pSensorMng == nullptr)
    {
        m_pSensorMng = dynamic_cast<CGPSManager *>(QueryPlugin("CGPSManager"));
    }

    // 当上报间隔不为0时，处于临时跟踪状态，需要根据情况结束状态
    if (m_iTempReportInterval > 0)
    {
        m_iTempReportLeft -= m_iTempReportInterval;
        if (m_iTempReportLeft <= 0)
        {
            m_iTempReportInterval = 0;
            m_iTempReportLeft = 0;
            stopTempGpsReport();
        }
    }

    QByteArray ba(4000, 0);
    BYTE *send_data = (BYTE *)ba.data();

    WORD lenth = 0;
    qDebug() << "-->>发送位置信息";
    if (m_pSensorMng)
    {
        GPS_INFO Gps_info ;        
        // 没有初始化的gps信息不上报,避免错误.
        if(!m_pSensorMng->getCurGPSInfo(Gps_info))
        {
            return;
        }
        
        lenth += fillLocationInfo(send_data, Gps_info);
        sendPayload2Jt808Serv(TERMINAL_UP_GPS_DATA, send_data, lenth);
    }
}

/*******************************************
    函数功能: 处理查询GPS信息命令
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
void CJt808Location::procQueryGpsData(int cycle_num, BYTE *payload, WORD lenth)
{
    qDebug() << "<<--查询位置信息";
    int idx = 0;

    qDebug() << "-->>查询位置信息应答";
    if (m_pSensorMng == nullptr)
    {
        m_pSensorMng = dynamic_cast<CGPSManager *>(QueryPlugin("CGPSManager"));
    }

    if (m_pSensorMng)
    {
        QByteArray ba(4000, 0);
        BYTE *send_data = (BYTE *)ba.data();
        /* 应答流水号 */
        short_to_two_char(cycle_num, send_data);
        idx += 2;

        /* 位置信息 */
        GPS_INFO Gps_info;
                
        // 没有初始化的gps信息不上报,避免错误.
        m_pSensorMng->getCurGPSInfo(Gps_info);
        
        idx += fillLocationInfo(&send_data[idx], Gps_info);
        //idx += fillLocationAppend(&send_data[idx], Gps_info);
        sendPayload2Jt808Serv(TERMINAL_ANWSER_GPS_DATA, send_data, idx);
    }
}

/*******************************************
函数功能: 组织位置信息
入口参数:
出口参数:
备    注:
*******************************************/
WORD CJt808Location::fillLocationInfo(BYTE *data, GPS_INFO &gps)
{
    WORD idx = 0;
    /* 报警标识 */
    long_to_four_char(getAlarmStatus().value, &data[idx]);
    idx += 4;
    /* 状态 */
    long_to_four_char(getCommonStatus().value, &data[idx]);
    idx += 4;
    /* 经维度 默认精度已经X100了, 这里只需要10^4即可*/
    long_to_four_char(gps.latitude * 10000, &data[idx]);
    idx += 4;
    long_to_four_char(gps.longitude * 10000, &data[idx]);
    idx += 4;

    /* 高度 */
    short_to_two_char(gps.high, &data[idx]);
    idx += 2;
    /* 速度 标准是0.1km/h 所以这里要X10*/
    short_to_two_char(gps.speed * 10, &data[idx]);
    idx += 2;
    /* 方向 */
    short_to_two_char(gps.dir, &data[idx]);
    idx += 2;
    /* 时间 */
    QDateTime tm = QDateTime::fromTime_t(gps.time_tick);
    ST_TIMER st_tm;
    st_tm.year = tm.date().year();
    st_tm.month = tm.date().month();
    st_tm.day = tm.date().day();
    st_tm.hour = tm.time().hour();
    st_tm.min = tm.time().minute();
    st_tm.sec = tm.time().second();
    st_tm.week = tm.date().weekNumber();
    time_hex2bcd(st_tm, &data[22]);
    idx += 6;

    /* 附加信息 */
    // 里程数
    data[idx++] = 0x1;
    data[idx++] = 4;
    long_to_four_char(1000, &data[idx]);
    idx += 4;
    //    /* 模拟量 */
    //    data[idx++] = 0x2B;
    //    data[idx++] = 4;
    //    short_to_two_char(0, &data[2]);
    //    idx += 2;
    //    short_to_two_char(1, &data[4]);
    //    idx += 2;
    //    /* GSM信号质量 */
    //    data[idx++] = 0x30;
    //    data[idx++] = 1;
    //    data[idx++] = 100;
    //    /* GNSS卫星数 */
    //    data[idx++] = (int)gps.sat_num;
    //    data[idx++] = 1;
    //    data[idx++] = 1;

    return idx;
}

/*******************************************
    函数功能: 临时位置跟踪
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
void CJt808Location::procGpsReportTemp(int cycle_num, BYTE *payload, WORD lenth)
{
    qDebug() << "<<--临时位置跟踪控制";
    int idx = 0;
    WORD interval = two_char_to_short(&payload[idx]);
    idx += 2;
    DWORD total = four_char_to_long(&payload[idx]);

    startTempGpsReport(total, interval);

    qDebug() << "-->>临时位置跟踪控制应答";
    sendCommonAnswer(cycle_num, SERVER_TEMPORARY_LOC_CTRL, 0);
}

/*******************************************
    函数功能: 开始临时位置跟踪
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
void CJt808Location::startTempGpsReport(int total_sec, int interval)
{
    m_iTempReportLeft = total_sec;
    m_iTempReportInterval = interval;

    // 定时器模块
    CTimerManager *pTimerMng = dynamic_cast<CTimerManager *>(::QueryPlugin("CTimerManager"));
    if (pTimerMng)
    {
        pTimerMng->RegTimer(MODULE_JT808_MNG, TIMER_JT808_GPS_REPORT, m_iTempReportInterval);
    }
}

/*******************************************
    函数功能: 结束临时位置跟踪
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
void CJt808Location::stopTempGpsReport()
{
    // 定时器模块
    CTimerManager *pTimerMng = dynamic_cast<CTimerManager *>(::QueryPlugin("CTimerManager"));
    if (pTimerMng)
    {
        pTimerMng->UnregTimer(TIMER_JT808_GPS_REPORT);
    }
}

/*******************************************
函数功能: 速度越限检查
入口参数: 
出口参数:
备    注:
*******************************************/
void CJt808Location::speedExceedCheck()
{
    
}
    