#ifndef CJT808LOCATION_H
#define CJT808LOCATION_H
#include "common.h"
#include "jt808_msg.h"
#include "jt808_common.h"

class CGPSManager;

class CJt808Location
{
public:
    CJt808Location();
    virtual ~CJt808Location() {}

    /*******************************************
    函数功能: 发送位置信息
    入口参数: 是否临时跟踪数据
    出口参数:
    备    注:
    *******************************************/
    void reportLocationInfo(bool bTempRep);

    /*******************************************
    函数功能: 速度越限检查
    入口参数: 
    出口参数:
    备    注:
    *******************************************/
    virtual void speedExceedCheck();
    
protected:
    /*******************************************
    函数功能: 发送数据到JT08服务器
    入口参数: WORD cmd,       命令字
            BYTE *payload,  净荷
            WORD lenth      净荷长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD sendPayload2Jt808Serv(WORD cmd, BYTE *payload, WORD lenth) = 0;

    /*******************************************
    函数功能: 获取jt808告警状态
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    virtual ALARM_STATUS &getAlarmStatus() = 0;

    /*******************************************
    函数功能: 获取jt808通用状态
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    virtual UNIT_STATUS &getCommonStatus() = 0;

    /*******************************************
    函数功能: 发送通用应答
    入口参数: cycle_num 流水号
             id  消息ID
             result 执行结果
    出口参数:
    备    注:
    *******************************************/
    virtual WORD sendCommonAnswer(WORD cycle_num, WORD id, BYTE result) = 0;

    /*******************************************
    函数功能: 处理查询GPS信息命令
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual void procQueryGpsData(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 组织位置信息
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    virtual WORD fillLocationInfo(BYTE *payload, GPS_INFO &gps);

    /*******************************************
    函数功能: 临时位置跟踪
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual void procGpsReportTemp(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 开始临时位置跟踪
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual void startTempGpsReport(int total_sec, int interval);

    /*******************************************
    函数功能: 结束临时位置跟踪
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual void stopTempGpsReport();

protected:
    int m_iTempReportLeft;
    int m_iTempReportInterval;

    CGPSManager *m_pSensorMng;
};

#endif // CJT808LOCATION_H
