#ifndef CJT808MEDIA_H
#define CJT808MEDIA_H
#include "common.h"
#include "jt808_msg.h"
#include "jt808_common.h"
#include <QMap>
#include <QString>
#include "file/CFileManager.h"

#pragma pack(1)

struct MediaEvent
{
    DWORD media_id;  // 媒体ID(>0)
    BYTE media_type; // 多媒体类型 图像(0) 音频(1) 视频(2)
    BYTE media_code; // JPEG(0) TIF(1) MP3(2) WAV(3) WMV(4) 其他保留
    BYTE event_type; // 事件编码 平台下发指令(0) 定时动作(1) 抢劫报警触发(2) 碰撞侧翻触发(3) 其他
    BYTE channel;    // 通道
};

struct MediaData
{
    MediaEvent event;
    GPS_INFO gps;
    QString data_path;
};

struct JT1078_ALARM_STATUS
{
    ALARM_STATUS alarm;
    DWORD append_alarm;
};

#pragma pack()

class CJt808Media
{
public:
    CJt808Media();

    virtual ~CJt808Media() {}

protected:
    /*******************************************
    函数功能: 发送数据到JT08服务器
    入口参数: WORD cmd,       命令字
            BYTE *payload,  净荷
            WORD lenth      净荷长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD sendPayload2Jt808Serv(WORD cmd, BYTE *payload, WORD lenth) = 0;

    /*******************************************
    函数功能: 发送通用应答
    入口参数: cycle_num 流水号
             id  消息ID
             result 执行结果
    出口参数:
    备    注:
    *******************************************/
    virtual WORD sendCommonAnswer(WORD cycle_num, WORD id, BYTE result) = 0;

    /*******************************************
    函数功能: 组织位置信息
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    virtual WORD fillLocationInfo(BYTE *payload, GPS_INFO &gps) = 0;
    /*******************************************
    函数功能: 多媒体上传应答
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procMediaReq(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 拍照
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procTakePicture(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 拍照应答
    入口参数: data 数据包
            lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD RspTakePicture(WORD cycl_num, int count, DWORD *pArray);

    /*******************************************
    函数功能: 存储多媒体数据检索
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procCheckMedia(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 媒体数据上传
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procReadMedia(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 开始录音
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procStartRecordSound(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 单条多媒体数据检索上传
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procUploadMediadata(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 多媒体数据上传
    入口参数: MediaData &media_data 多媒体信息
    出口参数:
    备    注:
    *******************************************/
    virtual WORD sendMediaData(MediaData &media_data);

    /*******************************************
    函数功能: 多媒体数据上传
    入口参数: MediaEvent &event 多媒体事件
    出口参数:
    备    注:
    *******************************************/
    virtual WORD sendMediaEvent(MediaEvent &event);

    /*******************************************
    函数功能: 查询终端音视频属性
    入口参数: int cycle_num,  流水号
            BYTE *payload,   净荷
            WORD lenth       长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procQueryMediaProperty(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 实时音视频请求
    入口参数: int cycle_num,  流水号
            BYTE *payload,   净荷
            WORD lenth       长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procRealtimeMediaTrans(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 实时音视频控制
    入口参数: int cycle_num,  流水号
            BYTE *payload,   净荷
            WORD lenth       长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procRealtimeMediaControl(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 服务端实时音视频传输状态通知
    入口参数: int cycle_num,  流水号
            BYTE *payload,   净荷
            WORD lenth       长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procMediaStatusNotify(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 查询终端媒体文件资源
    入口参数: int cycle_num,  流水号
            BYTE *payload,   净荷
            WORD lenth       长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procQueryMediaResource(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 录像文件列表应答
    入口参数: int cycle_num,  流水号
            pFileList*,   文件列表
            file_cnt       文件个数
    出口参数:
    备    注:
    *******************************************/
    WORD sendFilelistAnswer(int cycle_num, VCT_FILE_LIST &file_list);

    /*******************************************
    函数功能: 录像文件回放请求
    入口参数: int cycle_num,  流水号
            BYTE *payload,   净荷
            WORD lenth       长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procVideoPlay(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 录像文件回放控制
    入口参数: int cycle_num,  流水号
            BYTE *payload,   净荷
            WORD lenth       长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procVideoControl(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 文件上传请求
    入口参数: int cycle_num,  流水号
            BYTE *payload,   净荷
            WORD lenth       长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procFileUpload(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 文件上传控制
    入口参数: int cycle_num,  流水号
            BYTE *payload,   净荷
            WORD lenth       长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD procFileUploadControl(int cycle_num, BYTE *payload, WORD lenth);

protected:
    QMap<DWORD, QString> m_mapImageData;
    QMap<DWORD, QString> m_mapVideoData;
    QMap<DWORD, QString> m_mapAudioData;
};

#endif // CJT808MEDIA_H
