#include "jt808_common.h"
#include <iostream>
#include <string.h>
#include <cstdarg>
#include <QDebug>
#include <assert.h>

BYTE hex2bcd(BYTE hex)
{
    hex %= 100;
    return (((hex / 10) << 4) + (hex % 10));
}

void time_hex2bcd(ST_TIMER time, BYTE *bcdtime)
{
    bcdtime[0] = hex2bcd(time.year % 100);
    bcdtime[1] = hex2bcd(time.month);
    bcdtime[2] = hex2bcd(time.day);
    bcdtime[3] = hex2bcd(time.hour);
    bcdtime[4] = hex2bcd(time.min);
    bcdtime[5] = hex2bcd(time.sec);
}

void time2bcd(BYTE *time, BYTE *bcdtime)
{
    bcdtime[0] = hex2bcd(time[0]);
    bcdtime[1] = hex2bcd(time[1]);
    bcdtime[2] = hex2bcd(time[2]);
    bcdtime[3] = hex2bcd(time[3]);
    bcdtime[4] = hex2bcd(time[4]);
    bcdtime[5] = hex2bcd(time[5]);
}

void bcd2time(BYTE *bcdtime, BYTE *time)
{
    time[0] = bcd2value(bcdtime[0]);
    time[1] = bcd2value(bcdtime[1]);
    time[2] = bcd2value(bcdtime[2]);
    time[3] = bcd2value(bcdtime[3]);
    time[4] = bcd2value(bcdtime[4]);
    time[5] = bcd2value(bcdtime[5]);
}

BYTE bcd2value(BYTE bcd)
{
    BYTE value;
    value = ((bcd >> 4)) * 10;
    value += bcd & 0x0F;
    return value;
}

/*******************************************
函数名称: IPint2string
函数功能: 
入口参数: 
出口参数: 
备    注: 
*******************************************/
char *IPint2string(DWORD intIp, char *IP)
{
    BYTE a = intIp >> 24;
    BYTE b = intIp >> 16;
    BYTE c = intIp >> 8;
    BYTE d = intIp;
    sprintf(IP, "%d.%d.%d.%d", a, b, c, d);
    return IP;
}

/*******************************************
函数名称: stringIsSame
函数功能: 判断字符串是否相同
入口参数: 
出口参数: 
备    注: 
*******************************************/
bool stringIsSame(char *buf, const char *string)
{
    for (int i = 0; i < (int)strlen(string); i++)
    {
        if (buf[i] != string[i])
        {
            return false;
        }
    }
    return true;
}

//计算参数个数
BYTE check_para_num(BYTE *the_text)
{
    BYTE j = 0;

    for (; ((*the_text) != '\0') && ((*the_text) != '\r') && ((*the_text) != '\n'); the_text++)
    {
        if ((*the_text) == ':' || (*the_text) == ',')
        {
            j++;
        }
    }
    return (j);
}

/*******************************************
函数功能: 向指定号码发送短信
入口参数:
出口参数:
备    注:
*******************************************/
WORD sendSMS(char *num, const char *format, ...)
{
    WORD lenth = 0;
    char msg_data[1024] = {0};
    va_list args;

    if (strlen(num) == 0)
    {
        return (0);
    }
    va_start(args, format);
    lenth = vsprintf(msg_data + lenth, format, args);
    va_end(args);

    //send sms
    return (lenth);
}

std::string Jt808Msg::phone_number = "018888888888";

//////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////             Jt808Msg               ///////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////

Jt808Msg::Jt808Msg()
{
}

Jt808Msg::Jt808Msg(Jt808MsgNode &node)
{
    addMessageNode(node);
}

/*******************************************
函数功能: 析构函数
入口参数: 
出口参数: 
备    注: 
*******************************************/
Jt808Msg::~Jt808Msg()
{
    m_baPayload.clear();
    foreach (auto item, m_mapMsg)
    {
        SAFE_DELETE(item);
    }
    m_mapMsg.clear();
}

/*******************************************
函数功能: 释放消息
入口参数: 
出口参数: 
备    注: 
*******************************************/
void Jt808Msg::release()
{
    m_baPayload.clear();
    foreach (auto item, m_mapMsg)
    {
        SAFE_DELETE(item);
    }
    m_mapMsg.clear();
}

/*******************************************
函数功能: 获取净荷地址
入口参数: 
出口参数: 净荷信息
备    注: 测试通过
*******************************************/
QByteArray Jt808Msg::getPayload()
{
    m_baPayload.clear();
    auto it = m_mapMsg.begin();
    if (it != m_mapMsg.end())
    {
        // 是否分包
        Jt808MsgNode &node = *it.value();
        if (node.isMultiPack())
        {
            if (this->isMsgComplete())
            {
                for (int i = 1; i <= node.getPkgCount(); i++)
                {
                    auto it_idx = m_mapMsg.find(i);
                    if (it_idx != m_mapMsg.end())
                    {
                        m_baPayload.push_back(QByteArray((char *)it_idx.value()->getPayload(), it_idx.value()->getPayloadSize()));
                    }
                    else
                    {
                        m_baPayload.clear();
                        // 记录错误信息
                        break;
                    }
                }
            }
            else
            {
                // 还没有接收完毕,记录警告
            }
        }
        else
        {
            m_baPayload.push_back(QByteArray((char *)it.value()->getPayload(), it.value()->getPayloadSize()));
        }
    }
    return m_baPayload;
}

/*******************************************
函数功能:多包数据是否完整
入口参数:
出口参数: 多包数据是否完整
备    注: 多包数据是否完整
*******************************************/
bool Jt808Msg::isMsgComplete()
{
    int msg_size = m_mapMsg.size();
    if (msg_size > 0)
    {
        Jt808MsgNode *pNode = m_mapMsg.begin().value();
        if (pNode)
        {
            int cnt = pNode->getPkgCount();
            if (msg_size == cnt)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
    }
    else
    {
        return false;
    }
}
/*******************************************
函数功能: 获取净荷长度
入口参数: int idx 分包索引号
出口参数: 净荷长度
备    注: 测试通过
*******************************************/
WORD Jt808Msg::getPayloadSize(int idx)
{
    auto it = m_mapMsg.find(idx);
    if (it != m_mapMsg.end())
    {
        Jt808MsgNode *pNode = it.value();
        if (pNode)
        {
            return pNode->getPayloadSize();
        }
        else
        {
            return 0;
        }
    }
    return 0;
}

/*******************************************
函数功能: 获取(分包)消息
入口参数: int idx 分包索引号
出口参数: 净荷长度
备    注: 测试通过
*******************************************/
BYTE *Jt808Msg::getMsgBuff(int idx)
{
    auto it = m_mapMsg.find(idx);
    if (it != m_mapMsg.end())
    {
        Jt808MsgNode *pNode = it.value();
        if (pNode)
        {
            return (BYTE *)pNode->m_pbyBuffer;
        }
        else
        {
            return nullptr;
        }
    }
    return nullptr;
}

/*******************************************
函数功能: 附加分包信息
入口参数: int idx 分包索引号
出口参数: 净荷长度
备    注: 测试通过
*******************************************/
WORD Jt808Msg::addMessageNode(Jt808MsgNode &node)
{
    Jt808MsgNode *pNode = new Jt808MsgNode(node);
    if (pNode->isMultiPack())
    {
        // 多包场景下，先获取分包号
        WORD pkg_num = pNode->getPkgNum();
        auto it = m_mapMsg.find(pkg_num);
        // 如果已经找到对应序号的包，则直接丢弃
        if (it == m_mapMsg.end())
        {
            m_mapMsg[pkg_num] = pNode;
        }
    }
    else
    {
        m_mapMsg[1] = pNode;
    }
    return 0;
}

/*******************************************
函数功能: 获取msgid
入口参数: 
出口参数: msg id
备    注: 针对重建的消息使用
*******************************************/
WORD Jt808Msg::getMsgID()
{
    if (m_mapMsg.size() > 0)
    {
        Jt808MsgNode *pNode = m_mapMsg.begin().value();
        if (pNode)
        {
            return pNode->getMsgID();
        }
        else
        {
            return 0;
        }
    }
    return 0;
}

/*******************************************
函数功能: 获取流水号
入口参数: 
出口参数: 流水号id
备    注: 针对重建的消息使用
*******************************************/
WORD Jt808Msg::getCycle()
{
    if (m_mapMsg.size() > 0)
    {
        Jt808MsgNode *pNode = m_mapMsg.begin().value();
        if (pNode)
        {
            return pNode->getCycle();
        }
        else
        {
            return 0;
        }
    }
    return 0;
}

/*******************************************
函数功能: 获取流水号
入口参数: int idx,
        Jt808MsgNode& msg_node out 输出参数
出口参数: 流水号id
备    注: 针对重建的消息使用
*******************************************/
Jt808MsgNode *Jt808Msg::getMsgNode(int idx)
{
    auto it = m_mapMsg.find(idx);
    if (it != m_mapMsg.end())
    {
        return it.value();
    }

    return nullptr;
}

/*******************************************
函数功能: 获取总包数
入口参数:
出口参数: 总包数
备    注: 针对重建的消息使用
*******************************************/
WORD Jt808Msg::getTotalPktCount()
{
    if (m_mapMsg.size() > 0)
    {
        Jt808MsgNode *pNode = m_mapMsg.begin().value();
        if (pNode)
        {
            return pNode->getTotalPktCount();
        }
    }
    return 1;
}

/*******************************************
函数功能: 是否有多包
入口参数: 
出口参数: 是否有多包
备    注: 针对重建的消息使用
*******************************************/
bool Jt808Msg::isMultiPack()
{
    if (m_mapMsg.size() > 0)
    {
        Jt808MsgNode *pNode = m_mapMsg.begin().value();
        if (pNode)
        {
            return pNode->isMultiPack();
        }
    }
    return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////            Jt808MsgNode           ///////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////
Jt808MsgNode::Jt808MsgNode()
    : m_ilenth(0)
{
    m_pbyBuffer = new BYTE[2048];
    memset(m_pbyBuffer, 0, 2048);
    //memset(buffer, 0, sizeof(buffer));
}

Jt808MsgNode::Jt808MsgNode(Jt808MsgNode &node)
{
    m_pbyBuffer = new BYTE[2048];
    memcpy(m_pbyBuffer, node.m_pbyBuffer, 2048);
    m_ilenth = node.m_ilenth;
}

Jt808MsgNode::~Jt808MsgNode()
{
    m_ilenth = 0;
    SAFE_DELETE_ARRAY(m_pbyBuffer);
}

/*******************************************
函数功能: 释放消息
入口参数: 
出口参数: 
备    注: 
*******************************************/
void Jt808MsgNode::release()
{
    m_ilenth = 0;
    SAFE_DELETE_ARRAY(m_pbyBuffer);
}

/*******************************************
函数功能: 是否分包
入口参数:
出口参数: 净荷首地址
备    注: 测试通过
*******************************************/
bool Jt808MsgNode::isMultiPack()
{
    MSG_ATTR attr;
    attr.value = two_char_to_short(m_pbyBuffer + 3);
    return (bool)(attr.bit.package);
}

/*******************************************
函数功能: 获取总包数
入口参数:
出口参数: 是否有多包
备    注: 针对重建的消息使用
*******************************************/
WORD Jt808MsgNode::getPkgCount()
{
    WORD cnt = 0;
    if (isMultiPack())
    {
        cnt = two_char_to_short(m_pbyBuffer + JT808_HEAD_LENGTH_V1);
    }
    else
    {
        cnt = 1;
    }

    return cnt;
}

/*******************************************
函数功能: 获取当前包序号
入口参数:
出口参数: 包序号 1开始
备    注:
*******************************************/
WORD Jt808MsgNode::getPkgNum()
{
    WORD num = 0;
    if (isMultiPack())
    {
        num = two_char_to_short(m_pbyBuffer + JT808_HEAD_LENGTH_V1 + 2);
    }
    else
    {
        num = 1;
    }

    return num;
}

/*******************************************
函数功能: 获取净荷地址
入口参数:
出口参数: 净荷首地址
备    注: 测试通过
*******************************************/
BYTE *Jt808MsgNode::getPayload()
{
    BYTE *payload = nullptr;
    if (isMultiPack())
    {
        payload = m_pbyBuffer + JT808_HEAD_LENGTH_V1 + 4;
    }
    else
    {
        payload = m_pbyBuffer + JT808_HEAD_LENGTH_V1;
    }
    return payload;
}

/*******************************************
函数功能: 获取总包数
入口参数:
出口参数: 总包数
备    注: 针对重建的消息使用
*******************************************/
WORD Jt808MsgNode::getTotalPktCount()
{
    if (isMultiPack() == true)
    {
        return two_char_to_short(m_pbyBuffer + JT808_HEAD_LENGTH_V1);
    }
    else
    {
        return 1;
    }
}

/*******************************************
函数功能: 获取当前包数
入口参数:
出口参数: 当前包序号
*******************************************/
WORD Jt808MsgNode::getCurPktIndex()
{
    if (isMultiPack() == true)
    {
        return two_char_to_short(m_pbyBuffer + JT808_HEAD_LENGTH_V1 + 2);
    }
    else
    {
        return 1;
    }
}

/*******************************************
函数功能: 获取净荷长度
入口参数:
出口参数: 净荷长度
备    注: 测试通过
*******************************************/
WORD Jt808MsgNode::getPayloadSize()
{
    int size = 0;
    if (isMultiPack())
    {
        size = m_ilenth - JT808_HEAD_LENGTH_V1 - 2 - 4;
    }
    else
    {
        size = m_ilenth - JT808_HEAD_LENGTH_V1 - 2;
    }

    size = std::max(size, 0);

    return size;
}

/*******************************************
函数功能: 生成协议报文
入口参数: data,      发送地址
        send_len,   数据长度(不含分包的4个字节)
        total_pkt,  总包数
        cur_pkt     当前包序号
出口参数: 转义后的字节数
备    注: 测试通过
*******************************************/
WORD Jt808MsgNode::setPayload(BYTE *data, WORD send_len, WORD total_pkt, WORD cur_pkt)
{
    assert(cur_pkt <= total_pkt);
    if (data == nullptr && send_len > 0)
    {
        assert(0);
    }

    BYTE *payload = m_pbyBuffer + JT808_HEAD_LENGTH_V1;
    int copy_len = send_len;
    int idx = 0;

    // 是否多包
    if (total_pkt > 1)
    {
        // 极限值为1023-4
        copy_len = std::min((int)send_len, 1019);
        short_to_two_char(total_pkt, payload + idx);
        idx += 2;
        short_to_two_char(cur_pkt, payload + idx);
        idx += 2;
        memcpy(payload + idx, data, copy_len);
        idx += copy_len;
    }
    else
    {
        // 极限值为1023
        copy_len = std::min((int)send_len, 1023);
        memcpy(payload, data, copy_len);
        idx += copy_len;
    }
    return copy_len;
}

/*******************************************
函数功能: 生成协议报文
入口参数: id, 命令字
　　　　　lenth,净荷长度
出口参数: 转义后的字节数
备    注: 测试通过
*******************************************/
WORD Jt808MsgNode::generateMsg(std::string str_phone, WORD id, WORD lenth, bool bMultiPkt, WORD cycle_num)
{
    static WORD serial_num = 1;
    WORD ret_lenth = 0;
    BYTE check = 0;
    WORD i, j;

    BYTE *pBuff = m_pbyBuffer;
    int head_len = JT808_HEAD_LENGTH_V1;
    // 多包需要在净荷首地址添加总包和当前包序号,所以会需要添加额外的4个字节
    if (bMultiPkt)
    {
        head_len += 4;
    }

    /* 包标识 */
    pBuff[0] = 0x7E;
    /* 消息ID */
    short_to_two_char(id, pBuff + 1);
    /* 消息属性 */
    MSG_ATTR prop;
    prop.bit.encrypt = 0;
    prop.bit.version = 1;
    prop.bit.package = bMultiPkt;
    prop.bit.retain = 0;
    prop.bit.msglen = lenth;

    short_to_two_char(prop.value, pBuff + 3);
    // 版本信息，2019版本为1
    pBuff[5] = 1;
    /* 手机号 BCD[10] 不足位的情况下前补0*/
    int phone_len = std::min(20, (int)str_phone.length());
    char *phone_num = (char *)str_phone.c_str();

    for (int i = 0; i < phone_len; i += 2)
    {
        if (phone_len - i > 2)
        {
            pBuff[15 - i / 2] = ((phone_num[phone_len - i - 2] - '0') << 4) + (phone_num[phone_len - i - 1] - '0');
        }
        else
        {
            pBuff[15 - i / 2] = (phone_num[phone_len - i - 1] - '0');
        }
    }

    /* 流水号 */
    if (cycle_num == INVALID_CYCLE_NUM)
    {
        short_to_two_char(serial_num, pBuff + 16);
        serial_num++;
    }
    else //多包消息需要和之前的流水号保持一致
    {
        short_to_two_char(cycle_num, pBuff + 16);
    }
    /* 校验码 */
    check = 0;
    for (i = 0; i < head_len +lenth +  - 1; i++)
    {
        check ^= pBuff[i + 1];
    }

    pBuff[head_len + lenth] = check;

    /* 包标识 */
    pBuff[head_len + lenth + 1] = 0x7E;
    ret_lenth = head_len + lenth + 2;

    /* 转义 */
    for (i = 1; i < ret_lenth - 1; i++)
    {
        if (pBuff[i] == 0x7E || pBuff[i] == 0x7D)
        {
            for (j = 0; j < ret_lenth - i; j++)
            {
                pBuff[ret_lenth - j] = pBuff[ret_lenth - j - 1];
            }
            pBuff[i + 1] = (pBuff[i] == 0x7E) ? 0x02 : 0x01;
            pBuff[i] = 0x7D;
            i++;
            ret_lenth++;
        }
    }

    return ret_lenth;
}

/*******************************************
函数功能: 数据反转义
入口参数: data, 转义前的数据
          lenth,转以后的数据
出口参数: 转义后的字节数
备    注: 测试通过
*******************************************/
WORD Jt808MsgNode::revMsgEscape(BYTE *data, WORD lenth)
{
    WORD i, j;

    for (i = 0; i < lenth; i++)
    {
        if (data[i] == 0x7D && (data[i + 1] == 0x01 || data[i + 1] == 0x02))
        {
            data[i] += (data[i + 1] - 1);
            for (j = i + 1; j < lenth - 1; j++)
            {
                data[j] = data[j + 1];
            }
            lenth--;
        }
    }
    return (lenth);
}

/*******************************************
函数功能: 通过buff重建消息
入口参数: data, 接收的数据
          lenth,接收的数据长度
出口参数: 剩余的buff指针头
备    注: 测试通过
*******************************************/
BYTE *Jt808MsgNode::rebuildMsg(BYTE *data, WORD lenth)
{
    /* 查找包头标识，防止头部有其他非载荷*/
    int head_loc = 0;
    while (head_loc < lenth)
    {
        if (data[head_loc] == JT808_END_FLAG && data[head_loc + 1] != JT808_END_FLAG)
        {
            break;
        }
        head_loc++;
    }

    data = data + head_loc;
    lenth = lenth - head_loc;

    /* 查找包尾标识 */
    int tail_loc = 1;
    while (tail_loc < lenth)
    {
        if (data[tail_loc] == JT808_END_FLAG || tail_loc >= lenth)
        {
            break;
        }
        tail_loc++;
    }

    // 如果数据段没有找到7E，则直接返回0
    if (tail_loc >= lenth)
    {
        return nullptr;
    }

    /* 反转移 */
    int pack_lenth = revMsgEscape(data, tail_loc + 1);

    /* 检查长度是否合法 */
    int body_lenth = (((WORD)data[3] << 8) & 0x0300) + data[4];
    // 长度匹配
    if (body_lenth + JT808_HEAD_LENGTH_V1 + 2 == pack_lenth)
    {
        /* 检验位校验 */
        BYTE check = 0;
        for (int i = 1; i < pack_lenth - 2; i++)
        {
            check ^= data[i];
        }

        // 校验和匹配
        if (check == data[pack_lenth - 2])
        {
            //这时消息算是真正的完整，这里开始真正的恢复数据
            // 拷贝数据
            memcpy(m_pbyBuffer, data, pack_lenth);
            m_ilenth = pack_lenth;
        }
    }

    data = data + tail_loc;
    lenth -= tail_loc;
    return data + 1;
}

/*******************************************
    函数功能: 获取msgid
    入口参数: 
    出口参数: msg id
    备    注: 针对重建的消息使用
    *******************************************/
WORD Jt808MsgNode::getMsgID()
{
    return two_char_to_short(m_pbyBuffer + 1);
}

/*******************************************
    函数功能: 获取流水号
    入口参数: 
    出口参数: 流水号id
    备    注: 针对重建的消息使用
    *******************************************/
WORD Jt808MsgNode::getCycle()
{
    return two_char_to_short(m_pbyBuffer + 16);
}
