#ifndef CJT808_COMMON_H
#define CJT808_COMMON_H
#include <string.h>
#include <stdio.h>
#include "common.h"
#include "jt808_msg.h"
#include <QMap>

//时间结构体
typedef struct
{
    WORD year;
    BYTE month;
    BYTE day;
    BYTE hour;
    BYTE min;
    BYTE sec;
    BYTE week;
} ST_TIMER;

// typedef struct Gps_info_struct
// {
//     BYTE status;
//     ST_TIMER time;
//     DWORD time_dec;
//     BYTE ant_status;
//     BYTE satellites;
//     BYTE ew;
//     BYTE ns;
//     DWORD lo_data;
//     DWORD la_data;
//     WORD speed; // 1/100km/h
//     WORD dir;
//     BYTE hdop;
//     WORD hight; //¸ß¶È
//     BYTE bds_sate;
//     BYTE gps_sate;
// } Gps_info_ST;

enum JT808_COMMON_RESULT
{
    RESULT_SUCCESSED = 0,   // 成功
    RESULT_FAILED,          // 失败
    RESULT_MSG_ERROR,       // 消息错误
    RESULT_MSG_UNSUPPORTED, // 不支持

};

struct JT808CommonResponse
{
    WORD inc_idx;    // 流水号
    WORD message_id; // 消息ID
    BYTE result;     // 结果
};

/* 服务器运行状态 */
typedef enum
{
    SERVER_NO_CONNECT,       //未连接
    SERVER_CONNECT_REGOK,    //已注册
    SERVER_CONNECT_AUTHENOK, //已鉴权
    SERVER_CONNECT_UPDATE,   //升级
} SERVER_STATUS;

/* M_SERVERPARA.CTRL_STEP的定义 */
#define CTRL_STEP_WAIT_REG 0
#define CTRL_STEP_CONNECT_SERVER 1
#define CTRL_STEP_CONNECT_SERVER_WAIT 2
#define CTRL_STEP_BEFORE_REGIST_DEVICE 10
#define CTRL_STEP_REGIST_DEVICE 11
#define CTRL_STEP_REGIST_DEVICE_WAIT 12
#define CTRL_STEP_REGIST_DEVICE_WAIT_10MIN 13
#define CTRL_STEP_LOGIN_SERVER 20
#define CTRL_STEP_LOGIN_SERVER_WAIT 21
#define CTRL_STEP_DEREGIST 25
#define CTRL_STEP_DEREGIST_WAIT 26
#define CTRL_STEP_STAND_BY 30
#define CTRL_STEP_SERVER_ANSWER_WAIT 31
#define CTRL_STEP_SLEEP_CLOSE_SOCKET 40
#define CTRL_STEP_SLEEP_WAIT 41
#define CTRL_STEP_CONNECT_UPDATESERVER 50
#define CTRL_STEP_CONNECT_UPDATESERVER_WAIT 51
#define CTRL_STEP_LOGIN_UPDATESERVER 52
#define CTRL_STEP_LOGIN_UPDATESERVER_WAIT 53
#define CTRL_STEP_GET_UPDATEINFO 54
#define CTRL_STEP_GET_UPDATEINFO_WAIT 55
#define CTRL_STEP_GET_PACKAGE 56
#define CTRL_STEP_GET_PACKAGE_WAIT 57
#define CTRL_STEP_SLEEP 60

//服务器运行参数
typedef struct
{
    SERVER_STATUS state; //服务器连接状态
    BYTE ctrl_step;      //控制步骤
    BYTE err_times;      //错误计数
    BYTE main_or_back;   //用主服务还是备份服务
    WORD answer_wait_time;
    DWORD answer_wait_time_new;
    WORD answer_try_times_new;
    WORD sleep_wait_time;
    BYTE need_server_answer;
    BYTE need_server_ack;
    WORD send_serial;
    WORD heartbeat_time;
    WORD ex_buffer_delay_time;
    WORD received_data_count;
} SERVER_STRUCT;

#define REGIST_CODE_LENTH 50

struct DEVICE_INFO
{
    DEVICE_INFO()
    {
        memset(this, 0, sizeof(DEVICE_INFO));
        w_falg = 0;
        server_ip_int = 0x7f000001;
        server_port = 8088;
        char *phone = (char *)"018888888888";
        memcpy(phone_num, phone, strlen(phone) + 1);

        send_time_inteval_default = 60;
        send_time_inteval_sleep = 60;
        send_time_inteval_alarm = 0;
        heartbeat_time_inteval = 60;
        char *phone1 = (char *)"018888888888";
        char *phone2 = (char *)"018888888888";
        memcpy(alarm_sms_phone[0], phone1, strlen(phone1) + 1);
        memcpy(alarm_sms_phone[1], phone2, strlen(phone2) + 1);
    }

    BYTE w_falg;
    DWORD server_ip_int;
    WORD server_port;
    BYTE regist_code[REGIST_CODE_LENTH];
    BYTE phone_num[13];
    WORD send_time_inteval_default;
    WORD send_time_inteval_sleep;
    WORD send_time_inteval_alarm;
    WORD heartbeat_time_inteval;
    BYTE alarm_sms_phone[2][13];
};

extern BYTE hex2bcd(BYTE hex);
extern void time2bcd(BYTE *time, BYTE *bcdtime);
extern void bcd2time(BYTE *bcdtime, BYTE *time);
extern void time_hex2bcd(ST_TIMER time, BYTE *bcdtime);
extern BYTE bcd2value(BYTE bcd);

extern char *IPint2string(DWORD intIp, char *IP);
extern bool stringIsSame(char *buf, const char *string);
extern BYTE check_para_num(BYTE *the_text);

extern WORD sendSMS(char *num, const char *format, ...);

#define MAX_JT808_FRAMEBUF_LEN 1023
#define MAX_PHONE_NUMBER 13
#define JT808_HEAD_LENGTH 13
#define JT808_END_FLAG 0x7E

#define MAX_JT808_RECV_LEN_V1 2000
#define MAX_JT808_FRAMEBUF_LEN_V1 1023
#define MAX_PHONE_NUMBER_V1 13
#define JT808_HEAD_LENGTH_V1 18

#pragma pack(push, 1)

// JT808消息包，这个消息包包含分包信息
struct Jt808MsgNode;

typedef QMap<int, Jt808MsgNode *> JT808_MSG_NODE_MAP;

//////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////              Jt808Msg             ///////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////
struct Jt808Msg
{
    Jt808Msg();

    Jt808Msg(Jt808MsgNode &node);

    // 析构函数
    ~Jt808Msg();

    /*******************************************
    函数功能: 释放消息
    入口参数: 
    出口参数: 
    备    注: 
    *******************************************/
    void release();

    /*******************************************
    函数功能: 获取流水号
    入口参数: int idx,
            Jt808MsgNode& msg_node out 输出参数
    出口参数: 流水号id
    备    注: 针对重建的消息使用
    *******************************************/
    Jt808MsgNode *getMsgNode(int idx);

    /*******************************************
    函数功能: 获取净荷地址
    入口参数: 
    出口参数: 净荷信息
    备    注: 测试通过
    *******************************************/
    QByteArray getPayload();

    /*******************************************
    函数功能: 获取净荷长度
    入口参数: int idx 分包索引号
    出口参数: 净荷长度
    备    注: 测试通过
    *******************************************/
    WORD getPayloadSize(int idx = 0);

    /*******************************************
    函数功能: 获取总包数
    入口参数:
    出口参数: 总包数
    备    注: 针对重建的消息使用
    *******************************************/
    WORD getTotalPktCount();

    /*******************************************
    函数功能: 获取(分包)消息
    入口参数: int idx 分包索引号
    出口参数: 净荷长度
    备    注: 测试通过
    *******************************************/
    BYTE *getMsgBuff(int idx = 0);

    /*******************************************
    函数功能: 附加分包信息
    入口参数: int idx 分包索引号
    出口参数: 净荷长度
    备    注: 测试通过
    *******************************************/
    WORD addMessageNode(Jt808MsgNode &node);

    /*******************************************
    函数功能: 获取msgid
    入口参数: 
    出口参数: msg id
    备    注: 针对重建的消息使用
    *******************************************/
    WORD getMsgID();

    /*******************************************
    函数功能: 获取流水号
    入口参数: 
    出口参数: 流水号id
    备    注: 针对重建的消息使用
    *******************************************/
    WORD getCycle();

    /*******************************************
    函数功能: 是否有多包
    入口参数: 
    出口参数: 是否有多包
    备    注: 是否有多包
    *******************************************/
    bool isMultiPack();

    /*******************************************
    函数功能:多包数据是否完整
    入口参数:
    出口参数: 多包数据是否完整
    备    注: 多包数据是否完整
    *******************************************/
    bool isMsgComplete();

    // 终端电话号码
    static std::string phone_number;

    // jt808消息节点
    JT808_MSG_NODE_MAP m_mapMsg;

    QByteArray m_baPayload;
    int m_totalPkgCnt;
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////            Jt808MsgNode           ///////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////

#define INVALID_CYCLE_NUM 0xffff

struct Jt808MsgNode
{
public:
    Jt808MsgNode();
    Jt808MsgNode(Jt808MsgNode &node);

    ~Jt808MsgNode();

    /*******************************************
    函数功能: 释放消息
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    void release();

    /*******************************************
    函数功能: 获取净荷地址
    入口参数:int idx 分包索引号
    出口参数: 净荷首地址
    备    注: 测试通过
    *******************************************/
    BYTE *getPayload();

    /*******************************************
    函数功能: 生成协议报文
    入口参数: payload,   净荷地址
            send_len,   净荷长度
            total_pkt,  总包数
            cur_pkt     当前包序号
    出口参数: 转义后的字节数
    备    注: 测试通过
    *******************************************/
    WORD setPayload(BYTE *payload, WORD send_len, WORD total_pkt = 1, WORD cur_pkt = 1);

    /*******************************************
    函数功能: 获取净荷长度
    入口参数: int idx 分包索引号
    出口参数: 净荷长度
    备    注: 测试通过
    *******************************************/
    WORD getPayloadSize();

    /*******************************************
    函数功能: 获取当前包数
    入口参数:
    出口参数: 当前包序号
    *******************************************/
    WORD getCurPktIndex();

    /*******************************************
    函数功能: 获取总包数
    入口参数:
    出口参数: 总包数
    备    注: 针对重建的消息使用
    *******************************************/
    WORD getTotalPktCount();

    /*******************************************
    函数功能: 生成消息
    入口参数: data, 接收的原始数据
            lenth,原始数据长度
    出口参数: 转义后的字节数
    备    注: 测试通过
    *******************************************/
    WORD generateMsg(std::string phone_num, WORD id, WORD lenth, bool bMultiPkt = false, WORD cycle_num = INVALID_CYCLE_NUM);

    /*******************************************
    函数功能: 重建msg
    入口参数: data, 接收的原始数据
            lenth,原始数据长度
    出口参数: 转义后的字节数
    备    注: 测试通过
    *******************************************/
    BYTE *rebuildMsg(BYTE *data, WORD lenth);

    /*******************************************
    函数功能: 数据反转义
    入口参数: data, 转义前的数据
            lenth,转以后的数据
    出口参数: 转义后的字节数
    备    注: 测试通过
    *******************************************/
    WORD revMsgEscape(BYTE *data, WORD lenth);

    /*******************************************
    函数功能: 获取msgid
    入口参数: 
    出口参数: msg id
    备    注: 针对重建的消息使用
    *******************************************/
    WORD getMsgID();

    /*******************************************
    函数功能: 获取流水号
    入口参数: 
    出口参数: 流水号id
    备    注: 针对重建的消息使用
    *******************************************/
    WORD getCycle();

    /*******************************************
    函数功能: 是否有多包
    入口参数: 
    出口参数: 是否有多包
    备    注: 针对重建的消息使用
    *******************************************/
    bool isMultiPack();

    /*******************************************
    函数功能: 获取总包数
    入口参数:
    出口参数: 是否有多包
    备    注: 针对重建的消息使用
    *******************************************/
    WORD getPkgCount();

    /*******************************************
    函数功能: 获取当前包序号
    入口参数:
    出口参数: 包序号 1开始
    备    注:
    *******************************************/
    WORD getPkgNum();

public:
    BYTE *m_pbyBuffer;
    int m_ilenth;
};

#pragma pack(pop)

#endif
