#include "CLogManager.h"
#include "../socket/TcpServer.h"
#include "../base_module/factory.h"
#include "timer/CTimerManager.h"

REGISTER_MODULE(CLogManager, "CLogManager")

CLogManager::CLogManager(QObject *parent)
    : QObject(parent),
      IModule(),
      m_pTcpModule(nullptr),
      m_pTimerMng(nullptr)
{
}

CLogManager::~CLogManager()
{
    std::cout << name() << std::endl;
}

//初始化操作
bool CLogManager::Initialize()
{
    makePath("../log");

    //起主消息处理线程
    STARTUP_TASK(TaskMain)

    return true;
}

bool CLogManager::RegistMessage()
{
    m_pTcpModule = dynamic_cast<ITcpModule *>(QueryPlugin("TcpModule"));
    if (m_pTcpModule)
    {
        m_pTcpModule->RegistMessageCallback(this, MODULE_LOG_MNG);
    }
    return true;
}

//主消息任务，主要用于和其他模块之间的消息通讯
void CLogManager::TaskMain(void *pBuff)
{
    (void)pBuff;
    setCurrentTaskName("tskLogMain");
    MODULE_MESSAGE *pMsg = nullptr;
    RegistMessage();
    waitForStart();

    //起主消息处理线程
    STARTUP_TASK(TaskSaveLog)

    while (!isShutdown())
    {
        if (GetModuleMessage(pMsg)) // get msg from message queue
        {
            switch (pMsg->iCmd)
            {
            default:
                break;
            }

            SAFE_DELETE(pMsg);
        }
    }
}

//传感器收集任务, 用于定时轮询传感器相关的信息;
void CLogManager::TaskSaveLog(void *pBuff)
{
    (void)pBuff;
    while (this->isShutdown() == false)
    {
        //初步定在30分钟每次,太长更新不及时,太短更新太快占用系统资源
        mSleep(1800000);
        m_mtxMap.lock();
        foreach (auto pLog, m_mapLog)
        {
            pLog->backup();
        }
        m_mtxMap.unlock();
    }
}

CLoger *CLogManager::logCreate(QString log_name, int max_count)
{
    auto it = m_mapLog.find(log_name);
    if (it == m_mapLog.end())
    {
        CLoger *pLog = new CLoger("../log", log_name, max_count);
        pLog->restore();
        {
            QMutexLocker locker(&m_mtxMap);
            m_mapLog[log_name] = pLog;
        }
        return pLog;
    }
    else
    {
        return it.value();
    }
}

QString CLogManager::logQuery(QString log_name)
{
    auto it = m_mapLog.find(log_name);
    if (it != m_mapLog.end())
    {
        CLoger *pLog = it.value();
        // pLog
        pLog->setRecLevel(LOG_LEVEL_DEBUG); // test code
    }
    return "";
}
