#ifndef CLOGMANAGER_H
#define CLOGMANAGER_H
#include "base_module.h"
#include <QObject>
#include <QDateTime>
#include "CLoger.h"
#include <QString>
#include <QVector>
#include <QMap>

class ITcpModule;
class CLogManager;
class CTimerManager;

#define LOG_DECLARE() \
    static CLoger *s_pLoger;

#define LOG_DEFINITION(class_name) \
    CLoger *class_name::s_pLoger = nullptr;

#define LOG_CREATE(class_name, log_name)                                            \
    CLogManager *pLogMng = dynamic_cast<CLogManager *>(QueryPlugin("CLogManager")); \
    if (pLogMng)                                                                    \
    {                                                                               \
        class_name::s_pLoger = pLogMng->logCreate(log_name);                        \
    }

#define LOG_CREATE_WITH_COUNT(class_name, log_name, max_count)                                            \
    CLogManager *pLogMng = dynamic_cast<CLogManager *>(QueryPlugin("CLogManager")); \
    if (pLogMng)                                                                    \
    {                                                                               \
        class_name::s_pLoger = pLogMng->logCreate(log_name, max_count);                        \
    }

//#define LOG_INFO

//format: [LEVEL][yy-mm-dd h:m:s.ms][tid]file_name:line_no(func_name):content

#define filename(x) strrchr(x,'/')?strrchr(x,'/')+1:x

#define LOG_TRACE(class_name, fmt, args...) \
    class_name::s_pLoger->logWrite(LOG_LEVEL_TRACE, "[%s:%d](%s): " fmt, filename(__FILE__), __LINE__, __FUNCTION__, ##args);

#define LOG_DEBUG(class_name, fmt, args...) \
    class_name::s_pLoger->logWrite(LOG_LEVEL_DEBUG, "[%s:%d](%s): " fmt, filename(__FILE__), __LINE__, __FUNCTION__, ##args);

#define LOG_INFO(class_name, fmt, args...) \
    class_name::s_pLoger->logWrite(LOG_LEVEL_INFO, "[%s:%d](%s): " fmt, filename(__FILE__), __LINE__, __FUNCTION__, ##args);

#define LOG_WARN(class_name, fmt, args...) \
    class_name::s_pLoger->logWrite(LOG_LEVEL_WARN, "[%s:%d](%s): " fmt, filename(__FILE__), __LINE__, __FUNCTION__, ##args);

#define LOG_ERROR(class_name, fmt, args...) \
    class_name::s_pLoger->logWrite(LOG_LEVEL_ERROR, "[%s:%d](%s): " fmt, filename(__FILE__), __LINE__, __FUNCTION__, ##args);

#define LOG_FATAL(class_name, fmt, args...) \
    class_name::s_pLoger->logWrite(LOG_LEVEL_FATAL, "[%s:%d](%s): " fmt, filename(__FILE__), __LINE__, __FUNCTION__, ##args);

typedef QMap<QString, CLoger *> LOG_MAP;
typedef QVector<QString> SAVE_LOG_VCT;
class CLogManager : public QObject, public IModule
{
    Q_OBJECT
public:
    MODULE_NAME(CLogManager)
    explicit CLogManager(QObject *parent = nullptr);
    ~CLogManager();

    //初始化操作
    virtual bool Initialize();
    //模块注册接口
    bool RegistMessage();

    CLoger *logCreate(QString log_name, int max_count = 10000);
    QString logQuery(QString log_name);

Q_SIGNALS:
    //

public Q_SLOTS:

protected:
    //主消息任务，主要用于和其他模块之间的消息通讯
    THREAD_DEFINITION(CLogManager, TaskMain)
    //传感器收集任务, 用于定时轮询传感器相关的信息;
    THREAD_DEFINITION(CLogManager, TaskSaveLog)

    //主消息任务，主要用于和其他模块之间的消息通讯
    virtual void TaskMain(void *pBuff);
    //传感器收集任务, 用于定时轮询传感器相关的信息;
    virtual void TaskSaveLog(void *pBuff);

protected:
    ITcpModule *m_pTcpModule;
    CTimerManager *m_pTimerMng;

    LOG_MAP m_mapLog;
    QMutex m_mtxMap;
};

#endif // CLOGMANAGER_H
