#ifndef CLOGNODE_H
#define CLOGNODE_H
#include <QMutex>
#include <QVector>
#include <QFile>
#include <QStringList>

enum LOG_LEVEL
{
    LOG_LEVEL_FATAL = 1,
    LOG_LEVEL_ERROR,
    LOG_LEVEL_WARN,
    LOG_LEVEL_INFO,
    LOG_LEVEL_DEBUG,
    LOG_LEVEL_TRACE,
    LOG_LEVEL_MAX
};

class CLoger
{
public:
    CLoger(QString path, QString log_name, int max_count);
    CLoger();
    bool initialize();
    bool backup();
    bool restore();
    void setShowLevel(LOG_LEVEL level);
    void setRecLevel(LOG_LEVEL level);
    void logWrite(LOG_LEVEL level, const char *fmt, ...);
    void writeLogInfo(LOG_LEVEL level, const char *fmt, va_list arg_list);

protected:
    QMutex m_mtxLog;
    // 日志文件名(不含后缀) *.log *.inc *.tmplog分别对应全量日志/增量日志/临时日志

    // 全量持久化日志文件
    QFile m_fullLog;
    // 增量日志文件
    QFile m_incLog;
    // 临时日志文件
    QFile m_tmpLog;

    QString m_strPath;
    QString m_strName;

    QStringList m_LogInfo;

    QStringList m_incLogTemp;
    // 是否处于备份状态
    bool m_bBackup;
    LOG_LEVEL m_iShowLevel;
    LOG_LEVEL m_iRecLevel;

    QMutex m_mtxInc;

    QString m_strLogLevel[(int)LOG_LEVEL_MAX];

    int m_maxRecordCnt;
};

#endif // CLOGNODE_H
