#include "MainWindow.h"
#include <QApplication>
#include "base_module/base_module.h"
#include <QMessageBox>
#include "gst/gst.h"
#include "ui/setting/FormLogin.h"
#include "ui/setting/SplashScreen.h"
#include <QString>
#include <QVector>
#include <string>
#include <QDebug>
#include <QDir>
#include "libboot.h"
#include "common.h"

// 分离文件名和路径
extern QStringList sepratePathName(QString path_name);

struct MODULE_INFO
{
    MODULE_INFO(std::string class_name, QString module_name_cn)
        : strClass(class_name), strNameCN(module_name_cn)
    {
    }

    std::string strClass;
    QString strNameCN;
};
//
//模块加载可以在主线程中添加函数内处理，也可在主线程中调用该函数，改写本函数内容。
bool LoadPluginAll(SplashScreen &Splash)
{
    bool bSuccess = true;
    QVector<MODULE_INFO> modules =
        {
            MODULE_INFO("TcpModule", "通信"),
            MODULE_INFO("CConfigManager", "配置"),
            MODULE_INFO("CLogManager", "日志管理"),
            MODULE_INFO("CTimerManager", "定时器"),
            MODULE_INFO("CWebSocketClient", "网络"),
            MODULE_INFO("CJT808Terminal", "JT/T808管理"),
            MODULE_INFO("CFileManager", "文件管理"),
            MODULE_INFO("CSWDLManager", "包加载"),
            MODULE_INFO("CDeviceManager", "设备管理"),
            MODULE_INFO("CCameraManager", "相机管理"),
            MODULE_INFO("CGPSManager", "GPS管理"),
            MODULE_INFO("CAlarmManager", "告警管理"),
            MODULE_INFO("CGB28181Module", "GB28181模块"),
            MODULE_INFO("CMcAdptManager", "手咪模块"),
            MODULE_INFO("CdmmManager", "检测模块"),

        };

    int index = 0;
    const QString status_loading = "[%1/%2]%3模块加载中...";
    const QString status_ok = "[%1/%2]%3模块加载成功!";
    const QString status_error = "[%1/%2]%3模块加载失败!";

    Splash.setRange(0, modules.size());

    foreach (auto module, modules)
    {
        Splash.showProgressMessage(status_loading.arg(index).arg(modules.size()).arg(module.strNameCN));
        if (!LoadPlugin(module.strClass))
        {
            Splash.showProgressMessage(status_error.arg(index).arg(modules.size()).arg(module.strNameCN));
            mSleep(5000);
            bSuccess = false;
            break;
        }

        Splash.showProgressMessage(status_ok.arg(index).arg(modules.size()).arg(module.strNameCN));
        mSleep(50);
        index++;
        Splash.updateProgress(index);
    }

    return bSuccess;
}

int main(int argc, char *argv[])
{
    // 应用程序初始化操作
    QApplication a(argc, argv);

    // 这里先改下启动目录,因为在带参数启动时,是watchdog启动,启动目录为boot目录,这里要改成应用程序目录
    qDebug() << "////////////////////////////////////////////////////";

    // 获取当前运行的app名称

    QString app_name = a.applicationName();
    QString app_path = a.applicationDirPath();
    QString file_path = a.applicationFilePath();

    //    if (argc == 3)
    //    {
    //        QString strMain = argv[0];
    //        QString strBak = argv[1];
    //        QString strMode = argv[2];
    //        QStringList list= sepratePathName(argv[i]);
    //        if(QDir(list[0]).exists())
    //        {
    //            QDir::setCurrent(list[0]);
    //        }
    //    }

    system("pwd");
    qDebug() << "////////////////////////////////////////////////////";
    //

    // GStreamer 初始化函数
    gst_init(&argc, &argv);
    gst_debug_set_default_threshold(GST_LEVEL_ERROR);

    // 启动过程log动画
    SplashScreen splash(QPixmap(":/resources/logo/logo.png"));
    splash.show();

    //轻量化处理的模块加载
    if (!LoadPluginAll(splash))
    {
        QMessageBox::warning(nullptr, "LoadPlugin Failed!", "LoadPlugin Failed!");
        return 0;
    }

    // 关闭启动动画
    splash.close();

    // 主窗体
    MainWindow w;

    // 窗体启动后 真正启动任务
    startUpAll();

    // 登陆窗口，这里建立应用程序和服务器之间的联系
    //FormLogin *dlg = new FormLogin(&w);
	FormLogin *dlg = new FormLogin(&w);
    //   dlg->minimumSize();
    //if (QDialog::Accepted != dlg->exec())
    //{
        //关闭
        //ShutdownAll();
        //return 0;
   // }
    //delete dlg;

    //全屏显示
    w.showFullScreen();
    int ret = a.exec();

    //关闭
    ShutdownAll();
    return ret;
}
