#ifndef CWEBSOCKETMANAGE_H
#define CWEBSOCKETMANAGE_H
#include <QObject>
#include "base_module.h"
#include "log/CLogManager.h"
#include <QWebSocket>

class ITcpModule;
class CDeviceManager;
class CCameraManager;
class CWebSocketClient;
class CFileManager;


typedef bool (CWebSocketClient::*WS_COMMAND_PROC_FUNC)(QVariantMap& map);
typedef QMap<QString, WS_COMMAND_PROC_FUNC> MAP_FUNC_PROC;

class CWebSocketClient : public QObject, public IModule
{
    Q_OBJECT
public:
    MODULE_NAME(CWebSocketClient)
    LOG_DECLARE()
    explicit CWebSocketClient(QObject *parent = nullptr);
    ~CWebSocketClient();

    //初始化操作
    virtual bool Initialize();

    //模块注册接口
    bool RegistMessage();

    // 和业软服务端建立链接外部接口
    void connectToServer(QString strServIpPort = "");

    // 是否准备就绪
    bool isReady();

    // 发送消息到主机
    bool sendMessage(QVariant cmd = "");

Q_SIGNALS:
    // 通知连接到服务器
    void notifyServConnected();

    void notifyConnect2Sock(QString);

    void notifySendTextMessage(QString);
    
public slots:
    // 建立连接后处理。
    void onWebSockConnected();

    // 关闭连接
    void closeConnection();

    // 接收到文本消息后的处理
    void onTextMessageReceived(QString);

    // 发送消息到主机
    void sendTextMessage(QString);

    void onStateChanged(QAbstractSocket::SocketState state);

    // 和业软服务端建立链接
    void onConnect2Sock(QString strServIpPort = "");
protected:
    //主消息任务，主要用于和其他模块之间的消息通讯
    THREAD_DEFINITION(CWebSocketClient, TaskMain)

    //主消息任务，主要用于和其他模块之间的消息通讯
    virtual void TaskMain(void *pBuff);


    // 登陆响应
    bool onLogin(QVariantMap& prop_arr);

    // 获取参数响应
    bool onGetParams(QVariantMap& prop_arr);

    // 更新系统
    bool upgradeSystem(QVariantMap& prop_arr);

    // 回复出厂值
    bool restoreFactorySettings(QVariantMap& prop_arr);

    // 删除文件
    bool fileDelete(QVariantMap& prop_arr);

    // 实时视频rtmp推送
    bool realtimeVideo(QVariantMap& prop_arr);

    // 停止实时视频推流
    bool stopRealtimeVideo(QVariantMap& prop_arr);

    // 服务器要求获取录像文件的文件列表
    bool videoRecordList(QVariantMap& prop_arr);

    // 抓拍
    bool snapShot(QVariantMap& prop_arr);

    // 解析服务器下发命令或响应
    bool onPraseServCmd(QVariantMap& prop_arr);

protected:

    MAP_FUNC_PROC       m_pFuncList;
    //通信模块
    ITcpModule          *m_pTcpModule;
    //web socket 实例;
    QWebSocket          *m_pWebSock;
    //文件管理类
    CFileManager        *m_pFileMng;
    // 设备管理类
    CDeviceManager      *m_pDevMng;
    //相机管理类
    CCameraManager      *m_pCamMng;

    // 服务器IP:端口
    QString             m_strServIpPort;

    // 准备就绪状态
    bool                m_bReady;
};

#endif // CWEBSOCKETMANAGE_H
