﻿#include "TcpClient.h"
#include <iostream>
#include "../base_module/factory.h"
#include "../common.h"
#include <QDebug>

using namespace std;

#ifdef WIN32
#include <process.h>
#else
#include "unistd.h"
#include "signal.h"
#endif

//固定IP和端口号
const char *STR_IP_ADDR = "192.168.0.114";
const int iPort = 3600;

REGISTER_MODULE(CTcpClient, "TcpModule")

CTcpClient::CTcpClient()
    : m_ClientSocket(-1),
      m_needBoardcast(true),
      m_bConnected(false),
      m_strCurSrvIp(""),
      m_nThreadID(NULL)
{
    setComType(COM_TYPE_VIEWER);
}

CTcpClient::~CTcpClient()
{
#ifdef WIN32
    WSACleanup();
#endif
}

int CTcpClient::getSocketError(string str_info)
{
    int nError;
#ifdef WIN32
    nError = WSAGetLastError();
#else
    nError = SOCKET_ERROR;
#endif
    qDebug() << str_info.c_str() << nError;
    return nError;
}

bool CTcpClient::Initialize()
{
    //初始化socket
    if (!InitSocket())
    {
        return false;
    }

    //查找设备
    STARTUP_TASK(TaskDeviceFinder)

    //注册任务
    STARTUP_TASK(TaskMain)

#ifdef WIN32

#else
    //下面这段代码主要是用于linux下 socket没建立起来之前发送消息导致异常的处理
    sigset_t signal_mask;
    sigemptyset(&signal_mask);
    sigaddset(&signal_mask, SIGPIPE);
    int rc = pthread_sigmask(SIG_BLOCK, &signal_mask, NULL);
    if (rc != 0)
    {
        printf("block sigpipe error/n");
    }
#endif

    return true;
}

bool CTcpClient::InitSocket()
{
#ifdef WIN32
    WSADATA wsaData = {0};
    if (0 != WSAStartup(MAKEWORD(2, 2), &wsaData))
    {
        cout << (("socket 初始化失败"));
        return false;
    }
#endif

    m_udpSocket.bind(3602);                                                         //建立监听
    connect(&m_udpSocket, SIGNAL(readyRead()), this, SLOT(procNotifyDeviceInfo())); //接收准备

    return true;
}

void CTcpClient::procNotifyDeviceInfo()
{
    QByteArray datagram;
    do
    {
        datagram.resize(m_udpSocket.pendingDatagramSize());
        m_udpSocket.readDatagram(datagram.data(), datagram.size()); //接收数据
    } while (m_udpSocket.hasPendingDatagrams());

    MODULE_MESSAGE msg;

    QDataStream in(&datagram, QIODevice::ReadOnly);
    in.setVersion(QDataStream::Qt_4_7);
    in.readRawData((char *)&msg, sizeof(msg));
    if (msg.iCmd == CMD_NOTIFY_DEVICE_INFO)
    {
        int dst_ip = ipStr2Int((char *)msg.szBuff);
        Q_EMIT notifNewDeviceFind(dst_ip);
    }
}

bool CTcpClient::connect2Device(int ip_addr)
{
    //先判断当前是否已经连接，
    if (false == m_bConnected) // 没有连接的情况下
    {
        // 做TCP连接操作，
        if (Connect(ipInt2Str(ip_addr)))
        {
            // 成功的情况下，关闭广播，
            stopSearchDevice();
            // 更新当前ip信息
            m_strCurSrvIp = ipInt2Str(ip_addr);
        }
    }
    else // 有连接的情况下
    {
        // 判断当前连接是不是相同连接，
        if (ip_addr == ipStr2Int(m_strCurSrvIp)) // 相同连接的情况下直接返回；
        {
            return true;
        }
        else // 不相同连接的情况下
        {
            // 做TCP连接操作，
            if (Connect(ipInt2Str(ip_addr)))
            {
                // 成功的情况下，关闭广播，
                stopSearchDevice();
                // 更新当前ip信息
                m_strCurSrvIp = ipInt2Str(ip_addr);
                // 通知刷新本地其他模块的信息（主要是数据管理模块要清除遗留的工程信息）
                Q_EMIT notifyDeviceChanged();
            }
            else
            {
                Connect(m_strCurSrvIp);
            }
        }
    }
    return true;
}

bool CTcpClient::Connect(string ip_addr)
{
    // TODO: 在此添加控件通知处理程序代码
    m_ClientSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP); // 指定为tcp方式

    sockaddr_in m_nServeraddr;
    memset(&m_nServeraddr, 0, sizeof(m_nServeraddr));
    m_nServeraddr.sin_family = AF_INET;
    m_nServeraddr.sin_port = htons(iPort);                      // htons()作用是将端口号由主机字节序转换为网络字节序的整数值。
    m_nServeraddr.sin_addr.s_addr = inet_addr(ip_addr.c_str()); //inet_addr()作用是将一个IP字符串转化为一个网络字节序的整数值，

    if (SOCKET_ERROR == ::connect(m_ClientSocket, (sockaddr *)&m_nServeraddr, sizeof(m_nServeraddr)))
    {
        getSocketError("connect failed:");
        //连接失败的情况下，需要通知链接中断
        Q_EMIT notifyDisconnected();
        m_bConnected = false;
        return false;
    }

    m_bConnected = true;
    Q_EMIT notifyConnectionChanged(m_comType, true);
    return true;
}

void CTcpClient::startSearchDevice()
{
    m_needBoardcast = true;
}

void CTcpClient::stopSearchDevice()
{
    m_needBoardcast = false;
}

//广播获取设备信息
void CTcpClient::boardcastAskforDeviceInfo()
{
    string local_ip;
    if (getLocalMainIP(local_ip) == ERR_SUCCESS)
    {
        MODULE_MESSAGE msg;
        msg.dst_id = SRV_TCP_SERVER_MODULE;
        msg.dst_type = APP_TYPE_MAIN;
        msg.iCmd = CMD_ASKFOR_DEVICE_INFO;
        strncpy((char *)msg.szBuff, local_ip.c_str(), local_ip.length() + 1);
        QByteArray datagram;
        QDataStream out(&datagram, QIODevice::WriteOnly);
        out.setVersion(QDataStream::Qt_4_7);
        out.writeRawData((char *)&msg, sizeof(msg));
        //发送主机可以用QHostAddress("127.0.0.1")替换
        QHostAddress addr(local_ip.c_str());
        QStringList ip = QString(local_ip.c_str()).split(".");
        if (ip.length() == 4)
        {
            QString boardcast_ip = QString("%1.%2.%3.%4").arg(ip[0]).arg(ip[1]).arg(ip[2]).arg(255);
            QHostAddress boardcast = QHostAddress(boardcast_ip.toStdString().c_str());
            m_udpSocket.writeDatagram(datagram, boardcast, 3601); // UDP 发送数据
        }
    }
}

//查找设备任务
void CTcpClient::TaskDeviceFinder(void *pBuff)
{
    (void)pBuff;
    while (!m_bShutdown)
    {
        if (m_needBoardcast)
        {
            //这里只需要广播消息即可
            boardcastAskforDeviceInfo();
            mSleep(3000);
        }
        else
        {
            mSleep(100);
        }
    }
}

void CTcpClient::TaskMain(void *pBuff)
{
    CTcpClient *pThis = (CTcpClient *)pBuff;
    waitForStart();
    while (!m_bShutdown)
    {
        if (!m_bConnected)
        {
            mSleep(1000);
            continue;
        }

        MODULE_MESSAGE msg;
        int iLen = recv(pThis->m_ClientSocket, (char *)&msg, sizeof(MODULE_MESSAGE), 0); // 接收服务器端消息
        msg.pszExtBuff = nullptr;
        if (iLen <= 0)
        {
            getSocketError("客户端recv失败:");
            mSleep(500);
            Connect(m_strCurSrvIp);
        }
        else
        {
            if (msg.dst_id == MODULE_SRV_COMM)
            {
                procTcpMessage(msg);
                continue;
            }

            if (msg.dst_type == APP_TYPE_CLIENT)
            {
                //消息处理
                MODULE_MESSAGE *pMessage = new MODULE_MESSAGE;
                memcpy(pMessage, &msg, sizeof(msg));
                int recv_len = 0;
                if (pMessage->ext_length > 0)
                {
                    pMessage->pszExtBuff = new char[pMessage->ext_length];
                    memset(pMessage->pszExtBuff, 0, pMessage->ext_length);
                    while (recv_len < (int)pMessage->ext_length)
                    {
                        int len = recv(pThis->m_ClientSocket, pMessage->pszExtBuff + recv_len, pMessage->ext_length - recv_len, 0); // 接收服务器端消息
                        //接收失败
                        if (len < 0)
                        {
                            recv_len = len;
                            break;
                        }
                        else
                        {
                            recv_len += len;
                        }
                    }

                    if (recv_len < 0)
                    {
                        SAFE_DELETE(pMessage);
                        continue;
                    }
                }
                pThis->DispatchMessage2Thread(pThis->m_ClientSocket, pMessage);
                msg.ext_length = 0;
            }
        }
    }
}

void CTcpClient::procTcpMessage(MODULE_MESSAGE &msg)
{
    switch (msg.iCmd)
    {
    case CMD_NOTIFY_DEVICE_INFO:
        //Q_EMIT notifyDevice(str_ip);
        break;
    case CMD_ASK_FOR_APPTYPE:
    {
        MODULE_MESSAGE msg_send;
        msg_send.dst_type = APP_TYPE_MAIN;
        msg_send.dst_id = SRV_TCP_SERVER_MODULE;
        msg_send.iCmd = CMD_NOTIFY_APP_TYPE;
        int app_type = APP_TYPE_CLIENT;
        memcpy(msg_send.szBuff, &app_type, sizeof(int));
        SendMsg(msg_send, sizeof(MODULE_MESSAGE));
        break;
    }
    default:
        break;
    }
}

unsigned int CTcpClient::SendMsg(MODULE_MESSAGE &msg, int length)
{
    (void)length;
    // TODO: 在此添加控件通知处理程序代码
    DWORD dwRet = ERR_SUCCESS;
    // TODO: 在此添加控件通知处理程序代码
    //如果在本机找到注册的模块，则优先发到本机模块；
    IModule *pModule = m_MessageMap[msg.dst_id];
    if (msg.dst_type == APP_TYPE_CLIENT)
    {
        if (pModule != nullptr && pModule->isSafeModule())
        {
            //跨线程的内存需要重新申请，这里主动new内存防止本线程栈内存被意外释放
            MODULE_MESSAGE *pMsg = new MODULE_MESSAGE;
            memcpy(pMsg, &msg, sizeof(MODULE_MESSAGE));
            // printf("[发送内部消息]模块:%d,消息字:%d\n", msg.dst_id, msg.iCmd);
            SendLocalMessage(pModule, msg.iCmd, pMsg);
            return ERR_SUCCESS;
        }
    }
    else
    {
        if (m_bConnected)
        {
            //本机不存在的模块，通过tcp发送到对端
            dwRet = SendMsg(m_ClientSocket, msg);
        }
    }

    //成功的情况下返回ERROR_NO_ERROR
    return dwRet;
}

DWORD CTcpClient::SendMsg(SOCKET s, MODULE_MESSAGE &msg)
{
    DWORD dwRet = 0;
    // TODO: 在此添加控件通知处理程序代码
#ifdef WIN32
    int con_para = 0;
#else
    int con_para = MSG_NOSIGNAL;
#endif
    int ret = send(s, (char *)&msg, sizeof(msg), con_para);
    if (ret < 0)
    {
        OnSocketDisconnect(s);
        int nError = -1;
        cout << ("发送失败:") << nError << endl;
        dwRet = ERR_SEND_FAILED;
        return dwRet;
    }
    int send_ext = 0;
    while (send_ext < msg.ext_length)
    {
        int send_len = min(MAX_EXT_BUFF_SIZE, msg.ext_length - send_ext);
        int ret = send(s, msg.pszExtBuff + send_ext, send_len, con_para);
        if (ret < 0)
        {
            OnSocketDisconnect(s);
            int nError = -1;
            cout << ("发送失败:") << nError << endl;
            dwRet = ERR_SEND_FAILED;
            break;
        }
        send_ext += ret;
    }
    return 0;
}

void CTcpClient::OnSocketDisconnect(SOCKET aClientSocket)
{
    (void)aClientSocket;
    Q_EMIT notifyDisconnected();
    m_needBoardcast = true;
}
