﻿#pragma once
#include <QtNetwork>

#ifdef WIN32
#include <WinSock2.h>
#pragma comment(lib, "ws2_32.lib")
#else
#include <sys/socket.h>
#include <arpa/inet.h>
#endif

#include <vector>
#include <thread>
#include <mutex>
#include "message.h"
#include <sstream>
#include <map>
#include "./TcpModule.h"
#include <string>
#include "qobject.h"

typedef std::map<int, int> THREAD_VECTOR;

class CTcpClient : public ITcpModule
{
    Q_OBJECT
public:
    MODULE_NAME(CTcpClient)
    THREAD_DEFINITION(CTcpClient, TaskMain)
    THREAD_DEFINITION(CTcpClient, TaskDeviceFinder)

    CTcpClient();
    virtual ~CTcpClient() override;
    //发送消息
    virtual unsigned int SendMsg(MODULE_MESSAGE &msg, int length) override;

    //初始化函数
    virtual bool Initialize() override;
    //接收消息任务
    virtual void TaskMain(void *pBuff) override;

    //查找设备任务
    virtual void TaskDeviceFinder(void *pBuff);

    // 注册模块消息
    virtual bool RegistMessage() override { return true; };

    // 连接设备
    virtual bool connect2Device(int ip_addr);

    //初始化socket
    bool InitSocket();

    //连接服务端
    bool Connect(string ip_addr);

    //收消息处理函数
    static UINT RecvThreadProc(void *pParam);

    //开始搜索设备
    void startSearchDevice();

    //停止搜索设备
    void stopSearchDevice();

protected:
    //设置客户端模式
    void setClientViewMode(bool bClientMode = true);

    //处理TCP相关的消息
    void procTcpMessage(MODULE_MESSAGE &msg);

    //广播获取设备信息
    void boardcastAskforDeviceInfo();
public Q_SLOTS:
    // 连接终端消息处理
    void OnSocketDisconnect(SOCKET aClientSocket);

    // 设备信息处理
    void procNotifyDeviceInfo();
Q_SIGNALS :
    // 通知连接中断
    void notifyDisconnected();

    // 通知设备信息
    void notifNewDeviceFind(int);

    // 通知设备变更
    void notifyDeviceChanged();

protected:
    DWORD SendMsg(SOCKET s, MODULE_MESSAGE &msg);

    //获取socket错误信息，主要是windows下的错误 0表示成功 非0为错误码
    int getSocketError(string str_info);

    //本地客服端SOCKET
    SOCKET m_ClientSocket;

    //udp socket
    QUdpSocket m_udpSocket;
    //是否广播请求
    bool m_needBoardcast;
    //是否已经连接
    bool m_bConnected;
    //当前连接设备IP
    string m_strCurSrvIp;

    //接收线程ID
    unsigned int m_nThreadID;
    std::mutex m_mtxRegMsg;
    std::map<int, IModule *> m_MessageMap;
};
