﻿#include "TcpModule.h"
#include <iostream>
#include <thread>
#include <string>
#include "../base_module/factory.h"
#include <sys/types.h>
#include <assert.h>
#include <QDebug>

#ifdef WIN32
#include <winsock2.h>
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#endif
#include <QtNetwork>

using namespace std;

ITcpModule::ITcpModule()
    : IModule(),
      m_Socket((SOCKET)-1),
      m_comType(COM_TYPE_NONE)
{
}

ITcpModule::~ITcpModule()
{
#ifdef WIN32
#else
    if (m_Socket != (SOCKET)-1)
    {
        close(m_Socket); //关闭socket连接
    }
#endif

    m_Socket = -1;
}

//字符串IP地址转int
int ITcpModule::ipStr2Int(string strIP)
{
    QStringList ip = QString(strIP.c_str()).split(".");
    if(ip.length() == 4)
    {
        return (ip[0].toInt()<<24) + (ip[1].toInt()<<16) + (ip[2].toInt()<<8) + ip[3].toInt();
    }
    return 0;
}
//int型IP地址转字符串
string ITcpModule::ipInt2Str(int iIp)
{
    QString strIP;
    int ip1 = (iIp>>24) & 0xFF;
    int ip2 = (iIp>>16) & 0xFF;
    int ip3 = (iIp>>8) & 0xFF;
    int ip4 = iIp & 0xFF;
    strIP = QString("%1.%2.%3.%4").arg(ip1).arg(ip2).arg(ip3).arg(ip4);
    return strIP.toStdString();
}

int ITcpModule::getLocalMainIP(std::string &local_ip)
{
    int ret = -1;
    QList<QHostAddress> AddressList = QNetworkInterface::allAddresses();
    QHostAddress result;
    foreach(QHostAddress address, AddressList)
    {
        if(address.protocol() == QAbstractSocket::IPv4Protocol &&
           address != QHostAddress::Null &&
           address != QHostAddress::LocalHost)
        {
            if (address.toString().contains("127.0."))
            {
              continue;
            }
            result = address;
            ret = ERR_SUCCESS;
            break;
        }
    }
    local_ip = result.toString().toStdString();
    return ret;
}

bool ITcpModule::DispatchMessage2Thread(SOCKET socket, MODULE_MESSAGE *message)
{
    assert(message!=nullptr);
    (void)socket;
    IModule *pModule = m_MessageMap[message->dst_id]; 

    if (pModule != nullptr && pModule->isSafeModule())
    {
        QString strInfo;
        strInfo.sprintf("[收到TCP消息]模块:%d,消息字:%d\n", message->dst_id, message->iCmd);
        qDebug() << strInfo;
        pModule->PostModuleMessage(message);
        //PostThreadMessage(iThreadID, message.iCmd, (WPARAM)&message, (LPARAM)socket);
    }
    else
    {
        SAFE_DELETE(message);
        qDebug() << "================================ERROR MESSAGE!!=============================";
    }
    return true;
}

bool ITcpModule::SendLocalMessage(IModule *pModule, MODULE_COMMAND_ID cmd, MODULE_MESSAGE *msg)
{
    (void)cmd;
    if (pModule->isSafeModule())
    {
        pModule->PostModuleMessage(msg);
    }
    else
    {
        delete msg;
        return false;
    }

    return true;
}

void ITcpModule::RegistMessageCallback(IModule *pModule, MODULE_ID id)
{
    m_mtxRegMsg.lock();
    if (pModule)
    {
        // only one instance
        qDebug() << "RegistMessageCallback:" << id;
        //assert(m_MessageMap.find(id) == m_MessageMap.end());
        // add instance to map
        m_MessageMap[id] = pModule;
    }
    m_mtxRegMsg.unlock();
}
