﻿#ifndef ITCP_MODULE_H
#define ITCP_MODULE_H
#include "../base_module/base_module.h"
#include "message.h"
#include <QObject>

#define MAX_EXT_BUFF_SIZE 4096

enum TOWER_GUARD_COM_TYPE
{
    //默认连接方式
    COM_TYPE_NONE,
    //塔视设备
    COM_TYPE_TOWGUARD360,
    //预览终端
    COM_TYPE_VIEWER,
    //深度神经网络终端
    COM_TYPE_DNN,
};

class ITcpModule : public QObject, public IModule
{
    Q_OBJECT
public:
    ITcpModule();
    virtual ~ITcpModule();
    //发送消息
    virtual unsigned int SendMsg(MODULE_MESSAGE &msg, int length) = 0;

    virtual unsigned int boardcastMessage(MODULE_MESSAGE &msg) = 0;

    //注册线程消息
    virtual void RegistMessageCallback(IModule *pModule, MODULE_ID id);

    //发送消息到线程
    virtual bool DispatchMessage2Thread(SOCKET socket, MODULE_MESSAGE *message);

    //发送本地消息
    virtual bool SendLocalMessage(IModule *pModule, MODULE_COMMAND_ID cmd, MODULE_MESSAGE *msg);

    // 设置终端类型
    virtual void setComType(int type) { m_comType = type; }

    // 获取终端类型
    virtual int getComType() { return m_comType; }

    // 连接到设备
    virtual bool connect2Device(int ip_addr)
    {
        (void)ip_addr;
        return true;
    }

    //获取本地ip地址
    int getLocalMainIP(std::string &local_ip);

    //字符串IP地址转int
    int ipStr2Int(string string);
    //int型IP地址转字符串
    string ipInt2Str(int ip);
Q_SIGNALS :
    //通知连接状态变更
    void notifyConnectionChanged(int, bool);

Q_SIGNALS :
    void notifyClientModeChanged(bool bSrvOnline);

protected:
    //通讯sock
    SOCKET m_Socket;

    //模块消息映射表
    std::map<int, IModule *> m_MessageMap;

    //注册消息互斥体信号量
    mutex m_mtxRegMsg;

    //终端类型
    int m_comType;
};
#endif // ITCP_MODULE_H
