﻿#include "TcpServer.h"
#include <iostream>
#include <thread>
#include <string>
#include "../base_module/factory.h"
#include <unistd.h>
#include <sys/types.h>
#include <assert.h>
#include <QDebug>
#include <QDateTime>
#include <signal.h>
#ifdef WIN32
#include <winsock2.h>
#pragma comment(lib, "ws2_32.lib")
#else
#include <sys/socket.h>
#endif
using namespace std;

void LOG_ERROR(QString str)
{
    (void)str;
    qDebug() << str;
    return;
}

void LOG_INFO(QString str)
{
    (void)str;
    qDebug() << str;
    return;
}

REGISTER_MODULE(CTcpServer, "TcpModule")

void OutputMessage(const char *outstr, int level = 0)
{
    (void)level;
    std::cout << outstr << std::endl;
}

CTcpServer::CTcpServer()
    : m_ServerSocket(-1)
{
    setComType(COM_TYPE_TOWGUARD360);
}

CTcpServer::~CTcpServer()
{
    if (m_ServerSocket != -1)
        close(m_ServerSocket); //关闭socket连接

    m_ServerSocket = -1;
    std::cout << name() << std::endl;
}

void CTcpServer::destroy()
{
}

//初始化操作
bool CTcpServer::Initialize()
{
    if (!startListen("0.0.0.0", 3600))
    {
        LOG_ERROR("startListen init failed");
        return false;
    }

    // tcp消息接收线程
    STARTUP_TASK(TaskMain)
    return true;
}

// sock创建
bool CTcpServer::startListen(const char *ip, const unsigned short prot)
{
    m_ServerSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

    sockaddr_in saServer;
    saServer.sin_family = AF_INET;   //地址家族
    saServer.sin_port = htons(prot); //注意转化为网络节序
    saServer.sin_addr.s_addr = INADDR_ANY;

#ifdef WIN32
#else
    int opt = 1;
    setsockopt(m_ServerSocket, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt));
    signal(SIGPIPE, SIG_IGN);
#endif
    // 绑定
    if (bind(m_ServerSocket, (struct sockaddr *)&saServer, sizeof(saServer)) == -1)
    {
        OutputMessage(("绑定失败"));
        LOG_ERROR("bind failed");
        return false;
    }

    if (listen(m_ServerSocket, SOMAXCONN) == -1)
    {
        OutputMessage(("监听失败啊"));
        LOG_ERROR("listen failed");
        return false;
    }

    m_udpSocket.bind(3601);                                                         //建立监听
    connect(&m_udpSocket, SIGNAL(readyRead()), this, SLOT(procAskforDeviceInfo())); //接收准备

    return true;
}

// 设备连接应答
void CTcpServer::procAskforDeviceInfo()
{
    qDebug() << "procAskforDeviceInfo";
    QByteArray datagram;
    do
    {
        datagram.resize(m_udpSocket.pendingDatagramSize());
        m_udpSocket.readDatagram(datagram.data(), datagram.size()); //接收数据
    } while (m_udpSocket.hasPendingDatagrams());

    MODULE_MESSAGE msg;

    QDataStream in(&datagram, QIODevice::ReadOnly);
    in.setVersion(QDataStream::Qt_4_7);
    in.readRawData((char *)&msg, sizeof(msg));
    if (msg.iCmd == CMD_ASKFOR_DEVICE_INFO) //如果是请求设备信息
    {
        string dst_ip = (char *)msg.szBuff;
        //发送本设备信息给请求端
        string local_ip;
        if (getLocalMainIP(local_ip) == ERR_SUCCESS)
        {
            MODULE_MESSAGE msg;
            msg.dst_id = SRV_TCP_SERVER_MODULE;
            msg.dst_type = APP_TYPE_MAIN;
            msg.iCmd = CMD_NOTIFY_DEVICE_INFO;
            strncpy((char *)msg.szBuff, local_ip.c_str(), local_ip.length() + 1);
            QByteArray datagram;
            QDataStream out(&datagram, QIODevice::WriteOnly);
            out.setVersion(QDataStream::Qt_4_7);
            out.writeRawData((char *)&msg, sizeof(msg));

            QHostAddress boardcast = QHostAddress(dst_ip.c_str());
            m_udpSocket.writeDatagram(datagram, boardcast, 3602); // UDP 发送数据
        }
    }
}

// 主侦听任务
void CTcpServer::TaskAcceptMain(void *pBuff)
{
    (void)pBuff;
    setCurrentTaskName("tskAcceptServer");

    while (!isShutdown())
    {
        sockaddr_in client_addr;
        unsigned int length = sizeof(client_addr);

        SOCKET sClientSocket = accept(m_ServerSocket, (sockaddr *)&client_addr, &length);
        if (sClientSocket < 0)
        {
            OutputMessage("accept失败");
            LOG_ERROR("accept failed!");
            break;
        }
        else
        {
            OutputMessage("接收到请求，建立连接");

            u_short uPort = ntohs(client_addr.sin_port);
            string strIp = inet_ntoa(client_addr.sin_addr);
            OnSocketConnected(sClientSocket, strIp, uPort);
        }
    }

    return;
}

// 消息处理任务
void CTcpServer::TaskMain(void *pBuff)
{
    (void)pBuff;
    setCurrentTaskName("CTcpServer");
    setTaskRunType(SCHED_RR);
    waitForStart();

    //起侦听线程
    STARTUP_TASK(TaskAcceptMain);

    while (m_bShutdown == false)
    {
        timeval tv;
        tv.tv_sec = 5;
        tv.tv_usec = 0;

        fd_set fdsets;
        FD_ZERO(&fdsets);
        // Init fdsets
        m_mtxConnect.lock();
        int nSelectCount = 0;
        for (auto &it : m_mapClient)
        {
            nSelectCount++;
            FD_SET(it.second.sSocket, &fdsets);
        }
        m_mtxConnect.unlock();

        if (nSelectCount == 0)
        {
            sleep(1);
            continue;
        }

        select(FD_SETSIZE, &fdsets, nullptr, nullptr, &tv);

        m_mtxConnect.lock();
        std::vector<CClientInfo> RecvArray;
        for (auto &it : m_mapClient)
        {
            if (FD_ISSET(it.second.sSocket, &fdsets))
            {
                RecvArray.push_back(it.second);
            }
        }
        m_mtxConnect.unlock();

        for (size_t i = 0; i < RecvArray.size(); i++)
        {
            CClientInfo aClientInfo = RecvArray[i];
            RecvData(aClientInfo.sSocket);
        }
    }
}

// 接收数据处理
void CTcpServer::RecvData(SOCKET sock)
{
    MODULE_MESSAGE msg;
    ssize_t nRecvRet = recv(sock, (char *)&msg, sizeof(MODULE_MESSAGE), 0);
    msg.pszExtBuff = nullptr;
    if (nRecvRet < 0)
    {
        OnSocketDisconnect(sock);
        int nError = -1;
        std::ostringstream ostr;
        ostr << "服务器Recv失败:" << nError << endl;
        string strTmp = ostr.str();
        OutputMessage(strTmp.c_str());
        LOG_ERROR("server Recv failed%d");
    }
    else
    {
        //只处理本机消息
        if (msg.dst_type == APP_TYPE_MAIN)
        {
            //消息处理
            MODULE_MESSAGE *pMessage = new MODULE_MESSAGE;
            memcpy(pMessage, &msg, sizeof(msg));
            int recv_len = 0;
            if (pMessage->ext_length > 0)
            {
                pMessage->pszExtBuff = new char[pMessage->ext_length];
                memset(pMessage->pszExtBuff, 0, pMessage->ext_length);
                while (recv_len < (int)pMessage->ext_length)
                {
                    int len = recv(sock, pMessage->pszExtBuff + recv_len, pMessage->ext_length - recv_len, 0); // 接收服务器端消息
                    //接收失败
                    if (len < 0)
                    {
                        recv_len = len;
                        break;
                    }
                    else
                    {
                        recv_len += len;
                    }
                }

                if (recv_len < 0)
                {
                    SAFE_DELETE(pMessage);
                    return;
                }
            }

            msg.ext_length = 0;

            switch (msg.dst_id)
            {
            case SRV_TCP_SERVER_MODULE:
            {
                procTcpMessage(sock, *pMessage);
                delete pMessage;
                break;
            }
            default:
            {
                DispatchMessage2Thread(sock, pMessage);
                break;
            }
            }
        }
    }
}

// 处理tcp模块消息
void CTcpServer::procTcpMessage(SOCKET socket, MODULE_MESSAGE &msg)
{
    switch (msg.iCmd)
    {
    case CMD_NOTIFY_APP_TYPE:
    {
        auto it = m_mapClient.find(socket);
        if (it != m_mapClient.end())
        {
            int app_type = 0;
            memcpy(&app_type, msg.szBuff, sizeof(int));
            it->second.app_type = app_type;

            if (app_type == APP_TYPE_CLIENT)
            {
                Q_EMIT notifyClientModeChanged(true);
            }
        }
        break;
    }
    default:
        break;
    }
}

// 分发消息到线程
bool CTcpServer::DispatchMessage2Thread(SOCKET socket, MODULE_MESSAGE *message)
{
    assert(message != nullptr);
    (void)socket;
    IModule *pModule = m_MessageMap[message->dst_id];

    if (pModule && pModule->isSafeModule())
    {
        QString strInfo;
        strInfo.sprintf("[收到TCP消息]模块:%d,消息字:%d\n", message->dst_id, message->iCmd);
        qDebug() << strInfo;
        pModule->PostModuleMessage(message);
    }
    else
    {
        SAFE_DELETE(message);
    }
    return true;
}

// 发送本地消息
bool CTcpServer::SendLocalMessage(IModule *pModule, MODULE_COMMAND_ID cmd, MODULE_MESSAGE *msg)
{
    (void)cmd;
    if (pModule->isSafeModule())
    {
        // qDebug()<<"[发送内部消息]"<<pModule->name().c_str()<<"模块:"<<(int)msg->dst_id<<"消息字:"<<(int)cmd;
        pModule->PostModuleMessage(msg);
    }
    else
    {
        SAFE_DELETE(msg);
        return false;
    }

    return true;
}

// 注册消息回调
void CTcpServer::RegistMessageCallback(IModule *pModule, MODULE_ID id)
{
    m_mtxRegMsg.lock();
    if (pModule)
    {
        // only one instance
        qDebug() << "RegistMessageCallback:" << id;
        // assert(m_MessageMap.find(id) == m_MessageMap.end());
        //  add instance to map
        m_MessageMap[id] = pModule;
    }
    m_mtxRegMsg.unlock();
}

// 处理sock连接
void CTcpServer::OnSocketConnected(SOCKET sClientSocket, string ip, u_short port)
{
    CClientInfo aClientInfo;
    aClientInfo.nPort = port;
    aClientInfo.strIp = ip;
    aClientInfo.sSocket = sClientSocket;

    m_mtxConnect.lock();
    m_mapClient[sClientSocket] = aClientInfo;
    m_mtxConnect.unlock();
    askforAppType(sClientSocket);
}

// app类型应答
void CTcpServer::askforAppType(SOCKET sock)
{
    //请求获取终端类型
    MODULE_MESSAGE msg;
    msg.iCmd = CMD_ASK_FOR_APPTYPE;
    msg.dst_type = APP_TYPE_ALL;
    msg.dst_id = MODULE_APP_COMM;
    SendMsg(sock, msg);
}

// 断开处理
void CTcpServer::OnSocketDisconnect(SOCKET aClientSocket)
{
    m_mtxConnect.lock();
    auto it = m_mapClient.erase(aClientSocket);
    m_mtxConnect.unlock();

    bool bHasClient = false;
    for (auto it : m_mapClient)
    {
        if (it.second.app_type == APP_TYPE_CLIENT)
        {
            bHasClient = true;
            break;
        }
    }
    Q_EMIT notifyClientModeChanged(bHasClient);
}

// 广播消息
unsigned int CTcpServer::boardcastMessage(MODULE_MESSAGE &msg)
{
    foreach (IModule *pModule, m_MessageMap)
    {
        if (pModule != nullptr && pModule->isSafeModule())
        {
            //跨线程的内存需要重新申请，这里主动new内存防止本线程栈内存被意外释放
            MODULE_MESSAGE *pMsg = new MODULE_MESSAGE;
            memcpy(pMsg, &msg, sizeof(MODULE_MESSAGE));
            //接收方釋放
            msg.pszExtBuff = nullptr;
            // printf("[发送内部消息]模块:%d,消息字:%d\n", msg.dst_id, msg.iCmd);
            SendLocalMessage(pModule, msg.iCmd, pMsg);
        }
    }

    return ERR_SUCCESS;
}

// 发送消息
unsigned int CTcpServer::SendMsg(MODULE_MESSAGE &msg, int length)
{
    (void)length;
    DWORD dwRet = ERR_SUCCESS;
    // TODO: 在此添加控件通知处理程序代码
    //如果在本机找到注册的模块，则优先发到本机模块；
    IModule *pModule = m_MessageMap[msg.dst_id];

    if (msg.dst_type == APP_TYPE_MAIN && pModule != nullptr && pModule->isSafeModule())
    {
        //跨线程的内存需要重新申请，这里主动new内存防止本线程栈内存被意外释放
        MODULE_MESSAGE *pMsg = new MODULE_MESSAGE;
        memcpy(pMsg, &msg, sizeof(MODULE_MESSAGE));
        //接收方釋放
        msg.pszExtBuff = nullptr;
        // qDebug() << "[发送内部消息]模块:%d,消息字:%d\n"<< msg.dst_id << msg.iCmd;
        SendLocalMessage(pModule, msg.iCmd, pMsg);
        // PostThreadMessage(ThreadID, msg.iCmd, (WPARAM)pMsg, (LPARAM)nullptr);
        return ERR_SUCCESS;
    }
    else
    {
        try
        {
            //只向匹配的类型发送数据
            m_mtxConnect.lock();
            std::vector<CClientInfo> RecvArray;
            for (auto &it : m_mapClient)
            {
                if (it.second.app_type == msg.dst_type)
                {
                    RecvArray.push_back(it.second);
                }
            }
            m_mtxConnect.unlock();
            for (auto it : RecvArray)
            {
                if (it.app_type == msg.dst_type)
                {
                    SendMsg(it.sSocket, msg);
                }
                else if (it.app_type == APP_TYPE_NONE)
                {
                    askforAppType(it.sSocket);
                }
            }
        }
        catch (...)
        {
            dwRet = ERR_SEND_FAILED;
            cout << "未知异常" << endl;
        }
    }

    return dwRet;
}

// 发送消息
DWORD CTcpServer::SendMsg(SOCKET s, MODULE_MESSAGE &msg)
{
    DWORD dwRet = 0;
    // TODO: 在此添加控件通知处理程序代码
    int ret = send(s, (char *)&msg, sizeof(msg), MSG_NOSIGNAL);
    if (ret < 0)
    {
        OnSocketDisconnect(s);
        int nError = -1;
        cout << ("发送失败:") << nError << endl;
        LOG_INFO("send failed:");
        dwRet = ERR_SEND_FAILED;
        return dwRet;
    }
    int send_ext = 0;
    while (send_ext < msg.ext_length)
    {
        int send_len = min(MAX_EXT_BUFF_SIZE, msg.ext_length - send_ext);
        int ret = send(s, msg.pszExtBuff + send_ext, send_len, MSG_NOSIGNAL);
        if (ret < 0)
        {
            OnSocketDisconnect(s);
            int nError = -1;
            cout << ("发送失败:") << nError << endl;
            LOG_INFO("send failed:%d");
            dwRet = ERR_SEND_FAILED;
            break;
        }
        send_ext += ret;
    }
    return 0;
}
