﻿#pragma once
#include <vector>
#include <thread>
#include <mutex>
#include "message.h"
#include <sstream>
#include <QMap>
#include "../base_module/base_module.h"
#include <string>
#include "qobject.h"
#include "TcpModule.h"
#include <QObject>
#include <QMutex>
#include <QtNetwork>

#ifdef WIN32
#include <winsock2.h>
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#endif

using namespace std;

// 客户端信息
struct CClientInfo
{
    CClientInfo() : sSocket((SOCKET)-1), strIp(""), nPort(0), app_type(0) {}

    string GetShowText()
    {
        std::ostringstream ostr;
        ostr << strIp << ":" << nPort;
        return ostr.str();
    }

    SOCKET sSocket;
    string strIp;
    unsigned short nPort;
    int app_type;
};

// TCP服务端
class CTcpServer : public ITcpModule
{
    Q_OBJECT
public:
    MODULE_NAME(CTcpServer)
    CTcpServer();
    ~CTcpServer();

    // Although not required, it is adivced to use a member function for destroying
    virtual void destroy();

    THREAD_DEFINITION(CTcpServer, TaskMain)
    THREAD_DEFINITION(CTcpServer, TaskAcceptMain)

    //监听线程；
    virtual void TaskAcceptMain(void *pBuff);

    //TCP接收任务；
    virtual void TaskMain(void *pBuff) override;

    //初始化操作
    virtual bool Initialize() override;

    // 发送消息
    virtual DWORD SendMsg(SOCKET s, MODULE_MESSAGE &msg);

    //发送消息
    virtual unsigned int SendMsg(MODULE_MESSAGE &msg, int length) override;

    //会阻塞
    bool startListen(const char *ip, const unsigned short prot);

    // 关闭
    void stop();

    // 发送本地消息
    bool SendLocalMessage(IModule *pModule, MODULE_COMMAND_ID cmd, MODULE_MESSAGE *msg) override;

    //注册线程消息
    void RegistMessageCallback(IModule *pModule, MODULE_ID id) override;

    //分发消息到线程
    bool DispatchMessage2Thread(SOCKET socket, MODULE_MESSAGE *message) override;

    //注册消息，tcp模块自身不需要实现
    virtual bool RegistMessage() override { return true; }

    // 广播消息
    virtual unsigned int boardcastMessage(MODULE_MESSAGE &msg);

Q_SIGNALS :

    // 服务端状态变更
    void notifySrvChange(bool bSrvOnline);

protected Q_SLOTS:

    // 处理设备应答响应
    void procAskforDeviceInfo();


public Q_SLOTS:
    // 连接后处理
    void OnSocketConnected(SOCKET sClientSocket, string ip, u_short port);
    
    // 连接中断处理
    void OnSocketDisconnect(SOCKET aClientSocket);

private:
    //处理tcp连接相关的消息
    void procTcpMessage(SOCKET socket, MODULE_MESSAGE &message);

    //请求获取终端类型
    void askforAppType(SOCKET sock);

    // 接收数据
    void RecvData(SOCKET sock);

    // 服务端socket
    SOCKET m_ServerSocket;

    // 客户端列表
    map<SOCKET, CClientInfo> m_mapClient;

    // 保护信号量
    QMutex m_mtxRegMsg;
    QMutex m_mtxConnect;
    
    // 消息映射map表
    QMap<int, IModule *> m_MessageMap;

    //UDP socket
    QUdpSocket m_udpSocket;
};
