#include "JlCompress.h"
#include "CSWDLManager.h"
#include "socket/TcpServer.h"
#include "base_module/factory.h"
#include "device/CDeviceManager.h"
#include <QDebug>
#include <QMessageBox>
#include <QMetaType>
#include "CSwdlState.h"
#include <QDir>
#include "../libboot/libboot.h"
#include "timer/CTimerManager.h"
#include <assert.h>

REGISTER_MODULE(CSWDLManager, "CSWDLManager")

Q_DECLARE_METATYPE(State *)

CSWDLManager::CSWDLManager(QObject *parent)
    : QObject(parent),
      IModule(),
      m_pTcpModule(nullptr),
      m_pDevMng(nullptr),
      m_pTimerMng(nullptr),
      m_curStatus(SWDL_STATUS_IDEL),
      m_pCurState(nullptr),
      m_pIdelState(nullptr),
      m_pDloadState(nullptr),
      m_pSwapState(nullptr),
      m_pRollbackState(nullptr),
      m_bUpdating(false)
{
}

CSWDLManager::~CSWDLManager()
{
    std::cout << name() << std::endl;
}

//初始化操作
bool CSWDLManager::Initialize()
{
    qRegisterMetaType<State *>("State*"); //放在构造函数中使用

    m_pIdelState = IdelState::getInstace();
    m_pDloadState = DownloadState::getInstace();
    m_pSwapState = SwapState::getInstace();
    m_pRollbackState = RollbackState::getInstace();

    // 设置行径状态和回退状态
    m_pIdelState->setNextState(m_pDloadState);
    m_pDloadState->setNextState(m_pSwapState);
    m_pSwapState->setNextState(m_pIdelState);
    m_pRollbackState->setNextState(m_pIdelState);

    m_pIdelState->setRollbackState(m_pIdelState);
    m_pDloadState->setRollbackState(m_pRollbackState);
    m_pSwapState->setRollbackState(m_pRollbackState);
    m_pRollbackState->setRollbackState(m_pIdelState);

    // 因为下载态下,部分场景依赖于事件驱动,这里将状态机切换改为事件驱动方式
    connect(m_pIdelState, SIGNAL(notifyNext(State *)), this, SLOT(onNextState(State *)));
    connect(m_pIdelState, SIGNAL(notifyRollback(State *)), this, SLOT(onRollbackState(State *)));

    connect(m_pDloadState, SIGNAL(notifyNext(State *)), this, SLOT(onNextState(State *)));
    connect(m_pDloadState, SIGNAL(notifyRollback(State *)), this, SLOT(onRollbackState(State *)));

    connect(m_pSwapState, SIGNAL(notifyNext(State *)), this, SLOT(onNextState(State *)));
    connect(m_pSwapState, SIGNAL(notifyRollback(State *)), this, SLOT(onRollbackState(State *)));

    connect(m_pRollbackState, SIGNAL(notifyNext(State *)), this, SLOT(onNextState(State *)));
    connect(m_pRollbackState, SIGNAL(notifyRollback(State *)), this, SLOT(onRollbackState(State *)));

    m_pCurState = m_pIdelState;

    //起主消息处理线程
    STARTUP_TASK(TaskMain)

    return true;
}

// 获取软件包路径,目前只接受zip打包
QString CSWDLManager::getUpdatePkgPathname(QString ver)
{
    makePath("../package/");
    return QString("../package/%1.zip").arg(ver);
}

// 通过软件压缩包全包升级,外部接口,需要转本地线程处理
bool CSWDLManager::updateByPkg(int pkg_type, QString path_name)
{
    if (m_pCurState != IdelState::getInstace())
    {
        // 处于升级状态直接返回错误
        return false;
    }

    MODULE_MESSAGE msg;
    msg.dst_id = MODULE_SWDL_MNG;
    msg.iCmd = CMD_SWDL_PACKAGE_UPDATE;
    int idx = 0;
    memcpy(msg.szBuff + idx, &pkg_type, sizeof(int));
    idx += sizeof(int);
    strncpy((char *)msg.szBuff + idx, path_name.toStdString().c_str(), sizeof(msg.szBuff) - 10);
    m_pTcpModule->SendMsg(msg, sizeof(msg));
}

// 升级处理
void CSWDLManager::procUpdatePkg(MODULE_MESSAGE &msg)
{
    assert(m_pTcpModule);
    int update_type = 0;
    QString path_name;

    int idx = 0;
    memcpy(&update_type, msg.szBuff + idx, sizeof(int));
    idx += sizeof(int);

    path_name = QString((char *)msg.szBuff + idx);

    // 软件包升级
    updateByPackage(update_type, path_name);
}

// 软件包升级
bool CSWDLManager::updateByPackage(int update_type, QString path_name)
{
    bool bSuccess = false;
    if (update_type == 0)
    {
        // 先解压缩到临时目录
        int idx = path_name.lastIndexOf(".");
        QString unzip_path = path_name.left(idx);
        // 解压缩文件夹
        QStringList lsit = JlCompress::extractDir(path_name, unzip_path);
        if (lsit.size() > 0)
        {
            // 清除备区文件夹
            QString bak_path = this->getBakPath();
            QDir bak_dir;
            bak_dir.setPath(bak_path);
            bak_dir.removeRecursively();
            // 拷贝文件到备区
            if (copyDirectoryFiles(unzip_path, bak_path))
            {
                bSuccess = true;
                // 写启动文件
                CBootCfg boot;
                boot.setLaunchMode(APP_LUANCH_SWAP);
                boot.save();
                // 重启程序
                closeApp();
            }
        }
    }

    // 如果失败,需要发送升级结果
    if (bSuccess == false)
    {
        notifyUpdateResult(update_type, bSuccess);
    }
    return bSuccess;
}

// 广播升级结果
void CSWDLManager::notifyUpdateResult(int update_type, bool bSuccess)
{
    MODULE_MESSAGE msg;
    msg.iCmd = CMD_SWDL_UPDATE_RESULT;

    int idx = 0;
    memcpy(msg.szBuff + idx, &update_type, sizeof(int));
    idx += sizeof(int);

    memcpy(msg.szBuff + idx, &bSuccess, sizeof(bool));
    idx += sizeof(bool);

    m_pTcpModule->boardcastMessage(msg);
}

void CSWDLManager::procUpdateSuccess(MODULE_MESSAGE &pMsg)
{
    if (m_pCurState != m_pSwapState)
    {
        return;
    }

    bool bSuccess = false;
    CBootCfg boot;
    QString bak_path = boot.getBackupDir();
    QString cur_path = boot.getLaunchDir();
    // 拷贝文件到备区
    if (bak_path != cur_path)
    {
        QDir bak_dir;
        bak_dir.setPath(bak_path);
        bak_dir.removeRecursively();
        // 拷贝文件到备区
        if (copyDirectoryFiles(cur_path, bak_path))
        {
            bSuccess = true;
        }
    }
    else
    {
        bSuccess = true;
    }

    // 注销升级检查定时器
    m_pTimerMng = dynamic_cast<CTimerManager *>(::QueryPlugin("CTimerManager"));
    if (m_pTimerMng)
    {
        m_pTimerMng->UnregTimer(TIMER_UPDATE_SUCCESSED);
    }
    // 取消包加载状态
    m_pCurState = m_pIdelState;

    // 设置启动状态为正常态
    boot.setLaunchMode(APP_LUANCH_NORMAL);
    boot.save();

    // 广播升级成功消息
    notifyUpdateResult(0, bSuccess);
}

void CSWDLManager::closeApp()
{
    //执行多次，每半分钟查询一次在线设备
    CDeviceManager *pDevMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
    if (pDevMng)
    {
        pDevMng->closeApp();
    }
}

// 进入下一个状态
void CSWDLManager::onNextState(State *next_state)
{
    if (next_state)
    {
        next_state->process();
    }
}

// 进入回滚状态
void CSWDLManager::onRollbackState(State *rollback_state)
{
    if (rollback_state)
    {
        rollback_state->rollback();
    }
}

bool CSWDLManager::RegistMessage()
{
    m_pTcpModule = dynamic_cast<ITcpModule *>(QueryPlugin("TcpModule"));
    if (m_pTcpModule)
    {
        m_pTcpModule->RegistMessageCallback(this, MODULE_SWDL_MNG);
    }
    return true;
}

void CSWDLManager::loadSwdlState()
{
    // 起5m定时器
    m_pTimerMng = dynamic_cast<CTimerManager *>(::QueryPlugin("CTimerManager"));
    // 检查启动类型
    CBootCfg boot;
    APP_LAUNCH_MODE mode = boot.getLaunchMode();
    // 如果是倒换升级
    if (APP_LUANCH_SWAP == mode)
    {
        // 将状态改为倒换态
        m_pCurState = m_pSwapState;
        if (m_pTimerMng)
        {
            m_pTimerMng->RegTimer(MODULE_SWDL_MNG, TIMER_UPDATE_SUCCESSED, 10);
        }
    }
    // 正常态只需要将状态置为idel态
    else
    {
        m_pCurState = m_pIdelState;
    }
}

//主消息任务，主要用于和其他模块之间的消息通讯
void CSWDLManager::TaskMain(void *pBuff)
{
    (void)pBuff;
    setCurrentTaskName("tskSWDLMain");
    MODULE_MESSAGE *pMsg = nullptr;
    RegistMessage();
    waitForStart();

    loadSwdlState();

    while (!isShutdown())
    {
        if (GetModuleMessage(pMsg)) // get msg from message queue
        {
            switch (pMsg->iCmd)
            {
            case CMD_SWDL_PACKAGE_UPDATE:
                procUpdatePkg(*pMsg);
                break;
            case TIMER_UPDATE_SUCCESSED:
                procUpdateSuccess(*pMsg);
                break;
            default:
                break;
            }

            SAFE_DELETE(pMsg);
        }
    }
}

// 更新软件版本
bool CSWDLManager::updateSystem(QString sys_ver, QString sys_path, QString sw_ver, QString sw_path)
{
    if (m_pCurState != IdelState::getInstace())
    {
        // 处于升级状态直接返回错误
        respUpgradeResult(false);
    }
    else
    {
        // 开始升级,这里不堵塞,任务中会加信号量防止资源冲突
        qDebug() << "updateSystem";
        m_hardwareVer = sys_ver;
        m_hardwareUrl = sys_path;
        m_softwareVer = sw_ver;
        m_softwareUrl = sw_path;
        // 起任务下载
        STARTUP_TASK(TaskUpdateSystem)
    }

    return true;
}

// 升级处理任务,这里通过状态机控制流程;
void CSWDLManager::TaskUpdateSystem(void *pBuff)
{
    DownloadState *pDloadState = dynamic_cast<DownloadState *>(m_pDloadState);
    if (pDloadState)
    {
        m_bUpdating = true;
        pDloadState->setPackageUrl(m_softwareUrl);

        //
        IdelState::getInstace()->start();
    }

    //    QMutexLocker locker(&m_mtxUpdate);
    //    // 空闲态和交换态允许进入,其他状态进入直接回滚
    //    if(m_pCurState != IdelState::getInstace() &&
    //        m_pCurState != SwapState::getInstace())
    //    {
    //        m_pCurState = m_pCurState->rollback();
    //    }

    //    // 状态机状态切换
    //    do
    //    {
    //        if(m_pCurState)
    //        {
    //            m_pCurState = m_pCurState->process();
    //        }
    //    }while(m_pCurState && m_pCurState != IdelState::getInstace());
}

void CSWDLManager::respUpgradeResult(bool bSuccess)
{
    // 这里先打桩，等后续升级模块实现时，将这个处理移交到处理结果通知上
    m_pDevMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
    if (m_pDevMng)
    {
        m_pDevMng->upgradeSystemResult(m_hardwareVer, 1, m_softwareVer, bSuccess);
    }
}

// 获取系统版本，这里要调用底层函数，这里先打桩
QString CSWDLManager::getSystemVersion()
{
    QString strDevCode = "VERSION_0.1";
    return strDevCode;
}

// 获取软件版本
QString CSWDLManager::getSoftwareVersion()
{
    QString strDevCode = SOFTWARE_VERSION;
    return strDevCode;
}

// 制作信息
QString CSWDLManager::getManualFac()
{
    QString manual = "未知制造信息";
    return manual;
}

// 新软件版本
QString CSWDLManager::getNewSoftVer()
{
    QString ver = "";
    return ver;
}

// 升级路径
QString CSWDLManager::getUpdatePath()
{
    QString path = "";
    return path;
}

// 获取备区目录
QString CSWDLManager::getBakPath()
{
    QString cur_path = QDir::currentPath();
    int idx = cur_path.lastIndexOf("ofs");
    if (idx > 0)
    {
        QString root = cur_path.mid(idx);
        if (root == "ofs1")
        {
            return "../ofs2";
        }
        else if (root == "ofs2")
        {
            return "../ofs1";
        }
        else
        {
            return cur_path;
        }
    }

    return cur_path;
}
