#ifndef CSWDLMANAGER_H
#define CSWDLMANAGER_H
#include "base_module.h"
#include <QObject>
#include <QStateMachine>
#include "FileDownloader.h"
#include "CSwdlState.h"

class ITcpModule;
class CDeviceManager;
class CTimerManager;

enum SWDL_STATUS
{
    SWDL_STATUS_IDEL = 0,
    SWDL_STATUS_DOWNLOADING,
    SWDL_STATUS_UPGRADING,
    SWDL_STATUS_ROLLBACKING,
    SWDL_STATUS_SWAPING,
    SWDL_STATUS_REBOOTING,
    SWDL_STATUS_SUCCESSED
};

// 自定义事件,进入下一状态
class SWDL_GO_NEXT_EVENT : public QEvent
{
public:
    SWDL_GO_NEXT_EVENT() : QEvent(Type(eventType))
    {
    }
    static Type eventType;
};

// 自定义事件,回滚
class SWDL_ROLLBACK_EVENT : public QEvent
{
public:
    SWDL_ROLLBACK_EVENT() : QEvent(Type(eventType))
    {
    }
    static Type eventType;
};

class CSWDLManager : public QObject, public IModule
{
    Q_OBJECT
public:
    MODULE_NAME(CSWDLManager)
    explicit CSWDLManager(QObject *parent = nullptr);
    ~CSWDLManager();

    // 获取软件包路径
    QString getUpdatePkgPathname(QString ver);

    // 通过软件压缩包全包升级
    bool updateByPkg(int pkg_type, QString path_name);

    // 关闭APP
    void closeApp();

    //初始化操作
    virtual bool Initialize();

    //模块注册接口
    bool RegistMessage();

    // 版本更新操作
    bool updateSystem(QString sys_ver, QString sys_path, QString sw_ver, QString sw_path);

    // 获取系统版本，这里要调用底层函数，这里先打桩
    QString getSystemVersion();

    // 获取软件版本
    QString getSoftwareVersion();

    // 制作信息
    QString getManualFac();

    // 新软件版本
    QString getNewSoftVer();

    // 升级路径
    QString getUpdatePath();

    // 获取备区目录
    QString getBakPath();
Q_SIGNALS:
    //
    void error(QString);

    // 关闭APP
    void notifyClose();
public slots:
    // 进入下一个状态
    void onNextState(State *);
    // 进入回滚状态
    void onRollbackState(State *);

    // 软件包升级
    bool updateByPackage(int update_type, QString path_name);

protected:
    //主消息任务，主要用于和其他模块之间的消息通讯
    THREAD_DEFINITION(CSWDLManager, TaskMain)

    // 升级处理任务，其他模块调用触发，这里启动线程操作
    THREAD_DEFINITION(CSWDLManager, TaskUpdateSystem)

    //主消息任务，主要用于和其他模块之间的消息通讯
    virtual void TaskMain(void *pBuff);

    // 升级处理任务，下载完升级描述文件后需要升级时调用触发，这里启动线程操作
    virtual void TaskUpdateSystem(void *pBuff);

    //
    void upgradeProc();

    // 返回升级结果
    void respUpgradeResult(bool bSuccess);

    // 升级处理
    void procUpdatePkg(MODULE_MESSAGE &msg);

    // 升级成功处理
    void procUpdateSuccess(MODULE_MESSAGE &pMsg);

    // 广播升级结果
    void notifyUpdateResult(int update_type, bool bSuccess);

    // 加载升级状态
    void loadSwdlState();

protected:
    // 依赖模块
    ITcpModule *m_pTcpModule;
    CDeviceManager *m_pDevMng;
    CTimerManager *m_pTimerMng;

    // 系统版本
    QString m_hardwareVer;
    // 系统路径
    QString m_hardwareUrl;
    // 软件版本
    QString m_softwareVer;
    // 软件路径
    QString m_softwareUrl;

private:
    State *m_pCurState;

    State *m_pIdelState;
    State *m_pDloadState;
    State *m_pSwapState;
    State *m_pRollbackState;
    bool m_bUpdating;

    QMutex m_mtxUpdate;

    // 状态信息
    SWDL_STATUS m_curStatus;
};

#endif // CSWDLMANAGER_H
