#include "CSwdlState.h"
#include "FileDownloader.h"
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonArray>
#include <QMutexLocker>
#include <QDir>
#include "common.h"
#include <string>
#include "CSWDLManager.h"
#include "libboot.h"

IdelState *IdelState::m_instance = nullptr;
DownloadState *DownloadState::m_instance = nullptr;
SwapState *SwapState::m_instance = nullptr;
RollbackState *RollbackState::m_instance = nullptr;

const QString BOOT_INI_PATH = "../boot/boot.ini";

/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////

void IdelState::start()
{
    // backup db

    //

    next();
}

// 执行处理,返回值为下一状态指针
void IdelState::process()
{
}

/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////

const int MAX_DOWNLOAD_RETRY_COUNT = 3;
const QString PKG_ROOT_DIR = "../pkg/";
const QString UPGRADE_SCRIPT_FILENAME = PKG_ROOT_DIR + "upgrade.json";

// 构造函数
DownloadState::DownloadState()
    : m_pDownloader(nullptr),
      m_bScriptDloadSuccess(false),
      m_dlWaitSem(0)
{
    m_pDownloader = new FileDownloader;

    connect(m_pDownloader, SIGNAL(notifyDownloadFinished(QString, bool)), this, SLOT(onPackageDloadFinished(QString, bool)));
}

// 析构函数
DownloadState::~DownloadState()
{
    SAFE_DELETE(m_pDownloader);
}

// 执行处理,返回值为下一状态指针
void DownloadState::process()
{
    QDir dir(PKG_ROOT_DIR);
    if (dir.exists())
    {
        // 清空目录
        removeDir(PKG_ROOT_DIR);
    }

    // 创建升级包目录
    makePath(PKG_ROOT_DIR);

    if (!m_strPkgUrl.isEmpty())
    {
        int retry_cnt = MAX_DOWNLOAD_RETRY_COUNT;
        m_dlResult = DL_RESULT_NONE;

        // download script
        m_pDownloader->setDownloadPara(m_strPkgUrl, UPGRADE_SCRIPT_FILENAME);
        m_bScriptDloadSuccess = false;
        m_pDownloader->start();
    }
}

void DownloadState::onPackageDloadFinished(QString strPath, bool bSuccess)
{
    //下载完成,开始解压
    CSWDLManager *pSwdlMng = dynamic_cast<CSWDLManager *>(QueryPlugin("CSWDLManager"));
    if (pSwdlMng)
    {
        if (pSwdlMng->updateByPackage(0, strPath))
        {
            this->next();
        }
        else
        {
            this->rollback();
        }
    }
}

void DownloadState::clearDloadMap()
{
}

bool DownloadState::onParsePkgScript()
{
    // 先关闭下载器的下载
    //    m_pDownloader->stop();
    clearDloadMap();
    // open updates.json into a JSON paraser
    QFile scriptFile(UPGRADE_SCRIPT_FILENAME);
    QString bak_root = "../bak/";

    CSWDLManager *pSwdl = dynamic_cast<CSWDLManager *>(QueryPlugin("CSWDLManager"));
    if (pSwdl)
    {
        bak_root = pSwdl->getBakPath() + "/";
    }

    if (scriptFile.open(QIODevice::ReadOnly))
    {
        QJsonDocument jsonDocument = QJsonDocument::fromJson(scriptFile.readAll());
        if (jsonDocument.isNull())
        {
            return false;
        }

        QJsonObject jsonUpdates = jsonDocument.object().value("updates").toObject();
        std::string os_type = getOSType();
        QJsonArray jsonPlatformArray = jsonUpdates.value(os_type.c_str()).toArray();

        QString changeLogInfo = "";
        QJsonArray::iterator jsonIterator = jsonPlatformArray.begin();
        for (; jsonIterator != jsonPlatformArray.end(); jsonIterator++)
        {
            QJsonObject jsonUpdateObject = jsonIterator->toObject();
            UPDATE_PACKAGE_NODE node;
            if (jsonUpdateObject.value("operation") == "add")
            {
                node.strUrl = jsonUpdateObject.value("download-url").toString();
                node.strLocation = PKG_ROOT_DIR + jsonUpdateObject.value("download-location").toString();
                node.strDstPath = bak_root + jsonUpdateObject.value("extract-path").toString();
                node.op_type = OP_TYPE_ADD;
                m_mapPkgNode[node.strLocation] = node;
            }
            else if (jsonUpdateObject.value("operation") == "update")
            {
                node.strUrl = jsonUpdateObject.value("download-url").toString();
                node.strLocation = PKG_ROOT_DIR + jsonUpdateObject.value("download-location").toString();
                node.strDstPath = bak_root + jsonUpdateObject.value("extract-path").toString();
                node.op_type = OP_TYPE_UPDATE;
                m_mapPkgNode[node.strLocation] = node;
            }
            else if (jsonUpdateObject.value("operation") == "remove")
            {
                node.strDstPath = bak_root + jsonUpdateObject.value("file-location").toString();
                node.op_type = OP_TYPE_REMOVE;
                m_mapPkgNode[node.strDstPath] = node;
            }
            else if (jsonUpdateObject.value("operation") == "move")
            {
                node.strDstPath = bak_root + jsonUpdateObject.value("file-location").toString();
                node.strMovePath = bak_root + jsonUpdateObject.value("file-move-location").toString();
                node.op_type = OP_TYPE_MOVE;
                m_mapPkgNode[node.strDstPath] = node;
            }
            else if (jsonUpdateObject.value("operation") == "mkdir")
            {
                node.strDstPath = bak_root + jsonUpdateObject.value("directory-location").toString();
                node.op_type = OP_TYPE_MKDIR;
                m_mapPkgNode[node.strDstPath] = node;
            }
            else
            {
                continue;
            }
        }
    }

    // 只有节点大于0才需要做升级处理
    if (m_mapPkgNode.size() > 0)
    {
        m_dlResult = DL_RESULT_SUCCESS;
    }
    else
    {
        m_dlResult = DL_RESULT_FAILED;
    }

    // 释放信号量
    m_dlWaitSem.release();
    return true;
}

// // 解压升级包
// bool DownloadState::downloadPackage()
// {
//     PKG_NODE_MAP::iterator it;
//     for (it = m_mapPkgNode.begin(); it != m_mapPkgNode.end(); it++)
//     {
//         UPDATE_PACKAGE_NODE &node = it.value();
//         //
//         if (node.op_type == OP_TYPE_ADD || node.op_type == OP_TYPE_UPDATE)
//         {
//             node.result = DL_RESULT_NONE;
//             node.pDloader = new FileDownloader(node.strUrl, node.strLocation);
//             connect(node.pDloader, SIGNAL(notifyDownloadFinished(QString, bool)), this, SLOT(onFileDloadFinished(QString, bool)));
//             node.pDloader->start();
//         }
//         else
//         {
//             node.result = DL_RESULT_SUCCESS;
//         }
//     }

//     return true;
// }

// void DownloadState::onFileDloadFinished(QString strLocation, bool bSuccess)
// {
//     PKG_NODE_MAP::iterator it = m_mapPkgNode.find(strLocation);
//     if (it != m_mapPkgNode.end())
//     {
//         if (bSuccess == false)
//         {
//             it.value().result = DL_RESULT_FAILED;
//             if (it.value().pDloader)
//             {
//                 it.value().pDloader->start();
//             }
//             //            dloadRetry(it.value());
//             return;
//         }

//         it.value().result = DL_RESULT_SUCCESS;
//     }

//     bool bDloadOk = true;
//     for (it = m_mapPkgNode.begin(); it != m_mapPkgNode.end(); it++)
//     {
//         UPDATE_PACKAGE_NODE &node = it.value();
//         if (node.result != DL_RESULT_SUCCESS)
//         {
//             bDloadOk = false;
//             return;
//         }
//     }

//     // 进行操作
//     if (!execScript())
//     {
//         // 从主区回复文件
//         // 回滚状态
//         return rollback();
//     }

//     // 进入下一环节
//     next();
// }

void DownloadState::next()
{
    // 下载态到交换态过程需要2步实现
    // 1.写主备区域和启动状态到ini文件

    // 写启动文件
    // CBootCfg boot;
    // boot.setLaunchMode(APP_LUANCH_SWAP);
    // boot.save();

    // // 2.退出程序，通过看门狗重新启动
    // CSWDLManager *pSwdl = dynamic_cast<CSWDLManager *>(QueryPlugin("CSWDLManager"));
    // if (pSwdl)
    // {
    //     pSwdl->closeApp();
    // }
    // else
    // {
    //     exit(0);
    // }
}

// 执行升级脚本
bool DownloadState::execScript()
{
    PKG_NODE_MAP::iterator it;
    for (it = m_mapPkgNode.begin(); it != m_mapPkgNode.end(); it++)
    {
        UPDATE_PACKAGE_NODE &node = it.value();
        //
        switch (node.op_type)
        {
        case OP_TYPE_ADD:
        case OP_TYPE_UPDATE:
        {
            QFile file(node.strLocation);
            makePath(sepratePathName(node.strDstPath)[0]);
            file.copy(node.strDstPath);
            break;
        }
        case OP_TYPE_REMOVE:
        {
            QFile fileToRemove(node.strDstPath);
            fileToRemove.remove();
            break;
        }
        case OP_TYPE_MOVE:
        {
            QFile fileToMove(node.strDstPath);
            fileToMove.rename(node.strMovePath);
            break;
        }
        case OP_TYPE_MKDIR:
            makePath(node.strDstPath);
            break;
        default:
            break;
        }
    }
    return true;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////

// 执行处理,返回值为下一状态指针
void SwapState::process()
{
    mSleep(300000);
    return next();
}

// 处理完毕,没有异常,改倒换状态
void SwapState::next()
{
    CBootCfg watch;
    // 倒换启动信息
    CBootCfg boot;
    boot.setLaunchMode(APP_LUANCH_NORMAL);
    boot.save();

    return State::next();
}
// 执行处理,返回值为下一状态指针
void RollbackState::process()
{
    return next();
}
