#include "FileDownloader.h"
#include "common.h"
#include <QDir>
#include <algorithm>

using namespace std;

const int MAX_BUFF_SIZE = 4000;

FileDownloader::FileDownloader()
    :m_downloadPath(""),
      m_storePath(""),
      m_installPath(""),
      m_reply(nullptr),
      m_file(nullptr)
{
    m_pFileBuff = new BYTE[MAX_BUFF_SIZE];
}

FileDownloader::FileDownloader(QString downloadPath, QString storePath, QString installPath, bool start)
{
    setDownloadPara(downloadPath, storePath, installPath);
    if (start)
    {
        FileDownloader::start();
    }
}

void FileDownloader::setDownloadPara(QString downloadPath, QString storePath, QString installPath)
{
    m_downloadPath = downloadPath;
    m_storePath = storePath;
    m_installPath = (installPath == "") ? storePath : installPath;
    int index = max(storePath.lastIndexOf("/"),storePath.lastIndexOf("\\"));
    QString dir_path = storePath.left(index);
    QDir dir(dir_path);
    if(!dir.exists())
    {
        makePath(dir_path);
    }
}

FileDownloader::~FileDownloader()
{
    emit error("download finished");

    if (m_reply && m_reply->isOpen())
    {
        m_reply->close();
        SAFE_DELETE(m_reply)
    }

    delete m_manager;

    if (m_file&& m_file->isOpen())
    {
        m_file->close();
        SAFE_DELETE(m_file)
    }

    SAFE_DELETE_ARRAY(m_pFileBuff)
}

void FileDownloader::start()
{
    if(m_downloadPath.isEmpty())
    {
        return;
    }

    emit error("download started");

    m_file = new QFile(m_storePath);
    m_file->open(QIODevice::WriteOnly);

    m_manager = new QNetworkAccessManager();
    QUrl qurl(m_downloadPath);
    m_networkRequest = new QNetworkRequest(qurl);
    m_reply = m_manager->get(*m_networkRequest);

    connect(m_reply, SIGNAL(finished()), this, SLOT(downloadFinished()));
    connect(m_reply, SIGNAL(readyRead()), this, SLOT(onReadyRead()));
    connect(m_reply, SIGNAL(downloadProgress(qint64, qint64)), this, SLOT(mdownloadProgress(qint64, qint64)));
    connect(m_reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(networkError(QNetworkReply::NetworkError)));
}

void FileDownloader::stop()
{
    emit error("download stopped");

    if (m_reply&& m_reply->isOpen())
    {
        m_reply->abort();
    }

    if (m_file&& m_file->isOpen())
    {
        m_file->close();
    }
}

void FileDownloader::downloadFinished()
{
    if (m_bytesReceived == m_bytestotal)
    {
        emit error("download file complete");
        m_file->close();
        emit notifyDownloadFinished(m_storePath, true);
    }
}

void FileDownloader::onReadyRead()
{
    if (m_file->isOpen() && m_reply->isOpen())
    {
        QByteArray ba = m_reply->readAll();
        m_file->write(ba);   //将返回的数据进行读取，写入到临时文件中
    }
}

void FileDownloader::mdownloadProgress(qint64 bytesReceived, qint64 bytesTotal)
{
    m_bytesReceived = bytesReceived;
    m_bytestotal = bytesTotal;

//    if (m_file->isOpen() && m_reply->isOpen())
//    {
//         m_reply->read(m_pFileBuff, MAX_BUFF_SIZE);
//        m_file->write(ba);
//    }

    //emit downloadProgress(bytesReceived, bytesTotal);
}

void FileDownloader::networkError(QNetworkReply::NetworkError errorNumber)
{
    emit notifyDownloadFinished(m_storePath, false);
    emit error("Network Error: " + QString::number(errorNumber));
}
