#include "CTimerManager.h"
#include "../socket/TcpServer.h"
#include "../base_module/factory.h"

REGISTER_MODULE(CTimerManager, "CTimerManager")

CTimerManager::CTimerManager(QObject *parent)
    : QObject(parent),
      IModule(),
      m_pTcpModule(nullptr)
{
}

CTimerManager::~CTimerManager()
{
    std::cout << name() << std::endl;
}

//初始化操作
bool CTimerManager::Initialize()
{
    //起主消息处理线程
    STARTUP_TASK(TaskMain)

    return true;
}

bool CTimerManager::RegistMessage()
{
    m_pTcpModule = dynamic_cast<ITcpModule *>(QueryPlugin("TcpModule"));
    if (m_pTcpModule)
    {
        m_pTcpModule->RegistMessageCallback(this, MODULE_TIMER_MNG);
    }
    return true;
}

// 注册定时器
bool CTimerManager::RegTimer(MODULE_ID module, MODULE_COMMAND_ID timer, int interval, int type)
{
    QMutexLocker locker(&m_mtxMap);
    TIMER_NODE_MAP::Iterator it = m_mapTimerNode.find(timer);
    if (it != m_mapTimerNode.end())
    {
        return false;
    }

    CTimerNode *node = new CTimerNode();
    node->module = module;
    node->timer = timer;
    node->interval = interval;
    node->type = type;
    node->tickcount = 0;
    m_mapTimerNode[timer] = node;
    return true;
}

// 更改定时器间隔
bool CTimerManager::ChangeInterval(MODULE_COMMAND_ID timer, int interval)
{
    QMutexLocker locker(&m_mtxMap);
    TIMER_NODE_MAP::Iterator it = m_mapTimerNode.find(timer);
    if (it != m_mapTimerNode.end())
    {
        return false;
    }

    it.value()->interval = interval;

    return true;
}

// 注销定时器
bool CTimerManager::UnregTimer(MODULE_COMMAND_ID timer)
{
    QMutexLocker locker(&m_mtxMap);
    TIMER_NODE_MAP::Iterator it = m_mapTimerNode.find(timer);
    if (it == m_mapTimerNode.end())
    {
        return false;
    }

    m_mapTimerNode.remove(timer);

    return true;
}
//主消息任务，主要用于和其他模块之间的消息通讯
void CTimerManager::TaskMain(void *pBuff)
{
    (void)pBuff;
    setCurrentTaskName("tskTimerMain");
    setTaskRunType(SCHED_RR);
    MODULE_MESSAGE *pMsg = nullptr;
    RegistMessage();
    waitForStart();

    //起主消息处理线程
    STARTUP_TASK(TaskTimerTick)

    while (!isShutdown())
    {
        if (GetModuleMessage(pMsg)) // get msg from message queue
        {
            switch (pMsg->iCmd)
            {
            // case CMD_ASK_FOR_SET_COLLECT_MODE:
            //     ProcSetModeRequest(pMsg);
            //     break;
            default:
                break;
            }

            SAFE_DELETE(pMsg);
        }
    }
}

//主消息任务，主要用于和其他模块之间的消息通讯
void CTimerManager::TaskTimerTick(void *pBuff)
{
    (void)pBuff;
    setCurrentTaskName("tskTimerTick");
    while (!isShutdown())
    {
        m_mtxMap.lock();
        TIMER_NODE_MAP::Iterator it = m_mapTimerNode.begin();
        for (; it != m_mapTimerNode.end(); it++)
        {
            CTimerNode &node = *it.value();
            node.tickcount++;
            QTime tm = QTime::currentTime();
            int cur_tick = tm.hour() * 3600 + tm.minute() * 60 + tm.second();
            switch (node.type)
            {
            case TIMER_TYPE_INTERVAL:
                if (node.tickcount >= node.interval)
                {
                    SendTimeMsg(node);
                    node.tickcount = 0;
                }
                break;
            case TIMER_TYPE_INTEGRAL:
                if (cur_tick % node.interval == 0)
                {
                    SendTimeMsg(node);
                    node.tickcount = 0;
                }
            default:
                break;
            }
        }
        m_mtxMap.unlock();
        mSleep(1000);
    }
}

// 给注册的模块发送定时器事件
void CTimerManager::SendTimeMsg(CTimerNode &node)
{
    if (m_pTcpModule == nullptr)
    {
        m_pTcpModule = dynamic_cast<ITcpModule *>(QueryPlugin("TcpModule"));
    }

    //给应用层发送消息,同步传感器信息
    MODULE_MESSAGE msg;
    msg.dst_id = node.module;
    msg.dst_type = APP_TYPE_MAIN;
    msg.iCmd = node.timer;
    QDateTime date_time = QDateTime::currentDateTime();
    memcpy(msg.szBuff, &date_time, sizeof(date_time));

    m_pTcpModule->SendMsg(msg, sizeof(msg));
}
