#ifndef CTIMERMANAGER_H
#define CTIMERMANAGER_H
#include "base_module.h"
#include "message.h"
#include <QObject>
#include <QMap>
#include <QMutex>

class ITcpModule;

// 定时器类型，准点定时/按时间间隔
enum ENUM_TIMER_TYPE
{
    TIMER_TYPE_INTERVAL = 0,
    TIMER_TYPE_INTEGRAL,
};

// 定时器节点信息
struct CTimerNode
{
    MODULE_ID module;
    MODULE_COMMAND_ID timer;
    int interval;
    int tickcount;
    int type;
};

// 定时器节点MAP
typedef QMap<int, CTimerNode*> TIMER_NODE_MAP;

// 定时器管理类
class CTimerManager : public QObject, public IModule
{
    Q_OBJECT
public:
    MODULE_NAME(CTimerManager)
    explicit CTimerManager(QObject *parent = nullptr);
    ~CTimerManager();

    //初始化操作
    virtual bool Initialize();
    //模块注册接口
    bool RegistMessage();

    // 注册定时器
    bool RegTimer(MODULE_ID module, MODULE_COMMAND_ID timer, int interval, int type = 0);

    // 注销定时器
    bool UnregTimer(MODULE_COMMAND_ID timer);

    // 更改定时器间隔
    bool ChangeInterval(MODULE_COMMAND_ID timer, int interval);
Q_SIGNALS:
    //
    
public Q_SLOTS:

protected:
    //主消息任务，主要用于和其他模块之间的消息通讯
    THREAD_DEFINITION(CTimerManager, TaskMain)

    //定时器超时处理任务;
    THREAD_DEFINITION(CTimerManager, TaskTimerTick)

    //主消息任务，主要用于和其他模块之间的消息通讯
    virtual void TaskMain(void *pBuff);

    //传感器收集任务, 用于定时轮询传感器相关的信息;
    virtual void TaskTimerTick(void *pBuff);

    // 发送定时器消息
    void SendTimeMsg(CTimerNode& node);
protected:
    // 依赖模块
    ITcpModule*         m_pTcpModule;
    
    // 定时器节点
    TIMER_NODE_MAP      m_mapTimerNode;
    QMutex              m_mtxMap;
};

#endif // CTIMERMANAGER_H
