#include "CDlgServSetting.h"
#include "ui_CDlgServSetting.h"
#include "socket/CWebSocketClient.h"
#include "device/CDeviceManager.h"

#include <QTimer>

extern QString getMainMacAddress();

CDlgServSetting::CDlgServSetting(QString strHost,QWidget *parent) :
    QDialog(parent),
    ui(new Ui::CDlgServSetting),
    m_bServAddrOk(false),
    m_bClosedOnConnected(false),
    m_pWebSock(nullptr)
{
    ui->setupUi(this);
    if(strHost.isEmpty())
    {
        ui->url_path->setText("ws://171.217.92.33:58006/integratedDevice/websocket/");
    }
    else
    {
        ui->url_path->setText(strHost);
    }
    //
    //ui->url_path->setText("ws://192.168.0.60:8092/integratedDevice/websocket/"+getMainMacAddress());
    ui->dev_id->setText(getMainMacAddress());
    // WS连接必须在主窗体初始化以后连接， 否则会连接不上，切记!!!
    m_pWebSock = dynamic_cast<CWebSocketClient*>(QueryPlugin("CWebSocketClient"));
	m_devMsg = dynamic_cast<CDeviceManager*>(QueryPlugin("CDeviceManager"));
    // 设置ws的连接信息
    if(m_pWebSock)
    {
        connect(m_pWebSock, &CWebSocketClient::notifyServConnected,
                this, &CDlgServSetting::onServConnected);
    }
}

CDlgServSetting::~CDlgServSetting()
{
    delete ui;
}

// 检查按钮点击炒作
void CDlgServSetting::on_btn_check_clicked()
{
    // 增加定时器用于超时处理
    QTimer::singleShot(5000, this, SLOT(onConnTimeout()));
    m_bServAddrOk = false;
    ui->status->setText("正在登陆中，请耐心等待...");
    ui->btn_check->setDisabled(true);
    checkConnect();
}

// 点击确认
void CDlgServSetting::on_btn_ok_clicked()
{
    // 增加定时器用于超时处理
    QTimer::singleShot(10000, this, SLOT(onConnTimeout()));
    m_bServAddrOk = false;
    m_bClosedOnConnected = true;
    ui->status->setText("正在登陆中，请耐心等待...");
    ui->btn_ok->setDisabled(true);
    checkConnect();
}

// 连接超时处理
void CDlgServSetting::onConnTimeout()
{
    if(m_bServAddrOk != true)
    {
        ui->btn_check->setDisabled(false);
        ui->btn_ok->setDisabled(false);
        ui->status->setText("connect failed!");
    }
}

// 连接成功后的处理
void CDlgServSetting::onServConnected()
{
    m_bServAddrOk = true;
    ui->btn_check->setDisabled(false);
    ui->btn_ok->setDisabled(false);
    ui->status->setText("connect ok");
    if(m_bClosedOnConnected)
    {
        done(1);
    }
}

// 获取服务器HOST地址，这个地址不含设备ID，主要用于保存
QString CDlgServSetting::getServHost()
{
    QString strAddr = ui->url_path->text();
    if(!strAddr.isEmpty() && strAddr.at(strAddr.size()-1) != '/')
    {
        strAddr+='/';
    }
    return strAddr;
}

// 获取服务器地址，这个地址是加了设备ID的地址，直接用于连接服务端
QString CDlgServSetting::getServAddr()
{
    QString strAddr = ui->url_path->text();
    if(!strAddr.isEmpty() && strAddr.at(strAddr.size()-1) != '/')
    {
        strAddr+='/';
    }
	QString strId = "123456789";
	if (m_devMsg)
	{
		strId = m_devMsg->getDeviceID();
	}
    return strAddr + strId;
}

// 检查当前配置的连接
bool CDlgServSetting::checkConnect()
{
    m_pWebSock->connectToServer(getServAddr());
}

// 重载done函数，避免登陆窗口在未登陆的情况下，被手工取消而进行其他操作。
void CDlgServSetting::done(int )
{
    return QDialog::done(1);
}
