#include "DlgMainSetting.h"
#include "ui_DlgMainSetting.h"
#include "device/CDeviceManager.h"
#include "config/CConfigManager.h"
#include "camera/CCameraDevice.h"

DlgMainSetting::DlgMainSetting(CConfigManager &cfg_data, QWidget *parent) : QDialog(parent),
                                                                            ui(new Ui::DlgMainSetting),
                                                                            m_pDevMng(nullptr),
                                                                            m_bNeedUpdate(false),
                                                                            m_bInitialized(false),
                                                                            m_iDecType({1, 1, 1, 1})
{
    ui->setupUi(this);

    // device
    m_pDevMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
    if (m_pDevMng)
    {
        // 将配置数据回显到配置界面
        ui->txtDevCode->setText(m_pDevMng->getDeviceID());
        ui->txtCurrentSoftVer->setText(m_pDevMng->getSoftwareVersion());
        ui->txtDiskSpace->setText(QString("%1").arg(m_pDevMng->getDiskSpaceMB() / 1000000));
        ui->txtHeatBitFreq->setText(m_pDevMng->getHeartBitFrequency().toString());
        ui->txtManualFac->setText(m_pDevMng->getManualFac());
        ui->txtMileage->setText(QString("%1").arg(m_pDevMng->getMileage()));
        ui->txtMobileTraffic->setText(QString("%1").arg(m_pDevMng->getNetworkTraffic()));
        ui->txtNetStatus->setText(QString("%1").arg(m_pDevMng->getNetworkStatus()));
        ui->txtNetworkType->setText(QString("%1").arg(m_pDevMng->getNetwork()));
        ui->txtNewSoftVer->setText(m_pDevMng->getNewSoftVer());
        ui->txtUpdateFilePath->setText(QString("%1").arg(m_pDevMng->getUpdatePath()));
        ui->txtSoftVer->setText(m_pDevMng->getSoftwareVersion());
        ui->txtSysVer->setText(m_pDevMng->getSystemVersion());
        ui->txtVehicleInfo->setText(m_pDevMng->getVehicleNo().toString());
        ui->txtDiskType->setText(QString("%1").arg(m_pDevMng->getDisktype()));
        ui->txtRunStatus->setText(QString("%1").arg(m_pDevMng->getStatus()));
        //        ui->txtNameRule->setText(QString("%1").arg(m_pDevMng.getNameRule()));
        ui->txtReportFreq->setText(m_pDevMng->getReportFrequency().toString());
        ui->txtServAddr->setText(QString("%1").arg(m_pDevMng->getServHostess()));
    }

    //    /* RadioButtonGroup.cpp constructor*/
    //    for(int i = 0; i < MAX_CAMERA_COUNT; i++)
    //    {
    //        m_camDecGroup[i] = new QButtonGroup(this);
    //        CCamConfig* pCam =  m_localCfgData.getCamCfg(i);
    //        if(pCam)
    //        {
    //            int dec_type = pCam->getDecodeType();
    //            if(dec_type >=3 || dec_type < 0)
    //            {
    //                dec_type = 0;
    //                pCam->setDecodeType(dec_type);
    //            }
    //            m_iDecType[i] = dec_type;
    //        }
    //    }

    //    m_camDecGroup[0]->addButton(ui->cam1_jpeg, 0);
    //    m_camDecGroup[0]->addButton(ui->cam1_h264, 1);
    //    m_camDecGroup[0]->addButton(ui->cam1_h265, 2);
    m_bNeedUpdate = true;
    m_bInitialized = true;
}

DlgMainSetting::~DlgMainSetting()
{
    delete ui;
}

void DlgMainSetting::onSetCam1Dec(int id, bool checked)
{
    if (true == checked)
    {
        int cam_idx = 0;
        m_iDecType[cam_idx] = id;
        //        CCamConfig* pcam_cfg = m_localCfgData.getCamCfg(cam_idx);
        //        pcam_cfg->setDecodeType(id);
        //        ui->cam1_gstcode->setText(pcam_cfg->getGstSrc());
        //        ui->cam1_main->setText(pcam_cfg->getVideoMain());
        //        ui->cam1_sub->setText(pcam_cfg->getVideoSub());
    }
}

void DlgMainSetting::onSetCam2Dec(int id, bool checked)
{
    if (true == checked)
    {
        int cam_idx = 1;
        m_iDecType[cam_idx] = id;
        //        CCamConfig* pcam_cfg = m_localCfgData.getCamCfg(cam_idx);
        //        pcam_cfg->setDecodeType(id);
        //        ui->cam2_gstcode->setText(pcam_cfg->getGstSrc());
        //        ui->cam2_main->setText(pcam_cfg->getVideoMain());
        //        ui->cam2_sub->setText(pcam_cfg->getVideoSub());
    }
}

void DlgMainSetting::onSetCam3Dec(int id, bool checked)
{
    if (true == checked)
    {
        int cam_idx = 2;
        m_iDecType[cam_idx] = id;
        //        CCamConfig* pcam_cfg = m_localCfgData.getCamCfg(cam_idx);
        //        pcam_cfg->setDecodeType(id);
        //        ui->cam3_gstcode->setText(pcam_cfg->getGstSrc());
        //        ui->cam3_main->setText(pcam_cfg->getVideoMain());
        //        ui->cam3_sub->setText(pcam_cfg->getVideoSub());
    }
}

void DlgMainSetting::onSetCam4Dec(int id, bool checked)
{
    if (true == checked)
    {
        int cam_idx = 3;
        m_iDecType[cam_idx] = id;
        //        CCamConfig* pcam_cfg = m_localCfgData.getCamCfg(cam_idx);
        //        pcam_cfg->setDecodeType(id);
        //        ui->cam4_gstcode->setText(pcam_cfg->getGstSrc());
        //        ui->cam4_main->setText(pcam_cfg->getVideoMain());
        //        ui->cam4_sub->setText(pcam_cfg->getVideoSub());
    }
}

// 在其中一个源中已经选择的情况下，设置其他窗体中该视频源不可用
void DlgMainSetting::setCombItemDisable(QComboBox *pComb, QString str)
{
    QVariant v(0);
    // 序号为2的选项（第三个）不可选
    int index = pComb->findText(str);
    pComb->setItemData(index, v, Qt::UserRole - 1);
    // 选项背景置灰
    // pComb->setItemData(index, Qt::lightGray, Qt::BackgroundColorRole);
}

// 取消视频源不可用状态
void DlgMainSetting::setCombItemEnable(QComboBox *pComb, QString str)
{
    QVariant v(1 | 32);
    // 序号为2的选项（第三个）不可选
    int index = pComb->findText(str);
    pComb->setItemData(index, v, Qt::UserRole - 1);
    // 选项背景置灰
    // pComb->setItemData(index, Qt::lightGray, Qt::BackgroundColorRole);
}

// 视频源1选择变更
void DlgMainSetting::on_combCam1_currentIndexChanged(int index)
{
    if (!m_bNeedUpdate)
    {
        return;
    }

    // 先取消其他combox的不可选状态
    QString strSrc = ui->combCam1->currentText();
    int cam_idx = 0;
    CCamConfig cam_cfg(strSrc);
    if (strSrc != m_curVideoSrc[cam_idx])
    {
        setCombItemEnable(ui->combCam2, m_curVideoSrc[cam_idx]);
        setCombItemEnable(ui->combCam3, m_curVideoSrc[cam_idx]);
        setCombItemEnable(ui->combCam4, m_curVideoSrc[cam_idx]);
        m_curVideoSrc[cam_idx] = strSrc;
        if (!strSrc.isEmpty())
        {
            setCombItemDisable(ui->combCam2, strSrc);
            setCombItemDisable(ui->combCam3, strSrc);
            setCombItemDisable(ui->combCam4, strSrc);
        }

        //        // 刷新关联窗体
        //        ui->cam1_gstcode->setText(cam_cfg.getGstSrc());
        //        ui->cam1_main->setText(cam_cfg.getVideoMain());
        //        ui->cam1_sub->setText(cam_cfg.getVideoSub());
    }

    if (m_bInitialized)
    {
        //        m_localCfgData.setCamCfg(cam_idx, cam_cfg);
    }
}

// 视频源2选择变更
void DlgMainSetting::on_combCam2_currentIndexChanged(int index)
{
    if (!m_bNeedUpdate)
    {
        return;
    }

    QString strSrc = ui->combCam2->currentText();
    // 先取消其他combox的不可选状态
    int cam_idx = 1;
    CCamConfig cam_cfg(strSrc);
    if (strSrc != m_curVideoSrc[cam_idx])
    {
        setCombItemEnable(ui->combCam1, m_curVideoSrc[cam_idx]);
        setCombItemEnable(ui->combCam3, m_curVideoSrc[cam_idx]);
        setCombItemEnable(ui->combCam4, m_curVideoSrc[cam_idx]);

        m_curVideoSrc[cam_idx] = strSrc;
        // 在选择不为空的情况下，设置其他combox的不可选状态
        if (!strSrc.isEmpty())
        {
            setCombItemDisable(ui->combCam1, strSrc);
            setCombItemDisable(ui->combCam3, strSrc);
            setCombItemDisable(ui->combCam4, strSrc);
        }

        //        // 刷新关联窗体
        //        ui->cam2_gstcode->setText(cam_cfg.getGstSrc());
        //        ui->cam2_main->setText(cam_cfg.getVideoMain());
        //        ui->cam2_sub->setText(cam_cfg.getVideoSub());
    }

    if (m_bInitialized)
    {
        //        m_localCfgData.setCamCfg(cam_idx, cam_cfg);
    }
}

// 视频源3选择变更
void DlgMainSetting::on_combCam3_currentIndexChanged(int index)
{
    if (!m_bNeedUpdate)
    {
        return;
    }

    QString strSrc = ui->combCam3->currentText();
    int cam_idx = 2;
    CCamConfig cam_cfg(strSrc);
    // 先取消其他combox的不可选状态
    if (strSrc != m_curVideoSrc[cam_idx])
    {
        setCombItemEnable(ui->combCam1, m_curVideoSrc[cam_idx]);
        setCombItemEnable(ui->combCam2, m_curVideoSrc[cam_idx]);
        setCombItemEnable(ui->combCam4, m_curVideoSrc[cam_idx]);

        m_curVideoSrc[cam_idx] = strSrc;
        // 在选择不为空的情况下，设置其他combox的不可选状态
        if (!strSrc.isEmpty())
        {
            setCombItemDisable(ui->combCam1, strSrc);
            setCombItemDisable(ui->combCam2, strSrc);
            setCombItemDisable(ui->combCam4, strSrc);
        }

        //        ui->cam3_gstcode->setText(cam_cfg.getGstSrc());
        //        ui->cam3_main->setText(cam_cfg.getVideoMain());
        //        ui->cam3_sub->setText(cam_cfg.getVideoSub());
    }

    if (m_bInitialized)
    {
        //        m_localCfgData.setCamCfg(cam_idx, cam_cfg);
    }
}

// 视频源4选择变更
void DlgMainSetting::on_combCam4_currentIndexChanged(int index)
{
    if (!m_bNeedUpdate)
    {
        return;
    }

    QString strSrc = ui->combCam4->currentText();
    // 刷新关联窗体
    int cam_idx = 3;
    CCamConfig cam_cfg(strSrc);
    // 先取消其他combox的不可选状态
    if (strSrc != m_curVideoSrc[cam_idx])
    {
        setCombItemEnable(ui->combCam1, m_curVideoSrc[cam_idx]);
        setCombItemEnable(ui->combCam2, m_curVideoSrc[cam_idx]);
        setCombItemEnable(ui->combCam3, m_curVideoSrc[cam_idx]);

        m_curVideoSrc[3] = strSrc;
        // 在选择不为空的情况下，设置其他combox的不可选状态
        if (!strSrc.isEmpty())
        {
            setCombItemDisable(ui->combCam1, strSrc);
            setCombItemDisable(ui->combCam2, strSrc);
            setCombItemDisable(ui->combCam3, strSrc);
        }

        //        ui->cam4_gstcode->setText(cam_cfg.getGstSrc());
        //        ui->cam4_main->setText(cam_cfg.getVideoMain());
        //        ui->cam4_sub->setText(cam_cfg.getVideoSub());
    }

    if (m_bInitialized)
    {
        //        m_localCfgData.setCamCfg(cam_idx, cam_cfg);
    }
}

void DlgMainSetting::on_btnApply_clicked()
{
    // 这里要分状态对比，将产生变化的变量推送到具体的应用层
    if (m_pMainCfgData != nullptr)
    {
        //        m_pMainCfgData->updateConfig(m_localCfgData);
    }
}
