#include "DlgSetting.h"
#include "ui_DlgSetting.h"
#include <QDebug>
#include "factory.h"
#include "message.h"
#include <QMessageBox>

DlgSetting::DlgSetting(QWidget *parent)
    : QDialog(parent),
      ui(new Ui::DlgSetting),
      m_bInited(false)
{
    ui->setupUi(this);
    ui->show_pwd->setChecked(false);
    ui->sip_serv_pwd->setEchoMode(QLineEdit::PasswordEchoOnEdit);
    ui->ws_password->setEchoMode(QLineEdit::PasswordEchoOnEdit);
    ui->btnApply->setEnabled(false);

    // 屏蔽信号槽,等设置完毕后再放开,防止干扰
    this->blockSignals(true);
    loadGB28181Config();
    loadJT808Config();
    loadDeviceConfig();
    // 放开信号槽
    this->blockSignals(false);
    m_bInited = true;
}

DlgSetting::~DlgSetting()
{
    delete ui;
}

bool DlgSetting::checkChanged()
{
    bool bChanged = false;
    foreach (auto item, m_mapGB28181Cfg)
    {
        if (item.isChanged() == true)
        {
            bChanged = true;
            break;
        }
    }

    foreach (auto item, m_mapJT808Cfg)
    {
        if (item.isChanged() == true)
        {
            bChanged = true;
            break;
        }
    }

    foreach (auto item, m_mapDevCfg)
    {
        if (item.isChanged() == true)
        {
            bChanged = true;
            break;
        }
    }

    ui->btnApply->setEnabled(bChanged);

    return bChanged;
}

void DlgSetting::on_btnApply_clicked()
{
    bool bNeedReboot = GB28181Apply();
    bNeedReboot |= JT808Apply();
    bNeedReboot |= DeviceApply();
    if (bNeedReboot)
    {
        // 提示是否退出
        QMessageBox::StandardButton rb = QMessageBox::question(this, "配置变更", "当前配置发生变更,需要重启软件后生效,现在是否退出当前系统?", QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes);
        if (rb == QMessageBox::Yes)
        {
            notifyClose();
        }
        emit notifyConfigChanged();
    }

    ui->btnApply->setEnabled(false);
}

void DlgSetting::on_btnOK_clicked()
{
    on_btnApply_clicked();
    done(1);
}

void DlgSetting::on_btnCancel_clicked()
{
    done(0);
}
