#ifndef DLGSETTING_H
#define DLGSETTING_H

#include <QDialog>
#include <QMap>
#include <functional>

namespace Ui
{
    class DlgSetting;
}

typedef std::function<int(const QString &)> funcSetValue;

struct CONFIG_ITEM
{
    CONFIG_ITEM(QString value)
        : cfg_value(value),
          cur_value(value)
    {
    }
    CONFIG_ITEM()
    {
    }

    bool isChanged() { return cfg_value == cur_value ? false : true; }
    QString cfg_value;
    QString cur_value;
    funcSetValue func;
};

typedef QMap<QString, CONFIG_ITEM> CONFIG_MAP;

class DlgSetting : public QDialog
{
    Q_OBJECT

public:
    explicit DlgSetting(QWidget *parent = 0);
    ~DlgSetting();

private slots:
    bool checkChanged();

    /*************************************************************************************
    *******************************   gb28181   ******************************************
    *************************************************************************************/
    void loadGB28181Config();
    void setGB28181Value(QString key, QString value);
    bool GB28181Apply();

    // GB28181 配置变更事件
    void on_show_pwd_clicked(bool checked);
    void on_sip_serv_id_textChanged(const QString &arg1);
    void on_sip_serv_domain_textChanged(const QString &arg1);
    void on_sip_serv_ip_textChanged(const QString &arg1);
    void on_sip_serv_port_textChanged(const QString &arg1);
    void on_sip_serv_pwd_textChanged(const QString &arg1);
    void on_local_sip_id_textChanged(const QString &arg1);
    void on_local_sip_port_textChanged(const QString &arg1);
    void on_reg_interval_textChanged(const QString &arg1);
    void on_cam1_id_textChanged(const QString &arg1);
    void on_cam2_id_textChanged(const QString &arg1);
    void on_cam3_id_textChanged(const QString &arg1);
    void on_cam4_id_textChanged(const QString &arg1);

    /*************************************************************************************
    *******************************    jt808   *******************************************
    *************************************************************************************/
    void loadJT808Config();
    void setJT808Value(QString key, QString value);
    bool JT808Apply();

    void on_btnApply_clicked();
    void on_btnOK_clicked();
    void on_btnCancel_clicked();

    void on_jt808_serv_ip_textChanged(const QString &arg1);
    void on_jt808_tcp_port_textChanged(const QString &arg1);
    void on_jt808_slave_ip_textChanged(const QString &arg1);
    void on_jt808_udp_port_textChanged(const QString &arg1);
    void on_jt808_phone_number_textChanged(const QString &arg1);
    void on_jt808_vehicle_no_textChanged(const QString &arg1);

    void on_stream_code_currentIndexChanged(int index);

    /*************************************************************************************
    *******************************    device  *******************************************
    *************************************************************************************/
    void loadDeviceConfig();
    void setDeviceValue(QString key, QString value);
    bool DeviceApply();

    void on_dev_heartbeat_freq_textChanged(const QString &arg1);
    void on_dev_status_freq_textChanged(const QString &arg1);
    void on_dev_gps_freq_textChanged(const QString &arg1);
    void on_videoFrameSplittingInterval_textChanged(const QString &arg1);
    void on_dev_parkingAlarmInterval_textChanged(const QString &arg1);
    void on_dev_parkingAlarmEnable_toggled(bool checked);
    void on_dev_vehicle_no_textChanged(const QString &arg1);
    void on_dev_phone_num_textChanged(const QString &arg1);
    void on_ws_show_pwd_clicked(bool checked);
    void on_ws_servaddr_textChanged(const QString &arg1);
    void on_ws_username_textChanged(const QString &arg1);
    void on_ws_password_textChanged(const QString &arg1);

    void on_camViewCnt_currentIndexChanged(const QString &arg1);

signals:
    void notifyClose();

    // 通知配置变更
    void notifyConfigChanged();

private:
    Ui::DlgSetting *ui;

    bool m_bInited;

    CONFIG_MAP m_mapGB28181Cfg;
    CONFIG_MAP m_mapJT808Cfg;
    CONFIG_MAP m_mapDevCfg;
};

#endif // DLGSETTING_H
