#include "DlgSetting.h"
#include "ui_DlgSetting.h"
#include <QDebug>
#include "device/CDeviceManager.h"

void DlgSetting::loadDeviceConfig()
{
    CDeviceManager *pDevMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
    if (pDevMng)
    {
        m_mapDevCfg["pingFreq"] = CONFIG_ITEM(pDevMng->property("pingFreq").toString());
        m_mapDevCfg["commonStatusFrq"] = CONFIG_ITEM(pDevMng->property("commonStatusFrq").toString());
        m_mapDevCfg["uploadSpeedFreq"] = CONFIG_ITEM(pDevMng->property("uploadSpeedFreq").toString());
        m_mapDevCfg["videoFrameSplittingInterval"] = CONFIG_ITEM(pDevMng->property("videoFrameSplittingInterval").toString());
        m_mapDevCfg["parkingAlarmInterval"] = CONFIG_ITEM(pDevMng->property("parkingAlarmInterval").toString());
        m_mapDevCfg["parkingAlarmEnable"] = CONFIG_ITEM(pDevMng->property("parkingAlarmEnable").toString());
        m_mapDevCfg["vehicle_no"] = CONFIG_ITEM(pDevMng->property("vehicle_no").toString());
        m_mapDevCfg["phone_num"] = CONFIG_ITEM(pDevMng->property("phone_num").toString());

        m_mapDevCfg["ws_servaddr"] = CONFIG_ITEM(pDevMng->property("ws_servaddr").toString());
        m_mapDevCfg["ws_username"] = CONFIG_ITEM(pDevMng->property("ws_username").toString());
        m_mapDevCfg["ws_password"] = CONFIG_ITEM(pDevMng->property("ws_password").toString());
        m_mapDevCfg["cam_view_cnt"] = CONFIG_ITEM(pDevMng->property("cam_view_cnt").toString());

        ui->dev_heartbeat_freq->setText(m_mapDevCfg["pingFreq"].cur_value);
        ui->dev_status_freq->setText(m_mapDevCfg["commonStatusFrq"].cur_value);
        ui->dev_gps_freq->setText(m_mapDevCfg["uploadSpeedFreq"].cur_value);
        ui->videoFrameSplittingInterval->setText(m_mapDevCfg["videoFrameSplittingInterval"].cur_value);
        ui->dev_parkingAlarmInterval->setText(m_mapDevCfg["parkingAlarmInterval"].cur_value);
        ui->dev_parkingAlarmEnable->setChecked(QVariant(m_mapDevCfg["parkingAlarmEnable"].cur_value).toBool());
        ui->dev_vehicle_no->setText(m_mapDevCfg["vehicle_no"].cur_value);
        ui->dev_phone_num->setText(m_mapDevCfg["phone_num"].cur_value);
        ui->ws_servaddr->setText(m_mapDevCfg["ws_servaddr"].cur_value);
        ui->ws_username->setText(m_mapDevCfg["ws_username"].cur_value);
        ui->ws_password->setText(m_mapDevCfg["ws_password"].cur_value);

        ui->camViewCnt->setCurrentText(m_mapDevCfg["cam_view_cnt"].cur_value);
    }
}

void DlgSetting::on_dev_heartbeat_freq_textChanged(const QString &arg1)
{
    setDeviceValue("pingFreq", arg1);
}

void DlgSetting::on_dev_status_freq_textChanged(const QString &arg1)
{
    setDeviceValue("commonStatusFrq", arg1);
}

void DlgSetting::on_dev_gps_freq_textChanged(const QString &arg1)
{
    setDeviceValue("uploadSpeedFreq", arg1);
}

void DlgSetting::on_videoFrameSplittingInterval_textChanged(const QString &arg1)
{
    setDeviceValue("videoFrameSplittingInterval", arg1);
}

void DlgSetting::on_dev_parkingAlarmInterval_textChanged(const QString &arg1)
{
    setDeviceValue("parkingAlarmInterval", arg1);
}

void DlgSetting::on_dev_parkingAlarmEnable_toggled(bool checked)
{
    QString strEnable = checked ? "true" : "false";
    setDeviceValue("parkingAlarmEnable", QString("%1").arg(strEnable));
}

void DlgSetting::on_dev_vehicle_no_textChanged(const QString &arg1)
{
    setDeviceValue("vehicle_no", arg1);
}

void DlgSetting::on_dev_phone_num_textChanged(const QString &arg1)
{
    setDeviceValue("phone_num", arg1);
}

void DlgSetting::on_ws_servaddr_textChanged(const QString &arg1)
{
    setDeviceValue("ws_servaddr", arg1);
}

void DlgSetting::on_ws_username_textChanged(const QString &arg1)
{
    setDeviceValue("ws_username", arg1);
}

void DlgSetting::on_ws_password_textChanged(const QString &arg1)
{
    setDeviceValue("ws_password", arg1);
}

void DlgSetting::on_camViewCnt_currentIndexChanged(const QString &arg1)
{
    setDeviceValue("cam_view_cnt", arg1);
}

void DlgSetting::setDeviceValue(QString key, QString value)
{
    auto item = m_mapDevCfg.find(key);
    if (item != m_mapDevCfg.end())
    {
        item.value().cur_value = value;
        if (item.value().cfg_value != item.value().cur_value)
        {
            ui->btnApply->setEnabled(true);
        }
        else
        {
            checkChanged();
        }
    }
    else
    {
        qDebug() << "没有找到匹配的属性" << key;
    }
}

bool DlgSetting::DeviceApply()
{
    bool bChange = false;
    // 初始化过程中,会出现导入配置导致的变更,这种场景下,直接返回, 避免重复检查
    if (!m_bInited)
    {
        return bChange;
    }

    CDeviceManager *pDevMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
    if (pDevMng)
    {
        // 遍历属性表,
        auto it = m_mapDevCfg.begin();
        for (; it != m_mapDevCfg.end(); it++)
        {
            // 存在差异时调用设置函数,设置变量
            if (it.value().cfg_value != it.value().cur_value)
            {
                // 将当前值同步到配置值
                QString str = it.key();
                const char *prop_name = str.toLocal8Bit().data();
                pDevMng->setProperty(prop_name, it.value().cur_value);
                it.value().cfg_value = it.value().cur_value;
                bChange = true;
            }
        }

        if (bChange)
        {
            pDevMng->saveConfig();
        }
    }

    return bChange;
}

void DlgSetting::on_ws_show_pwd_clicked(bool checked)
{
    if (checked == true)
    {
        ui->ws_password->setEchoMode(QLineEdit::Normal);
    }
    else
    {
        ui->ws_password->setEchoMode(QLineEdit::PasswordEchoOnEdit);
    }
}
