#include "DlgSetting.h"
#include "ui_DlgSetting.h"
#include "gb28181/CGB28181Module.h"
#include <QDebug>

void DlgSetting::loadGB28181Config()
{
    CGB28181Module *pGB28181Mng = dynamic_cast<CGB28181Module *>(QueryPlugin("CGB28181Module"));
    if (pGB28181Mng)
    {
        m_mapGB28181Cfg["cam1_id"] = CONFIG_ITEM(pGB28181Mng->property("cam1_id").toString());
        m_mapGB28181Cfg["cam2_id"] = CONFIG_ITEM(pGB28181Mng->property("cam2_id").toString());
        m_mapGB28181Cfg["cam3_id"] = CONFIG_ITEM(pGB28181Mng->property("cam3_id").toString());
        m_mapGB28181Cfg["cam4_id"] = CONFIG_ITEM(pGB28181Mng->property("cam4_id").toString());

        m_mapGB28181Cfg["serv_ip"] = CONFIG_ITEM(pGB28181Mng->property("serv_ip").toString());
        m_mapGB28181Cfg["serv_port"] = CONFIG_ITEM(pGB28181Mng->property("serv_port").toString());
        m_mapGB28181Cfg["serv_id"] = CONFIG_ITEM(pGB28181Mng->property("serv_id").toString());
        m_mapGB28181Cfg["serv_pwd"] = CONFIG_ITEM(pGB28181Mng->property("serv_pwd").toString());

        m_mapGB28181Cfg["local_id"] = CONFIG_ITEM(pGB28181Mng->property("local_id").toString());
        m_mapGB28181Cfg["local_port"] = CONFIG_ITEM(pGB28181Mng->property("local_port").toString());
        m_mapGB28181Cfg["reg_interval"] = CONFIG_ITEM(pGB28181Mng->property("reg_interval").toString());
        m_mapGB28181Cfg["stream_code"] = CONFIG_ITEM(pGB28181Mng->property("stream_code").toString());

        ui->cam1_id->setText(m_mapGB28181Cfg["cam1_id"].cur_value);
        ui->cam2_id->setText(m_mapGB28181Cfg["cam2_id"].cur_value);
        ui->cam3_id->setText(m_mapGB28181Cfg["cam3_id"].cur_value);
        ui->cam4_id->setText(m_mapGB28181Cfg["cam4_id"].cur_value);

        ui->sip_serv_id->setText(m_mapGB28181Cfg["serv_id"].cur_value);
        ui->sip_serv_domain->setText(pGB28181Mng->getCode(m_mapGB28181Cfg["serv_id"].cfg_value.toStdString()).c_str());
        ui->sip_serv_ip->setText(m_mapGB28181Cfg["serv_ip"].cur_value);
        ui->sip_serv_port->setText(m_mapGB28181Cfg["serv_port"].cur_value);
        ui->sip_serv_pwd->setText(m_mapGB28181Cfg["serv_pwd"].cur_value);

        ui->local_sip_id->setText(m_mapGB28181Cfg["local_id"].cur_value);
        ui->local_sip_port->setText(m_mapGB28181Cfg["local_port"].cur_value);
        ui->reg_interval->setText(m_mapGB28181Cfg["reg_interval"].cur_value);

        ui->stream_code->setCurrentIndex(m_mapGB28181Cfg["stream_code"].cur_value.toInt());
    }
}

void DlgSetting::on_show_pwd_clicked(bool checked)
{
    if (checked == true)
    {
        ui->sip_serv_pwd->setEchoMode(QLineEdit::Normal);
    }
    else
    {
        ui->sip_serv_pwd->setEchoMode(QLineEdit::PasswordEchoOnEdit);
    }
}

void DlgSetting::on_sip_serv_id_textChanged(const QString &arg1)
{
    setGB28181Value("serv_id", arg1);
    CGB28181Module *pGB28181Mng = dynamic_cast<CGB28181Module *>(QueryPlugin("CGB28181Module"));
    if (pGB28181Mng)
    {
        ui->sip_serv_domain->setText(pGB28181Mng->getCode(m_mapGB28181Cfg["serv_id"].cfg_value.toStdString()).c_str());
    }
}

void DlgSetting::on_sip_serv_domain_textChanged(const QString &arg1)
{
    // setGB28181Value("sip_serv_domain", arg1);
}

void DlgSetting::on_sip_serv_ip_textChanged(const QString &arg1)
{
    setGB28181Value("serv_ip", arg1);
}

void DlgSetting::on_sip_serv_port_textChanged(const QString &arg1)
{
    setGB28181Value("serv_port", arg1);
}

void DlgSetting::on_sip_serv_pwd_textChanged(const QString &arg1)
{
    setGB28181Value("serv_pwd", arg1);
}

void DlgSetting::on_local_sip_id_textChanged(const QString &arg1)
{
    setGB28181Value("local_id", arg1);
}

void DlgSetting::on_local_sip_port_textChanged(const QString &arg1)
{
    setGB28181Value("local_port", arg1);
}

void DlgSetting::on_reg_interval_textChanged(const QString &arg1)
{
    setGB28181Value("reg_interval", arg1);
}

void DlgSetting::on_cam1_id_textChanged(const QString &arg1)
{
    setGB28181Value("cam1_id", arg1);
}

void DlgSetting::on_cam2_id_textChanged(const QString &arg1)
{
    setGB28181Value("cam2_id", arg1);
}

void DlgSetting::on_cam3_id_textChanged(const QString &arg1)
{
    setGB28181Value("cam3_id", arg1);
}

void DlgSetting::on_cam4_id_textChanged(const QString &arg1)
{
    setGB28181Value("cam4_id", arg1);
}

void DlgSetting::on_stream_code_currentIndexChanged(int index)
{
    setGB28181Value("stream_code", QVariant(index).toString());
}

void DlgSetting::setGB28181Value(QString key, QString value)
{
    // 初始化过程中,会出现导入配置导致的变更,这种场景下,直接返回, 避免重复检查
    if (!m_bInited)
    {
        return;
    }

    auto item = m_mapGB28181Cfg.find(key);
    if (item != m_mapGB28181Cfg.end())
    {
        item.value().cur_value = value;
        if (item.value().cfg_value != item.value().cur_value)
        {
            ui->btnApply->setEnabled(true);
        }
        else
        {
            checkChanged();
        }
    }
    else
    {
        qDebug() << "没有找到匹配的属性" << key;
    }
}

bool DlgSetting::GB28181Apply()
{
    bool bChange = false;
    CGB28181Module *pGB28181Mng = dynamic_cast<CGB28181Module *>(QueryPlugin("CGB28181Module"));
    if (pGB28181Mng)
    {
        // 遍历属性表,
        auto it = m_mapGB28181Cfg.begin();
        for (; it != m_mapGB28181Cfg.end(); it++)
        {
            // 存在差异时调用设置函数,设置变量
            if (it.value().cfg_value != it.value().cur_value)
            {
                // 将当前值同步到配置值
                const char *prop_name = it.key().toLocal8Bit().data();
                pGB28181Mng->setProperty(prop_name, it.value().cur_value);
                it.value().cfg_value = it.value().cur_value;
                bChange = true;
            }
        }

        if (bChange)
        {
            pGB28181Mng->saveConfig();
        }
    }

    return bChange;
}
