#include "DlgSetting.h"
#include "ui_DlgSetting.h"
#include <QDebug>
#include "jt808/CJT808Terminal.h"

void DlgSetting::loadJT808Config()
{
    CJT808Terminal *pJT808Mng = dynamic_cast<CJT808Terminal *>(QueryPlugin("CJT808Terminal"));
    if (pJT808Mng)
    {
        m_mapJT808Cfg["main_ip"] = CONFIG_ITEM(pJT808Mng->property("main_ip").toString());
        m_mapJT808Cfg["slave_ip"] = CONFIG_ITEM(pJT808Mng->property("slave_ip").toString());
        m_mapJT808Cfg["tcp_port"] = CONFIG_ITEM(pJT808Mng->property("tcp_port").toString());
        m_mapJT808Cfg["udp_port"] = CONFIG_ITEM(pJT808Mng->property("udp_port").toString());
        m_mapJT808Cfg["phone_num"] = CONFIG_ITEM(pJT808Mng->property("phone_num").toString());
        m_mapJT808Cfg["vehicle_no"] = CONFIG_ITEM(pJT808Mng->property("vehicle_no").toString());

        ui->jt808_serv_ip->setText(m_mapJT808Cfg["main_ip"].cur_value);
        ui->jt808_slave_ip->setText(m_mapJT808Cfg["slave_ip"].cur_value);
        ui->jt808_tcp_port->setText(m_mapJT808Cfg["tcp_port"].cur_value);
        ui->jt808_udp_port->setText(m_mapJT808Cfg["udp_port"].cur_value);
        ui->jt808_phone_number->setText(m_mapJT808Cfg["phone_num"].cur_value);
        ui->jt808_vehicle_no->setText(m_mapJT808Cfg["vehicle_no"].cur_value);
    }
}

void DlgSetting::on_jt808_serv_ip_textChanged(const QString &arg1)
{
    setJT808Value("main_ip", arg1);
}

void DlgSetting::on_jt808_tcp_port_textChanged(const QString &arg1)
{
    setJT808Value("tcp_port", arg1);
}

void DlgSetting::on_jt808_slave_ip_textChanged(const QString &arg1)
{
    setJT808Value("slave_ip", arg1);
}

void DlgSetting::on_jt808_udp_port_textChanged(const QString &arg1)
{
    setJT808Value("udp_port", arg1);
}

void DlgSetting::on_jt808_phone_number_textChanged(const QString &arg1)
{
    setJT808Value("phone_num", arg1);
}

void DlgSetting::on_jt808_vehicle_no_textChanged(const QString &arg1)
{
    setJT808Value("vehicle_no", arg1);
}

void DlgSetting::setJT808Value(QString key, QString value)
{
    auto item = m_mapJT808Cfg.find(key);
    if (item != m_mapJT808Cfg.end())
    {
        item.value().cur_value = value;
        if (item.value().cfg_value != item.value().cur_value)
        {
            ui->btnApply->setEnabled(true);
        }
        else
        {
            checkChanged();
        }
    }
    else
    {
        qDebug() << "没有找到匹配的属性" << key;
    }
}

bool DlgSetting::JT808Apply()
{
    bool bChange = false;
    // 初始化过程中,会出现导入配置导致的变更,这种场景下,直接返回, 避免重复检查
    if (!m_bInited)
    {
        return bChange;
    }

    CJT808Terminal *pJT808Mng = dynamic_cast<CJT808Terminal *>(QueryPlugin("CJT808Terminal"));
    if (pJT808Mng)
    {
        // 遍历属性表,
        auto it = m_mapJT808Cfg.begin();
        for (; it != m_mapJT808Cfg.end(); it++)
        {
            // 存在差异时调用设置函数,设置变量
            if (it.value().cfg_value != it.value().cur_value)
            {
                // 将当前值同步到配置值
                const char *prop_name = it.key().toLocal8Bit().data();
                pJT808Mng->setProperty(prop_name, it.value().cur_value);
                it.value().cfg_value = it.value().cur_value;
                bChange = true;
            }
        }

        if (bChange)
        {
            pJT808Mng->saveConfig();
        }
    }

    return bChange;
}
