#include "FormLogin.h"
#include "ui_FormLogin.h"
#include <QMessageBox>
#include "base_module.h"
#include "./device/CDeviceManager.h"
#include <QtXml>
#include <QDir>
#include <QMessageBox>
#include "socket/CWebSocketClient.h"
#include "ui/setting/CDlgServSetting.h"
#include "device/CDeviceManager.h"

extern QString getMainMacAddress();

const QString LOGIN_DB_NAME = "login";

// const QString DEFAULT_SERV_ADDR = "ws://171.217.92.33:58006/integratedDevice/websocket/";
const QString DEFAULT_SERV_ADDR = "ws://171.217.92.33:58006/integratedDevice/websocket/";

//LOG_DEFINITION(FormLogin)

FormLogin::FormLogin(QWidget *parent) : QDialog(parent), 
						m_pDevMng(nullptr),
                        m_bLogined(false),
                        m_pWebSock(nullptr),
                        m_pDbLogin(nullptr),
                        m_pConfigMng(nullptr)
{
    // 查询websocket通信模块
    //LOG_CREATE(FormLogin, "web_login");
    m_pDevMng = dynamic_cast<CDeviceManager *>(::QueryPlugin("CDeviceManager"));

    connect(m_pDevMng, SIGNAL(notifyLogin(bool)), this, SLOT(onLogined(bool)));

    m_pConfigMng = dynamic_cast<CConfigManager *>(QueryPlugin("CConfigManager"));
    if (m_pConfigMng)
    {
        m_pDbLogin = m_pConfigMng->getDatabase(MODULE_UI_ADPT, LOGIN_DB_NAME);
        if (!m_pDbLogin)
        {
            createDatebase();
        }

        restoreFromDB(m_pDbLogin);
    }

    //QString serv_addr = m_pDevMng->getWsServaddr().toString();
	/*
	if (serv_addr.isEmpty())
    {
        CDlgServSetting dlg(nullptr);
        if (QDialog::Accepted == dlg.exec())
        {
            m_pDevMng->setWsServaddr(dlg.getServHost());
            m_pDevMng->saveConfig();
        }
    }
	*/
    // WS连接必须在主窗体初始化以后连接， 否则会连接不上，切记!!!
    m_pWebSock = dynamic_cast<CWebSocketClient *>(QueryPlugin("CWebSocketClient"));
    // 设置ws的连接信息
    if (m_pWebSock)
    {
        connect(m_pWebSock, &CWebSocketClient::notifyServConnected,
                this, &FormLogin::onConnected);
        m_pWebSock->connectToServer(genConnectAddr(m_pDevMng->getWsServaddr().toString()));
    }

    m_TimerMonitor.setInterval(2000);

    connect(&m_TimerMonitor, SIGNAL(timeout()), this, SLOT(onCheckLogin()));

    m_TimerMonitor.start();
}

FormLogin::~FormLogin()
{
    //delete ui;
    m_TimerMonitor.stop();
}

// 获取服务器地址（带设备ID）
QString FormLogin::genConnectAddr(QString strAddr)
{
    if (!strAddr.isEmpty() && strAddr.at(strAddr.size() - 1) != '/')
    {
        strAddr += '/';
    }
	QString strId = "12345678";
	if(m_pDevMng)
	{
		strId = m_pDevMng->getDeviceID();
	}
    return strAddr + strId;
}

// 创建数据库
bool FormLogin::createDatebase()
{
    m_pDbLogin = m_pConfigMng->regDatabase(MODULE_UI_ADPT, LOGIN_DB_NAME);
    m_pDbLogin->regKey("rem_user", true);
    m_pDbLogin->regKey("auto_login", true);
    m_pDbLogin->save();
}

// 从数据库恢复
bool FormLogin::restoreFromDB(CDatebase *pDatebase)
{
    
    bool bRemUser, bAutoLogin;

    strUser = m_pDevMng->getWsUsername().toString();
    strPwd = m_pDevMng->getWsPassword().toString();
    bRemUser = pDatebase->getValue("rem_user").toBool();
    bAutoLogin = pDatebase->getValue("auto_login").toBool();

    // 刷新到界面
    //ui->autoLogin->setChecked(bAutoLogin);
    //ui->remUser->setChecked(bRemUser);
    //ui->user_name->setText(strUser);
   // ui->psd->setText(strPwd);
    return true;
}

// 保存配置文件
void FormLogin::saveLoginCfg()
{
/*
    QString strUser, strPwd;
    bool bRemUser, bAutoLogin;
    if (ui->autoLogin->isChecked() == true)
    {
        // 保存用户名和密码到登陆配置文件
        bAutoLogin = true;
        bRemUser = true;
        strUser = ui->user_name->text();
        strPwd = ui->psd->text();
    }
    else
    {
        bAutoLogin = false;
        strPwd = "";

        if (ui->remUser->isChecked() == true)
        {
            bRemUser = true;
            strUser = ui->user_name->text();
        }
        else
        {
            bRemUser = false;
            strUser = "";
        }
    }

    m_pDevMng->setWsUsername(strUser);
    m_pDevMng->setWsPassword(strPwd);
    m_pDevMng->saveConfig();

    m_pDbLogin->setValue("rem_user", bRemUser);
    m_pDbLogin->setValue("auto_login", bAutoLogin);
    m_pDbLogin->save();
    */
}

// 处理登陆结果
void FormLogin::onLogined(bool bSuccess)
{
    if (bSuccess)
    {
       // saveLoginCfg();
        //执行多次，每半分钟查询一次在线设备
        CDeviceManager *pDevMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
        if (pDevMng)
        {
            pDevMng->getParams();
        }
        m_bLogined = true;
       // this->hide();
       // QDialog::done(1);
    }
    else
    {
        //ui->login_status->setText("用户名密码错误，请重新输入！");
        on_btnLogin_clicked();
    }

    //ui->btnLogin->setEnabled(true);
    
}

// 检查登录
void FormLogin::onCheckLogin()
{
	
    CDeviceManager *pDevMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
    if (pDevMng)
    {
        m_bLogined = pDevMng->isLogined();
		/*
        if (m_bLogined)
        {
            QDialog::done(1);
        }
        */
    }
}

// 登陆操作
void FormLogin::on_btnLogin_clicked()
{

/*
    if (ui->user_name->text().isEmpty() || ui->psd->text().isEmpty())
    {
        ui->login_status->setText("用户名密码错误，请重新输入！");
        return;
    }
*/
    if (!m_pDevMng->login(strUser, strPwd))
    {
        //如果连接未就绪，提示连接状态
        //ui->login_status->setText("连接未准备就绪，请稍后尝试.");
        return;
    }
}

// 连接处理
void FormLogin::onConnected()
{

        // 增加定时器用于超时处理
    QTimer::singleShot(10000, this, SLOT(onLoginTimeout()));

    //ui->login_status->setText("正在登陆中，请耐心等待...");
    // 最后去使能登陆按钮状态
    //ui->btnLogin->setEnabled(false);
	//EasyLog::Write("onConnected");
    on_btnLogin_clicked();
}

// 登陆超时处理
void FormLogin::onLoginTimeout()
{
    // 变更登陆按钮使能状态
    //ui->btnLogin->setEnabled(true);
    // 更新状态消息
   // ui->login_status->setText("登陆超时，请重新登陆.");
	on_btnLogin_clicked();
}

// 退出程序
void FormLogin::on_btnQuit_clicked()
{
    // 提示是否退出
    /*
    QMessageBox::StandardButton rb = QMessageBox::question(this, "退出系统", "是否退出当前系统?", QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes);
    if (rb == QMessageBox::Yes)
    {
        ::exit(1);
    }
	*/
    return;
}

// 设置自动登陆
void FormLogin::on_autoLogin_clicked(bool checked)
{
/*
    if (checked == true)
    {
       // ui->autoLogin->setChecked(true);
    }
    */
}

// 重载done函数，避免登陆窗口在未登陆的情况下，被手工取消而进行其他操作。


// 配置服务器按钮点击操作
void FormLogin::on_pushButton_clicked()
{
	/*

    if (m_pWebSock)
    {
        disconnect(m_pWebSock, &CWebSocketClient::notifyServConnected,
                   this, &FormLogin::onConnected);
    }
	
    CDlgServSetting dlg(m_pDevMng->getWsServaddr().toString(), this);
    if (QDialog::Accepted == dlg.exec())
    {
        m_pDevMng->setWsServaddr(dlg.getServHost());
        m_pDevMng->saveConfig();
    }
    */
}
